MODULE proginterface

!
! Questo modulo contiene funzioni di interfaccia tra vecchia gestione e nuova gestione
!

!
!                 Routine di interfaccia per la gestione del contenuto
!S nk_to_elem(elem)                      Trasferisco in elem le informazioni sul contenuto
!S elem_to_nk(elem)                      Prendo le informazioni sul contenuto da elem
!S fillcounts()                          Prendo informazioni sui conteggi da expo
!
implicit none

CONTAINS

   subroutine nk_to_elem(elem)
!
!  Trasferisco in elem le informazioni sul contenuto
!
   USE elements
   USE general
   type(element_type), dimension(:), intent(out), allocatable :: elem
   integer                                       :: i
!
   call reallocate_elem(elem,nk)
   do i=1,nk
      elem(i)%z = no(i)
      elem(i)%lab = Species(i)
      elem(i)%c_radius = radii(i,1)
      elem(i)%w_radius = radii(i,2)
      select case (neutro)
         case (0)
          elem(i)%ax(:) = al(:,i)
          elem(i)%bx(:) = bs(:,i)
          elem(i)%cx = cl(i)
         case (1)
          elem(i)%fact = cl(i)
         case (2)
          elem(i)%ae(:) = al(:,i)
          elem(i)%be(:) = bs(:,i)
          elem(i)%ce = cl(i)
      end select
      elem(i)%zeff = zeff(i)
      elem(i)%nw = nw(i)
      elem(i)%radtype = neutro
      !!mrelem(i)%name = nomeat(i)
      elem(i)%ptab = pxen_from_specie(elem(i)%lab)
      elem(i)%name = eleminfo(elem(i)%ptab)%name
   enddo
!
   end subroutine nk_to_elem

!---------------------------------------------------------------------      

   subroutine elem_to_nk(elem)
!
!  Prendo le informazioni sul contenuto da elem
!
   USE general
   USE Properties
   USE elements
   USE progtype
   type(element_type), dimension(:), intent(in) :: elem
   integer                                      :: i, j, k, iorde(18)
   real                                         :: noTmp(18), somma
   character(len=4)                             :: spec
   integer                                      :: lenspec, lennome
!
   nk = min(size(elem),NMAXSPECIE)
   lenspec = len(Species(1))  ! to remove warning with gfortran
   lennome = len(nomeat(1))
!
   noTmp(1:nk) = elem(1:nk)%z
   do j=1, nk
      iorde(j) = j
   enddo
   call sort_index(1, noTmp, iorde, nk)
   k = 0
   WhichProps = k
   AtmProperties(k)%ifMottFormula(:) = 0
   AtmProperties(k)%zeff(:) = 0
!
   do j=1,nk
      i = iorde(j)
      no(i) = elem(i)%z
      Species(i) = elem(i)%lab(1:lenspec)
      read(Species(i),'(a4)') nalf(i)
      radii(i,1) = elem(i)%c_radius
      radii(i,2) = elem(i)%w_radius
      !neutro = elem(i)%radtype
      select case (neutro)
         case (0)   ! Raggi X
           al(:,i) = elem(i)%ax
           bs(:,i) = elem(i)%bx
           cl(i) = elem(i)%cx 
         case (1)   ! Neutroni
           al(:,i) = 0
           bs(:,i) = 0
           cl(i) = elem(i)%fact 
         case (2)   ! Elettroni 
           al(:,i) = elem(i)%ae
           bs(:,i) = elem(i)%be
           cl(i) = elem(i)%ce 
      end select
      zeff(i) = elem(i)%zeff
      nw(i) = nint(elem(i)%nw)
      nomeat(i) = elem(i)%name(1:lennome)
!
      AtmProperties(k)%zeff(i)    = zeff(i)
      AtmProperties(k)%AtmW(i)    = elem(i)%weight
      AtmProperties(k)%z(i)       = no(i)
      AtmProperties(k)%natsInCell(i) = nw(i)
      AtmProperties(k)%radii(i,1) = radii(i,1)
      AtmProperties(k)%radii(i,2) = radii(i,1)
!
      AtmProperties(k)%a(1:4,i) = al(1:4,i)
      AtmProperties(k)%b(1:4,i) = bs(1:4,i)
      AtmProperties(k)%c(i)     = cl(i)
      AtmProperties(k)%fprimo(i,1)   = elem(i)%fMo(1)
      AtmProperties(k)%fsecondo(i,1) = elem(i)%fMo(2)
      AtmProperties(k)%fmuro(i)      = elem(i)%fMo(3)
      AtmProperties(k)%NomeSpecie(i) = nomeat(i)
      AtmProperties(k)%Species(i) = Species(i)
!
      somma = sum(abs(AtmProperties(k)%a(:,i)))
      somma = somma + sum(abs(AtmProperties(k)%b(:,i)))
      somma = somma + AtmProperties(k)%c(i)
      if (somma < 0.1) then
          AtmProperties(k)%ifMottFormula(k) = 1
          AtmProperties(k)%a(1:4,k) = elem(i)%ax
          AtmProperties(k)%b(1:4,k) = elem(i)%bx
          AtmProperties(k)%c(k)     = elem(i)%cx
      endif
!
   enddo
!
   nat = 0
   do i=1, nk
      if (AtmProperties(k)%z(i) > 1) then
          nat = nat + AtmProperties(k)%natsInCell(i)
      endif
   enddo
   AtmProperties(k)%nSpecies = nk
   AtmProperties(k)%nat = nat
!
   end subroutine elem_to_nk


!-------------------------------------------------------------------

   subroutine examing(ref1,ref2)
!
!  Calcola errore di phase (erore) e shift di fase (shiftexam)
!
   USE progtype
   USE General, only:knw
   USE Riflessi, only:mkang
   USE PhaseModule, only:ifExamin
   type(reflection_type), dimension(:), intent(in) :: ref1,ref2
   integer                                         :: nrefex
   integer                                         :: i,j
   integer, dimension(3,size(ref1))                :: khl
   integer                                         :: knwsav, ifExaminsav
   integer, dimension(:), allocatable              :: mkg
!
   nrefex = size(ref1)
   do i=1,nrefex
      khl(:,i) = ref1(i)%hkl
   enddo
   knwsav = knw
   ifExaminsav = ifExamin
   knw = 1        ! force call to examin
   ifExamin = 1   ! force call to examin
   if (allocated(mkang)) then
       call examin(1,mkang,khl,ref1(:)%ph,ref2(:)%ph,nrefex,0,(/(1.0,j=1,nrefex)/),0)
   else
       allocate(mkg(nrefex),source=1)
       call examin(1,mkg,khl,ref1(:)%ph,ref2(:)%ph,nrefex,0,(/(1.0,j=1,nrefex)/),0)
   endif
   knw = knwsav
   ifExamin = ifExaminsav
!
   end subroutine examing

END MODULE proginterface
