 MODULE polyhedra
!
!S find_polyhedra(atom,legm,poly,npoly)    Utilizza la connettivita' per cercare dei poliedri
!F function numpoly(poly)                  Il numero di poliedri
!S polyhedra_delete(poly,vpos)             Elimina poliedri indicati nel vettore vpos
!S copia_polyhedra(poly1,poly2)            Copia poly2 in poly1
!S update_polyhedra(poly,iorder)           Aggiorna i poliedri se il modello e' cambiato
!F number_of_faces(poly) result(nface)     Numero di facce del poliedro
!F is_in_poly(poly,kat) result(okpol)      L'atomo kat e' coinvolto in qualche poliedro
!S search_in_poly(poly,kat,pvet,np)        Restituisci in pvet tutti gli np poliedri che contengono l'atom kat
!S get_triangles(triang,npoint)            Calcola tutti i possibili triangoli 
!S print_poly(kpr,poly)                    Print polyhedra
!S reallocate_poly(vetr,n,savevet)         Rialloca ad n un vettore di tipo polyhedron_type
!
!corr USE atom_basic, only: atom_type
 USE progtype, only: symminfo_type

 implicit none

 type triangle_type
   integer :: n1=0,n2=0,n3=0
 end type triangle_type

 type polyhedron_type
   integer                           :: nvert = 0  ! numero di vertici del poliedro
   integer                           :: xc    = 0  ! centro del poliedro
   type(triangle_type), dimension(8) :: face       ! facce triangolari del poliedro
   real                              :: d_index = 0.0 ! distortion index

 contains

   procedure :: number_of_faces
 end type polyhedron_type


 CONTAINS

   integer function number_of_faces(poly)
   class(polyhedron_type), intent(in) :: poly
   select case (poly%nvert)
      case (4)
        number_of_faces = 4
      case (6)
        number_of_faces = 8
   end select
   end function number_of_faces

!--------------------------------------------------------------------------------------------------

   subroutine find_polyhedra_sym(atom,legm,cellt,atoms,legms,infos,poly,npoly,lsym)
!
!  Utilizza la connettivita' per cercare dei poliedri
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE realloc
   type(atom_type), dimension(:), allocatable, intent(in)               :: atom
   type(bond_type), dimension(:), allocatable, intent(in)               :: legm
   type(cell_type), intent(in)                                          :: cellt
   type(atom_type), dimension(:), allocatable, intent(inout)            :: atoms
   type(bond_type), dimension(:), allocatable, intent(inout)            :: legms
   type(symminfo_type), dimension(:), allocatable, intent(inout)        :: infos
   type(polyhedron_type), dimension(:), allocatable, intent(inout)      :: poly
   integer, intent(out)                                                 :: npoly
   logical, intent(in)                                                  :: lsym
   integer                                                              :: nat0,nleg0
   integer                                                              :: i,j
   integer, dimension(:), allocatable                                   :: iorder
   integer                                                              :: natsym
   logical                                                              :: lrem
   integer                                                              :: natadd
   !real                                                                 :: init_time,end_time
!
   !call cpu_time(init_time)
   !ortom = orthomatrix(cell)
   if (.not. lsym) then     ! non e' attiva la simmetria
       call copia_atoms(atoms,atom)
       call init_infos(infos,size(atoms))
       call copia_legm(legms,legm)
   endif
   nat0 = numatomtype(atoms)
   if (nat0 == 0) then ! possible if apply symmetry with 'all atoms fit'
       call reallocate_poly(poly,0)  ! delete all polyhedra if present
       return
   endif
   nleg0 = numbonds(legms)
!
!  Prova ad espandere i contatti
   call expand_contacts(atom,cellt,atoms,infos,legms)
     !write(0,*)'NUMAT=',numatomtype(atoms),nat0
!
!  Cerca poliedri
   call find_polyhedra(atoms,legms,cellt,poly,npoly)
!
!  Elimina gli atomi aggiunti che non sono coinvolti in poliedri
   natsym = numatomtype(atoms)
   natadd = natsym - nat0
   if (npoly > 0) then
       if (natadd > 0) then
           lrem = .false.
           do i=nat0+1,natsym
              do j=1,npoly
                 if (.not.is_in_poly(poly,i)) then
                     atoms(i)%nz = -999   ! marca l'atomo
                     infos(i)%oper = 0    ! marca infos
                     lrem = .true.
                 endif
              enddo
           enddo
           if (lrem) then  ! c'e' qualche atomo da rimuovere?
               allocate(iorder(natsym))
               call remove_atoms_from_list(atoms,atoms%nz,-999,legm=legms,iord=iorder)
               call update_polyhedra(poly,iorder)
               call remove_infos(infos)
           endif
       endif
   else
!
!      Se non ci sono poliedri, ripristina tutto
       if (natadd > 0) then
           call reallocate(atoms,nat0)
           call reallocate_legm(legms,nleg0)
           call reallocate_infos(infos,nat0)
       endif
   endif
     !write(0,*)'NUMAT=',numatomtype(atoms)
   !call print_poly(0,poly,atoms)
   !call cpu_time(end_time)
   !write(0,*)'time=',end_time-init_time
!
   end subroutine find_polyhedra_sym
 
!--------------------------------------------------------------------------------------------------

   subroutine find_polyhedra(atom,legm,cellt,poly,npoly)
!
!  Utilizza la connettivita' per cercare dei poliedri
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE nr
   USE unit_cell
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(cell_type), intent(in)                                     :: cellt
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, intent(out)                                            :: npoly
   type(atom_type), dimension(:), allocatable                      :: atomc
   integer                                                         :: natom
   integer                                                         :: nleg
   type(connection_type), dimension(:), allocatable                :: conn
   integer                                                         :: i,j,   k
   integer                                                         :: npolmax
   type(triangle_type), dimension(4)                               :: triangt
   type(triangle_type), dimension(20)                              :: triango
!!!!corr   type(triangle_type), dimension(8)                               :: triangord
   integer, dimension(3)                                           :: na
   real, dimension(20)                                             :: dist
   integer, dimension(20)                                          :: iord
   integer                                                         :: code, tpos, ntri
   real, dimension(4)                                              :: vp
   integer, dimension(6)                                           :: ntriang
!
   npoly = 0
   natom = numatomtype(atom)
   nleg = numbonds(legm)
!
   if (nleg > 0) then
!
!      genera permutazioni 
       call get_triangles(triangt,4)   ! per il tetraedro
       call get_triangles(triango,6)   ! per il ottaedro
!
       call legm_to_connect(natom,legm,conn)       ! estrai conn da legm
!
!      calcola il numero massimo di poliedri
       npolmax = count(conn(:)%nat == 4 .or. conn(:)%nat == 6)
       call reallocate_poly(poly,npolmax,.false.)  ! alloca poly sul numero massimo
!
!      converti in cartesiane
!corr       ortom = orthomatrix(cell)
       call frac_to_cart_copy(atom,atomc,cellt%get_ortom())
!
       do i=1,natom
          select case (conn(i)%nat)
             case (4)    ! tetrahedro
!
!              Controlla che il poliedro sia chimicamente plausibile
               call check_atoms_poly(i,conn(i)%pos,atomc,code)
               if (code  == 0) then  ! il poliedro e' buono se code = 0
!
!                  Aggiungi nuovo poliedro
                   npoly = npoly + 1
                   poly(npoly)%nvert = conn(i)%nat
                   poly(npoly)%xc = i
                   do j=1,4
                      poly(npoly)%face(j)%n1 = conn(i)%pos(triangt(j)%n1)
                      poly(npoly)%face(j)%n2 = conn(i)%pos(triangt(j)%n2)
                      poly(npoly)%face(j)%n3 = conn(i)%pos(triangt(j)%n3)
                   enddo
               endif
               
             case (6)    ! ottaedro
!
!              Controlla che il poliedro sia chimicamente plausibile
               call check_atoms_poly(i,conn(i)%pos,atomc,code)
               if (code  == 0) then  ! il poliedro e' buono se code = 0
!
!                  calcola distanza baricentro-centro poliedro per tutte le facce
                   do j=1,20
                      na(1) = conn(i)%pos(triango(j)%n1)
                      na(2) = conn(i)%pos(triango(j)%n2)
                      na(3) = conn(i)%pos(triango(j)%n3)
                      vp(:) = plane3points(atomc(na(1))%xc,atomc(na(2))%xc,atomc(na(3))%xc)
!
!                     after structure cagly_true.cif medium distance from all vertices was chosen
                      dist(j) = 0
                      do k=1,6
                         dist(j) = dist(j) + abs(dot_product(vp(:3),atomc(conn(i)%pos(k))%xc)+vp(4))
                      enddo
                      !dist(j) = abs(dot_product(vp(:3),atomc(i)%xc)+vp(4))
                   enddo
!
!                  scegli le 8 facce che massimizzano la distanza e con atomi coinvolti in 4 facce
                   call indexx(dist,iord)
                   iord(:) = iord(20:1:-1)
!
                   npoly = npoly + 1
                   poly(npoly)%nvert = conn(i)%nat
                   poly(npoly)%xc = i
                   ntriang(:) = 0   ! save the number of triangle for atom (must be == 4)
                   ntri = 0
                   do j=1,20
                      tpos = iord(j)
                      !write(0,*)'valuto tri ',tpos,triango(tpos)%n1,triango(tpos)%n2,triango(tpos)%n3
                      if (ntriang(triango(tpos)%n1) + 1 > 4) cycle
                      if (ntriang(triango(tpos)%n2) + 1 > 4) cycle
                      if (ntriang(triango(tpos)%n3) + 1 > 4) cycle
                      ntriang(triango(tpos)%n1) = ntriang(triango(tpos)%n1) + 1
                      ntriang(triango(tpos)%n2) = ntriang(triango(tpos)%n2) + 1
                      ntriang(triango(tpos)%n3) = ntriang(triango(tpos)%n3) + 1
                      na(1) = conn(i)%pos(triango(tpos)%n1)
                      na(2) = conn(i)%pos(triango(tpos)%n2)
                      na(3) = conn(i)%pos(triango(tpos)%n3)
                      ntri = ntri + 1
                      poly(npoly)%face(ntri)%n1 = na(1)
                      poly(npoly)%face(ntri)%n2 = na(2)
                      poly(npoly)%face(ntri)%n3 = na(3)
                      !write(0,*)ntri,'face=',na(:)
                      if (ntri == 8) exit
                   enddo
                   if (ntri /= 8) npoly = npoly - 1  ! Cannot find 8 faces: polyhedra rejected
               endif
          end select
       enddo
   endif
   call reallocate_poly(poly,npoly,.true.) ! ricompatta poly
!
   end subroutine find_polyhedra

!--------------------------------------------------------------------------------------------------

#if 0
   subroutine find_polyhedra_eq(polyau,npolyau,poly,npoly,infosau,atoms,infos,legms)
   USE atom_type_util
   USE connect_mod
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: polyau
   integer, intent(in)                                          :: npolyau
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, intent(inout)                                          :: npoly
   type(symminfo_type), dimension(:), allocatable, intent(in)      :: infosau
   type(atom_type), dimension(:), allocatable, intent(in)          :: atoms
   type(symminfo_type), dimension(:), allocatable, intent(inout)   :: infos
   type(bond_type), dimension(:), allocatable, intent(in)          :: legms
   type(connection_type), dimension(:), allocatable                :: conns
   integer :: natom, npoly0
   integer :: i,j,nf,nc
!
   natom = numatomtype(atoms)
   call legm_to_connect(natom,legms,conns)       ! estrai conn da legm
       call reallocate_poly(poly,50,.true.)  !FIXME
   npoly = 0
   do i=1,natom
      do j=1,npolyau
         if (infos(i)%asym == infosau(poly(j)%xc)%asym .and. conns(i)%nat == polyau(j)%nvert) then
          write(0,*)'ATOM ',i,'CONN=',conns(i)%pos
          write(0,*)'ATOM ',i,'ASYM=',infos(conns(i)%pos)%asym
             npoly = npoly + 1
             poly(npoly)%nvert = polyau(j)%nvert
             poly(npoly)%xc = i
             do nf=1,poly(j)%number_of_faces()
                write(0,*)'ASYM faccia:',infosau(polyau(j)%face(nf)%n1)%asym,infosau(polyau(j)%face(nf)%n2)%asym,infosau(polyau(j)%face(nf)%n3)%asym
                do nc=1,conns(i)%nat
                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n1)%asym) then
                       poly(npoly)%face(nf)%n1 = conns(i)%pos(nc)
                       cycle
                   endif
                enddo
                do nc=1,conns(i)%nat
                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n2)%asym) then
                       poly(npoly)%face(nf)%n2 = conns(i)%pos(nc)
                       cycle
                   endif
                enddo
                do nc=1,conns(i)%nat
                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n3)%asym) then
                       poly(npoly)%face(nf)%n3 = conns(i)%pos(nc)
                       cycle
                   endif
                enddo
                write(0,*)'faccia n.',nf,poly(npoly)%face(nf)%n1,poly(npoly)%face(nf)%n2,poly(npoly)%face(nf)%n3
             enddo
          !   if (poly%num_face())
!             poly(npoly) = poly
             write(0,*)'probabile poliedro ',j,' su atomo ',slabnum(atoms(i)%lab,i)
             write(0,*)'at=',conns(i)%nat
         endif
      enddo
   enddo
!
   call reallocate_poly(poly,npoly,.true.) ! ricompatta poly
!
   end subroutine find_polyhedra_eq
#endif

!--------------------------------------------------------------------------------------------------

!corr   function poly_eq(poly,infos,xc,vconn)
!corr   type(polyhedron_type), intent(in) :: poly
!corr   type(polyhedron_type)             :: poly_eq
!corr!
!corr!
!corr   end function poly_eq

!--------------------------------------------------------------------------------------------------

   subroutine check_atoms_poly(kat,vet,atomc,code)
   USE atom_type_util
   integer, intent(in)                         :: kat
   integer, dimension(:), intent(in)           :: vet
   type(atom_type), dimension(:), intent(in)   :: atomc  ! coordinate cartesiane
   integer, intent(out)                        :: code
!
   call check_atoms(kat,vet,atomc,code)  ! check sugli angoli
   if (code == 0) then                   ! check sulle specie
!
!      non accettare poliedri con non-metallo al centro e metallo ai vertici
       if (.not.is_metal(atomc(kat)) .and. all(is_metal(atomc(vet)))) code = 1
   endif
!
   end subroutine check_atoms_poly

!--------------------------------------------------------------------------------------------------

   integer function numpoly(poly)
!
!  Il numero di poliedri
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
!
   if (allocated(poly)) then
       numpoly = size(poly)
   else
       numpoly = 0
   endif
!
   end function numpoly

!--------------------------------------------------------------------------------------------------

   subroutine polyhedra_delete(poly,vpos)
!
!  Elimina poliedri indicati nel vettore vpos
!
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, dimension(:), intent(in)                               :: vpos
   integer                                                         :: npoly
   type(polyhedron_type), dimension(:), allocatable                :: polynew
   integer                                                         :: nrem
   integer                                                         :: i
   integer                                                         :: newpoly
!
   npoly = numpoly(poly)
   nrem = size(vpos)
   if (npoly > 0 .and. nrem > 0) then
       newpoly = npoly - nrem
       call reallocate_poly(polynew,newpoly)
       newpoly = 0
       do i=1,npoly
          if (any(vpos == i)) cycle
          newpoly = newpoly + 1
          polynew(newpoly) = poly(i)
       enddo
   endif
   call copia_polyhedra(poly,polynew)
!
   end subroutine polyhedra_delete

!--------------------------------------------------------------------------------------------------

   subroutine copia_polyhedra(poly1,poly2)
!
!  Copia poly2 in poly1
!
   USE realloc
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly1
   type(polyhedron_type), dimension(:), allocatable, intent(in)    :: poly2
   integer                                                         :: npoly2
!
   npoly2 = numpoly(poly2)
   call reallocate_poly(poly1,npoly2,savevet=.false.)
   if (npoly2 > 0) then
       poly1(:) = poly2(:)
   endif
!
   end subroutine copia_polyhedra

!--------------------------------------------------------------------------------------------------

   subroutine update_polyhedra(poly,iorder)
!
!  Aggiorna i poliedri se il modello e' cambiato
!  iorder(vecchia poisizione dell'atomo) = nuova posizione
!  iorder(vecchia poisizione dell'atomo) = 0 -> atomo cancellato
!
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, dimension(:), intent(in)                               :: iorder
   integer                                                         :: npoly
   integer                                                         :: nat
   integer                                                         :: i,j
   integer, dimension(size(poly))                                  :: pvet
   integer                                                         :: np
!corr   integer                                                         :: nface
!
   npoly = numpoly(poly)
   if (npoly > 0) then
!
!      Rimuovi i poliedri che contengono atomi cancellati
       nat = size(iorder)
       do i=1,nat
          if (iorder(i) == 0) then                               ! atomo i eliminato
              call search_in_poly(poly,i,pvet,np)                ! cerca i nel poliedro
              if (np > 0) call polyhedra_delete(poly,pvet(:np))  ! cancella i poliedri in pvet         
          endif
       enddo
!
!      Aggiorna il numero d'ordine degli atomi
       do i=1,numpoly(poly)
          poly(i)%xc = iorder(poly(i)%xc)
!corr          nface = number_of_faces(poly(i))
          do j=1,poly(i)%number_of_faces()
             poly(i)%face(j)%n1 = iorder(poly(i)%face(j)%n1)
             poly(i)%face(j)%n2 = iorder(poly(i)%face(j)%n2)
             poly(i)%face(j)%n3 = iorder(poly(i)%face(j)%n3)
          enddo
       enddo
   endif
!
   end subroutine update_polyhedra

!--------------------------------------------------------------------------------------------------
  
#if 0
   integer function number_of_faces(poly) result(nface)
!
!  Numero di facce del poliedro
!
   type(polyhedron_type) :: poly
!
   select case (poly%nvert)
      case (4)
        nface = 4
      case (6)
        nface = 8
   end select
!
   end function number_of_faces
#endif

!--------------------------------------------------------------------------------------------------

   logical function is_in_poly(poly,kat) result(okpol)
!
!  L'atomo kat e' coinvolto in qualche poliedro
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   integer, intent(in)                                          :: kat
!corr   integer                                                      :: nface
   integer                                                      :: i,j
!
   okpol = .false.
   do i=1,numpoly(poly)
!corr      nface = number_of_faces(poly(i))
      okpol = poly(i)%xc == kat     ! atomo centrale?
      if (.not.okpol) then          ! ... o un atomo sulle facce
          do j=1,poly(i)%number_of_faces()
             if (poly(i)%face(j)%n1 == kat .or. poly(i)%face(j)%n2 == kat     &
                 .or. poly(i)%face(j)%n3 == kat) then
                 okpol = .true.
                 exit
             endif
          enddo
      endif
      if (okpol) exit
   enddo 
!
   end function is_in_poly

!--------------------------------------------------------------------------------------------------

   subroutine search_in_poly(poly,kat,pvet,np)
!
!  Restituisci in pvet tutti gli np poliedri che contengono l'atom kat
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   integer, intent(in)                                          :: kat
   integer, dimension(size(poly)), intent(out)                  :: pvet
   integer, intent(out)                                         :: np
   integer                                                      :: i,j
!corr   integer                                                      :: nface
   logical                                                      :: okpol
!
   np = 0
   do i=1,numpoly(poly)
!corr      nface = number_of_faces(poly(i))
      okpol = poly(i)%xc == kat     ! atomo centrale?
      if (.not.okpol) then          ! o un atomo sulle facce
          do j=1,poly(i)%number_of_faces()
             if (poly(i)%face(j)%n1 == kat .or. poly(i)%face(j)%n2 == kat     &
                 .or. poly(i)%face(j)%n3 == kat) then
                 okpol = .true.
                 exit
             endif
          enddo
      endif
      if (okpol) then
          np = np + 1
          pvet(np) = i
      endif
   enddo
!
   end subroutine search_in_poly

!--------------------------------------------------------------------------------------------------
  
   subroutine get_triangles(triang,npoint)
!
!  Calcola tutti i possibili triangoli 
!  
   type(triangle_type), dimension(:), intent(inout) :: triang
   integer, intent(in)                              :: npoint
   integer                                          :: k
   logical                                          :: mtc
   integer, dimension(npoint)                       :: vet
   integer                                          :: nt
!
   k = 3
   mtc = .false.
   nt = 0
   do 
     call nexksb(npoint,k,vet,mtc)
     nt = nt + 1
     triang(nt)%n1 = vet(1)
     triang(nt)%n2 = vet(2)
     triang(nt)%n3 = vet(3)
     if (.not.mtc) exit
   enddo
!
   end subroutine get_triangles

!--------------------------------------------------------------------------------------------------

   subroutine print_poly(kpr,poly,atom,info)
!
!  Print polyhedra
!
   USE atom_basic
   integer, intent(in)                                          :: kpr
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   type(atom_type), dimension(:), allocatable                   :: atom
   type(symminfo_type), dimension(:), allocatable, intent(inout) :: info
   integer                                                      :: i,j
!corr   integer                                                      :: nface
   integer :: numau, asym, npoly
   integer, dimension(:), allocatable :: vetd
!
   npoly = numpoly(poly)
   if (npoly == 0) return
!
   write(kpr,'("Tutti i poliedri")')
   do i=1,numpoly(poly)
      write(kpr,'(a,i5,a,i5,a,a,i5)')'nvert=',poly(i)%nvert,' cent=',poly(i)%xc,' at='//trim(atom(poly(i)%xc)%lab),  &
                  ' asym=',info(poly(i)%xc)%asym
                 cycle
!corr      if (poly(i)%nvert == 4) then
!corr          nface = 4
!corr      else
!corr          nface = 8
!corr      endif
      do j=1,poly(i)%number_of_faces()
         write(kpr,*)'face ',j,poly(i)%face(j)
      enddo
   enddo
!
   write(kpr,'("Poliedri distinti")')
   numau = maxval(info(poly(:)%xc)%asym)
   allocate(vetd(numau),source=0)
   do i=1,numpoly(poly)
      asym = info(poly(i)%xc)%asym
      if (asym <= numau .and. vetd(asym) == 0) then
          vetd(asym) = i
          write(kpr,'(a,i5,a,i5,a)')'nvert=',poly(i)%nvert,' cent=',poly(i)%xc,' at='//trim(atom(poly(i)%xc)%lab)
      endif
  enddo
!
  end subroutine print_poly

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_poly(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo polyhedron_type
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(polyhedron_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                               :: n
   logical, optional, intent(in)                     :: savevet
   logical                                           :: savev
   integer                                           :: nv
   type(polyhedron_type), allocatable                :: vsav(:)
   integer                                           :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
           allocate(vsav(n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
!
!          salva vetr fino a nsav
!!!corr           allocate(vsav(nsav))
!!!corr           vsav = vetr(:nsav)
!
!          dealloca vetr
!!!corr           deallocate(vetr)
!
!          rialloca vetr su n
!!!corr           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
!!!corr           vetr(:nsav) = vsav(:nsav)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_poly
   
!--------------------------------------------------------------------------------------------------

   subroutine nexksb(n,k,a,mtc)
!
!  From Chapter 3: Next k-subset of an n-set(p32) - Combinatorial Algorithms
!  Algorithm:Generating k-subset of {1,2,...,n} of an 
!          n-set, in lexicographical order. 
!
   integer a(k) 
   logical mtc
   integer, save :: m2,h
   integer :: k,n
   integer :: i,j
   if(k.gt.0)goto 30
   do 1 i=1,n
 1 a(i)=0
   mtc=.false.
   return
30 if(mtc) go to 40
   m2=0
   h=k
   go to 50
40 if(m2.lt.n-h) h=0
   h=h+1
   m2=a(k+1-h)
50 do 51 j=1,h
51 a(k+j-h)=m2+j
   mtc=a(1).ne.n-k+1
   return
   end subroutine nexksb

 END MODULE polyhedra
