!--------------------------------------------------------------------------------------------
        MODULE nrtype
               INTEGER, PARAMETER  :: I4B = SELECTED_INT_KIND(9)
               INTEGER, PARAMETER  :: SP  = KIND(1.0)
               INTEGER, PARAMETER  :: DP  = KIND(1.0d0)
               REAL(DP), PARAMETER :: PI_D=3.141592653589793238462643383279502884197_dp
        END MODULE nrtype
!--------------------------------------------------------------------------------------------
        MODULE nr

        INTERFACE
            FUNCTION chebev(a,b,c,x)
            USE nrtype
            REAL(SP), INTENT(IN) :: a,b,x
            REAL(SP), DIMENSION(:), INTENT(IN) :: c
            REAL(SP) :: chebev
            END FUNCTION chebev
        END INTERFACE
        INTERFACE
            FUNCTION chebft(a,b,n,func)
            USE nrtype
            REAL(SP), INTENT(IN) :: a,b
            INTEGER(I4B), INTENT(IN) :: n
            REAL(SP), DIMENSION(n) :: chebft
            INTERFACE
            FUNCTION func(x)
            USE nrtype
            implicit none
            real(sp), dimension(:), intent(in) :: x
            real(sp), dimension(size(x)) :: func
            END FUNCTION func
            END INTERFACE
            END FUNCTION chebft
        END INTERFACE
        INTERFACE
            FUNCTION chder(a,b,c)
            USE nrtype
            REAL(SP), INTENT(IN) :: a,b
            REAL(SP), DIMENSION(:), INTENT(IN) :: c
            REAL(SP), DIMENSION(size(c)) :: chder
            END FUNCTION chder
        END INTERFACE
        INTERFACE
            SUBROUTINE choldc(a,p,ier)
            USE nrtype
            REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
            REAL(DP), DIMENSION(:), INTENT(OUT) :: p
            INTEGER, INTENT(INOUT) :: ier
            END SUBROUTINE choldc
        END INTERFACE
        INTERFACE
            FUNCTION invltrimatrix(n,a,p)
            USE nrtype
            implicit none
            integer, intent(in)                  :: n
            real(dp), dimension(n,n), intent(in) :: a
            real(dp), dimension(n), intent(in)   :: p
            real(dp), dimension(n,n)             :: invltrimatrix
            END FUNCTION invltrimatrix
        END INTERFACE
!!mr        INTERFACE
!!mr            SUBROUTINE inverti_lu(mato,matinv,ier)
!!mr            USE nrtype
!!mr            real(dp), dimension(:,:), intent(in)                        :: mato
!!mr            real(dp), dimension(size(mato,2),size(mato,2)), intent(out) :: matinv
!!mr            real(dp), dimension(size(mato,2),size(mato,2))              :: mat
!!mr            integer(i4b), dimension(size(mato,2))                       :: indx
!!mr            integer                                                     :: ier
!!mr            END SUBROUTINE inverti_lu
!!mr        END INTERFACE
        INTERFACE
            SUBROUTINE cholsl(a,p,b,x)
            USE nrtype
            REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
            REAL(DP), DIMENSION(:), INTENT(IN) :: p,b
            REAL(DP), DIMENSION(:), INTENT(INOUT) :: x
            END SUBROUTINE cholsl
        END INTERFACE
        INTERFACE
            FUNCTION dfridr(func,x,h,err)
            USE nrtype
            REAL(SP), INTENT(IN) :: x,h
            REAL(SP), INTENT(OUT) :: err
            REAL(SP) :: dfridr
            INTERFACE
            FUNCTION func(x)
            USE nrtype
            REAL(SP), INTENT(IN) :: x
            REAL(SP) :: func
            END FUNCTION func
            END INTERFACE
            END FUNCTION dfridr
        END INTERFACE
        INTERFACE indexx
            SUBROUTINE indexx_sp(arr,index)
            USE nrtype
            REAL(SP), DIMENSION(:), INTENT(IN) :: arr
            INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: index
            END SUBROUTINE indexx_sp
            SUBROUTINE indexx_i4b(iarr,index)
            USE nrtype
            INTEGER(I4B), DIMENSION(:), INTENT(IN) :: iarr
            INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: index
            END SUBROUTINE indexx_i4b
        END INTERFACE
        INTERFACE
            SUBROUTINE lubksb_new(a,indx,b)
            USE nrtype
            REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
            INTEGER(I4B), DIMENSION(:), INTENT(IN) :: indx
            REAL(DP), DIMENSION(:), INTENT(INOUT) :: b
            END SUBROUTINE lubksb_new
        END INTERFACE
        INTERFACE
            SUBROUTINE ludcmp_new(a,indx,d,ier)
            USE nrtype
            REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
            INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: indx
            REAL(DP), INTENT(OUT) :: d
            INTEGER, INTENT(OUT) :: ier
            END SUBROUTINE ludcmp_new
        END INTERFACE

        END MODULE nr
!--------------------------------------------------------------------------------------------
        MODULE nrutil
          USE nrtype
          IMPLICIT NONE
          integer(I4B), parameter :: NPAR_ARTH=16,NPAR2_ARTH=8
          INTEGER(I4B), PARAMETER :: NPAR_GEOP=4,NPAR2_GEOP=2
          INTEGER(I4B), PARAMETER :: NPAR_CUMSUM=16
          INTERFACE arth
                  MODULE PROCEDURE arth_r, arth_d, arth_i
          END INTERFACE
          INTERFACE cumsum
                  MODULE PROCEDURE cumsum_r,cumsum_i
          END INTERFACE
          INTERFACE swap
                  MODULE PROCEDURE swap_i,swap_r,swap_d,swap_rv
          END INTERFACE
          INTERFACE assert
                  MODULE PROCEDURE assert1
          END INTERFACE
          INTERFACE assert_eq
                  MODULE PROCEDURE assert_eq2,assert_eq3,assert_eqn
          END INTERFACE
          INTERFACE geop
                  MODULE PROCEDURE geop_r
          END INTERFACE

          CONTAINS

               FUNCTION iminloc(arr)
               REAL(SP), DIMENSION(:), INTENT(IN) :: arr
               INTEGER(I4B), DIMENSION(1) :: imin
               INTEGER(I4B) :: iminloc
               imin=minloc(arr(:))
               iminloc=imin(1)
               END FUNCTION iminloc

               SUBROUTINE assert1(n1,string)
               CHARACTER(LEN=*), INTENT(IN) :: string
               LOGICAL, INTENT(IN) :: n1
               if (.not. n1) then
                   write (*,*) 'nrerror: an assertion failed with this tag:',string
                       !'
                   call cstop('program terminated by assert1')
               end if
               END SUBROUTINE assert1

               FUNCTION assert_eq2(n1,n2,string)
               CHARACTER(LEN=*), INTENT(IN) :: string
               INTEGER, INTENT(IN) :: n1,n2
               INTEGER :: assert_eq2
               if (n1 == n2) then
                   assert_eq2=n1
               else
                   write (*,*) 'nrerror: an assert_eq failed with this tag:',string
                   call cstop('program terminated by assert_eq2')
               end if
               END FUNCTION assert_eq2

               FUNCTION assert_eq3(n1,n2,n3,string)
               CHARACTER(LEN=*), INTENT(IN) :: string
               INTEGER, INTENT(IN) :: n1,n2,n3
               INTEGER :: assert_eq3
               if (n1 == n2 .and. n2 == n3) then
                   assert_eq3=n1
               else
                   write (*,*) 'nrerror: an assert_eq failed with this tag:',string
                   call cstop('program terminated by assert_eq3')
               end if
               END FUNCTION assert_eq3

               FUNCTION assert_eqn(nn,string)
               CHARACTER(LEN=*), INTENT(IN) :: string
               INTEGER, DIMENSION(:), INTENT(IN) :: nn
               INTEGER :: assert_eqn
               if (all(nn(2:) == nn(1))) then
                   assert_eqn=nn(1)
               else
                   write (*,*) 'nrerror: an assert_eq failed with this tag:',string
                   call cstop('program terminated by assert_eqn')
               end if
               END FUNCTION assert_eqn

               FUNCTION geop_r(first,factor,n)
               REAL(SP), INTENT(IN) :: first,factor
               INTEGER(I4B), INTENT(IN) :: n
               REAL(SP), DIMENSION(n) :: geop_r
               INTEGER(I4B) :: k,k2
               REAL(SP) :: temp
               if (n > 0) geop_r(1)=first
               if (n <= NPAR_GEOP) then
                   do k=2,n
                      geop_r(k)=geop_r(k-1)*factor
                   end do
               else
                   do k=2,NPAR2_GEOP
                      geop_r(k)=geop_r(k-1)*factor
                   end do
                   temp=factor**NPAR2_GEOP
                   k=NPAR2_GEOP
                   do
                     if (k >= n) exit
                     k2=k+k
                     geop_r(k+1:min(k2,n))=temp*geop_r(1:min(k,n-k))
                     temp=temp*temp
                     k=k2
                   end do
               end if
               END FUNCTION geop_r

               function imaxloc(arr)
               real(dp), dimension(:), intent(in) :: arr
               integer(i4b) :: imaxloc
               integer(i4b), dimension(1) :: imax
               imax = maxloc(arr(:))
               imaxloc=imax(1)
               end function imaxloc

               function outerprod(a,b)
               real(dp), dimension(:), intent(in) :: a,b
               real(dp), dimension(size(a),size(b)) :: outerprod
               outerprod = spread(a,dim=2,ncopies=size(b)) * &
                       spread(b,dim=1,ncopies=size(a))
               end function outerprod

               SUBROUTINE swap_i(a,b)
               INTEGER(I4B), INTENT(INOUT) :: a,b
               INTEGER(I4B) :: dum
               dum=a
               a=b
               b=dum
               END SUBROUTINE swap_i

               SUBROUTINE swap_r(a,b)
               REAL(SP), INTENT(INOUT) :: a,b
               REAL(SP) :: dum
               dum=a
               a=b
               b=dum
               END SUBROUTINE swap_r

               subroutine swap_d(a,b)
               real(dp), dimension(:), intent(inout) :: a,b
               real(dp), dimension(size(a)) :: dum
               dum = a
               a=b
               b=dum
               end subroutine swap_d

               SUBROUTINE swap_rv(a,b)
               REAL(SP), DIMENSION(:), INTENT(INOUT) :: a,b
               REAL(SP), DIMENSION(SIZE(a)) :: dum
               dum=a
               a=b
               b=dum
               END SUBROUTINE swap_rv

               FUNCTION arth_r(first,increment,n)
               REAL(SP), INTENT(IN) :: first,increment
               INTEGER(I4B), INTENT(IN) :: n
               REAL(SP), DIMENSION(n) :: arth_r
               INTEGER(I4B) :: k,k2
               REAL(SP) :: temp
               if (n > 0) arth_r(1)=first
               if (n <= NPAR_ARTH) then
                        do k=2,n
                           arth_r(k)=arth_r(k-1)+increment
                        end do
               else
                        do k=2,NPAR2_ARTH
                           arth_r(k)=arth_r(k-1)+increment
                        end do
                        temp=increment*NPAR2_ARTH
                        k=NPAR2_ARTH
                        do
                           if (k >= n) exit
                           k2=k+k
                           arth_r(k+1:min(k2,n))=temp+arth_r(1:min(k,n-k))
                           temp=temp+temp
                           k=k2
                        end do
               end if
               END FUNCTION arth_r

               FUNCTION arth_d(first,increment,n)
               REAL(DP), INTENT(IN) :: first,increment
               INTEGER(I4B), INTENT(IN) :: n
               REAL(DP), DIMENSION(n) :: arth_d
               INTEGER(I4B) :: k,k2
               REAL(DP) :: temp
               if (n > 0) arth_d(1)=first
               if (n <= NPAR_ARTH) then
                        do k=2,n
                           arth_d(k)=arth_d(k-1)+increment
                        end do
               else
                        do k=2,NPAR2_ARTH
                           arth_d(k)=arth_d(k-1)+increment
                        end do
                        temp=increment*NPAR2_ARTH
                        k=NPAR2_ARTH
                        do
                          if (k >= n) exit
                          k2=k+k
                          arth_d(k+1:min(k2,n))=temp+arth_d(1:min(k,n-k))
                          temp=temp+temp
                          k=k2
                        end do
               end if
               END FUNCTION arth_d

               FUNCTION arth_i(first,increment,n)
               INTEGER(I4B), INTENT(IN) :: first,increment,n
               INTEGER(I4B), DIMENSION(n) :: arth_i
               INTEGER(I4B) :: k,k2,temp
               if (n > 0) arth_i(1)=first
               if (n <= NPAR_ARTH) then
                        do k=2,n
                           arth_i(k)=arth_i(k-1)+increment
                        end do
               else
                        do k=2,NPAR2_ARTH
                           arth_i(k)=arth_i(k-1)+increment
                        end do
                        temp=increment*NPAR2_ARTH
                        k=NPAR2_ARTH
                        do
                           if (k >= n) exit
                           k2=k+k
                           arth_i(k+1:min(k2,n))=temp+arth_i(1:min(k,n-k))
                           temp=temp+temp
                           k=k2
                        end do
               end if
               END FUNCTION arth_i

               RECURSIVE FUNCTION cumsum_r(arr,seed) RESULT(ans)
               REAL(SP), DIMENSION(:), INTENT(IN) :: arr
               REAL(SP), OPTIONAL, INTENT(IN) :: seed
               REAL(SP), DIMENSION(size(arr)) :: ans
               INTEGER(I4B) :: n,j
               REAL(SP) :: sd
               n=size(arr)
               if (n == 0_i4b) RETURN
               sd=0.0_sp
               if (present(seed)) sd=seed
               ans(1)=arr(1)+sd
               if (n < NPAR_CUMSUM) then
                       do j=2,n
                          ans(j)=ans(j-1)+arr(j)
                       end do
               else
                       ans(2:n:2)=cumsum_r(arr(2:n:2)+arr(1:n-1:2),sd)
                       ans(3:n:2)=ans(2:n-1:2)+arr(3:n:2)
               end if
               END FUNCTION cumsum_r

               RECURSIVE FUNCTION cumsum_i(arr,seed) RESULT(ans)
               INTEGER(I4B), DIMENSION(:), INTENT(IN) :: arr
               INTEGER(I4B), OPTIONAL, INTENT(IN) :: seed
               INTEGER(I4B), DIMENSION(size(arr)) :: ans
               INTEGER(I4B) :: n,j,sd
               n=size(arr)
               if (n == 0_i4b) RETURN
               sd=0_i4b
               if (present(seed)) sd=seed
               ans(1)=arr(1)+sd
               if (n < NPAR_CUMSUM) then
                       do j=2,n
                          ans(j)=ans(j-1)+arr(j)
                       end do
               else
                       ans(2:n:2)=cumsum_i(arr(2:n:2)+arr(1:n-1:2),sd)
                       ans(3:n:2)=ans(2:n-1:2)+arr(3:n:2)
               end if
               END FUNCTION cumsum_i

               SUBROUTINE nrerror(string)
               CHARACTER(LEN=*), INTENT(IN) :: string
               write (*,*) 'nrerror: ',string
               call cstop('program terminated by nrerror')
               END SUBROUTINE nrerror

        END MODULE nrutil
!--------------------------------------------------------------------------------------------
               SUBROUTINE ludcmp_new(a,indx,d,ier)
               USE nrtype; USE nrutil, ONLY : assert_eq,imaxloc,outerprod,swap
               IMPLICIT NONE
               REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
               INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: indx
               REAL(DP), INTENT(OUT) :: d
               INTEGER, INTENT(OUT) :: ier
               REAL(DP), DIMENSION(size(a,1)) :: vv
               REAL(DP), PARAMETER :: TINY=1.0e-20_dp
               INTEGER(I4B) :: j,n,imax
               ier = 0
               n=assert_eq(size(a,1),size(a,2),size(indx),'ludcmp')
               d=1.0
               vv=maxval(abs(a),dim=2)
               if (any(vv == 0.0)) then
                   ier = -1           ! Singular matrix
                   return
               endif
               vv=1.0_dp/vv
               do j=1,n
                       imax=(j-1)+imaxloc(vv(j:n)*abs(a(j:n,j)))
                       if (j /= imax) then
                               call swap(a(imax,:),a(j,:))
                               d=-d
                               vv(imax)=vv(j)
                       end if
                       indx(j)=imax
                       if (a(j,j) == 0.0) a(j,j)=TINY
                       a(j+1:n,j)=a(j+1:n,j)/a(j,j)
                       a(j+1:n,j+1:n)=a(j+1:n,j+1:n)-outerprod(a(j+1:n,j),a(j,j+1:n))
               end do
               END SUBROUTINE ludcmp_new

               SUBROUTINE lubksb_new(a,indx,b)
               USE nrtype; USE nrutil, ONLY : assert_eq
               IMPLICIT NONE
               REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
               INTEGER(I4B), DIMENSION(:), INTENT(IN) :: indx
               REAL(DP), DIMENSION(:), INTENT(INOUT) :: b
               INTEGER(I4B) :: i,n,ii,ll
               REAL(DP) :: summ
               n=assert_eq(size(a,1),size(a,2),size(indx),'lubksb')
               ii=0
               do i=1,n
                       ll=indx(i)
                       summ=b(ll)
                       b(ll)=b(i)
                       if (ii /= 0) then
                               summ=summ-dot_product(a(i,ii:i-1),b(ii:i-1))
                       else if (summ /= 0.0) then
                               ii=i
                       end if
                       b(i)=summ
               end do
               do i=n,1,-1
                       b(i) = (b(i)-dot_product(a(i,i+1:n),b(i+1:n)))/a(i,i)
               end do
               END SUBROUTINE lubksb_new
!--------------------------------------------------------------------------------------------
        FUNCTION chebev(a,b,c,x)
        USE nrtype;             !USE nrutil, ONLY : nrerror
        IMPLICIT NONE
        REAL(SP), INTENT(IN) :: a,b,x
        REAL(SP), DIMENSION(:), INTENT(IN) :: c
        REAL(SP) :: chebev
        INTEGER(I4B) :: j,m
        REAL(SP) :: d,dd,sv,y,y2
      !  if ((x-a)*(x-b) > 0.0) call nrerror('x not in range in chebev_s')
        m=size(c)
        d=0.0
        dd=0.0
        y=(2.0_sp*x-a-b)/(b-a)
        y2=2.0_sp*y
        do j=m,2,-1
                sv=d
                d=y2*d-dd+c(j)
                dd=sv
        end do
        chebev=y*d-dd+0.5_sp*c(1)
        END FUNCTION chebev

        FUNCTION chebft(a,b,n,func)
        USE nrtype; USE nrutil, ONLY : arth,outerprod
        IMPLICIT NONE
        REAL(SP), INTENT(IN) :: a,b
        INTEGER(I4B), INTENT(IN) :: n
        REAL(SP), DIMENSION(n) :: chebft
        INTERFACE
                FUNCTION func(x)
   USE nrtype
!
   implicit none
   real(sp), dimension(:), intent(in) :: x
   real(sp), dimension(size(x)) :: func
                END FUNCTION func
        END INTERFACE
        REAL(DP) :: bma,bpa
        REAL(DP), DIMENSION(n) :: theta
        bma=0.5_dp*(b-a)
        bpa=0.5_dp*(b+a)
        theta(:)=PI_D*arth(0.5_dp,1.0_dp,n)/n
        chebft(:)=matmul(cos(outerprod(arth(0.0_dp,1.0_dp,n),theta)), &
                func(real(cos(theta)*bma+bpa,sp)))*2.0_dp/n
        END FUNCTION chebft

        FUNCTION chder(a,b,c)
        USE nrtype; USE nrutil, ONLY : arth,cumsum
        IMPLICIT NONE
        REAL(SP), INTENT(IN) :: a,b
        REAL(SP), DIMENSION(:), INTENT(IN) :: c
        REAL(SP), DIMENSION(size(c)) :: chder
        INTEGER(I4B) :: n
        INTEGER(I4B) :: n_1, meno1
        REAL(SP) :: con
        REAL(SP), DIMENSION(size(c)) :: temp
        n=size(c)
        temp(1)=0.0
!       temp(2:n)=2.0_sp*arth(n-1,-1,n-1)*c(n:2:-1)
        n_1 = n-1
        meno1 = -1
        temp(2:n)=2.0_sp*arth(n_1,meno1,n_1)*c(n:2:-1)
        chder(n:1:-2)=cumsum(temp(1:n:2))
        chder(n-1:1:-2)=cumsum(temp(2:n:2))
        con=2.0_sp/(b-a)
        chder=chder*con
        END FUNCTION chder
!--------------------------------------------------------------------------------------------
        SUBROUTINE indexx_sp(arr,index)
        USE nrtype; USE nrutil, ONLY : arth,assert_eq,nrerror,swap
        IMPLICIT NONE
        REAL(SP), DIMENSION(:), INTENT(IN) :: arr
        INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: index
        INTEGER(I4B), PARAMETER :: NN=15, NSTACK=50
        REAL(SP) :: a
        INTEGER(I4B) :: n,k,i,j,indext,jstack,l,r
        INTEGER(I4B) :: uno
        INTEGER(I4B), DIMENSION(NSTACK) :: istack
        n=assert_eq(size(index),size(arr),'indexx_sp')
!       index=arth(1,1,n)
        uno = 1
        index=arth(uno,uno,n)
        jstack=0
        l=1
        r=n
        do
                if (r-l < NN) then
                        do j=l+1,r
                                indext=index(j)
                                a=arr(indext)
                                do i=j-1,l,-1
                                        if (arr(index(i)) <= a) exit
                                        index(i+1)=index(i)
                                end do
                                index(i+1)=indext
                        end do
                        if (jstack == 0) RETURN
                        r=istack(jstack)
                        l=istack(jstack-1)
                        jstack=jstack-2
                else
                        k=(l+r)/2
                        call swap(index(k),index(l+1))
                        call icomp_xchg(index(l),index(r))
                        call icomp_xchg(index(l+1),index(r))
                        call icomp_xchg(index(l),index(l+1))
                        i=l+1
                        j=r
                        indext=index(l+1)
                        a=arr(indext)
                        do
                                do
                                        i=i+1
                                        if (arr(index(i)) >= a) exit
                                end do
                                do
                                        j=j-1
                                        if (arr(index(j)) <= a) exit
                                end do
                                if (j < i) exit
                                call swap(index(i),index(j))
                        end do
                        index(l+1)=index(j)
                        index(j)=indext
                        jstack=jstack+2
                        if (jstack > NSTACK) call nrerror('indexx: NSTACK too small')
                        if (r-i+1 >= j-l) then
                                istack(jstack)=r
                                istack(jstack-1)=i
                                r=j-1
                        else
                                istack(jstack)=j-1
                                istack(jstack-1)=l
                                l=i
                        end if
                end if
        end do
        CONTAINS

        SUBROUTINE icomp_xchg(i,j)
        INTEGER(I4B), INTENT(INOUT) :: i,j
        INTEGER(I4B) :: swp
        if (arr(j) < arr(i)) then
                swp=i
                i=j
                j=swp
        end if
        END SUBROUTINE icomp_xchg
        END SUBROUTINE indexx_sp

        SUBROUTINE indexx_i4b(iarr,index)
        USE nrtype; USE nrutil, ONLY : arth,assert_eq,nrerror,swap
        IMPLICIT NONE
        INTEGER(I4B), DIMENSION(:), INTENT(IN) :: iarr
        INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: index
        INTEGER(I4B), PARAMETER :: NN=15, NSTACK=50
        INTEGER(I4B) :: a
        INTEGER(I4B) :: n,k,i,j,indext,jstack,l,r
        INTEGER(I4B) :: uno
        INTEGER(I4B), DIMENSION(NSTACK) :: istack
        n=assert_eq(size(index),size(iarr),'indexx_sp')
!       index=arth(1,1,n)
        uno = 1
        index=arth(uno,uno,n)
        jstack=0
        l=1
        r=n
        do
                if (r-l < NN) then
                        do j=l+1,r
                                indext=index(j)
                                a=iarr(indext)
                                do i=j-1,1,-1
                                        if (iarr(index(i)) <= a) exit
                                        index(i+1)=index(i)
                                end do
                                index(i+1)=indext
                        end do
                        if (jstack == 0) RETURN
                        r=istack(jstack)
                        l=istack(jstack-1)
                        jstack=jstack-2
                else
                        k=(l+r)/2
                        call swap(index(k),index(l+1))
                        call icomp_xchg(index(l),index(r))
                        call icomp_xchg(index(l+1),index(r))
                        call icomp_xchg(index(l),index(l+1))
                        i=l+1
                        j=r
                        indext=index(l+1)
                        a=iarr(indext)
                        do
                                do
                                        i=i+1
                                        if (iarr(index(i)) >= a) exit
                                end do
                                do
                                        j=j-1
                                        if (iarr(index(j)) <= a) exit
                                end do
                                if (j < i) exit
                                call swap(index(i),index(j))
                        end do
                        index(l+1)=index(j)
                        index(j)=indext
                        jstack=jstack+2
                        if (jstack > NSTACK) call nrerror('indexx: NSTACK too small')
                        if (r-i+1 >= j-l) then
                                istack(jstack)=r
                                istack(jstack-1)=i
                                r=j-1
                        else
                                istack(jstack)=j-1
                                istack(jstack-1)=l
                                l=i
                        end if
                end if
        end do
        CONTAINS

        SUBROUTINE icomp_xchg(i,j)
        INTEGER(I4B), INTENT(INOUT) :: i,j
        INTEGER(I4B) :: swp
        if (iarr(j) < iarr(i)) then
                swp=i
                i=j
                j=swp
        end if
        END SUBROUTINE icomp_xchg
        END SUBROUTINE indexx_i4b
!--------------------------------------------------------------------------------------------
        FUNCTION dfridr(func,x,h,err)
        USE nrtype; USE nrutil, ONLY : assert,geop,iminloc
        IMPLICIT NONE
        REAL(SP), INTENT(IN) :: x,h
        REAL(SP), INTENT(OUT) :: err
        REAL(SP) :: dfridr
        INTERFACE
                FUNCTION func(x)
                USE nrtype
                IMPLICIT NONE
                REAL(SP), INTENT(IN) :: x
                REAL(SP) :: func
                END FUNCTION func
        END INTERFACE
        INTEGER(I4B),PARAMETER :: NTAB=10
        REAL(SP), PARAMETER :: CON=1.4_sp,CON2=CON*CON,BIG=huge(x),SAFE=2.0
        INTEGER(I4B) :: ierrmin,i,j,NTAB_1
        REAL(SP) :: hh
        REAL(SP), DIMENSION(NTAB-1) :: errt,fac
        REAL(SP), DIMENSION(NTAB,NTAB) :: a
        call assert(h /= 0.0, 'dfridr arg')
        hh=h
        a(1,1)=(func(x+hh)-func(x-hh))/(2.0_sp*hh)
        err=BIG
!       fac(1:NTAB-1)=geop(CON2,CON2,NTAB-1)
        NTAB_1 = NTAB-1
        fac(1:NTAB-1)=geop(CON2,CON2,NTAB_1)
        do i=2,NTAB
                hh=hh/CON
                a(1,i)=(func(x+hh)-func(x-hh))/(2.0_sp*hh)
                do j=2,i
                        a(j,i)=(a(j-1,i)*fac(j-1)-a(j-1,i-1))/(fac(j-1)-1.0_sp)
                end do
                errt(1:i-1)=max(abs(a(2:i,i)-a(1:i-1,i)),abs(a(2:i,i)-a(1:i-1,i-1)))
                ierrmin=iminloc(errt(1:i-1))
                if (errt(ierrmin) <= err) then
                        err=errt(ierrmin)
                        dfridr=a(1+ierrmin,i)
                end if
                if (abs(a(i,i)-a(i-1,i-1)) >= SAFE*err) RETURN
        end do
        END FUNCTION dfridr

      !--------------------------------------------------------------------------------------------
        SUBROUTINE choldc(a,p,ier)
        USE nrtype; USE nrutil, ONLY : assert_eq
        IMPLICIT NONE
        REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
        REAL(DP), DIMENSION(:), INTENT(OUT) :: p
!???!   INTEGER,  INTENT(OUT) :: ier
        INTEGER, INTENT(INOUT):: ier
        INTEGER(I4B) :: i,n
        REAL(DP) :: summ
        n=assert_eq(size(a,1),size(a,2),size(p),'choldc')
        do i=1,n
                summ=a(i,i)-dot_product(a(i,1:i-1),a(i,1:i-1))
                if (summ < 0.0) then
                   ier=1
                   return
                endif
                p(i)=sqrt(summ)
                a(i+1:n,i)=(a(i,i+1:n)-matmul(a(i+1:n,1:i-1),a(i,1:i-1)))/p(i)
        end do
        END SUBROUTINE choldc

      !--------------------------------------------------------------------------------------------
        SUBROUTINE cholsl(a,p,b,x)
        USE nrtype; USE nrutil, ONLY : assert_eq
        IMPLICIT NONE
        REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
        REAL(DP), DIMENSION(:), INTENT(IN) :: p,b
        REAL(DP), DIMENSION(:), INTENT(INOUT) :: x
        INTEGER(I4B) :: i,n
        n=assert_eq((/size(a,1),size(a,2),size(p),size(b),size(x)/),        'cholsl')
        do i=1,n
                x(i)=(b(i)-dot_product(a(i,1:i-1),x(1:i-1)))/p(i)
        end do
        do i=n,1,-1
                x(i)=(x(i)-dot_product(a(i+1:n,i),x(i+1:n)))/p(i)
        end do
        END SUBROUTINE cholsl
!-------------------------------------------------------------------------------

        function invltrimatrix(n,a,p)       result(matr)
!
!       Inverte una matrice triangolare inferiore
!       La matrice da invertire e' contenuta nel triangolo inferiore di a
!       eccetto gli elementi diagonali che sono contenuti in p
!
        USE nrtype
!
        implicit none
        integer, intent(in)                  :: n
        real(dp), dimension(n,n)             :: matr
        real(dp), dimension(n,n), intent(in) :: a
        real(dp), dimension(n), intent(in)   :: p
        integer                              :: i,j
!
        matr = a
        do i=1,n
           matr(i,i) = 1.0/p(i)
           do j=i+1,n
              matr(j,i) = -dot_product(matr(j,i:j-1),matr(i:j-1,i))/p(j)
           enddo
        enddo
        forall(i=1:n,j=1:n,j>i)matr(i,j) = 0.0  ! azzera triangolo superiore
!
        end function invltrimatrix
!-------------------------------------------------------------------------------

      subroutine inverti_lu(mato,matinv,ier)
      USE nrtype
      USE nr
      real(dp), dimension(:,:), intent(in)                        :: mato
      real(dp), dimension(size(mato,2),size(mato,2)), intent(out) :: matinv
      real(dp), dimension(size(mato,2),size(mato,2))              :: mat
      integer(i4b), dimension(size(mato,2))                       :: indx
      real(dp)                                                    :: d
      integer                                                     :: i,j
      integer                                                     :: ier
      integer                                                     :: n
!
      mat(:,:) = mato(:,:)
      call ludcmp_new(mat,indx,d,ier)
      if (ier == 0) then
          n = size(mat,2)
          do i=1,n
             do j=1,n
                matinv(i,j) = 0.0
             enddo
             matinv(i,i) = 1.0
          enddo
          do j=1,size(indx)
             call lubksb_new(mat,indx,matinv(:,j))
          enddo
      endif
!
      end subroutine inverti_lu
!----------------------------------------------------------------------------------
 MODULE LINLSQ

 implicit none

 CONTAINS

   subroutine solve_linlsq(a,b,x,ier)
!
!  Solve || ax - b ||2
!
   USE nr; USE nrtype
   real(dp), dimension(:,:), intent(in)        :: a
   real(dp), dimension(:), intent(in)          :: b
   real(dp), dimension(size(a,2)), intent(out) :: x
   integer, intent(out) :: ier
   real(dp), dimension(size(a,2))              :: bb,p
   real(dp), dimension(size(a,2),size(a,2))    :: ata
!
   ier=0
   bb = matmul(transpose(a),b)
   ata = matmul(transpose(a),a)
   call choldc(ata,p,ier)
   if(ier==1) return
   call cholsl(ata,p,bb,x)
!
   end subroutine solve_linlsq

 END MODULE

!----------------------------------------------------------------------------------
!  Subroutine to find the inverse of a square matrix
!  Author : Louisda16th a.k.a Ashwith J. Rego
!  Reference : Algorithm has been well explained in:
!  http://math.uww.edu/~mcfarlat/inverse.htm           
!  http://www.tutor.ms.unimelb.edu.au/matrix/matrix_inverse.html
      SUBROUTINE FINDInv(matrix, inverse, n, errorflag)
      IMPLICIT NONE
      !Declarations
      INTEGER, INTENT(IN) :: n
      INTEGER, INTENT(OUT) :: errorflag  !Return error status. -1 for error, 0 for normal
      REAL, INTENT(IN), DIMENSION(n,n) :: matrix  !Input matrix
      REAL, INTENT(OUT), DIMENSION(n,n) :: inverse !Inverted matrix
      
      LOGICAL :: FLAG = .TRUE.
      INTEGER :: i, j, k, l
      REAL :: m
      REAL, DIMENSION(n,2*n) :: augmatrix !augmented matrix
      
      !Augment input matrix with an identity matrix
      DO i = 1, n
          DO j = 1, 2*n
             IF (j <= n ) THEN
                 augmatrix(i,j) = matrix(i,j)
             ELSE IF ((i+n) == j) THEN
                 augmatrix(i,j) = 1
             Else
                 augmatrix(i,j) = 0
             ENDIF
          END DO
      END DO
      
      !Reduce augmented matrix to upper traingular form
      DO k =1, n-1
         IF (augmatrix(k,k) == 0) THEN
         FLAG = .FALSE.
         DO i = k+1, n
            IF (augmatrix(i,k) /= 0) THEN
               DO j = 1,2*n
                  augmatrix(k,j) = augmatrix(k,j)+augmatrix(i,j)
               END DO
               FLAG = .TRUE.
               EXIT
            ENDIF
            IF (FLAG .EQV. .FALSE.) THEN
                PRINT*, "Matrix is non - invertible"
                inverse = 0
                errorflag = -1
                return
            ENDIF
         END DO
         ENDIF
         DO j = k+1, n
            m = augmatrix(j,k)/augmatrix(k,k)
            DO i = k, 2*n
               augmatrix(j,i) = augmatrix(j,i) - m*augmatrix(k,i)
            END DO
         END DO
      END DO
      
      !Test for invertibility
      DO i = 1, n
         IF (augmatrix(i,i) == 0) THEN
             PRINT*, "Matrix is non - invertible"
             inverse = 0
             errorflag = -1
             return
         ENDIF
      END DO
      
      !Make diagonal elements as 1
      DO i = 1 , n
         m = augmatrix(i,i)
         DO j = i , (2 * n)
            augmatrix(i,j) = (augmatrix(i,j) / m)
         END DO
      END DO
      
      !Reduced right side half of augmented matrix to identity matrix
      DO k = n-1, 1, -1
         DO i =1, k
            m = augmatrix(i,k+1)
            DO j = k, (2*n)
               augmatrix(i,j) = augmatrix(i,j) -augmatrix(k+1,j) * m
            END DO
         END DO
      END DO
      
      !store answer
      DO i =1, n
         DO j = 1, n
            inverse(i,j) = augmatrix(i,j+n)
         END DO
      END DO
      errorflag = 0
      END SUBROUTINE FINDinv
!-------------------------------------------------------------------------------------
