MODULE crystal_phase
   USE progtype
   USE atom_basic, only: atom_type
   USE spginfom
   USE unit_cell

   type crystal_phase_type 
     type(atom_type), dimension(:), allocatable :: at
     type(bond_type), dimension(:), allocatable :: leg
     type(spaceg_type)                          :: spg
     type(cell_type)                            :: cell
     character(len=100)                         :: phase_name = ' '
   contains

     procedure :: exist_spg
     procedure :: set => set_phase
     procedure :: print => print_phase

   end type crystal_phase_type

   CONTAINS
  
   logical function exist_spg(phase)
   class(crystal_phase_type), intent(in) :: phase
   exist_spg = phase%spg%num /= 0
   end function exist_spg

!----------------------------------------------------------------------------------------------

   subroutine set_phase(phase,at,leg,pname,spg,cell)
   USE atom_type_util
   USE connect_mod
   class(crystal_phase_type), intent(inout)             :: phase
   type(atom_type), dimension(:), allocatable, optional :: at
   type(bond_type), dimension(:), allocatable, optional :: leg
   type(spaceg_type), intent(in), optional              :: spg
   type(cell_type), intent(in), optional                :: cell
   character(len=*), intent(in), optional               :: pname
!
   if (present(at)) then
       call copia_atoms(phase%at,at)
   endif
!
   if (present(leg)) then
       call copia_legm(phase%leg,leg)
   endif
!
   if (present(pname)) then
       phase%phase_name = trim(pname)
   endif
!
   if (present(spg)) then
       phase%spg = spg
   endif
!
   if (present(cell)) then
       phase%cell = cell
   endif
!
   end subroutine set_phase

!----------------------------------------------------------------------------------------------

   subroutine print_phase(phase,kpr)
   USE atom_type_util
   USE spginfom
   USE ccryst
   class(crystal_phase_type), intent(in) :: phase
   integer, intent(in)                   :: kpr
   call phase%spg%prn(kpr,short=.true.)
   write(kpr,'(/)')
   call print_cell(phase%cell%get_par(),kpr=kpr)
   end subroutine print_phase

!----------------------------------------------------------------------------------------------

   integer function numphase(phase)
   type(crystal_phase_type), dimension(:), allocatable, intent(in) :: phase
   if (allocated(phase)) then
       numphase = size(phase)
   else
       numphase = 0
   endif
   end function numphase

!----------------------------------------------------------------------------------------------

   subroutine reallocate_phase(phase,n,savevet)
!
!  Rialloca ad n un vettore reale.
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(crystal_phase_type), allocatable, intent(inout) :: phase(:)
   integer, intent(in)                                  :: n
   logical, optional, intent(in)                        :: savevet
   logical                                              :: savev
   integer                                              :: nv
   type(crystal_phase_type), allocatable                 :: vsav(:)
   integer                                              :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(phase)) deallocate(phase)
       return
   endif
!
   if (.not.allocated(phase)) then
       allocate(phase(n))
   else
!
       nv = size(phase)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di phase da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
           allocate(vsav(n))
           vsav(:nsav) = phase(:nsav)
           call move_alloc(vsav,phase)
       else
           deallocate(phase)
           allocate(phase(n))
       endif
   endif
!
   end subroutine reallocate_phase

END MODULE crystal_phase
