MODULE wlist

implicit none

integer, parameter, private :: NMAXLEN = 80

type column_type 
   character(len=1)                              :: ctype
   integer, dimension(:), allocatable            :: ivet
   real, dimension(:), allocatable               :: rvet
   character(NMAXLEN), dimension(:), allocatable :: svet
end type

interface set_column
   module procedure set_column_i,set_column_r,set_column_s,set_column_check
end interface

CONTAINS 
   
  subroutine wlist_create(list_code,ncol,nrow,coltitles,wintitle,msg,wtype,isel,func,col)
  USE strutil
  interface
    subroutine func(code)
    integer, intent(in) :: code
    end subroutine func
  end interface
  integer, intent(in)                                :: list_code   ! codice identificativo della lista
  integer, intent(in)                                :: nrow,ncol   ! numero di colonne e righe
  character(len=*), dimension(:), intent(in)         :: coltitles   ! titoli delle colonne
  character(len=*), intent(in)                       :: wintitle    ! titolo della finestra
  character(len=*), intent(in), optional             :: msg         ! messaggio opzionale prima della lista
  integer, intent(in), optional                      :: wtype       ! tasti della window
  integer, intent(in), optional                      :: isel        ! riga selezionata
  type(column_type), dimension(:)                    :: col         ! le colonne da inserire nella lista
  character(len=NMAXLEN), dimension(size(coltitles)) :: titles
  character(len=NMAXLEN)                             :: wtitle,wmsg
  integer                                            :: i
  integer                                            :: iselrow
  integer                                            :: wintype
  integer, dimension(ncol)                           :: coltype
  character(len=NMAXLEN), dimension(nrow)            :: svet
!
! Add char(0)
  do i=1,size(coltitles)
     titles(i) = add_c0(coltitles(i),NMAXLEN)
  enddo
  wtitle = add_c0(wintitle,NMAXLEN)
!
  if (present(msg)) then
      wmsg = add_c0(msg,NMAXLEN)
  else
      wmsg = char(0)
  endif
!
  if (present(isel)) then
      iselrow = isel
  else
      iselrow = 1
  endif
!
  if (present(wtype)) then
      wintype = wtype
  else
      wintype = 2
  endif
!
  do i=1,ncol
     coltype(i) = ichar(col(i)%ctype)
  enddo
!
! Crea la lista
  call list_create(list_code,ncol,nrow,coltype,titles,wtitle,wmsg,wintype,iselrow,func)
!
! Riempi la lista
  do i=1,ncol
     select case (col(i)%ctype)
       case ('I')
         call list_set_column(list_code, i, col(i)%ivet, 0, 0) 
       case ('R')
         call list_set_column(list_code, i, 0, col(i)%rvet, 0) 
       case ('S')
         !call list_set_column(list_code, i, 0, 0, add_c0(col(i)%svet,NMAXLEN))   !non funziona
         svet(:) = add_c0(col(i)%svet(:),NMAXLEN)
         call list_set_column(list_code, i, 0, 0, svet)
     end select
  enddo
!
  end subroutine wlist_create

!--------------------------------------------------------------------------------------

  type(column_type) function set_column_check(n)  result(col)
  USE realloc
  integer, intent(in) :: n
  call reallocate(col%ivet,n,.false.)
  col%ctype = 'B'
  col%ivet = 0
  end function set_column_check

!--------------------------------------------------------------------------------------

  type(column_type) function set_column_i(ivet)  result(col)
  USE realloc
  integer, dimension(:), intent(in) :: ivet
  call reallocate(col%ivet,size(ivet),.false.)
  col%ctype = 'I'
  col%ivet = ivet
  end function set_column_i

!--------------------------------------------------------------------------------------

  type(column_type) function set_column_r(rvet)  result(col)
  USE realloc
  real, dimension(:), intent(in) :: rvet
  call reallocate(col%rvet,size(rvet),.false.)
  col%ctype = 'R'
  col%rvet = rvet
  end function set_column_r

!--------------------------------------------------------------------------------------

  type(column_type) function set_column_s(svet)  result(col)
  USE realloc
  character(len=*), dimension(:), intent(in) :: svet
  call reallocate(col%svet,size(svet),.false.,NMAXLEN)
  col%ctype = 'S'
  col%svet = svet
  end function set_column_s

END MODULE wlist
