#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <cassert>

#include <glib.h>
#include <utils.h>
#include <g_ext.h>
#include "ilm_defs.h"
#include "application.h"
#include "util.h"

extern "C" const char *mr_setenv_(char *string);

namespace Ilm
{

  bool Application::Initialize(int argc, const char *const argv[])
  {
    bool result = false;
    g_assert( argc!=0 && argv!=NULL && argv[0]!=NULL );
    gchar *_inpath;
    _inpath = find_program_in_path(argv[0]);
    if( _inpath==NULL )
      throw UserError("fatal : argv0 is not found\n");
//    strMainDir = gext::path_get_dirpath(_inpath);
    strMainDir = get_prog_dir(_inpath);
    g_free(_inpath);

//mr#   ifdef PKGDATADIR
//mr    strPkgDataDir = PKGDATADIR G_DIR_SEPARATOR_S;
//mr#   else
    strPkgDataDir = gext::path_get_dirpath(strMainDir.c_str()) + G_DIR_SEPARATOR_S + "share"
                    + G_DIR_SEPARATOR_S + "ilmilione" + G_DIR_SEPARATOR_S;
//mr#   endif

    if(!g_file_test(strPkgDataDir.c_str(), G_FILE_TEST_IS_DIR))
    {
        throw UserError("fatal: Directory '" + strPkgDataDir + "' does not exist!\n");
    }
    if(Ambiente("PATH") != NULL)
    {
       std::string path = Ambiente("PATH");
       path = "PATH=" + path + ";" + strMainDir;
       mr_setenv_(g_strdup_printf("%s", path.c_str()));
    }
    else
    {
       std::string path = "PATH=" + strMainDir;
       mr_setenv_(g_strdup_printf("%s", path.c_str()));
    }

    // for ccp4
//#   ifdef _WIN32
    std::string ccp4sym = "SYMINFO=" + strPkgDataDir + "files"  + 
                          G_DIR_SEPARATOR_S + "syminfo.lib";
    mr_setenv_(g_strdup_printf("%s", ccp4sym.c_str()));

    std::string pixmapdir = strPkgDataDir + "pixmaps";
    add_pixmap_directory(pixmapdir.c_str());

    std::string ccp4file = strPkgDataDir + "files"  + G_DIR_SEPARATOR_S + "syminfo.lib";
    if(!g_file_test(ccp4file.c_str(), G_FILE_TEST_EXISTS))
    {
        throw UserError("fatal: File '" + ccp4file + "' does not exist!\n");
    }
    gchar *confdir=NULL, *conffn=NULL;

    const gchar *dir = g_get_home_dir();
    if( dir==NULL )
      dir = strMainDir.c_str();

    g_assert(dir!=NULL);

#ifdef _WIN32
    const char *arch = SystemArch();
    char *tmp = g_strdup_printf(".%s%s", PACKAGE, arch);
    confdir = g_build_path(G_DIR_SEPARATOR_S, dir, tmp, NULL);
#else
    confdir = g_build_path(G_DIR_SEPARATOR_S, dir, "." PACKAGE, NULL);
#endif
    if( !g_file_test(confdir, G_FILE_TEST_IS_DIR) )
      dir_create(confdir);
    strConfDir = confdir;
    std::string suffix;
#ifdef NDEBUG
    suffix.assign(g_strdup_printf("%s%d", PACKAGE, MY_MAJOR_VERSION(VERSION)));
    conffn = g_build_filename(confdir, suffix.c_str(), NULL);
#else
    suffix.assign(g_strdup_printf("dbg%s%d", PACKAGE, MY_MAJOR_VERSION(VERSION)));
    conffn = g_build_filename(confdir, suffix.c_str(), NULL);
#endif
    g_free(confdir);

    strConfFile.assign(conffn);
    if( g_file_test(conffn, G_FILE_TEST_EXISTS) )
    {
#ifdef _WIN32
        result = SameVersion((char *)fromUTF8(conffn).c_str());
#else
        result = SameVersion(conffn);
#endif
    }

    prjAllocation.x = 0;
    prjAllocation.y = 0;
    prjAllocation.width = 500;
    prjAllocation.height = 800;
    auto_update = 1;
    strMainDir += G_DIR_SEPARATOR_S;
    for( unsigned i=0; i<2; i++ )
      vstrExecutables.push_back(strMainDir+SZExecutables[i]); // programs
    vstrExecutables.push_back(GetBrowser()); // internet browser
    vstrExecutables.push_back(strMainDir+SZExecutables[IExePDBViewer]); // ilmView
/*
    gchar *cootp = find_program_in_path(SZExecutables[IExePDBViewer]);
    if(cootp != NULL)
       vstrExecutables.push_back(cootp); // coot
    else
       vstrExecutables.push_back("unknown");
*/
    vstrLast.clear();
    vstrExamples.clear();

    if(argc > 1)
    {
       const gchar *fn = argv[1];
       if(strcmp(fn, "setup") == 0)
       {
          Cancella(conffn);
          result = false;
       }
    }
    g_free(conffn);
    return result;
  }

  bool Application::SameVersion(char*conffn)
  {
    std::ifstream ff;
    std::stringstream strm;
    std::string str;

    ff.open(conffn, std::ios::in | std::ios::binary);
    if( !ff.is_open() || !ff )
      throw UserError("failure opening configuration file:\n" + strConfFile);

    char buf[10];
    memset(buf, 0, sizeof(buf));
    ff.read(buf, sizeof(buf));
    ff.close();
    char buf1[10];
    memset(buf1, 0, sizeof(buf1));
    ScriviStringa(buf1,sizeof(buf1), "%s",VERSION);
    return( memcmp(buf, buf1, sizeof(buf)) == 0 );
  }

  void Application::SetMenuIconSize(int w, int h)
  {
     wmenu = w;
     hmenu = h;
  }

  std::string Application::DateInfo()
  {
   const char *ddir = g_path_get_dirname(PkgDataDir().c_str());
   char *filinfo = g_build_filename(ddir, "files", ".instdate", NULL);
   return dateinfo(filinfo);
  }

  int Application::LastUpdate(bool ifSilent, int *versionNumber)
  {
   const char *ddir = g_path_get_dirname(PkgDataDir().c_str());
   char *filinfo = g_build_filename(ddir, "files", ".instdate", NULL);
   return IsUpdateAvailable(filinfo, ifSilent, versionNumber);
  }

  void Application::ReadSettings()
  {
    std::ifstream ff;
    std::stringstream strm;
    std::string str;
    unsigned i;

    //const char *conffn = fromUTF8(strConfFile).c_str();
    const char *conffn = strConfFile.c_str();
    ff.open(conffn, std::ios::in | std::ios::binary);
    if( !ff.is_open() || !ff )
      throw UserError("failure opening configuration file:\n" + strConfFile);
    char buf[20];
    memset(buf, 0, sizeof(buf));
    ff.read(buf, sizeof(buf));
    memset(buf, 0, sizeof(buf));
    ff.read(buf, sizeof(buf));
    strm.str(buf);
    strm >> prjAllocation.x >> prjAllocation.y >> prjAllocation.width >> prjAllocation.height;
    strm >> auto_update;
    std::vector<std::string> exe;
    for( i=0; i<NExecutables; i++ )
    {
      strm.str("");
      ff.get(*strm.rdbuf(), '\0');
      char zero;
      ff.get(zero);
      str = strm.str();
      if( (gext::str_is_white(str.c_str()) ) ) //&& (i!= IExePDBViewer))
        throw UserError("wrong data in the configuration file:\n" + strConfFile);
      exe.push_back(str);
    }
    vstrExecutables = exe;
    assert( vstrExecutables.size()==NExecutables );
    for( i=0; i<NEXAMPLES; i++ )
    {
      if( ff.eof() )
        break;
      strm.str("");
      ff.get(*strm.rdbuf(), '\0');
      char zero;
      ff.get(zero);
      str = strm.str();
      if( !gext::str_is_white(str.c_str()) )
        vstrExamples.push_back(str);
    }
    for( i=0; i<NRECENT; i++ )
    {
      if( ff.eof() )
        break;
      strm.str("");
      ff.get(*strm.rdbuf(), '\0');
      char zero;
      ff.get(zero);
      str = strm.str();
      if( !gext::str_is_white(str.c_str()) )
        vstrLast.push_back(str);
    }
  }

  void Application::WriteSettings() const
  {
    std::ofstream of;
    //const char *conffn = fromUTF8(strConfFile).c_str();
    const char *conffn = strConfFile.c_str();
    //g_print("%s\n", conffn);
    of.open(conffn, std::ios::out | std::ios::binary);
    if( !of.is_open() || !of )
      throw UserError("failure writing to configuration file:\n" + strConfFile);
    char buf[20];
    memset(buf, 0, sizeof(buf));
    ScriviStringa(buf,sizeof(buf), "%s",VERSION);
    of.write(buf, sizeof(buf));
    ScriviStringa(buf,sizeof(buf), "%d %d %d %d %d",
    prjAllocation.x, prjAllocation.y, prjAllocation.width, prjAllocation.height, auto_update);
    of.write(buf, sizeof(buf));
    size_t i;
    for( i=0; i<vstrExecutables.size(); i++ )
      of << vstrExecutables[i] << '\0';
    for( i=0; i<vstrExamples.size(); i++ )
      of << vstrExamples[i] << '\0';
    for( i=vstrExamples.size(); i< NEXAMPLES; i++ )
      of << " \0";
    for( i=0; i<vstrLast.size(); i++ )
      of << vstrLast[i] << '\0';
  }

  void Application::AddLast(const std::string &fn)
  {
    vstring lst;

    if( gext::str_is_white(fn.c_str()) )
      return;

    lst.push_back(fn);
       
    for( unsigned i=0; i<vstrLast.size(); i++ ) 
    {
      if( !gext::str_is_white(vstrLast[i].c_str()) && !compare_names(vstrLast[i],fn) )
        lst.push_back(vstrLast[i]);
    }
    vstrLast = lst;
  }

  void Application::UpdateRecentMenu()
  {
    for( unsigned i=0; i<NRECENT; i++ ) 
    {
      const char *s;
      std::stringstream name;
      name << "recent" << i+1;
      gpointer w = get_object(SplashWidget(), name.str().c_str());
      if( w==NULL )
        break;
      if( i<Last().size() )
      {
        s = Last()[i].c_str();
        obj_set_sensitive(w, TRUE);
        obj_set_visible(w, TRUE);
      }
      else
      {
        s = "  ";
        obj_set_visible(w, FALSE);
      }
      obj_set_label(w, s);
    }
  }


  void Application::AddExample(const std::string &fn)
  {
    vstring lst;

    if( gext::str_is_white(fn.c_str()) )
      return;

    lst.push_back(fn);
       
    for( unsigned i=0; i<vstrExamples.size(); i++ ) 
    {
      if( !gext::str_is_white(vstrExamples[i].c_str()) && !compare_names(vstrExamples[i],fn) )
        lst.push_back(vstrExamples[i]);
    }
    vstrExamples = lst;
  }

  void Application::UpdateExamplesMenu()
  {
    for( unsigned i=0; i<NEXAMPLES; i++ ) 
    {
      const char *s;
      std::stringstream name;
      name << "example" << i+1;
      gpointer w = get_object(SplashWidget(), name.str().c_str());
      if( w==NULL )
        break;
      if( i<LastE().size() )
      {
        s = LastE()[i].c_str();
        obj_set_sensitive(w, TRUE);
      }
      else
      {
        s = "  ";
        obj_set_sensitive(w, FALSE);
      }
      obj_set_label(w, s);
    }
  }

  void Application::SetAllocation(GtkAllocation alloc)
  {
     memcpy(&prjAllocation, &alloc, sizeof(GtkAllocation));
  }

} // namespace Ilm
