#ifndef __ILM_APPLICATION_H_mrt
#define __ILM_APPLICATION_H_mrt

#include <string>
#include <vector>
#include <cassert>

namespace Ilm
{
  class Application
  {
  public:
    void SetMenuIconSize(int w, int h);
    const std::string &MainDir() const {return strMainDir;}
    const std::string ConfFile() const {return strConfFile;}
    const std::string ConfDir() const {return strConfDir;}
    const std::string PkgDataDir() const {return strPkgDataDir;}
    const std::vector<std::string> &Executables() const {return vstrExecutables;}
    const std::vector<std::string> &Last()  const {return vstrLast;}
    const std::vector<std::string> &LastE() const {return vstrExamples;}
    std::string strFontName;
    std::string SIRProtConfigDir() const {return PkgDataDir() + "sirprotdir" G_DIR_SEPARATOR_S;}
    PangoFontDescription *FontDesc;
    GtkWidget *SplashWidget() const {assert(GTK_IS_WIDGET(poSplashWdgt)); return poSplashWdgt;}
    GtkWindow *SplashWindow() const {assert(GTK_IS_WIDGET(poSplashWdgt)); return GTK_WINDOW(poSplashWdgt);}
    void SetExecutables(const std::vector<std::string> &exe) {vstrExecutables=exe;}
    bool Initialize(int argc, const char *const argv[]);
    bool SameVersion(char *conffn);
    std::string DateInfo();
    int LastUpdate(bool ifSilent, int *versionNumber);
    void ReadSettings();
    void WriteSettings() const;
    void AddLast(const std::string &fn);
    void UpdateRecentMenu();
    void AddExample(const std::string &fn);
    void UpdateExamplesMenu();
    void SetSplash(GtkWidget *w) {poSplashWdgt=w;}
    const GtkAllocation *GetAllocation() const {return &prjAllocation;}
    void  SetAllocation(GtkAllocation alloc);
    const int AutoUpdate() const {return auto_update;}
    void  SetAutoUpdate(int flag) {auto_update = flag;}
    int MenuIconW()  {return wmenu;}
    int MenuIconH()  {return hmenu;}

  private:
    std::string strMainDir; // where exe is
    std::string strConfDir; // home
    std::string strConfFile;
    std::string strPkgDataDir; // /usr/local/share/ilmilione ili strMainDir
    std::vector<std::string> vstrExecutables;
    std::vector<std::string> vstrLast;
    std::vector<std::string> vstrExamples;
    GtkWidget *poSplashWdgt;
    GtkAllocation prjAllocation;
    int auto_update, wmenu, hmenu;
  };
}

#endif
