#ifndef __ILM_DEFS_H_mrt
#define __ILM_DEFS_H_mrt

#ifdef ENABLE_NLS
#error "not using gettext"
#endif

#define RED "#DD0000"

#define NRECENT 5
//#ifdef _WIN32
//#define NEXAMPLES 6
//#else
#define NEXAMPLES 8
//#endif
#define MAX_MODELS 20

namespace Ilm
{
  const char *const SZMainName = PACKAGE;

  enum ProjectType {
    MAD     = 1,
    MIR     = 2,
    MRS     = 3, // MIRAS/SIRAS
    EDM     = 4,
    MOLREP  = 5, // Molecular Replacement
    ABI     = 6, // ab-initio sir200x
    DEA     = 7  // DEA
  };
  enum Pages { MTZ, CONT, RUN };

  const char *const ambp[] = {"     ", "ARP/wARP", "PHENIX", "Buccaneer", "Nautilus"};
  const char *const amb_prg[] = {" ", "auto_tracing.sh", "phenix.autobuild",
                                 "buccaneer_pipeline", "nautilus_pipeline"};
  const char *const ambpre[] = {"     ", "arp", "phenix", "buccaneer", "nautilus"};

  const double DProteinVm = 1.34; //!< (Volume of pure protein without solvent / its weight ), in A**3 / Da 
/*
  const double DResidueWeight = 108.7; //!< Weight of one residue in Daltons, 1Da = 1 g/mol

  const double DHPerResidue = 8.0;  //!< Average number of Hydrogen atoms per residue 
  const double DCPerResidue = 5.0;  //!< Average number of Carbon atoms per residue
  const double DNPerResidue = 1.35; //!< Average number of Nitrogen atoms per residue
  const double DOPerResidue = 1.5;  //!< Average number of Oxygen atoms per residue
  const double DSPerResidue = 0.05; //!< Average number of Sulphur atoms per residue
*/
  const char *const PROJECT_EXT      = ".ilm";
  const char *const PROJECT_AEXT     = ".ailm";
  const char *const INP_EXT          = ".sir";
  const char *const INFO_EXT         = "_files.htm";
  const char *const OUT_EXT          = ".out";
  const char *const LOG_EXT          = ".log";
  const char *const ILOG_EXT         = "_files.log";
  const char *const BIN_EXT          = ".bin";
  const char *const GRAPH_EXT        = ".png";
  const char *const FGRAPH_EXT       = "_fPlot.png";
  const char *const LOCK_EXT         = ".lock";
  const char *const MTZ_EXT          = ".mtz";
  const char *const REC_EXT          = "_SIR200x.pha";
  const char *const TYPE_EXT []      = {"_task0", "_task1", "_task2", "_edm", "_remo", "_abi", "_dea"};

  const char *const PHASE_EXT        = "_phase.mtz";
  const char *const EDM_EXT          = "_edm.mtz";
  const char *const STOP_EXT         = "_task1.stop";
  const char *const EDM_EXT1         = "_flunch.mtz";
//  const char *const EDM_EXT2         = "_flunch_extra.mtz";
  const char *const EDM_MAP1         = "_edm.map";
  const char *const EDM_MAP2         = "_flunch.map";
  const char *const FRACLSQ_EXT      = "_lsq.fra";
  const char *const SEQ_EXT          = ".seq";
  const char *const MAD1FRAC_EXT     = "_SIR200x.fra";
  const char *const PROFLEXFRAC_EXT  = "_SIRPROT.fra";
  const char *const HTML_EXT         = ".htm";
#ifdef _WIN32
  const char *const SHELL_EXT        = ".cmd";
  const char *const CONT_CHAR        = "^";
  const char *const COM_CHAR         = "REM";
  const char *const REF_CHAR         = "%";
  const char *const COPYF            = "copy";
  const char *const DEA_SH           = "dea_run.cmd";
  const char *const CADPH_SH         = "cad_phases.cmd";
  const char *const CADMD_SH         = "cad_obs.cmd";
  const char *const PHEMD_SH         = "phenixrun_model.cmd";
  const char *const PHEPH_SH         = "phenixrun_phases.cmd";
  const char *const ARPMD_SH         = "arprun_model.cmd";
  const char *const ARPPH_SH         = "arprun_phases.cmd";
  const char *const BUCMD_SH         = "buccaneerrun_model.cmd";
  const char *const BUCPH_SH         = "buccaneerrun_phases.cmd";
  const char *const NAUMD_SH         = "nautilusrun_model.cmd";
  const char *const NAUPH_SH         = "nautilusrun_phases.cmd";
#else
  const char *const SHELL_EXT        = ".sh";
  const char *const CONT_CHAR        = "\\";
  const char *const COM_CHAR         = "#";
  const char *const COPYF            = "cp";
  const char *const REF_CHAR         = "$";
  const char *const DEA_SH           = "dea_run.sh";
  const char *const CADPH_SH         = "cad_phases.sh";
  const char *const CADMD_SH         = "cad_obs.sh";
  const char *const PHEMD_SH         = "phenixrun_model.sh";
  const char *const PHEPH_SH         = "phenixrun_phases.sh";
  const char *const ARPMD_SH         = "arprun_model.sh";
  const char *const ARPPH_SH         = "arprun_phases.sh";
  const char *const BUCMD_SH         = "buccaneerrun_model.sh";
  const char *const BUCPH_SH         = "buccaneerrun_phases.sh";
  const char *const NAUMD_SH         = "nautilusrun_model.sh";
  const char *const NAUPH_SH         = "nautilusrun_phases.sh";
#endif

  const char *const CADPH_INP        = "cad_phases.inp";
  const char *const CADMD_INP        = "cad_obs.inp";
  const char *const FREEMD_INP       = "freemd.inp";
  const char *const FREEPH_INP       = "freeph.inp";

  const unsigned short       MAX_WAVES          = 5;
  const unsigned short       MAX_DERIVATIVES    = 5;
  const unsigned short       MAX_PAGES          = 4;
  const unsigned short       MAX_ELEMENTS       = 4;
  const unsigned short       MAX_ANOM_ELEMENTS  = 5;
  const unsigned short       MAX_ABI_BLOCKS     = 10;
  const unsigned short       MAX_SYMBOL_LEN     = 4;
  const unsigned short       PATH_LEN           = 1024;

  const char *const SZExecutables[] =
  {
    "sir2014",
    "sirprot",
    "firefox",
    "jav"                      //"coot"
  };

  const char *const SZLbExecutables[] =
  {
    "Sir200X:",
    "Sirprot:",
    "Browser:",
//  "Coot:"
    "Viewer:"
  };

  const char *const SZTip[] =
  {
    "Executable for the SIR2011 program (relative or absolute path)",
    "Executable for the SIRProt program (relative or absolute path)",
    "Executable for the help end Internet Browser",
//  "Executable for the Coot"
    "Executable for the Viewer"
  };

  static const char *const Waves[]  = {"Cu", "Mo", "Other"};
  static const char *const Sources[]  = {"X Ray Data", "Neutron Data", "Electron Data"};

  const size_t NExecutables = sizeof(SZExecutables)/sizeof(char*);
  enum EExe { IExeSir = 0, IExeSirProt = 1, 
              IExeBrowser = 2, IExePDBViewer = 3 };
  class Project;
  class Application;
  class GtkProject;
  class UserError;

} // namespace Ilm

#endif  // __ILM_DEFS_H_mrt

