#ifndef __SIR_APPLICATION_H
#define __SIR_APPLICATION_H

#ifndef _WIN32
#include <stdlib.h>
#include <unistd.h>
#endif
#include <string>
#include <cstring>
#include <vector>
#include <cassert>

#define MAX_LEN       280
#define N_OBJS        16
#define N_FIXED       16
#define PALETTELINES  16+N_OBJS
#define NUM_STYLES    8
#define MAXPROJ       5
#define GREXT         ".gru"

enum {colorBLACK = N_OBJS+NUM_COLORS, colorBRIGHTWHITE, colorWHITE, colorGRAY,
      colorBLUE, colorGREEN, colorCYAN, colorRED, colorMAGENTA,
      colorBROWN, colorLIGHTBLUE, colorLIGHTGREEN, colorLIGHTCYAN,
      colorLIGHTRED, colorLIGHTMAGENTA, colorYELLOW};

enum {param_SUPPORT, param_ENABLE, param_BUFFERS, param_FORMAT};
/* ---------------------------------------------------------------- */
/*       User-defined types                                         */
/* ---------------------------------------------------------------- */

typedef struct {
                guint16 red;
                guint16 green;
                guint16 blue;
               } rgbCol;

typedef float speCol[4];

namespace main_app
{
  const rgbCol  bcolors[] = {{  0,   0,   0},
                             {255, 255, 255},
                             {192, 192, 192},
                             {127, 127, 127},
                             {  0,   0, 160},
                             {  0, 160,   0},
                             {  0, 160, 160},
                             {160,   0,   0},
                             {160,   0, 160},
                             {160, 160,   0},
                             {  0,   0, 255},
                             {  0, 255,   0},
                             {  0, 255, 255},
                             {255,   0,   0},
                             {255,   0, 255},
                             {255, 255,   0}};

  class Application
  {
  public:
    Application(const char *pack, const char *vers, const char *shnam,
       const char *prgfiles, const char *exampdir="smallmolecules", int col=1,
       const char *about_d=" ", const char *about_p=" ");
    int InitGraphics();
    std::string DateInfo(const char *fil=NULL) ;
    int LastUpdate(GtkWidget *mw, bool ifSilent, int *version=NULL, bool g=true);
    const char *PackageName() const {return strPackageName;}
    const char *PackageVersion() const {return Version;}
    const char *PkName() const {return ShortName;}
    const std::string &MainDir() const {return strMainDir;}
    const std::string WorkingDirectory() const {return strWorkingDirectory;}
    const std::string ConfFile() const {return strConfFile;}
    const std::string ConfDir() const {return strConfDir;}
    const std::string PkgDataDir() const {return strPkgDataDir;}
    const std::string AuxDataDir() const {return strAuxDataDir;}
    const std::string ExamplesDir() const {return strUserExamplesDir;}
    const std::string InternetBrowser() const {return strInternetBrowser;}
    const std::string PovrayP() const {return strPovrayP;}
    const std::string GroupsFile() const {return strGroupsFile;}
    const std::string PkgMTime() const {return strMtime;}
    const std::vector<std::string> &Last() const {return vstrLast;}
    const std::string FontN() const {return strFontN;}
    const std::string FontP() const {return strFontP;}
    const std::string FontU() const {return strFontU;}
    int MenuIconW()  {return wmenu;}
    int MenuIconH()  {return hmenu;}
    int   nRecentProjects;
    int   DefBg;
    bool  standalone;
    bool  nocfg;               //22/07/2016
    nomefile RecentProj[5];
    bool Initialize(int argc, const char *const argv[]);
    bool Initialize();
    bool  CheckConfig();
    bool ReadConfig();
    bool RicreaConfig(int errore);
    bool CreaConfig(void);
    bool UpdateCfg(int nline, const char *line);
    bool UpdateMisc1();
    void UpdateDime(int w, int h);
    void UpdatePosition(int x, int y);
    void UpdateWD(const char *str);
    void UpdateIB(char *str);
    void UpdatePS(int val);
    void UpdatePovray(char *str);
    void UpdateGrf(char *str);
    void UpdateBestv(int val);
    void UpdateRend(int val);
    void UpdateBG(int val);
    void UpdateLabel(int val);
    void UpdateAutoU(int val);
    bool UpdateAA();
    void LeggiFonts();
    void ScriviFonts();
    void SetFont(char c, char *fontname);
    bool UpdateFonts();
    void AddLast(const std::string &fn);
    void ResettaPalette();
    void ReadPalette(FILE *fp, int *errore, int err_cod);
    bool WritePalette();
    void ScegliPalette(int quale);
    void GetExamples();
    void OpenAbout(GtkWidget *main_window);
    GdkColor *getgdkCol(int quale);
    GdkColor colorPalette[16];
    float *getfCol(int quale);
    float *getdefCol(int quale);
    void   putfCol(int quale, float *color);
    int PlotType, IfLabels;
    int Kscrx, Kscry, Bestv, Background, IfMaximized;
    int XStart, YStart;
    nomefile filesir, filebin, fileout;
    rgbCol pal_colors[16];
    speCol app_color[NUM_COLORS+N_OBJS+N_FIXED];
    speCol app_defs[NUM_COLORS+N_OBJS+N_FIXED];
    speCol fix_color[N_FIXED];
    int AppError;
    bool GtkOn;
    const int AutoUpdate() const {return auto_update;}
    const bool ForceSoftRendering() const {return (IfSoftRend!=0);}
    void  SetAutoUpdate(int flag) {auto_update = flag;}
    static void SetProgName(const char *nome) {strProgName = g_strdup(nome); }
    static const char *GetProgName() {return strProgName; }
    const bool IsMSEnabled() const {return (MSEnabled!=0);}
    const bool IsMSSupported() const {return (MSSupported!=0);}
    const int  MSBuffers() const {return (NSamples);}
    const int  MSFormat() const {return (nPixelFormat);}
    void  Set_AA(int param, int val);

  private:
    int Argc, wmenu, hmenu;
    char **Argv;
    static char *strProgName;
    
    const char *strPackageName;
    const char *Version;
    const char *ShortName;
    const char *PrgFiles;
    const char *ExampDir;
    const char *AboutDesc;
    const char *AboutPixmap;
    std::string strMainDir; 
    std::string strWorkingDirectory; 
    std::string strConfDir;
    std::string strConfFile;
    std::string strPkgDataDir; 
    std::string strAuxDataDir; 
    std::string strExamplesDir;
    std::string strUserExamplesDir;
    std::string strInternetBrowser; 
    std::string strPovrayP; 
    std::string strGroupsFile; 
    std::string strFontN;
    std::string strFontP;
    std::string strFontU;
    std::string strMtime;
    std::vector<std::string> vstrLast; 
    int  auto_update;
    int  IfSoftRend;
    int  MSSupported;
    int  MSEnabled;
    int  NSamples;
    int  nPixelFormat;
    GtkWidget *AboutDlg;
    void MsgWinErr(const char *cmsg, int type);
  };
}
#endif
