MODULE filereading

 USE realloc
 USE errormod
 USE progtype, only:point_type
 
 implicit none

 integer, parameter :: FTYPE_ALL = 1
 integer, parameter :: FTYPE_COUNTS = 2
 integer, parameter :: FTYPE_XY = 3
 integer, parameter :: FTYPE_GSAS = 4
 integer, parameter :: FTYPE_CIF = 5
 integer, parameter :: FTYPE_XYE = 6
 integer, parameter :: FTYPE_POW = 7
 integer, parameter :: FTYPE_DOUBLE = 105

 CONTAINS

   subroutine read_file_from_type(unitinp,ftype,tth_min,tth_step,tth_max,counts,ncounts,use_step,wavel,ier,emess)
!
   integer, intent(in)                          :: unitinp
   integer, intent(in)                          :: ftype
   real, intent(inout)                          :: tth_min,tth_max,tth_step
   type(point_type), allocatable, intent(inout) :: counts(:)
   integer, intent(out)                         :: ncounts
   logical, intent(in)                          :: use_step
   real, intent(out)                            :: wavel
   integer, intent(out)                         :: ier
   logical, optional, intent(in)                :: emess
   type(error_type)                             :: error,error1
   logical                                      :: mess
!
   ier = 0
   wavel = -100
   select case(ftype)

       case (FTYPE_XY,FTYPE_XYE)                  ! leggi 2 colonne con 2-theta e conteggi
         call read2columns(unitinp,tth_min,tth_max,tth_step,counts,ncounts,wavel,use_step,error)

       case (FTYPE_COUNTS)                   ! leggi file di soli conteggi
         call readonlyc(unitinp,tth_min,tth_step,tth_max,counts,ncounts,.false.,error)

       case (FTYPE_DOUBLE)                   ! leggi riga conteggi e riga 2theta
         call readonlyc(unitinp,tth_min,tth_step,tth_max,counts,ncounts,.true.,error)

       case (FTYPE_GSAS)                     ! leggi file GSAS
         call readgsas(unitinp,tth_min,tth_step,tth_max,counts,ncounts,error)

       case (FTYPE_CIF)                      ! leggi file pdCIF
         call readcif(unitinp,tth_min,tth_step,tth_max,counts,ncounts,wavel,error)

       case (FTYPE_POW)
!
!        leggi prima xy ed eventualmente segnala l'errore sulla lettura di xy
         call read2columns(unitinp,tth_min,tth_max,tth_step,counts,ncounts,wavel,use_step,error)
         if (error%signal) then
             rewind(unitinp)
             call readonlyc(unitinp,tth_min,tth_step,tth_max,counts,ncounts,.false.,error1)
             if (.not.error1%signal) error%signal = .false.
         endif
   end select
!
   if (error%signal) then
       mess = .true.
       if (present(emess)) mess = emess
       if (mess) call error%print()
       ier = 1
   endif
!
   end subroutine read_file_from_type

  !---------------------------------------------------------------------------------------------

   integer function filetype_from_string(str) result(ftype)
   USE strutil
   character(len=*), intent(in) :: str
   character(len=len_trim(str)) :: str1
!
   str1 = upper(str)
   select case(str1)
      case ('COUNTS')
        ftype = FTYPE_COUNTS
      case ('XY')
        ftype = FTYPE_XY
      case ('DOUBLE')
        ftype = FTYPE_DOUBLE
      case ('GSAS')
        ftype = FTYPE_GSAS
      case ('CIF')
        ftype = FTYPE_CIF
      case default
        ftype = 0
   end select
!
   end function filetype_from_string

  !---------------------------------------------------------------------------------------------

   integer function filetype_from_filename(filename) result(ftype)
!
!  Identifica il tipo di file dall'estensione
!
   USE fileutil
   USE strutil
   character(len=*), intent(in)      :: filename
   character(len=len_trim(filename)) :: ext
!corr   integer                           :: ier
!
!corr   call get_extension(filename,ext,ier)
   ext = get_extension(filename)
   if (len_trim(ext) == 0) then
       ftype = 0    !!!!!FTYPE_POW
   else
       ext = upper(ext)
       select case (ext(:Lung(ext)))
         case ('XY')
           ftype = FTYPE_XY

         case ('CIF','RTV')
           ftype = FTYPE_CIF

         case ('GDA')
           ftype = FTYPE_GSAS

         case ('POW')
           ftype = FTYPE_POW

         case ('DAT','TXT')
           ftype = FTYPE_POW

         case ('XYE')
           ftype = FTYPE_XYE

         case default   
           ftype = 0   !!!!!FTYPE_POW
           !ftype = 0

       end select
   endif

   end function filetype_from_filename

  !---------------------------------------------------------------------------------------------

   subroutine jump_rows(unit)
!
!  Salta le righe iniziali contrassegnate dal carattere '#' o '>' o '!'
!
   integer, intent(in)          :: unit
   character(len=80)            :: line
   integer                      :: ierror
!
   do
      read(unit,*,iostat=ierror)line
      if (ierror < 0) return             ! controllo per fine file
      line = adjustl(line)
      if (scan(line(1:1),'#>!') == 0) exit
   enddo
!
!  spostati all'ultimo record
   backspace (unit)
!
   end subroutine jump_rows
   
  !---------------------------------------------------------------------------------------------

   subroutine readonlyc(unit,tmino,tstepo,tmaxo,dati,nco,double,error)
!
!  Legge un file di soli conteggi.
!  2theta min,2theta step,2theta max in prima riga e conteggi nelle righe successive.
!  Se si genera errore le variabili tmin,tstep,tmax,xco,yco,nc non vengono modificate. 
!
   USE strutil
   integer, intent(in)             :: unit                 ! unit� di lettura
   real, intent(inout)             :: tmino,tstepo,tmaxo   ! 2theta min., 2theta step, 2theta max.
   type(point_type), allocatable, intent(inout) :: dati(:)
   integer, intent(inout)          :: nco                  ! numero di conteggi
   logical, intent(in)             :: double
   type(error_type), intent(out)   :: error
   real                            :: tmin,tstep,tmax
   type(point_type), allocatable        :: datic(:)              
   integer                         :: nc
   character(len=80)               :: line
   real, dimension(40)             :: vet
   integer, dimension(40)          :: ivet
   integer                         :: iv
   character(len=5)                :: form = '(a80)'
   character(len=20)               :: strt1,strt2,strt3
   integer                         :: iendf
   integer                         :: n
   integer                         :: nini,nfin
   integer                         :: i
!
   call error%reset()
!
!  salta le prime righe con '#'
   call jump_rows(unit)
!
!  Leggi thmin,thmax,thstep
   read(unit,form) line
   call s_filter(line)              !filtra linea
   call getnum(line,vet,ivet,iv)
   if (iv < 3) then
       call error%set('It expects 2theta min., 2theta step, 2theta max. before the countings')
       go to 10
   endif
   tmin=vet(1)
   tstep=vet(2)
   tmax=vet(3)
!
!  Esegue controllo su thmin,thmax,thstep
   if (tmin >= tmax) then
       write(strt1,'(f0.3)') tmin
       write(strt2,'(f0.3)') tmax
       call error%set('2theta min. >= 2theta max.'//char(10)//'2theta min. ='    &
             //trim(strt1)//char(10)//'2theta max. ='//trim(strt2)//char(10)//   &
            'Please insert correct 2theta min. and 2theta max. before the countings')
       go to 10
   endif
   if (tmax >= 180) then
       write(strt1,'(f0.3)') tmin
       write(strt2,'(f0.3)') tmax
       call error%set('2theta min. ='//trim(strt1)//char(10)//'2theta max. =' &
              //trim(strt2)//char(10)//                                             &
              'Please insert correct 2theta min. and 2theta max. before the countings')
       go to 10
   endif
!
!  Calcola il numero di conteggi che andrebbero letti
   nc = nint((tmax - tmin)/tstep) + 1
   if (nc <= 1) then
       write(strt1,'(f0.3)') tmin
       write(strt2,'(f0.3)') tmax
       write(strt3,'(f0.4)') tstep
       call error%set('2theta min ='//trim(strt1)//char(10)//'2theta max. ='//trim(strt2)//char(10)//    &
                            '2theta step ='//trim(strt3)//char(10)//                                     &
       'Please insert correct 2theta min., 2theta step, 2theta max. before the countings or select the correct format of file')
       go to 10
   endif
!
!  Alloca conteggi ad ncounts
   allocate(datic(nc))
!
!  Leggi conteggi
   nfin = 0
   n = 0
   do
      read(unit,form,iostat=iendf) line
      if (iendf < 0) exit
      call s_filter(line)              !filtra linea
      n = n + 1
      call getnum(line,vet,ivet,iv)
      if (iv == 0 .or. err_string) then
          write(strt1,'(i0)')n
          call error%set('Error in reading row '//trim(strt1)//char(10)//trim(line))
          go to 10
      endif
      nini = nfin + 1
      nfin = nini + iv - 1
      if (nfin > nc) then !se thmax non coincide con l'ultimo conteggio
          nfin = nc 
          iv = nfin - nini + 1
      endif
      datic(nini:nfin)%y = vet(:iv)
      if (nfin == nc) exit
      if (double) then
          read(unit,form,iostat=iendf) line
          if (iendf < 0) exit
      endif
   enddo
!
!  tmax potrebbe essere pi� grande del numero di conteggi letto
   nc = nfin
   call reallocate(datic,nc)
!
!  assegna i 2 theta
   datic(:)%x = tmin + (/(i*tstep,i=0,nc-1)/)
   tmax = datic(nc)%x
!
!  In assenza di errore puoi aggiornare tmino,tstepo,tmaxo,xco,yco,nco 
   nco = nc
   tmino = tmin
   tmaxo = tmax
   tstepo = tstep
   call reallocate(dati,nc,.false.)
   do i=1,nco
      dati(i) = datic(i)
   enddo   

   return

10 continue
!
   end subroutine readonlyc
   
 !---------------------------------------------------------------------------------------------

   subroutine read2columns(unit,tmino,tmaxo,tstep,dati,nco,wavel,usestp,error)
!
!  Legge un file con due colonne (2theta e conteggi)   
!  Se si genera errore le variabili tmin,tstep,tmax,xco,yco,nc non vengono modificate. 
! 
   USE strutil
   integer, intent(in)                          :: unit
   real, intent(out)                            :: tmino,tmaxo
   real, intent(inout)                          :: tstep
   type(point_type), allocatable, intent(inout) :: dati(:)
   integer, intent(out)                         :: nco
   logical, intent(in)                          :: usestp
   type(error_type), intent(out)                :: error
   real, intent(out)                            :: wavel ! if <0 wave is not available
   type(point_type), allocatable                :: datic(:)
   integer                                      :: nc
   integer                                      :: i
   character(len=200)                           :: line
   integer                                      :: ierror
   real, dimension(40)                          :: vet
   integer, dimension(40)                       :: ivet
   integer                                      :: iv
   character(len=6)                             :: strn
   integer                                      :: ncc
   real                                         :: wval
!
   !call reset_error(error)
   call error%reset()
!
!  salta le prime righe con '#'
   call jump_rows(unit)
!
!  Calcola il numero di conteggi
   call count_line(unit,nc,ierror)
!
!  Alloca variabili su ncounts
   allocate(datic(nc))
!
!  Leggi 2-theta e conteggi
   call jump_rows(unit)
   ncc = 0
   wavel = -1
   wval = -1
   do i=1,nc
      read(unit,'(a)') line
      call s_filter(line)              !filtra linea
      if (len_trim(line) == 0) cycle   !salta le linee vuote
      call getnum(line,vet,ivet,iv)
      if (ncc == 0 .and. iv == 1) then !wavelength found!
          wval = vet(1)
          cycle
      endif
      if (iv < 2) then   !!!!corr .or. err_string) then
          write(strn,'(i6)')i
          call error%set('Error on reading row '//trim(adjustl(strn))//char(10)//trim(line))
          go to 10
      endif
      ncc = ncc + 1
      datic(ncc)%x = vet(1)
      datic(ncc)%y = vet(2)
   enddo
!
!  Esegui controllo sui 2theta: ogni 2theta deve essere inferiore al suo successivo
   if (any(datic(2:ncc)%x < datic(1:ncc-1)%x)) then
       call error%set('Error on reading column of 2theta')
       go to 10
   endif
!
!  ora in assenza di errore puoi trasferire in xco e yco   
   if (wval > 0) wavel = wval
   nco = ncc
   tmino = datic(1)%x
   tmaxo = datic(ncc)%x   
   call reallocate(dati,ncc,.false.)
   do i=1,ncc
      dati(i) = datic(i)
   enddo
!
   if (usestp) then
!
!     Ricalcola i conteggi usando lo step 
      dati%x = tmino + (/(i*tstep,i=0,ncc-1)/)
      tmaxo = dati(ncc)%x
   else
!
!     Calcola lo step e usa i conteggi del file
      tstep = sum(dati(2:ncc)%x - dati(1:ncc-1)%x)/(ncc-1)
   endif
!
10 return
!
   end subroutine read2columns

 !---------------------------------------------------------------------------------------------

   subroutine readgsas(unit,tmino,tstepo,tmaxo,dati,nco,error)
!
!  Legge un file di soli conteggi.
!  2theta min,2theta step,2theta max in prima riga e conteggi nelle righe successive.
!  Se si genera errore le variabili tmin,tstep,tmax,xco,yco,nc non vengono modificate. 
!
   USE strutil
   integer, intent(in)                          :: unit                 ! unit� di lettura
   real, intent(inout)                          :: tmino,tstepo,tmaxo   ! 2theta min., 2theta step, 2theta max.
   type(point_type), allocatable, intent(inout) :: dati(:)
   integer, intent(inout)                       :: nco                  ! numero di conteggi
   type(error_type), intent(out)                :: error
   real                                         :: tmin,tstep,tmax
   type(point_type), allocatable                :: datic(:)              
   integer                                      :: nc
   character(len=80)                            :: line,line1
   integer                                      :: nlong1,nlong2
   character(len=5)                             :: form = '(a80)'
   integer                                      :: ierror
   integer                                      :: n
   integer                                      :: nini,nfin
   integer                                      :: i
   integer                                      :: typef
   integer                                      :: ndatap
!
   call error%reset()
!
!  Cerca la linea con la stringa 'BANK' 
   do 
      read(unit,form,iostat=ierror) line
      if (ierror < 0) then
          call error%set("It doesn't seem GSAS file")
          go to 10
      endif
      call s_filter(line)  !filtra linea
      if (line(1:4) == 'BANK') then
          call cutst(line,nlong1,line1,nlong2)    ! cut 'BANK'
          call cutst(line,nlong1,line1,nlong2)    ! cut IBANK
          call cutst(line,nlong1,line1,nlong2)    ! leggi il numero di punti
          call s_to_i(line1,nc,ierror,nlong2)
          call cutst(line,nlong1,line1,nlong2)    ! cut NREC
          call cutst(line,nlong1,line1,nlong2)    ! leggi BYNTYP
          if (trim(line1) /= 'CONST') then
              call error%set('Cannot read GSAS file with BYNTYP='//trim(line1))
              go to 10
          endif
          call cutst(line,nlong1,line1,nlong2)    ! leggi BCOEF(1) = thmin
          call s_to_r(line1,tmin,ierror,nlong2)
          tmin = tmin*0.01
          call cutst(line,nlong1,line1,nlong2)    ! leggi BCOEF(2) = thstep
          call s_to_r(line1,tstep,ierror,nlong2)
          tstep = tstep*0.01
          call cutst(line,nlong1,line1,nlong2)    ! cut BCOEF(3)
          call cutst(line,nlong1,line1,nlong2)    ! cut BCOEF(4) 
          call cutst(line,nlong1,line1,nlong2)    ! leggi type
          select case (line1)
            case ('STD')
              ndatap = 10
              typef = 1
            case ('ESD')
              ndatap = 5
              typef = 2
            case ('ALT','FXY','FXYE')
              call error%set('Cannot read GSAS file with keyword TYPE='//trim(line1))
              go to 10
            case default
              ndatap = 10
              typef = 1
          end select
          exit
      endif
   enddo
!
!  Alloca conteggi ad ncounts
   allocate(datic(nc))
!
!  Leggi conteggi
   nfin = 0
   n = 0
   do
      read(unit,form,iostat=ierror) line
      if (ierror < 0) then
          nc = nfin  ! potrebbe aver letto meno conteggi degli nc previsti
          exit
      endif
      call s_filter(line)   
      nini = nfin + 1
      nfin = nini + ndatap - 1
      if (nfin > nc) nfin = nc !se thmax non coincide con l'ultimo conteggio
      select case (typef)
         case (1)     ! 'STD' or blank
           read(line,'(10(2x,f6.0))') datic(nini:nfin)%y

         case (2)     ! 'ESD'
           read(line,'(10(f8.0,8x))') datic(nini:nfin)%y
      end select
      if (nfin == nc) exit
   enddo
   datic%x = tmin + (/(n*tstep,n=0,nc-1)/)
   tmax = datic(nc)%x
!
!  In assenza di errore puoi aggiornare tmino,tstepo,tmaxo,xco,yco,nco 
   nco = nc
   tmino = tmin
   tmaxo = tmax
   tstepo = tstep
   call reallocate(dati,nc,.false.)
   do i=1,nco
      dati(i) = datic(i)
   enddo   

   return

10 continue 
!
   end subroutine readgsas

  !---------------------------------------------------------------------------------------------

   subroutine readcif(unit,tmino,tstepo,tmaxo,dati,nco,wavel,error)
!
!  Legge un file con due colonne (2theta e conteggi)   
!  Se si genera errore le variabili tmin,tstep,tmax,xco,yco,nc non vengono modificate. 
! 
   USE strutil
   USE cif_frm
   integer, intent(in)                           :: unit
   real, intent(out)                             :: tmino,tstepo,tmaxo
   type(point_type), allocatable, intent(inout)  :: dati(:)
   integer, intent(out)                          :: nco
   real, intent(out)                             :: wavel ! if <0 wave is not available
   type(error_type), intent(out)                 :: error
   type(point_type), allocatable                 :: datic(:)
   integer                                       :: nc
   integer                                       :: i
   integer, parameter                            :: NW=10
   integer, parameter                            :: NAVET = 1000
   type(cifword_type), dimension(:), allocatable :: cifword
   real                                          :: range_min,range_max,range_inc
   integer                                       :: n
   logical                                       :: countx_fill,county_fill
   logical                                       :: newdata
   integer                                       :: nline 
   character(len=100)                            :: dataname
!
   call error%reset()
!
   call load_cif_dictionary(cifword)
!
   nline = 0
   call cifword_fill(cifword,unit,newdata,nline,dataname,error)
!
   nc = 0
   range_min = -1
   range_max = -1
   range_inc = -1
   county_fill = .false.
   countx_fill = .false.
   wavel = -1
!
   do i=size(cifword),1,-1
      if (cifword(i)%wok) then
          select case (trim(cifword(i)%str))
             case ('_pd_meas_2theta_scan','_pd_proc_2theta_corrected','_pd_proc_2theta_fixed')
               if (.not.countx_fill) then
                   nc = cifword(i)%nv
                   if(.not.allocated(datic))allocate(datic(nc))
                   datic(:)%x = cifword(i)%vet(:nc)
                   countx_fill = .true.
               endif 
             case ('_pd_meas_intensity_total','_pd_proc_intensity_total','_pd_meas_counts_total','_pd_proc_intensity_net')
               if (.not.county_fill) then
!
!                  riempio le y solo una volta 
                   county_fill = .true.
                   nc = cifword(i)%nv
                   if(.not.allocated(datic))allocate(datic(nc))
                   datic(:)%y = cifword(i)%vet(:nc)
               endif
             case ('_pd_meas_2theta_range_min')
               range_min = cifword(i)%vet(1)
             case ('_pd_meas_2theta_range_max')
               range_max = cifword(i)%vet(1)
             case ('_pd_meas_2theta_range_inc')
               range_inc = cifword(i)%vet(1)
             case ('_diffrn_radiation_wavelength')
               wavel = cifword(i)%vet(1)
          end select
      endif
   enddo
!
!  ora in assenza di errore puoi trasferire i conteggi
   if (nc > 0 .and. county_fill) then
       if(countx_fill) then
          if (range_inc < 0) range_inc = sum(datic(2:nc)%x - datic(1:nc-1)%x)/(nc-1)
       else
          if (range_min < 0 .and. range_inc < 0) then
              call error%set('Undefined range in pdCIF file')
              go to 10
          endif
          datic%x = range_min + (/(n*range_inc,n=0,nc-1)/)
       endif
       tstepo = range_inc
       nco = nc
       tmino = datic(1)%x
       tmaxo = datic(nc)%x   
       call reallocate(dati,nc,.false.)
       do i=1,nc
          dati(i) = datic(i)
       enddo
   else
       call error%set('Cannot find intensity measurements in pdCIF file')
       go to 10
   endif
!
   return
!
10 continue 
!
   end subroutine readcif

  !---------------------------------------------------------------------------------------------

   subroutine read_reflection_file(filename,refl,numref,stype,error)
!
!  Legge un file di riflessi con nelle prime 3 colonne gli indici h,k,l
!  fvet1 e fvet2 conterranno le altre colonne
!
   USE strutil
   USE fileutil
   character(len=*), intent(in)                        :: filename
   type(reflection_type), dimension(:), allocatable    :: refl
   integer, intent(out)                                :: numref
   type(error_type), intent(out)                       :: error
   character(len=*), intent(in)                        :: stype
   integer                                             :: ierror
   integer                                             :: nline
   character(len=250)                                  :: line
   real, dimension(40)                                 :: vet
   integer, dimension(40)                              :: ivet
   integer                                             :: iv
   integer                                             :: i
   character(len=20)                                   :: strn
   type(file_handle)                                   :: fileh
   integer                                             :: ierl
!
   call error%reset()
   call fileh%fopen(filename,'r')
   if (fileh%good()) then
!
       call count_line(fileh%handle(),nline,ierror)
       if (ierror > 0 .or. nline == 0) go to 10
!
       call reallocate(refl,nline)
!
       numref = 0
       do i=1,nline
          read(fileh%handle(),'(a)') line
          call s_filter(line)  !filtra linea
          if (is_comment_line(line,'#','>','!')) cycle
          select case (stype)
             case ('shelx')
               read(line,'(3i4,2f8.2)',iostat=ierl)ivet(1:3),vet(4),vet(5)
             case ('fc')
               read(line,'(3i4,3f8.2)',iostat=ierl)ivet(1:3),vet(4),vet(5),vet(6)
               if (ierl == 0) refl(numref+1)%ph = nint(vet(6))
             case default
               call getnum(line,vet,ivet,iv)
               if (iv < 4) then
                   ierl = 1
               else
                   if (iv == 4) vet(5) = 0.0  ! sigma(Fo) is absent
                   ierl = 0
               endif
          end select
          if (ierl /= 0) then
              write(strn,'(i0)')i
              call error%set('Error on reading row '//trim(adjustl(strn))//    &
                             ' in reflection file '//trim(filename)//char(10)//trim(line))
              return
          endif
          numref = numref + 1
          refl(numref)%hkl = ivet(1:3)
          refl(numref)%fo = vet(4)
          refl(numref)%fc = vet(5)  ! sigma(Fo)
       enddo
       call reallocate(refl,numref)
   else
       call set_file_error(filename,fileh%err(),error)
   endif
   call fileh%fclose()
!
   return
!
10 call error%set('Error in reading reflection file '//trim(filename))
!
   end subroutine read_reflection_file

  !---------------------------------------------------------------------------------------------
   
   subroutine count_line(unit,nline,ierror)
!
!  Conta il numero di linee in un file 
!
   integer, intent(in)  :: unit
   integer, intent(out) :: nline
   integer, intent(out) :: ierror ! > 0 se si verifica errore
!
   nline = 0
   ierror = 0
   do
      read(unit,*,iostat=ierror)
      if (ierror /= 0) exit                   ! fine file o errore
      nline = nline + 1
   enddo
   if (ierror < 0) ierror = 0
   rewind unit
!
   end subroutine count_line

  !---------------------------------------------------------------------------------------------

   subroutine write_column(filename,jfile,xcol1,xcol2)
!
!  Scrive su file in colonne
!
   USE fileutil
   character(len=*), intent(in), optional  :: filename
   integer, intent(in), optional           :: jfile
   real, dimension(:), intent(in)          :: xcol1
   real, dimension(:), intent(in),optional :: xcol2
   integer                                 :: j_in
   integer                                 :: i
   integer                                 :: ier
!
!  Apertura file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile
       ier = 0
   endif
!
!  Scrittura su file
   if (ier == 0) then
       if (present(xcol2)) then
           write(j_in,'(f0.6,1x,f0.6)')(xcol1(i),xcol2(i),i=1,min(size(xcol1),size(xcol2)))
       else
           write(j_in,'(f0.6)')(xcol1(i),i=1,size(xcol1))
       endif
       close(j_in)
   endif
!
   end subroutine write_column

  !---------------------------------------------------------------------------------------------

   subroutine read_column(filename,xcol,ncol,error)
!
!  Legge un file su colonne
!
   USE fileutil
   USE strutil
   character(len=*), intent(in)                   :: filename
   real, dimension(:), allocatable, intent(inout) :: xcol
   integer, intent(out)                           :: ncol
   type(error_type), intent(out)                  :: error
   integer                                        :: j_in
   integer                                        :: ncol0
   integer                                        :: ierror
   character(len=250)                             :: line
   character(len=20)                              :: strn
   real, dimension(40)                            :: vet
   integer, dimension(40)                         :: ivet
   integer                                        :: iv
   integer                                        :: i
!
   ncol = 0
!
!  Apertura file
   call get_unit(j_in)
   open(unit=j_in,file=filename,iostat=ierror,status='old')
!
   if (ierror == 0) then
       call error%reset()
!
       call count_line(j_in,ncol0,ierror)   ! conta le linee nel file
       if (ierror > 0 .or. ncol0 == 0) go to 10
!
       call reallocate(xcol,ncol0,.false.)  ! prealloca xcol al num. di linee
!
       do i=1,ncol0
          read(j_in,'(a)') line
          call s_filter(line)  !filtra linea
          if (is_comment_line(line,'#','>','!')) cycle
          call getnum(line,vet,ivet,iv)
          if (iv == 0) then
              write(strn,'(i0)')i
              call error%set('Error on reading row '//trim(adjustl(strn))//' in '//trim(filename)//char(10)//trim(line))
              return
          endif
          ncol = ncol + 1
          xcol(ncol) = vet(1)
       enddo
       call reallocate(xcol,ncol)
   else
       if (.not.file_exist(filename)) then
          call error%set("Can't find file "//trim(filename))
       else
          call error%set('Error opening the file '//trim(filename))
       endif
   endif
   return
!
10 call error%set('Error in reading file '//trim(filename))
!
   end subroutine read_column

  !---------------------------------------------------------------------------------------------

   subroutine get_info_datafile(fname,tmin,tmax,wavel,ier)
   USE fileutil
   character(len=*), intent(in)                :: fname
   integer, intent(out)                        :: ier
   real, intent(out)                           :: tmin,tmax,wavel
   integer                                     :: j_in
   type(point_type), allocatable, dimension(:) :: countp
   real                                        :: tstep
   integer                                     :: ftype
   integer                                     :: ncount
!
!  Open file
   call get_unit(j_in)
   open(unit=j_in,file=fname,iostat=ier,status='old',action='read')
   if (ier == 0) then
       ftype =  filetype_from_filename(fname)
       call read_file_from_type(j_in,ftype,tmin,                    &
            tstep,tmax,countp,ncount,.false.,wavel,ier)
       if (ier == 0 .and. wavel < 0) wavel = 1.54056
       close(j_in)
   endif
!
   end subroutine get_info_datafile

END MODULE filereading

   subroutine get_info_datafileC(fname,tmin,tmax,wavel,ier)
   USE filereading
   character(len=*), intent(in)                :: fname
   integer, intent(out)                        :: ier
   real, intent(out)                           :: tmin,tmax,wavel

   call  get_info_datafile(fname,tmin,tmax,wavel,ier)
!
   end subroutine get_info_datafileC

