!#define FFTW

MODULE fouriermap

implicit none

integer, parameter             :: FOUR_FO=1, FOUR_2FO_FC=2, FOUR_FO_FC=3, FOUR_FC=4
character(len=8), dimension(4) :: four_name = [ 'Fo      ', '2Fo - Fc', 'Fo - Fc ', 'Fc      ' ] 

real :: maxfmap

type map_info_type 
   logical :: active   ! map visualization active?
   integer :: ftype    ! map type? 
end type map_info_type

type(map_info_type) :: mapinfo

CONTAINS


   subroutine labelx(icalfo,atmold,nPeaksToFind,ier)
   USE General, only: lo, StructureName, cell
   USE Symmetry, only: nsym, ncentr, ddmin, icent
   USE Atomi
   USE Molcom, only: It
   USE sinterface
   USE ilmviewmod
   USE variables
   USE atom_type_util
   USE fragmentmod
   USE connect_mod
   USE unit_cell
   USE PhaseModule, only:biso
   USE realloc
   USE strutil
!
   implicit none
   integer, intent(in)                            :: icalfo
   type(atom_type), dimension(:), intent(in)      :: atmold
   integer, intent(in)                            :: nPeaksToFind
!corr      real, intent(in)                               :: umapel
   type(fragment_type), dimension(:), allocatable :: fragm
   integer                                        :: nfrag
   type(atom_type), dimension(:), allocatable     :: atmc
   real, dimension(3)                             :: xn,xo,xeq,s
   real, dimension(2)                             :: vet
   real, dimension(:), allocatable                :: tempdist
   real, allocatable, dimension(:)                :: distminv
   character(len=16), dimension(:), allocatable   :: labmin
   integer, allocatable, dimension(:)             :: kminv,selected
   integer                                        :: iAction !Gianluca 13/01/2016
   integer                                        :: i, j, k, ifi, kt, jpunt, ifin
   integer                                        :: Naccept
   integer                                        :: ier
   integer                                        :: iExitC,MultiSel,NumCan
   real                                           :: distmin, d, distm
   integer                                        :: kmin, iat
   integer                                        :: nequal, nnzo
   real                                           :: scalatom, sumnew, sumold
   integer                                        :: nat_old, nat_new
   type(atom_type), dimension(:), allocatable     :: atmnew
   logical                                        :: kpr = .true.
   integer                                        :: new_peaks
   real                                           :: coef
   character(len=:), allocatable                  :: msg
!
   nat_old = size(atmold)
   call xyzs_to_atom(atmnew)
   nat_new = numatomtype(atmnew)
   call atom_string(atmnew)  ! peaks will be as Q1, Q2, ...
   atmnew(:)%biso=biso
   if (kpr) then
       call stampa_struttura(atmold,title='Old atoms list')
       call stampa_struttura(atmnew,title='Fourier peaks')
   endif
   allocate(tempdist(nat_new))
   vet(:) = (/1.0,-1.0/)
   ifin=icent+1
   qq(:) = 0.0
   do i=1,nat_old
      nnzo=iabs(atmold(i)%nz)/nzConst
      xo(:) = atmold(i)%xc(:)
      distm=9999
      jpunt=0
      do j=1,nat_new
         if (qq(j).lt.0.5) then
             xn(:) = atmnew(j)%xc(:)
             do kt=1,ncentr
                do ifi=1,ifin
                   coef=vet(ifi)
                   do k=1,nsym
                      call xequi(xo,k,coef,xeq,kt)
                      call xdist(xn,xeq,d,s)
                      if (d.lt.ddmin) then
                          distm=d
                          jpunt=j
                          qq(jpunt) = i
                          tempdist(jpunt) = d
                      endif
                   enddo
                enddo
             enddo
         endif
      enddo
   enddo
!
   if (kpr) then
       do j=1,nat_new
          i=nint(qq(j))
          if (i.gt.0) then
              write(lo,'(2x,a,i5,4f10.3,10x,a8,i4,4f10.3,1x,f10.3)')             &
              atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso,    &
              atmold(i)%lab,i,atmold(i)%xc(:),bfac(i),tempdist(j)
          else
              write(lo,'(2x,a,i5,4f10.3)') atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso
          endif
       enddo
   endif
   new_peaks = 0
   nequal = 0
   sumold = 0.0
   sumnew = 0.0
!
   do j=1,nat_new
!
!-- accept the new peak only if has no counterpart
!-- in the atom list
!
      i=nint(qq(j))
      if (i.le.0) then
          new_peaks = new_peaks + 1
          atmnew(new_peaks)%xc = atmnew(j)%xc
          atmnew(new_peaks)%biso = biso
      elseif (icalfo.ne.3) then
          nequal = nequal + 1
          sumold = sumold + atmold(i)%inte
          sumnew = sumnew + atmnew(j)%inte
      endif
   enddo
   scalatom = 1.0
!
! Riscala intensita
!
  !!!!FIXME-non funziona                     atmnew(:)%inte = atmnew(:)%inte / umapel
   if (nequal.gt.0.and.icalfo.ne.3) then
       if (sumnew.gt.0.0) scalatom = sumold/sumnew
!test          do j=1,new_peaks
!test             atmnew(j)%inte = scalatom * atmnew(j)%inte
!test          enddo
   endif
   if (new_peaks > nPeaksToFind) new_peaks = nPeaksToFind
!
   nat_new = new_peaks + nat_old
   call reallocate(atmnew,nat_new)
   if (new_peaks > 0) then
       write(lo,265) new_peaks
  265     format(  10x,i5, '  new peak(s) found')
       call calcola_occ(atmnew(:new_peaks))
!
!         move atoms at the end of arrays
!         to make room for old atoms
       atmnew(nat_old+1:) = atmnew(:new_peaks)
   endif
!
!  copy old atoms at the beginning of arrays
   atmnew(:nat_old) = atmold(:)
!
   if (new_peaks > 0) then
       nats = nat_old + nats
       call copia_atoms(atm,atmnew)
!
!      Frammenta il modello
       call get_fragments(atm,lconn,nfrag,fragm)
!
!      Raggruppa i frammenti 
       call regroup_fragment(fragm,atm)
!
!      Disegna il modello e i nuovi atomi come ghost
       if (.not.jav_opened())call ilmview_open(StructureName)
       call jav_set_modify(0)
       call ilmview_draw(atm,lconn)
!
!      Calcola distanza minime
       call copia_atoms(atmc,atm)
       call frac_to_cart(atmc,orthomatrix(cell))
       allocate(distminv(new_peaks),labmin(new_peaks), kminv(new_peaks))
       do i=1,new_peaks
          iat = nat_old+i
          call get_minimum_distance(atmc(iat),atmc(:nat_old),kmin,distmin)
          distminv(i) = distmin
          kminv(i) = kmin          !!mr
          labmin(i) = atmc(kmin)%lab
          write(It(i),'(a,i0,4x,a,f0.3)')trim(atmc(iat)%lab)//'  I=', int(atmc(iat)%inte),  &
            'dmin('//trim(atmc(iat)%lab)//'-'//trim(atmc(kmin)%lab)//')=' ,distmin
           It(i) = trim(It(i))//char(0)
       enddo
!
!!Gianluca 13/01/2016
!      iAction = 2
!      call ClessidraOnOff(iAction)
!!Gianluca 13/01/2016
       call ClearMessage()
! --
       msg = trim(i_to_s(new_peaks))//' new peaks found.'//char(10)//  &
             'Check Intensities &amp; distances and select peaks to retain.'//char(0)
       NumCan = new_peaks
       MultiSel = 1
       allocate (selected(NumCan))
       selected(:) = 0
       call c_NewPeekList('Select peaks to accept'//char(0), new_peaks,     &
                          MultiSel, selected,iExitC, nat_old, msg, atmc, distminv, kminv)
       call jav_set_modify(1)           !!mr 13/01/2016
       if (iExitC == 3) then
           selected(:) = 0
       elseif (iExitC.eq.2) then
           selected(:) = 1
       endif
       Naccept = 0
       do i=1, NumCan
          if (selected(i).eq.1) then
              Naccept = Naccept + 1
              atm(nat_old+Naccept) = atm(nat_old+i)
          endif
       enddo
       if (Naccept == 0) ier = 1 ! segnala che non c'e' modifica
       nats = nat_old + Naccept
       call reallocate(atm,nats)
       call ilmview_draw(atm,lconn)
       call jav_set_modify(1)
!Gianluca 13/01/2016
      iAction = 2
      call ClessidraOnOff(iAction)
!Gianluca 13/01/2016
   else
       nats=nat_old
       ier = -1
       call OpenWarning('No new peaks found !!!')
!!mr 5/06/2016
       call jav_set_modify(1)
       iAction = 2
       call ClessidraOnOff(iAction)
!!end mr
   endif
   call atom_to_xyzs(atm)
!Gianluca 13/01/2016
!         iAngolo = 0
!         leggi = 1
!         call toMolPlo3(leggi)
          call toMolPlo3(0)
          call RotazTot
          call GetBonds(1)
!Gianluca 13/01/2016
   !call print_connect(atm%lab,legm=lconn,kpri=0)
!
   end subroutine labelx

END MODULE fouriermap
