MODULE fastfcal
!Questo modulo contiene procedure per velocizzare il calcolo dei fattori di struttura attraverso
!la memorizzazione di opportune variabili in sfactor e limitando il calcolo solo agli atomi per i
!quali si verificano variazioni di coord,fattori termici,occ.
!
!S fcalc_init(ref,atx)                Subroutine di inizializzazione
!S fcalc_close()                      Dealloca e azzera tutto   
!S fcalc_for_tfocc(ref,atx,pafc)      Ricalcola tfocc soltanto per gli atomi atx 
!S fcalc_from_atm(ref,atx,pafc)       Calcola fattori di struttura per atomi atx se cambiano le coordinate 
!S fcalc_from_thermalf((ref,atx,pafc) Calcola fattori di struttura per atomi atx se cambiano fatt. term e occ.
!S fcalc_save_atm(pafc)               Salva i sen e cos per gli atomi indicati nel vettore pafc
!S fcalc_save_thermalf(pafc)          Salva tfocc per gli atomi indicati nel vettore pafc
!S fcalc_ripr_atm(pafc)               Ripristina sen e cos per gli atomi indicati nel vettore pafc
!S fcalc_ripr_thermalf(pafc)          Ripristina tfocc per gli atomi indicati nel vettore pafc
!S fcalc_get_phases(ref)              Calcola le fasi
USE progtype, only:reflection_type
USE atom_basic, only: atom_type

implicit none

type sfactor_type
     real, dimension(:),   allocatable :: ff             !fattori di scattering(f.s.)
     real, dimension(:,:), allocatable :: hkle           !matrici di simmetria
     real, dimension(:),   allocatable :: ht
     real, dimension(:),   allocatable :: fcosat,fsinat  !contributo per atomo al coseno e seno
     real, dimension(:),   allocatable :: tfocc          !termine dipendente da f.s.,occ.,fattore termico
end type

type(sfactor_type), dimension(:), allocatable, private :: sfactor,sfactorc !sfactor e una sua copia
integer, private                                       :: ninforef         !numero di riflessi
integer, private                                       :: ninfoatoms       !numero di atomi
integer, dimension(:), allocatable, private            :: kpscat           !punta al fattore di scattering
!
! generazione tabella seni/coseni
integer, private                 :: ipt
real, parameter, private         :: to_rad = 3.141592653589793238462643383279502884197 / 180
real, dimension(0:3599), parameter :: sintable = (/(sin(to_rad*(ipt)/10),ipt=0,3599)/)
real, dimension(0:3599), parameter :: costable = (/(cos(to_rad*(ipt)/10),ipt=0,3599)/)
!
!!usa la tabella sin/cos solo se il compilatore non e' intel
!#if !defined(__INTEL_COMPILER)
#define TRIG_TABLE 1
!#endif

CONTAINS

   subroutine fcalc_init(ref,atx)
!
!  Subroutine di inizializzazione. Alloca sfactor(c) e precalcola tutto.
!   
   USE General, only : al,cl,bs,nk,twopi
   USE Symmetry, only : nsym,icent,kmat,tmat   !!!!!,ismat,tsmat
   USE Atomi, only:nzConst
   USE realloc
!
   type(reflection_type), dimension(:), intent(inout) :: ref    ! i riflessi
   type(atom_type), dimension(:), intent(in)          :: atx    ! modello strutturale
   integer, dimension(3)                              :: i1
   integer                                            :: i,j   !!!!!,k,l
!!!corr   real                                               :: t
   !integer                                            :: m
   real                                               :: rho
   real                                               :: const
!
!  Numero di riflessi e di atomi su cui verra eseguito il calcolo di fattori di struttura
   ninforef = size(ref)
   ninfoatoms = size(atx)
!
!  Alloca sfactor
   if (.not.allocated(sfactor)) then
        allocate(sfactor(ninforef),sfactorc(ninforef))
        do i=1,ninforef
           !allocate(sfactor(i)%ff(nk),sfactorc(i)%ff(nk))
           allocate(sfactor(i)%ff(nk))
           !allocate(sfactor(i)%hkle(3,nsym),sfactorc(i)%hkle(3,nsym),sfactor(i)%ht(nsym),sfactorc(i)%ht(nsym))
           allocate(sfactor(i)%hkle(3,nsym),sfactorc(i)%hkle(3,nsym),sfactor(i)%ht(nsym))
           allocate(sfactor(i)%fcosat(ninfoatoms),sfactorc(i)%fcosat(ninfoatoms))
           if (icent == 0) allocate(sfactor(i)%fsinat(ninfoatoms),sfactorc(i)%fsinat(ninfoatoms))
           allocate(sfactor(i)%tfocc(ninfoatoms),sfactorc(i)%tfocc(ninfoatoms))
        enddo
   endif
!
!  Calcola matrici e fattori di scattering
   loop_riflessi: do i=1,ninforef
      i1 = ref(i)%hkl
      !do j=1,nsym
      !   t=0.0   !!!1000.0
      !   do l=1,3
      !      t=t+i1(l)*tsmat(l,j)
      !      sfactor(i)%hkle(l,j)=0
      !   enddo
      !   do l=1,3
      !      do k=1,2
      !       m=iabs(ismat(k,l,j))
      !       if(m.gt.0) sfactor(i)%hkle(m,j)=sfactor(i)%hkle(m,j)+i1(l)*isign(1,ismat(k,l,j))
      !      enddo
      !   enddo
      !   sfactor(i)%ht(j)=t
      !enddo

      do j=1,nsym
         sfactor(i)%ht(j) = dot_product(i1,tmat(j,:))   ! H*T
         sfactor(i)%hkle(:,j) = matmul(i1,kmat(j,:,:))  ! H*R
      enddo
      rho=ref(i)%slaq
!
      sfactor(i)%ff(:nk) = cl(:nk) + sum(al(:,:nk)*exp(-bs(:,:nk)*rho),dim=1)
   enddo loop_riflessi
!
!  Ingloba la constante nei valori precalcolati
   if (icent == 1) then
       const = twopi
   else
#ifdef TRIG_TABLE
       const = 360
#else
       const = twopi
#endif
   endif
   do i=1,ninforef
      sfactor(i)%hkle(:,:) = sfactor(i)%hkle(:,:)*const
      sfactor(i)%ht(:) = sfactor(i)%ht(:)*const
   enddo
!
!  Memorizza il puntatore al fattore di scattering
   call reallocate(kpscat,ninfoatoms)
   kpscat(:) = iabs(atx(:)%nz/nzConst)
!
!  Calcola tfocc per tutti gli atomi
   call fcalc_for_tfocc(ref,atx,(/(i,i=1,ninfoatoms)/))
!
!  Calocla fattori di struttura per tutti gli atomi
   call fcalc_from_atm(ref,atx,(/(i,i=1,ninfoatoms)/))
   call fcalc_save_atm((/(i,i=1,ninfoatoms)/))
!
   end subroutine fcalc_init

!-----------------------------------------------------------------------

   subroutine fcalc_close()
!
!  Dealloca e azzera tutto   
!
   if (allocated(sfactor)) then
       deallocate(sfactor,sfactorc)
       ninforef = 0
       ninfoatoms = 0
   endif
!
   end subroutine fcalc_close

!-----------------------------------------------------------------------

   subroutine fcalc_for_tfocc(ref,atx,pafc)
!
!  Ricalcola tfocc soltanto per gli atomi atx 
!   
   USE Symmetry, only : pts
!
   type(reflection_type), dimension(:), intent(in) :: ref    ! i riflessi
   type(atom_type), dimension(:), intent(in)       :: atx    ! atomi per i quali calcolare tfocc
   integer, dimension(:), intent(in)               :: pafc   ! num. d'ordine degli atomi atx
   integer                                         :: i,jj
   real                                            :: rho
   integer                                         :: kz
   integer                                         :: natfc
!
   natfc = size(atx)
   do i=1,ninforef
      rho=ref(i)%slaq
      do jj=1,natfc
         kz=kpscat(pafc(jj))
         sfactor(i)%tfocc(pafc(jj))=pts*sfactor(i)%ff(kz)*exp(-atx(jj)%biso*rho)*atx(jj)%och*atx(jj)%ocry
      enddo 
   enddo 
!
   end subroutine fcalc_for_tfocc
   
!-----------------------------------------------------------------------
#if 0
   subroutine fcalc_from_atm_old(ref,atx,pafc)
!
!  Calcola Fc**2 dopo aver ricalcolato i contibuti fcosat e fsinat solo per atx
!  Va usata se cambiano le coordinate di uno o piu atomi
!   
   USE General, only : pi        , rtod,twopi
   USE Symmetry, only : nsym,icent
!
   type(reflection_type), dimension(:), intent(inout) :: ref    ! i riflessi
   type(atom_type), dimension(:), intent(in)          :: atx    ! atomi per i quali ricalcolare cos e sin
   integer, dimension(:), intent(in)                  :: pafc   ! num. d'ordine degli atomi atx
   integer                                            :: i,j,jj
   real                                               :: at,bt
   real                                               :: argg
   integer                                            :: natfc
   integer :: k
   real :: dot_pro
   real :: cs,ss
   integer :: pos
!
   natfc = size(atx)
   if (icent == 0) then
       do i=1,ninforef
          do jj=1,natfc
             at=0.0
             bt=0.0
             do j=1,nsym
                dot_pro = 0.0
                do k = 1,3
                   dot_pro = dot_pro + sfactor(i)%hkle(k,j)*atx(jj)%xc(k)
                enddo                
                !argg = twopi*(dot_pro + sfactor(i)%ht(j))
                !tval = tan(argg*0.5)
!new
                argg = twopi*(dot_pro + sfactor(i)%ht(j))
                !! scommenta se riattivi la s call sincos(argg,ss,cs)
                !ss = sin(argg)
                !cs = cos(argg)
                  !write(66,*)argg,ss,cs,sin(argg),cos(argg)
!new
!old                argg = pi*(dot_pro + sfactor(i)%ht(j))
!old                tval = tan(argg)
!old                ttval = tval*tval
!old                ttval1 = 1.0/(1+ttval)
!old                cs = (1-ttval)*ttval1
!old                !ss = 2.0*tval*ttval1 !ho spostato il 2 fuori dal loop
!old                ss = tval*ttval1
                at = at + cs
                bt = bt + ss
             enddo
             pos = pafc(jj)
             !sfactor(i)%fcosat(pafc(jj)) = at   
             !sfactor(i)%fsinat(pafc(jj)) = bt
             sfactor(i)%fcosat(pos) = at   
             sfactor(i)%fsinat(pos) = bt
          enddo
          ref(i)%fc = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)**2 +  &
                         dot_product(sfactor(i)%fsinat,sfactor(i)%tfocc)**2                              
       enddo 
   else
       do i=1,ninforef
          do jj=1,natfc
             at=0.0
             do j=1,nsym
                dot_pro = 0
                do k = 1,3
                   dot_pro = dot_pro + sfactor(i)%hkle(k,j)*atx(jj)%xc(k)
                enddo                             
                argg = twopi*(dot_pro + sfactor(i)%ht(j))
                at = at + cos(argg)
             enddo
             sfactor(i)%fcosat(pafc(jj)) = at
          enddo 
          dot_pro = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)
          !ref(i)%fc = 4.0*dot_pro*dot_pro  ! eliminato il 4
          ref(i)%fc = dot_pro*dot_pro
       enddo 
   endif
!
   end subroutine fcalc_from_atm_old
#endif

!-----------------------------------------------------------------------

   subroutine fcalc_from_atm(ref,atx,pafc)
!
!  Calcola Fc**2 dopo aver ricalcolato i contibuti fcosat e fsinat solo per atx
!  Va usata se cambiano le coordinate di uno o piu atomi
!   
   USE General, only : pi, rtod,twopi
   USE Symmetry, only : nsym,icent
!
!
   type(reflection_type), dimension(:), intent(inout) :: ref    ! i riflessi
   type(atom_type), dimension(:), intent(in)          :: atx    ! atomi per i quali ricalcolare cos e sin
   integer, dimension(:), intent(in)                  :: pafc   ! num. d'ordine degli atomi atx
   integer                                            :: i,j,jj
   real                                               :: at,bt
   real                                               :: argg
   integer                                            :: natfc
   integer :: k
   real :: dot_pro
   real :: cs,ss
   integer :: pos
   integer :: iangle
!
   natfc = size(atx)
   if (icent == 0) then
       do i=1,ninforef
          do jj=1,natfc
             at=0.0
             bt=0.0
             do j=1,nsym
                dot_pro = 0.0
                do k = 1,3
                   dot_pro = dot_pro + sfactor(i)%hkle(k,j)*atx(jj)%xc(k)
                enddo                
!
                argg = (dot_pro + sfactor(i)%ht(j))
#ifdef TRIG_TABLE
                iangle = mod(argg+36000,360.)*10 
                cs = costable(iangle)
                ss = sintable(iangle)
                at = at + cs
                bt = bt + ss
#else
                !call sincos(argg,ss,cs)
                !at = at + cs
                !bt = bt + ss
                at = at + cos(argg)
                bt = bt + sin(argg)
#endif
             enddo
             pos = pafc(jj)
             sfactor(i)%fcosat(pos) = at   
             sfactor(i)%fsinat(pos) = bt
          enddo
          ref(i)%fc = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)**2 +  &
                         dot_product(sfactor(i)%fsinat,sfactor(i)%tfocc)**2                              
       enddo 
   else
       do i=1,ninforef
          do jj=1,natfc
             at=0.0
             do j=1,nsym
                dot_pro = 0
                do k = 1,3
                   dot_pro = dot_pro + sfactor(i)%hkle(k,j)*atx(jj)%xc(k)
                enddo                             
                argg = (dot_pro + sfactor(i)%ht(j))
                at = at + cos(argg)
             enddo
             sfactor(i)%fcosat(pafc(jj)) = at
          enddo 
          dot_pro = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)
          !ref(i)%fc = 4.0*dot_pro*dot_pro  ! eliminato il 4
          ref(i)%fc = dot_pro*dot_pro
       enddo 
   endif
!
   end subroutine fcalc_from_atm

!-----------------------------------------------------------------------

   real function cos_32(angle)
   USE General, only : pi,twopi
!
!  This is the main cosine approximation "driver"
!  It reduces the input argument's range to [0, pi/2],
!  and then calls the approximator. 
!
   real, intent(in) :: angle
   real, parameter  :: two_over_pi= 2.0/3.1415926535897932384626433    
   real             :: x
   integer          :: quad              ! what quadrant are we in?
   real, parameter :: c1= 0.99940307
   real, parameter :: c2=-0.49558072
   real, parameter :: c3= 0.03679168
   real            :: x2
!
   x=mod(angle, twopi)                   ! Get rid of values > 2* pi
   if(x<0)x=-x                           ! cos(-x) = cos(x)
   quad=int(x * two_over_pi)             ! Get quadrant # (0 to 3) we're in
   select case(quad)
      case (0)
   x2=x * x
   cos_32 = (c1 + x2*(c2 + c3 * x2))
         !cos_32 = cos_32s(x)
      case (1)
   x = pi - x
   x2=x * x
   cos_32 = -(c1 + x2*(c2 + c3 * x2))
         !cos_32 = -cos_32s(pi-x)
      case (2)
   x = x - pi
   x2=x * x
   cos_32 = -(c1 + x2*(c2 + c3 * x2))
         !cos_32 = -cos_32s(x-pi)
      case (3)
   x = twopi - x
   x2=x * x
   cos_32 = -(c1 + x2*(c2 + c3 * x2))
         !cos_32 =  cos_32s(twopi-x)
   end select 
   end function cos_32

!-----------------------------------------------------------------------

   subroutine fcalc_from_thermalf(ref,atx,pafc)
!
!  Calcola i fattori di struttura dopo aver ricalcolato i contibuti fcosat e fsinat solo per atx
!  Va usata se cambiano i fattori termici o le occupanze
!   
   USE Atomi, only : nzConst
   USE Symmetry, only : pts,icent
!
   type(reflection_type), dimension(:), intent(inout) :: ref    ! i riflessi
   type(atom_type), dimension(:), intent(in)              :: atx    ! modello strutturale
   integer, dimension(:), intent(in)                  :: pafc
   integer                                            :: i,jj
   real                                               :: rho
   integer                                            :: kz
   integer                                            :: natfc
   real                                               :: dot_pro
!
   natfc = size(atx)
   if (icent == 0) then
       do i=1,ninforef
          rho=ref(i)%slaq
          do jj=1,natfc
             kz=iabs(atx(jj)%nz/nzConst)
             sfactor(i)%tfocc(pafc(jj))=pts*sfactor(i)%ff(kz)*exp(-atx(jj)%biso*rho)*atx(jj)%och*atx(jj)%ocry
          enddo
          ref(i)%fc = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)**2 +     &
                         dot_product(sfactor(i)%fsinat,sfactor(i)%tfocc)**2
       enddo
   else
       do i=1,ninforef
          rho=ref(i)%slaq
          do jj=1,natfc
             kz=iabs(atx(jj)%nz/nzConst)
             sfactor(i)%tfocc(pafc(jj))=pts*sfactor(i)%ff(kz)*exp(-atx(jj)%biso*rho)*atx(jj)%och*atx(jj)%ocry
          enddo
          !ref(i)%fc = sqrt(4.0*dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)**2)
          !ref(i)%fc = 2.0*abs(dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc))
          dot_pro = dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)
          ref(i)%fc = 4.0*dot_pro*dot_pro
       enddo
   endif
!
   end subroutine fcalc_from_thermalf

!-----------------------------------------------------------------------

   subroutine fcalc_get_phases(ref)
   USE Symmetry, only : icent
!
!  Calcola le fasi. Utile se e' prevista chiamata alla examin o to_coord
!   
   USE General, only:rtod
   type(reflection_type), dimension(:), intent(inout) :: ref    
   real                                               :: a,b
   real                                               :: ph
   integer                                            :: i
!   
   b = 0
   do i=1,size(ref)
      a = 2.0*dot_product(sfactor(i)%fcosat,sfactor(i)%tfocc)
      if (icent == 0) then
          b = dot_product(sfactor(i)%fsinat,sfactor(i)%tfocc)
      endif
      ph = rtod*atan2(b,a)
      if (ph < 0) ph = ph + 360
!      ph=rtod*atan2(b,a)+360.0
!      ph = mod(int(ph+0.5),360)
      ref(i)%ph = nint(ph)
   enddo
!   
   end subroutine fcalc_get_phases
   
!-----------------------------------------------------------------------

   subroutine fcalc_save_atm(pafc)
!
!  Salva i contributi per gli atomi indicati nel vettore pafc
!   
   USE Symmetry, only : icent
!
   integer, dimension(:), intent(in) :: pafc
   integer                           :: jj
   integer                           :: natfc
   integer                           :: i
   integer                           :: pos
!
   natfc = size(pafc)
   if (icent == 0) then
       do i=1,ninforef
          do jj=1,natfc
             pos = pafc(jj)
             sfactorc(i)%fcosat(pos) = sfactor(i)%fcosat(pos)
             sfactorc(i)%fsinat(pos) = sfactor(i)%fsinat(pos)
          enddo 
       enddo
   else
       do i=1,ninforef
          do jj=1,natfc
             pos = pafc(jj)
             sfactorc(i)%fcosat(pos) = sfactor(i)%fcosat(pos)
          enddo 
       enddo   
   endif
!
   end subroutine fcalc_save_atm

!-----------------------------------------------------------------------

   subroutine fcalc_save_thermalf(pafc)
!
!  Salva i contributi per gli atomi indicati nel vettore pafc
!   
   integer, dimension(:), intent(in) :: pafc
   integer                           :: jj
   integer                           :: natfc
   integer                           :: i
   integer                           :: pos
!
   natfc = size(pafc)
   loop_riflessi: do i=1,ninforef
      loop_atomi: do jj=1,natfc
         pos = pafc(jj)
         sfactorc(i)%tfocc(pos)=sfactor(i)%tfocc(pos)
      enddo loop_atomi
   enddo loop_riflessi
!
   end subroutine fcalc_save_thermalf

!-----------------------------------------------------------------------

   subroutine fcalc_ripr_atm(pafc)
!
!  Ripristina i contributi per gli atomi indicati nel vettore pafc
!      
   USE Symmetry, only : icent
   integer, dimension(:), intent(in) :: pafc
   integer                           :: jj
   integer                           :: natfc
   integer                           :: i
   integer                           :: pos
!
!--------Warning: prova ad ottimizzare parametrizzando pafc(jj) e spostando la copia
! in una variabile di tipo derivato con solo cos e sen
!
   natfc = size(pafc)
   if (icent == 0) then
       do i=1,ninforef
          do jj=1,natfc
             pos = pafc(jj)
             sfactor(i)%fcosat(pos) = sfactorc(i)%fcosat(pos)
             sfactor(i)%fsinat(pos) = sfactorc(i)%fsinat(pos)
          enddo
       enddo
   else
       do i=1,ninforef
          do jj=1,natfc
             pos = pafc(jj)
             sfactor(i)%fcosat(pos) = sfactorc(i)%fcosat(pos)
          enddo
       enddo
   endif
!
   end subroutine fcalc_ripr_atm

!-----------------------------------------------------------------------

   subroutine fcalc_ripr_thermalf(pafc)
!
!  Ripristina i contributi per gli atomi indicati nel vettore pafc
!    
   integer, dimension(:), intent(in) :: pafc
   integer                           :: jj
   integer                           :: natfc
   integer                           :: i
   integer                           :: pos
!
   natfc = size(pafc)
   loop_riflessi: do i=1,ninforef
      loop_atomi: do jj=1,natfc
         pos = pafc(jj)
         sfactor(i)%tfocc(pos)=sfactorc(i)%tfocc(pos)
      enddo loop_atomi
   enddo loop_riflessi
!
   end subroutine fcalc_ripr_thermalf


END MODULE fastfcal
 
!!!!corrMODULE checkroutine

!!!!corrCONTAINS

!!!!corr   logical function check_container(vet1,vet2)  result(okcont)
!
!  okcont=.true. se tutto vet2 e' contenuto in vet1
!
!!!!corr   integer, dimension(:), intent(in) :: vet1,vet2
!!!!corr   integer                           :: i
!!!!corr   integer                           :: nv1,nv2
!
!!!!corr   nv1 = size(vet1)
!!!!corr   nv2 = size(vet2)
!!!!corr   if (nv2 > nv1) then
!!!!corr       okcont = .false.
!!!!corr   else
!!!!corr       okcont = .true.
!!!!corr       do i=1,size(vet2)
!!!!corr          if (.not.any(vet1 == vet2(i))) then
!!!!corr              okcont = .false.
!!!!corr              exit
!!!!corr          endif
!!!!corr       enddo
!!!!corr   endif
!
!!!!corr   end function check_container
   
!!!!corrEND MODULE checkroutine
