MODULE messagemod

implicit none

integer, parameter, private  :: NMAXMESS = 12
integer, private             :: prog_current_value, prog_final_value
integer, private             :: lastpos = 1

CONTAINS

    subroutine init_messages()
!
!   Inizializzi messaggi
!
    USE molcom
    integer :: i
    if (jscreen > 0) then
        call SetMesg(color_LIGHTRED,10,1,2)
        do i=2,NMAXMESS
           call SetMesg(color_LIGHTBLUE,10,i,0)
        enddo
        call ClearMesg()
        lastpos = 1
    endif
    end subroutine init_messages

!--------------------------------------------------------------

    subroutine clear_messages(pos,ipos,fpos)
    USE molcom, only:jscreen
    integer, intent(in), optional :: pos
    integer, intent(in), optional :: ipos,fpos
    character(len=1)              :: str = ' '
    integer                       :: i
!   
    if (jscreen > 0) then
        if (present(pos)) then
            call WriteMesg(str//char(0), pos)
        elseif (present(ipos)) then
            if (present(fpos)) then
                do i=ipos,fpos
                   call WriteMesg(str//char(0), i)
                enddo
                lastpos = ipos - 1
            else
                do i=ipos,NMAXMESS
                   call WriteMesg(str//char(0), i)
                enddo
                
            endif
        else
            call ClearMesg()  ! clear di tutti i messaggi
            lastpos = 0
        endif
    endif
!
    end subroutine clear_messages

!--------------------------------------------------------------

    subroutine write_message(string,inum,rnum,pos)
    USE strutil
    USE molcom, only:jscreen
    character(len=*), intent(in)  :: string
    integer, intent(in), optional :: inum
    real, intent(in), optional    :: rnum
    integer, intent(in)           :: pos
    logical                       :: is_num,is_int,is_real
    integer                       :: posm
!
    if (jscreen > 0 .and. pos >= 0) then
!
!       Se pos = 0 scrivi nella riga successiva all'ultima
        if (pos  == 0) then
            posm = lastpos + 1
        else
            posm = pos
        endif
!
        is_int = present(inum)
        is_real = present(rnum)
        is_num = is_int .or. is_real
        if (is_num) then
            if (is_int) then
                call WriteMesg(string//trim(i_to_s(inum))//char(0), posm)
            else
                call WriteMesg(string//trim(r_to_s(rnum,3))//char(0), posm)
            endif
        else
            call WriteMesg(trim(string)//char(0), posm)
        endif
        lastpos = posm
    endif
!
    end subroutine write_message

!--------------------------------------------------------------

    subroutine progress_bar_init(string,fval)
    character(len=*), intent(in), optional :: string  ! stringa da scrivere
    integer, intent(in), optional          :: fval    ! valore finale della progress bar
!
    if (present(string)) then
        call scrper(string,0,0)
    else
        call scrper(' ',0,0)
    endif
!
!   inizializza il valore corrente
    prog_current_value = 0       
!
!   inizializza il valore finale
    if (present(fval)) then
        prog_final_value = fval
    else
        prog_final_value = 100
    endif
!
    end subroutine progress_bar_init

!--------------------------------------------------------------

    subroutine progress_bar_update(string,cval,fval)
    character(len=*), intent(in), optional :: string  ! stringa da scrivere
    integer, intent(in), optional          :: cval    ! valore corrente
    integer, intent(in), optional          :: fval    ! valore finale
!
    if (present(cval)) then
        prog_current_value = cval
    else
        prog_current_value = prog_current_value + 1
    endif
!
    if (present(fval)) then
        prog_final_value = fval
    endif
!
    if (present(string)) then
        call scrper(string,prog_current_value,prog_final_value)
    else
        call BarraDiStato(0,4,0,0,' ')   ! per rimuovare stringa
        call scrper(' ',prog_current_value,prog_final_value)
    endif
!
    end subroutine progress_bar_update

!--------------------------------------------------------------

    subroutine progress_bar_close()
!
    call scrper(' ',0,-1)
    prog_final_value = 0
    prog_current_value = 0
!
    end subroutine progress_bar_close 

END MODULE messagemod
