MODULE Conteggi
   integer, parameter :: npunti=0
END MODULE Conteggi
MODULE sannel
USE simulated_anneal, only:SimAnn_Conditions_type
USE fragmentmod
USE rotationmod
USE grexpomod
USE connect_mod, only: connection_type, angle_type
USE rrestr, only: restraint_type, constraint_type
USE anti_bump, only: bump_settings
USE model_util, only: model_type
implicit none

public :: sa_prelim,tfunk,xpar_to_atm,chisquared_bond,chisquared_angle

type shift_atom_type
  integer, dimension(3) :: rcod ! codice di affinamento  
end type

integer, parameter :: THERMALF_TYPE = 1
integer, parameter :: PO_TYPE = 2
integer, parameter :: OTHERS_TYPE   = 3
integer, parameter :: EXT_TYPE = 4
integer, parameter :: INT_TYPE = 5

enum, bind(c) 
  enumerator :: SA_LOCAL=1,BBBC,SA_GLOBAL,HARMONYS
endenum

integer, parameter                               :: CHI2PROF_NEW=3,&   ! CF profilo new
                                                    CHI2I=2,       &   ! CF intensita'
                                                    CHI2FACT=1,    &   ! CF fattori di struttura (tutti i riflessi)
                                                    CHI2I_SD=4,    &   ! CF fattori di struttura (sing e doppietti)
                                                    CHI2I_NEW=5,   &   ! CF intensita' integrate
                                                    CHI2PROF=6,    &   ! CF profilo 
                                                    CHI2GRU=7          ! CF intensita' per gruppi

type(fragment_type), dimension(:), allocatable   :: fragment           ! i frammenti
type(rotation_type), dimension(:), allocatable   :: rotation           ! le rotazioni
type(shift_atom_type), dimension(:), allocatable :: shift_atom         ! gli shift sugli atomi
type(shift_atom_type), dimension(:), allocatable :: shift_atom_copy    ! gli shift sugli atomi (copia)
type(opti_info_type)                             :: ginfo              ! informazioni generali sulla procedura
type(opti_info_type)                             :: goptim             ! informazioni generali sulla procedura
type(param_type), dimension(:), allocatable      :: gpar               ! i parametri dell'algoritmo di SA
integer                                          :: numfrag            ! il numero di frammenti
integer                                          :: numrot             ! il numero di rotazioni
real                                             :: shiftatomval       ! shift applicato agli atomi
integer                                          :: numcyc_calc        ! calculated number of CF evaluation for run
real                                             :: rwden              ! denominatore funzione costo
real, dimension(:), allocatable, private         :: intoss,intcal      ! intensita' per funzione costo CHI2I
!corrtype(error_type)                                 :: saerr              ! errore in SA
type(reflection_type), dimension(:), allocatable :: refi               ! copia di tutti i riflessi
#if defined(DISTRIB)
logical, private                                 :: print_tb = .false. ! stampa info per tabella riepilogativa
#else
logical, private                                 :: print_tb = .true.  ! stampa info per tabella riepilogativa
#endif
integer                                          :: ncuold 
integer                                          :: ifLabelsSav        
real, dimension(:), allocatable :: ycossb
real :: sumycossb,sumintoss
integer :: mcountlim    !!! variabile da sistemare
real, dimension(:), allocatable :: ycountonew,weichi
integer                         :: nco_new
type(connection_type), dimension(:), allocatable   :: connect   ! connettivita'    !!!!FIXME
type(angle_type), dimension(:), allocatable        :: ang       ! angoli
type(rparam_type)                                  :: gpo_sav
type(model_type), dimension(:), allocatable :: samodel
integer :: nsamodel
real, parameter, private :: DISTDOC = 1.0   ! distanza limite per la d.o.c.
real, parameter, private :: DISTOCC = 0.9   ! distanza limite per atomi in posizione speciale
character(len=*), dimension(4), parameter :: algoname_long = (/'Simulated Annealing       ',  &
                                                               'Hybrid Big Bang Big Crunch',  &
                                                               'Simulated Annealing       ',  &
                                                               'Harmony Search            '/)  
character(len=*), dimension(4), parameter :: algoname_short= (/'SA     ','HBB-BC ','SA     ','HS     '/)
logical :: map_enabled = .true.
real :: sasogmap
real :: radmap
!
! restraints variables
type(restraint_type), dimension(:), allocatable  :: res_sa, bump_sa
type(restraint_type), dimension(:), allocatable  :: bumptab
type(bump_settings)                              :: bumpset
type(constraint_type), dimension(:), allocatable :: sconstr

real, parameter :: SASIGD = 0.2
real, parameter :: SAWEID = 100
real, parameter :: SHIFT_DEF = 0.5

type(atom_type), dimension(:), allocatable :: atom0

type doc_type
   integer              :: dtype = 0  ! 0 = not active, 1 = active on all atoms, 2 = active on selected atoms
   integer, dimension(:), allocatable :: vet   ! atoms involved
contains
   procedure :: active => doc_active
   procedure :: all_atoms => doc_all_atoms
   procedure :: prn => doc_print
end type
type (doc_type) :: sadoc

CONTAINS

   subroutine sa_prelim(gopti,sc,bcond,read_dir,irefb,saerr)
   USE Riflessi
   USE Atomi
   USE connect_mod
   USE variables, only:atm,ref,lconn
   USE realloc
   USE General, only:knw,bt,no,nk,nrefl,resmin,neutro,spacegr,cell
   USE simulated_anneal
   USE atom_type_util
   USE nr
   USE reflection_type_util
   USE Conteggi, only:npunti
   USE Molcom, only:ifLabels, jscreen
   USE bb_bc
   USE messagemod
   USE compare_struct
   USE factor
   USE anti_bump
   USE errormod
   USE spginfom
   USE unit_cell
   type(opti_info_type), intent(inout)       :: gopti    
   type(SimAnn_Conditions_type), intent(out) :: sc                     
   type(bb_bc_condition), intent(out)        :: bcond                     
   logical, intent(in)                       :: read_dir
   integer, intent(out)                      :: irefb
   type(error_type), intent(out)             :: saerr
   integer                                   :: i
   real                                      :: resmaxg
   integer                                   :: ishifta
   integer                                   :: nang
   real, parameter                           :: reslimit = 2.0
   integer                                   :: code
   integer                                   :: nrefmin
   type(error_type)                          :: errdir
   integer, dimension(1) :: pfrag
!
   write(6,'(1x,79("+"),//,30x,"Global Optimization Procedure",//,1x,79("+"))')
!
   call saerr%reset()
   call InitSA_sir()
!   
   if (jscreen > 0) then
       call StartStopMsg(1)
       call init_messages()
       call write_message('Simulated Annealing',pos=1)
   endif      
!
   call xyz_to_atom(atm,xyzs(:3,:nats),bfac,bij,xyzs(5,:),(/(1.0, i=1, nats)/),nz,aLab,xyzs(4,:))
   call fillref(ref)
   call indexx(ref%slaq, iwork1)
   ref(:) = ref(iwork1)
!
   call fillatom(atm)
   !call to_coord(1)
   !call to_coord_ref(1)
!
   call iconn_to_legm(iconn,lconn,atm)
!
!  Converti iconn in connect
   call legm_to_connect(nats,lconn,connect)
!
!  Genera leg e ang da connect
   call connect_to_ang(connect,atm,ang,nang)
!
!  Stampa la connettivita nelle sue differenti forme   
   call print_connect(atm%lab,connt=connect)
!
!  Inizializza
       call sa_set_default(sc)
       call bb_bc_initialize(bcond)
       gopti%cftype = set_function_type(npunti)  ! set only cost function
   call reset_restraints(res_sa)
   call reset_restraints(bump_sa)
   call reset_constraints(sconstr)
!!!!!!!!!!!!!!test
      !call add_constraint_on_biso(atm,lconn,celem,sconstr)
      !call stampa_constraints(sconstr,atm,0,.true.)
      !sconstr%ref  = 1
      !write(0,*)'REF=',sconstr%ref
!!!!!!!!!!!!!!test
   call create_bump_table(atm,bumptab)
   numfrag = 0
   numrot = 0
   call reallocate_shift(shift_atom,nats,.false.)
   do i=1,nats
      shift_atom(i)%rcod = 0
   enddo
   shiftatomval = SHIFT_DEF
   call get_fragments_from_leg(atm,lconn,connect,numfrag,fragment)
!
!  Estrai tutte le rotazioni dal modello strutturale
   call get_rotation_from_leg(atm,connect,lconn,set_cell_type(cell),numrot,rotation)
!
!  reset refinement codes
   forall (i=1:nats) atm(i)%rcod = 0
!
!  Leggi le direttive se e' attivo il comando %sannel
   if (read_dir) then
       call sa_directives(gopti,sc,bcond,resmaxg,ishifta,errdir)  
       if (errdir%signal) call errdir%print()
   else
       resmaxg = -1      ! 
       irefb = 0         ! non affinare i fattori termici
   endif
           !call set_antibump_auto(bump_sa,bumptab,atm,fragment)  !!!Force anti-bump for test
           !if (nrestraints(bump_sa) == 0) stop
!
!  Porta i frammenti in cella
   call fragment_in_cell(fragment,atm)
!
!  Fix traslation in case of polar axis only for fragment with more atoms
!  uncomment only for test if (.not.spacegr%polar()) stop
   if (spacegr%polar()) then
       pfrag = maxloc(fragment(:)%nat)
       call spacegr%code_for_polar(fragment(pfrag(1))%rcod)
   endif
!!
!!  Azzera codici di affinamento per parametri atomici 
!!   do i=1,nats
!!      atm(i)%rcod(1:3) = 0
!!      atm(i)%rcod(4) = irefb
!!      atm(i)%rcod(5) = 0
!!   enddo
!
!  Calcola il numero iniziale di paramatri, serve per il calcolo del num. movimenti e per la risoluzione
   call sa_conta_parametri(gopti,atm,fragment,rotation)
!
!  Calcola valore iniziali per algoritmi di ottimizzazione
   if (gopti%autoalgo) call gopt_set_algo(gopti,numrot,numfrag)
   if (sc%autont) call gopt_set_ncycle(gopti,sc%nt)  
   if (gopti%algo == BBBC) then
       call set_bb_bc_condition(bcond)
   endif
                !deallocate(res_sa)
!   
   call print_fragment(fragment,atm)
   if (numrot > 0) then
      call print_rotation(atm%lab,rotation,1)
      !!do i=1,numrot   ! now in add_rotation!
      !!   call set_torsion_for_rotat(connect,rotation(i))
      !!   rotation(i)%torsval = torsion_value_for_rotat(rotation(i),atm)
      !!enddo
   endif
!
!  Memorizza tutti i riflessi
   gopti%nreftot = nrefl
   call reallocate(refi,gopti%nreftot)
   refi = ref
   ncuold = 0
!
!  Definisci i limiti risoluzione dello spettro 
   if (resmaxg < 0) then
!gl!       resmin = resvalue(thmax)
!gl!   if (resmin > reslimit) then
       if (resmin < reslimit) then
           gopti%res = resmin
       else
           gopti%res = RESLIMIT
           call sa_conta_parametri(gopti,atm,fragment,rotation)
           call check_resp(gopti%res,gopti%npar)
       endif
   else    
       gopti%res = resmaxg
   endif
      !deallocate(bump_sa)
!
   ginfo = gopti  ! save in ginfo for graphic and external function
!   
   ifLabelsSav = ifLabels      
!corr   if (nrestraints(res_sa)>0) write(0,*)'chi2res=',chisquared_res(res_sa, atm,kpr=1)
!
   if (jscreen > 0) then
       call clear_messages()
       call write_message(trim(algoname_long(gopti%algo)), pos=1)
       call write_message('Number of refined parameters: ',inum=gopti%npar,pos=2)
       call write_message('DOF(ext): ',inum=gopti%dofe,pos=3)
       call write_message('DOF(int): ',inum=gopti%dofi,pos=4)
       if (knw > 0) then
           call fcalcan(ref,atm)
           call to_coord_ref(-1)
           call write_message_found(5)
       endif
   endif
!   
   end subroutine sa_prelim     

!----------------------------------------------------------------------------------------------------      

   subroutine global_init(gopti)
   USE conteggi, only:npunti
   type(opti_info_type), intent(out)  :: gopti     
!
   gopti%cftype = set_function_type(npunti)
   !!gopti%cftype = CHI2FACT
   !!gopti%cftype = CHI2PROF
   !!gopti%cftype = CHI2I_NEW
!corr   gopti%doc = .false.
   gopti%seed = 1      ! If different from zero, holds the seed for random number generator
   gopti%nrun = 10     ! Number of runs
   gopti%nrunc = 1     ! Contatore dei run
   !gopti%algo = SA_LOCAL  ! Tipo di algoritmo usato
   gopti%algo = BBBC   ! Tipo di algoritmo usato
   gopti%autoalgo = .true.
   gopti%popt = 'PMVESB'   ! Opzioni di output
   gopti%sogCF = -1    ! Soglia sul valore della Cost Function
!
   end subroutine global_init

!----------------------------------------------------------------------------------------------------      

   integer function set_function_type(np)  result(cftype)
   integer, intent(in)                  :: np
!
   if (np > 0) then
       cftype = CHI2PROF_NEW
   else
       cftype = CHI2FACT
   endif
!
   end function set_function_type

!----------------------------------------------------------------------------------------------------      
 
   subroutine sa_run(gopti,sc,bcond)
   USE lsqalgo
   USE variables, only:atm,ref,ortom,ortoi,lconn
   USE calctime
   USE Molcom, only:ifBrekke,jscreen
   USE General, only:xgg,cell,nrefl,StructureName,lo,resm
   USE simulated_anneal
   USE unit_cell
   USE compare_struct
   USE atom_type_util
   USE Conteggi, only:npunti
   USE messagemod
   USE bb_bc
   USE realloc
   USE Symmetry, only: nsymtot 
   USE ilmviewmod
   USE anti_bump
   USE connect_mod
   type(opti_info_type), intent(inout)         :: gopti    
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(in)           :: bcond                     
   real                                        :: resmaxg
   integer                                     :: i
   real                                        :: res,resi,minres
   integer                                     :: ncc
   integer, dimension(size(atm))               :: rcodbsav
   integer                                     :: posfin,posini
   real, dimension(6)                          :: cellold
   real                                        :: vol_per_atom
   type(bond_type), dimension(:), allocatable  :: legm
   type(atom_type), dimension(:), allocatable  :: atomcart
!
   fortran_work = .true.
!
!  Conta gradi di liberta
   call sa_conta_parametri(gopti,atm,fragment,rotation)
!   
!  Assegna valore per la risoluzione
   resmaxg = gopti%res
   resi = resmaxg
!   
!  Controlla che la risoluzione assegnata non sia inferiore al valore minimo gestibile
!gl!   ncc = rfcond%lim%nctot   
!gl!   minres = resvalue(refi(nrefl)%tthd)
   minres = resm
   res = max(resi,minres)
!   
   gopti%res = res
!gl!   gopti%ttmax = tthvalue(res)
!gl!   gopti%ttmin = refi(1)%tthd
!gl!   call set_limit_ref(rfcond%lim,refi,gopti%ttmax)
!      
!!!!prova
!      write(0,*)'nc prima=',rfcond%lim%ncu
!      ncusav = rfcond%lim%ncu
!      rfcond%lim%ncu = min(rfcond%lim%ncu*1.1,1.0*rfcond%lim%nctot)
!      write(0,*)'nc dopo =',rfcond%lim%ncu
!!!!prova
   gopti%nref = numref_from_res(res, refi)
   call reallocate(ref, gopti%nref)
!gl!   gopti%nco = rfcond%lim%ncu
!
   call abilita_runtb((/0,0,0,-1/))   ! disabilita runtb
!
        !write(0,*)'ncuold=',ncuold,rfcond%lim%ncu
       call stampa_struttura(atm,title='Starting structure')
       call to_coord_ref(1)
!
!  Tutti gli atomi affinati vanno in posizione generale
   do i=1,numfrag
      if (any(fragment(i)%rcod(:) > 0)) then
          atm(fragment(i)%pos)%ocry = 1.0
      endif
   enddo
!
!  Checking sul volume per atomo
   vol_per_atom = volume_per_atom(atm, nsymtot, cell_volume(cell(1:6)))
   write(6,'("   Volume per atom calculated by starting model = ",f0.3)') vol_per_atom
   if (vol_per_atom < 5 .or. vol_per_atom > 30)    &
   write(6,'("   WARNING: volume per atom seems unresonable. Check the starting fragment")')
!
!  Stampa restraints
   call stampa_restraints(lo,res_sa,atm)
   call set_equivalent_for_angle(res_sa,lconn)
!corr   if (nrestraints(bump_sa) > 0) then
!corr       fbump = antibump_function(bump_sa, atm, .true.)  ! only for test
!corr   endif
   call print_antibump(lo,bump_sa,atm)

   call RunToolbar(1,'Starting simulated annealing')
!
!  Attiva la gestione dei tempi   
   timeactive = .true.
   call init_timestep(6,(/'SA step di temperatura     ', &
                          'Cost function calculation  ', &
                          'Moves                      ', &
                          'Graphic                    ', &
                          'Tempo/run                  ', &
                          'Total time                 '/))          
!
   call gopt_init_costf(gopti)   ! inizializzazione per funzioni costo
!
!  Inizializza gpar, randomizza se richiesto
   call sa_prerefine(gopti,sc%rand)
!
!  Stampa informazioni generali:range,num. di riflessi,dof,etc.   
   call gopt_print_info(gopti,6,sadoc,atm)
!
!  Operazioni per la grafica, imponi visualizzazione cella   
   if (jscreen > 0) then
       call ilmview_set_cell(1)
       call jav_set_style(5,1)
       call ilmview_open(StructureName)
       call ilmview_draw(atm,lconn)
       call jav_set_sym(0,0,1)
       call set_polyhedra_button(0)
       call set_contacts_button(0)
       call set_grow_fragment_button(0)
       call jav_set_modify(3)
   endif
!   
   call timestep(6)        ! inizio misura tempo totale
!
!  Calcolo automatico del numero di cicli per temperatura
   if (sc%autont) call gopt_set_ncycle(gopti,sc%nt)
!
!  Calcolo automatico della temperatura iniziale
   if (sc%autoti) call sa_set_temperature_new(sc,gopti,gpar,safunc_temp)
   !ginfo = gopti; if (sc%autoti) call sa_set_temperature(sc,gopti,gpar,tfunk,safunc1)
                     !sc%t_ini = 50
!
!  Setting della convergenza
   call sa_set_convergence(sc,gopti%npar)
!
   call RunToolbar(1,'Running Simulated Annealing')
   call progress_bar_init('Starting simulated annealing')   !  Inizializza progress bar      
   call abilita_runtb((/1,1,1,1/))   ! riabilita runtb
!
!  Definisce su che righe scrivere i vari messaggi
       call sa_set_messages(gopti%algo, 0, nrestraints(bump_sa),nrestraints(res_sa))
!
!  Initialize map visualization: compute radius and define sogmap
   call copia_atoms(atomcart,atm)
   if (numfrag > 1) then
       call regroup_fragment(fragment,atomcart)
   endif
   call frac_to_cart(atomcart,ortom)
   call get_radius_molecule(atomcart,radmap)
   call map_set_rad(radmap)
   sasogmap = 120
!!!   call sogmap_init(FOUR_FO_FC,ref,atm,sasogmap)   ! non funziona ad es. con clomipra
!!!   sasogmap = sasogmap * 0.003
!!!      write(0,*)'sogmap=',sasogmap
!
   ifBrekke = 0      !azzera codice di interazione con la grafica
   call simann(gopti,sc,bcond) 
   !call ptempering(sc) 
!
   call gopt_close_costf()            ! dealloca info
   call set_atm_in_frag(fragment,atm)
!
!  Chiudi gestione tempi   
   call timestep(6)        ! fine misura tempi totali   
   call set_status_print_time(1,.false.)
   call set_status_print_time(5,.false.)
   call print_timestep()
   call end_timestep()
!
   call progress_bar_close()                             ! chiudi progress bar
   if (jscreen > 0) then
!
!      disegna modello
       call copia_legm(legm,lconn)
       if (numfrag > 1) then 
           call connect_fragment(fragment,atm,legm)
       endif
       call ilmview_draw(atm,legm)
!
   endif
!   
   fortran_work = .false.
!
   end subroutine sa_run

!----------------------------------------------------------------------------------------------------      

   subroutine sa_set_messages(algo, data_type, nbump, nres)
!
!  Set order of messages for SA in the split window
!
   USE simulated_anneal
   integer, intent(in) :: algo
   integer, intent(in) :: data_type
   integer, intent(in) :: nbump, nres
   integer             :: nline
!
   nline = 1
   if (algo == BBBC) then
       nline = nline + 1; vmess(LINE_RUN) = nline
                          vmess(LINE_TEMP) = -100   ! non mostrare la temperatura
       nline = nline + 1; vmess(LINE_MOVES) = nline
       nline = nline + 1; vmess(LINE_CF) = nline
       nline = nline + 1; vmess(LINE_CFMIN) = 5
   else
       nline = nline + 1; vmess(LINE_RUN) = nline
       nline = nline + 1; vmess(LINE_TEMP) = nline
       nline = nline + 1; vmess(LINE_MOVES) = nline
       nline = nline + 1; vmess(LINE_CF) = nline
       nline = nline + 1; vmess(LINE_CFMIN) = nline
   endif
   if (nres > 0) then
       nline = nline + 1; vmess(LINE_CFRES) = nline
   endif
   if (nbump > 0) then
       nline = nline + 1; vmess(LINE_CFBUMP) = nline
   endif
   if (data_type > 0) then
       nline = nline + 1; vmess(LINE_RATIO) = nline
       nline = nline + 1; vmess(LINE_TIME) = nline
   else
       vmess(LINE_RATIO) = -100
       nline = nline + 1; vmess(LINE_TIME) = nline + 1
   endif
!
   end subroutine sa_set_messages

!----------------------------------------------------------------------------------------------------      

   subroutine gopt_init_costf(gopti)
!
!  Inizializzazione della funzione costo
!   
   USE fastfcal
   USE lsqalgo
   USE variables, only:ref,atm          
   USE realloc
   type(opti_info_type), intent(inout) :: gopti    
   integer                             :: i
   integer                             :: nnn
!!!!!!!!!!new
   integer                             :: maxlim
   integer, dimension(:), allocatable  :: ivetc,puntc
   integer                             :: ncc
!!!!!!!!!!new
!   
   select case (gopti%cftype)
      case (CHI2PROF)     
               
      case (CHI2FACT)
        rwden = sqrt(sum(ref%fo**2))/100.0
        gopti%Cost_function_name = 'R structure factor'       
        gopti%Cost_function_name_short = 'R-factor'
       !write(6,*)'chi2init=',sqrt(chisquared_factor(0))/rwden
      
      case (CHI2I)
        call reallocate(intoss,intcal,gopti%nref,.false.)
        call reallocate(weichi,gopti%nref,.false.)
!        allocate(intoss(gopti%nref),intcal(gopti%nref))
        !intoss = ref%m*ref%lp*ref%fo**2      
        !intoss = ref%m*ref%fo**2      
        intoss = ref%fo**2      
        !intoss = ref%lp*ref%fo**2      
        sumintoss = sum(intoss)
        !rwden = sqrt(sum(intoss**2))/100.0
        weichi(:) = 1.0/(1.0+intoss(:))
        rwden = sumintoss/100.0
        gopti%Cost_function_name = 'R intensities'
        gopti%Cost_function_name_short = 'R-int'
        
      case (CHI2I_SD)
        
      case (CHI2I_NEW)

      case (CHI2PROF_NEW)     

   end select
   call fcalc_init(ref,atm)
!  
   end subroutine gopt_init_costf
        
!----------------------------------------------------------------------------------------------------         

   subroutine gopt_close_costf()
   USE fastfcal
!
   call fcalc_close()     ! dealloca info sui riflessi
!
   end subroutine gopt_close_costf
   
!----------------------------------------------------------------------------------------------------         

   subroutine sa_set_convergence(sc,npar)
!
!  Setting della convergenza in base al numero di parametri
!
   type(SimAnn_Conditions_type), intent(inout) :: sc
   integer, intent(in)                         :: npar
!
   select case (npar)
      case (1:3)
        sc%eps = 0.01

      case (4:5)
        sc%eps = 0.1

      case (6:)
        sc%eps = 2.0

   end select
!
   end subroutine sa_set_convergence

!----------------------------------------------------------------------------------------------------         

   subroutine sa_directives(gopti,sc,bcond,resmaxg,ishifta,errdir) 
!
!  Inizializza parametri e leggi direttive
!
   !USE General, only:card
!gl!   USE CommonExpo, only:ipdir,icomat
   USE Direttive
   USE simulated_anneal                  
   USE variables, only:lconn
   USE Atomi, only:nats
   USE commandsmod
   USE bb_bc
   USE errormod
   USE connect_mod
                  USE variables, only: atm, lconn
   USE rrestr
   type(opti_info_type), intent(inout)         :: gopti     
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(inout)        :: bcond                     
   real, intent(out)                           :: resmaxg
!corr   integer, intent(out)                        :: irefb
   integer, intent(out)                        :: ishifta
   type(error_type), intent(out)               :: errdir
   real                                        :: shiftaval
   integer                                     :: i
   integer, dimension(2)                       :: vfrot
   integer                                     :: kposf
   logical, dimension(nats)                    :: vetshift, vfragt, vfragr, vetocc
   integer                                     :: icpos
   type(restraint_type), dimension(:), allocatable :: resb
   integer :: posb, irefb
   logical :: build_frag 
!
!  Inizializza parametri dell'algoritmo
   resmaxg = -1
   irefb = 0
   ishifta = 0
   shiftaval = 0.0
   vetshift(:) = .false.
   vfragt(:) = .false.
   vfragr(:) = .false.
   vetocc(:) = .false.
!
!  Leggi direttive
   if (icomat /= 0) then
!
!      cerca il comando %sannel che non è necessariamente l'ultimo
       icpos = command_position(commands,'sannel')
       do i=1,commands(icpos)%ndir
          ipdir = ipdir + 1
          call sa_read_line_dir(commands(icpos)%strdir(i),gopti,sc,bcond,vfrot,vfragt,vfragr,            &
                                vetocc,resmaxg,irefb,ishifta,shiftaval,vetshift,resb,errdir)
          if (errdir%signal) exit
          if (numrot > 0) then
              kposf = rotation_position(rotation,vfrot)
              if (kposf > 0) rotation(kposf)%rcod = 0
          endif
       enddo
   endif
!
   if (errdir%signal) return
!
   if (nrestraints(resb) > 0) then
!corr       call stampa_restraints(0,resb,atm)
       build_frag = .false.
       do i=1,nrestraints(resb)
          posb = bond_position(lconn,resb(i)%na(1),resb(i)%na(2))
          if (posb > 0) then
              call remove_bond(lconn, posb)
              call add_restraint_to_list(res_sa,resb(i))
              build_frag = .true.
               !write(0,*)'rimuovo legame posb'
          endif
       enddo
!!!!!!!!!!!!!!test per convertire tutti i legami in restraints
       ! do i=1,numbonds(lconn)
       !    call set_restr(res_sa,1,(/lconn(i)%n1,lconn(i)%n2/),lconn(i)%dist,lconn(i)%sigma,lconn(i)%dist)
       ! enddo
       !    call reallocate_legm(lconn,0)
       !    build_frag = .true.
!!!!!!!!!!!!!!test per convertire tutti i legami in restraints
             !write(0,*)'NUMFRAG=',numfrag,fragment(1)%rcod,fragment(2)%rcod
       if (build_frag) then
          call legm_to_connect(nats,lconn,connect)
          call reallocate_fragment(fragment,0)   
          call get_fragments_from_leg(atm(:nats),lconn,connect,numfrag,fragment)
             !write(0,*)'NUMFRAG=',numfrag
       endif
   endif
!
!  Fix translation and rotation of fragment if required
   do i=1, numfrag
      if (vfragt(i)) fragment(i)%rcod(1:3) = 0
      if (vfragr(i)) fragment(i)%rcod(4:6) = 0
   enddo
!
!  Shift sugli atomi
   if (ishifta > 0) then
       shiftatomval = shiftaval
!
!      Azzera vetshift se l'atomo e' isolato
       do i=1,nats
          if (vetshift(i) .and. fragment_natom(fragment,i) > 1) then
              shift_atom(i)%rcod(1:3) = 1
          else
              shift_atom(i)%rcod(1:3) = 0
          endif
       enddo
   endif 
!
!  Set refcode for DOC on atoms
   where (vetocc) atm%rcod(5) = 1
!
!  Set code for thermal factor
   if (irefb == 1) then
       atm%rcod(4) = 1
   endif 
!
   end subroutine sa_directives

!----------------------------------------------------------------------------------------------------

   subroutine sa_read_line_dir(strdir0,gopti,sc,bcond,vfrot,vfragt,vfragr,            &
                               vetocc,resmaxg,irefb,ishifta,shiftaval,vetshift,resb,errdir)
   USE strutil
   USE simulated_anneal                 
   USE variables, only:atm
   USE bb_bc
!corr   USE RefinecomRef, only:hklpo,gpo,kopcorr
   USE anti_bump
   USE errormod
   USE atom_type_util
   USE realloc
   character(len=*), intent(in)                :: strdir0
   type(opti_info_type), intent(inout)         :: gopti     
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(inout)        :: bcond                     
   integer, dimension(2), intent(out)          :: vfrot
   logical, intent(inout), dimension(:)        :: vfragt, vfragr, vetocc
   logical, dimension(size(vfragt))            :: vrot
   real, intent(inout)                         :: resmaxg
   integer, intent(inout)                      :: irefb 
   integer, intent(inout)                      :: ishifta
   real, intent(out)                           :: shiftaval
   logical, dimension(:), intent(inout)        :: vetshift
   type(restraint_type), dimension(:), allocatable, intent(inout) :: resb
   type(error_type), intent(out)               :: errdir
   character(len=len(strdir0))                 :: strdir,word
   real, dimension(100)                        :: vet
   integer, dimension(100)                     :: ivet
   integer                                     :: iv
   integer                                     :: nlongs
   logical                                     :: errpar
   type(restraint_type)                        :: restemp
   integer                                     :: ier
   integer                                     :: n1
   integer                                     :: i
   integer, dimension(size(atm))               :: ratom
   integer                                     :: natr
   real                                        :: rshift
!
   vfrot(:) = 0
   strdir = strdir0
   call Cutst(strdir,nlongs,word) 
   call Getnum(strdir(:nlongs),vet,ivet,iv)
!
   errpar = .false.
   if (match_word(word,'TEMP')) then ! temperature        
           if (iv >= 1) then
               sc%t_ini = vet(1)
               sc%autoti = .false.
           else
               errpar = .true.
           endif
               
   elseif (match_word(word,'TFAC')) then ! temperature factor
           if (iv >= 1) then
               sc%temp_fact = vet(1)
           else
               errpar = .true.
           endif
               
   elseif (match_word(word,'NITE')) then ! number of iterations
           if (iv >= 1) then
               sc%nt = ivet(1)
               sc%autont = .false.
           else
               errpar = .true.
           endif

   elseif (match_word(word,'NRUN')) then ! number of runs
           if (iv >= 1) then
               gopti%nrun = ivet(1)
           else
               errpar = .true.
           endif

   elseif (match_word(word,'NCYC')) then ! numero di cicli per bb-bc
           if (iv >= 1) then
               bcond%niter = ivet(1)
               bcond%auto = .false.
           else
               errpar = .true.
           endif

   elseif (match_word(word,'RESM')) then ! max resolution
           if (iv > 0) then
               resmaxg = vet(1)
           else
               errpar = .true.
           endif

   elseif (match_word(word,'FIXR')) then ! fix rotation
           vrot(:) = .false.
           call get_atoms_from_string(strdir,atm,vrot,err=errdir)
           if (count(vrot) /= 2) call errdir%set('Error')
           if (errdir%signal) then
               call errdir%add(' in directive fixrotation')
           else
               n1 = 0
               do i=1,size(vrot)
                  if(vrot(i)) then
                     n1 = n1 + 1
                     vfrot(n1) = i
                  endif
               enddo
           endif

   elseif (match_word(word,'FFTR')) then ! fix translation in fragment
           call get_atoms_from_string(strdir,atm,vfragt,err=errdir)
           if (errdir%signal) then
               call errdir%add(' in directive fftranslation')
           endif

   elseif (match_word(word,'FFRO')) then ! fix rotation in fragment
           call get_atoms_from_string(strdir,atm,vfragr,err=errdir)
           if (errdir%signal) then
               call errdir%add(' in directive ffrotation')
           endif

   elseif (match_word(word,'RAND')) then ! randomize fragment
           if (iv == 0) then
               sc%rand = 0
           else 
               sc%rand = ivet(1)
           endif
           
   elseif (match_word(word,'REFI')) then ! refine thermal factor         
           irefb = 1

   elseif (match_word(word,'SHIF')) then ! shift to atoms: At1 At2 ... shift
           call get_atoms_from_string(strdir,atm,vetshift,shiftaval,SHIFT_DEF,errdir)
           if (errdir%signal) then
               call errdir%add(' in directive shift_atom')
           else
               if (all(vetshift .eqv. .false.)) then           
!
!                  Only shiftval was assigned
                   ishifta = size(vetshift)
                   vetshift(:) = .true.
               else
                   ishifta = count(vetshift)
               endif
           endif

   elseif (match_word(word,'COST')) then ! cost function
           if (iv > 0) then
               gopti%cftype = ivet(1)
           else
               errpar = .true.
           endif
         
   elseif (match_word(word,'ROTATE_AROUND_A')) then ! rotate_around_axis
           call get_atoms_from_string(strdir,atm,ratom,natr,rshift,-1.0,err=errdir)
           if (errdir%signal) then
               call errdir%add(' in directive rotate_around_axis')
           else
               call add_rotation(connect,numrot,rotation,ratom(1:2),ratom(3:natr),atm,rshift)
           endif
         
   elseif (match_word(word,'DOC')) then ! dynamical occupancy correction
           if (len_trim(strdir) > 0) then
               call get_atoms_from_string(strdir,atm,vetocc,err=errdir)
               if (errdir%signal) then
                   call errdir%add(' in directive DOC')
               endif
           else
               vetocc(:) = .true.
           endif

   elseif (match_word(word,'ALGO')) then ! tipo di algoritmo
           if (iv > 0) then
               if (ivet(1) > 0 .and. ivet(1) < 5) then
                   gopti%algo = ivet(1)
                   gopti%autoalgo = .false.
               endif
           endif

   elseif (match_word(word,'BUMP')) then ! set bump as: bump specie1 specie2 target_distance
           call bump_from_string(strdir, atm, bump_sa, bumptab, fragment, bumpset%scaledist, ier)
           if (ier == 0) then
               if (len_trim(strdir) == 0) then
                   bumpset%baction = 1  ! default strategy has been required!
               else
                   bumpset%baction = 2  ! user strategy
               endif
           endif
           errpar = ier /= 0

   elseif (match_word(word,'BWEIGHT')) then
           if (iv > 0) bumpset%weight =  vet(1)

   elseif (match_word(word,'BSCALE')) then
           if (iv > 0) bumpset%scaledist =  vet(1)

   elseif (match_word(word,'PO')) then ! orientazione preferenziale
           !if (iv == 3) then
           !    kopcorr = 1
           !    gpo%rcod = 1
           !    hklpo(:) = ivet(1:3)
           !endif

   elseif (match_word(word,'RES')) then ! prova a vedere se e' un restraint
           call get_restraint_from_string(strdir,atm,restemp,ier,sigd=SASIGD,weid=SAWEID)
           if (ier /= 0) then
               errpar = .true.
           else
               call add_restraint_to_list(res_sa,restemp)
           endif
   elseif (match_word(word,'BOND_TO_RES')) then ! prova a vedere se e' un restraint
           call get_restraint_from_string(strdir,atm,restemp,ier,sigd=SASIGD,weid=SAWEID)
              !write(0,*)'restemp=',restemp%sigma,restemp%wei
           if (ier /= 0) then
               errpar = .true.
           else
               call add_restraint_to_list(resb,restemp)
           endif
   endif
!   
!  Segnale errore nell'inserimento del paramatro da associare alla stringa
   if (errpar) then
       call errdir%set('Error in directive "'//trim(strdir0)//'"')
   endif        
!
   end subroutine sa_read_line_dir
   
!----------------------------------------------------------------------------------------------------

!----------------------------------------------------------------------------------------------------         

   subroutine sa_conta_parametri(info,atomo,frag,rotat)
   USE rrestr
   type(opti_info_type), intent(inout)           :: info
   type(atom_type), dimension(:), intent(in)     :: atomo
   type(fragment_type), dimension(:), intent(in) :: frag
   type(rotation_type), dimension(:), intent(in) :: rotat
   integer                                       :: i
!   
!  Conta gradi di libert� esterni
   info%dofe = 0           
   do i=1,numfrag
      info%dofe = info%dofe + count(frag(i)%rcod > 0)
   enddo
!   
!  Conta i B
   info%npb = count(atomo%rcod(4) > 0)
   if (nconstraints(sconstr) > 0) info%npb = info%npb + count(sconstr%ref > 0)
!
!  Conta le rotazioni   
   if (numrot > 0) then
       info%nprot = count(rotat%rcod > 0)  
   else
       info%nprot = 0
   endif    
!
!  Conta gli shifts   
   info%npshift = 0
   do i=1,size(atomo)
      info%npshift = info%npshift + count(shift_atom(i)%rcod > 0)
   enddo
!
!  DOF interni   
   info%dofi = info%nprot
!
!  Numero totale di parametri   
   info%npar = info%dofi + info%dofe + info%npb + info%npshift
!
   end subroutine sa_conta_parametri

!----------------------------------------------------------------------------------------------------

   subroutine sa_prerefine(gopti,randomod)
!
!  Conta DOF, definisce i parametri (gpar) e randomizza i parametri se richiesto
!   
   USE Atomi, only:nats    
   USE variables
   USE simulated_anneal
   USE compare_struct
   USE General, only:cell,StructureName,package_alt_name
   USE atom_type_util
   USE chem_file_fmt
   USE errormod
   USE rand_mod
   USE connect_mod
   USE elements
   USE unit_cell
   USE rrestr
   type(opti_info_type), intent(inout)       :: gopti     
   integer, intent(in)                       :: randomod 
   integer                                   :: i,j
   integer                                   :: nkref
   character(len=30)                         :: strat
   real, dimension(3)                        :: bar0
   character(len=*), dimension(5), parameter :: atomstring = (/'x   ','y   ','z   ','B   ','occ.'/)
   character(len=*), dimension(3), parameter :: string_rot =(/'theta','phi  ','z    '/)
   real, dimension(3)                        :: xroti = (/0.5,0.5,0.0/) 
   real, dimension(3)                        :: rangef
   logical, dimension(3)                     :: not_periodic, out_range
   real, dimension(3)                        :: newpos
   real                                      :: newang
   type(error_type)                          :: error
   integer                                   :: ndoc
!test   integer :: no_H
!
!  Randomizza il modello se richiesto 
   if (randomod >= 0) then
       call Init_Ran(randomod)
       call randomize_molecule(atm,lconn,set_cell_type(cell),fragment,rotation)
       call set_atm_in_frag(fragment,atm)                    ! update fragment info
       do i=1,numrot                                         ! update torsion value
          rotation(i)%torsval = torsion_value_for_rotat(rotation(i),atm)
       enddo
!
!      Crea mol file dalle coordinate random
       call create_molfile(trim(StructureName)//'_rand.mol',cella=cell,atom=atm,legm=lconn,progname=package_alt_name,error=error)
   endif
!
!  Alloca xpar al numero di parametri       
   if (allocated(gpar)) deallocate(gpar)
   allocate(gpar(gopti%npar))
!
   gpar(:)%stp = 1.0
   nkref = 0      
!   
!  Parametri termici
   do j=1,nats
      write(strat,'(i0,a)')j,' ('//trim(atm(j)%lab)//')'     
      if (atm(j)%rcod(4) > 0) then      
         nkref = nkref + 1
         gpar(nkref)%str = trim(atomstring(4))//strat
         gpar(nkref)%val = atm(j)%biso
         atm(j)%rcod(4) = nkref
         gpar(nkref)%low = max(0.0,atm(j)%biso-10)
         gpar(nkref)%high = atm(j)%biso+10
         gpar(nkref)%bound = 0         
         allocate(gpar(nkref)%pat(1))
         gpar(nkref)%pat(1) = j         
         gpar(nkref)%typepar = THERMALF_TYPE
      endif
   enddo
!
   do i=1,nconstraints(sconstr)
      if (sconstr(i)%ref > 0) then
          select case (sconstr(i)%code)
            case (1)
              nkref = nkref + 1
              gpar(nkref)%str = 'constraint on '//trim(specie_from_pxen(atm(sconstr(i)%vat(1))%ptab))
              sconstr(i)%ref = nkref
              gpar(nkref)%low = 0
              gpar(nkref)%high = 20
              gpar(nkref)%bound = 0         
              gpar(nkref)%val = constr_aveb(sconstr(i),atm,gpar(nkref)%low,gpar(nkref)%high)
          !write(0,*)'MED=',gpar(nkref)%val,atm(sconstr(i)%vat)%biso
              allocate(gpar(nkref)%pat(sconstr(i)%npar))
              gpar(nkref)%pat(:) = sconstr(i)%vat
              gpar(nkref)%typepar = THERMALF_TYPE
             case (4)
          end select
      endif
   enddo
!
!  Loop sui frammenti
   do i=1,numfrag
!
!     Traslazioni del frammento
!     Calcola il baricentro del frammento
      bar0 = baricentro(atm(fragment(i)%pos)) 
!
!     calcola il range di movimento in ognuna delle 3 direzioni
      rangef(:) = fragment(i)%thigh(:) - fragment(i)%tlow(:)    
!
!     controlla se il range e' diverso da 1 e quindi non pu� essere periodico
      not_periodic(:) = abs(rangef(:) - 1.0) > 0.01
!
!     controlla se il frammento e' fouri range
      out_range(:) = any(bar0(:) > fragment(i)%thigh) .or. any(bar0(:) < fragment(i)%tlow)      
!
!     Traslo il frammento all'interno del range se si verificano contemporaneamente le 2 condizioni:
!     1) Il range non e' periodico in almeno una direzione
!     2) Il frammento e' fuori range in almeno una direzione
      if (any(not_periodic(:)) .and. (any(out_range(:))) ) then
!
!         Modifica le coordinate del frammento solo lungo le direzioni che non
!         soddisfano le 2 condizioni precedenti      
          where(not_periodic(:) .and. out_range) 
                newpos(:) = rangef(:)/2
          elsewhere
                newpos(:) = bar0(:)
          endwhere
!!!!          write(6,*)'Frammento spostato in ',newpos,' da ',bar0
!
!         Ora sposta il frammento nella nuova posizione          
          call set_fragment_position(fragment(i),newpos,atm)   
      endif       
!
      do j=1,3      
         if (fragment(i)%rcod(j) > 0) then             
             nkref = nkref + 1
             gpar(nkref)%str = atomstring(j)
             gpar(nkref)%val = 0  
             fragment(i)%rcod(j) = nkref         
             if (not_periodic(j)) then     ! range non periodico
                 gpar(nkref)%low = fragment(i)%tlow(j)/2.0
                 gpar(nkref)%high = fragment(i)%thigh(j)/2.0
                 gpar(nkref)%bound = 0             
             else                          ! range periodico
                 gpar(nkref)%low = -bar0(j)
                 gpar(nkref)%high = 1-bar0(j)
                 gpar(nkref)%bound = 1             
             endif
             !no_H = fragment(i)%nat - number_of_hydrogens(atm(fragment(i)%pos))
             !write(0,*)'NO_H=',no_H
             !allocate(gpar(nkref)%pat(no_H))
             !gpar(nkref)%pat = pack(fragment(i)%pos, mask = atomic_number(atm(fragment(i)%pos)) /= H_at)
             !      write(0,*)'PAR=',gpar(nkref)%pat
             allocate(gpar(nkref)%pat(fragment(i)%nat))
             gpar(nkref)%pat = fragment(i)%pos   
             gpar(nkref)%typepar = EXT_TYPE
         endif
      enddo
!
!     Rotazione frammento      
!
!     calcola il range per la rotazione random del frammento
      rangef(1) = fragment(i)%rbound(2) - fragment(i)%rbound(1)    
!
!     controlla se il range e' diverso da 1 e quindi non puo' essere periodico
      not_periodic(1) = abs(rangef(1) - 1.0) > 0.01
      do j=4,6
         if (fragment(i)%rcod(j) > 0) then
             nkref = nkref + 1
             gpar(nkref)%str = string_rot(j-3)
             gpar(nkref)%val = xroti(j-3)   
             fragment(i)%rcod(j) = nkref
             if (not_periodic(1)) then
                 gpar(nkref)%low = max(xroti(j-3) - rangef(1)/2,0.0)
                 gpar(nkref)%high = min(xroti(j-3) + rangef(1)/2,1.0)
                 gpar(nkref)%bound = 0
             else
                 gpar(nkref)%low = 0.0
                 gpar(nkref)%high = 1.0
                 gpar(nkref)%bound = 1
             endif
             allocate(gpar(nkref)%pat(fragment(i)%nat))
             gpar(nkref)%pat = fragment(i)%pos  
             gpar(nkref)%typepar = EXT_TYPE
         endif          
      enddo
   enddo   
!
!  Loop sulle rotazioni
   do i=1,numrot
      if (rotation(i)%rcod > 0) then
          nkref = nkref + 1         
          write(gpar(nkref)%str,'(a,i0)')'DOF(int) n.',i 
          gpar(nkref)%val = 0.0
          rotation(i)%rcod = nkref
          rangef(1) = rotation(i)%trbound(2) - rotation(i)%trbound(1)  ! range di rotazione
          not_periodic(1) = abs(rangef(1) - 360) > 0.1                 ! valuta periodicit�
          if (not_periodic(1)) then     ! range non periodico
!
!             Converti i limiti delle torsioni in limiti degli angoli di rotazione normalizzati 
              !write(6,*)i,'tors lim=',rotation(i)%trbound(1),rotation(i)%trbound(2)
              gpar(nkref)%low = -(rotation(i)%torsval - rotation(i)%trbound(1))*dtor/twopi
              gpar(nkref)%high = (rotation(i)%trbound(2) - rotation(i)%torsval)*dtor/twopi
                   !write(0,*)'TORS=',gpar(nkref)%low,gpar(nkref)%high
              gpar(nkref)%bound = 0          
              !write(6,*)i,'ang     =', -(rotation(i)%torsval - rotation(i)%trbound(1)),   &
              !                          (rotation(i)%trbound(2) - rotation(i)%torsval)
              !write(6,*)i,'ang norm=',gpar(nkref)%low,gpar(nkref)%high
              out_range(1) = 0.0 < gpar(nkref)%low .or. 0.0 > gpar(nkref)%high
              if (out_range(1)) then   ! torsione fuori range
              !    write(0,*)'torsione fuori range'
                  newang = (gpar(nkref)%high - gpar(nkref)%low) / 2
              endif
          else                          ! range periodico
              gpar(nkref)%low = -0.5
              gpar(nkref)%high = 0.5
              gpar(nkref)%bound = 1          
          endif
          allocate(gpar(nkref)%pat(rotation(i)%nat))
          gpar(nkref)%pat = rotation(i)%pat
          gpar(nkref)%typepar = INT_TYPE
      endif
   enddo
!
!  Loop per gli shifts
   do i=1,nats  
      do j=1,3
         if (shift_atom(i)%rcod(j) > 0) then
             nkref = nkref + 1             
             gpar(nkref)%val = 0.0
             shift_atom(i)%rcod(j) = nkref
             gpar(nkref)%low = 0.0
             gpar(nkref)%high = shiftatomval 
             gpar(nkref)%bound = 0             
             allocate(gpar(nkref)%pat(1))
             gpar(nkref)%pat(1) = i  
             gpar(nkref)%typepar = INT_TYPE
             write(strat,'(i0,a)')gpar(nkref)%pat(1),' ('//trim(atm(gpar(nkref)%pat(1))%lab)//')'
             gpar(nkref)%str = trim(atomstring(j))//strat
         endif
      enddo
   enddo
!
!  Correzione per PO
   !if (gpo%rcod > 0) then
   !    nkref = nkref + 1
   !    gpo%val = 1.0
   !    gpo%rcod = nkref
   !    gpar(nkref)%str = 'G'
   !    gpar(nkref)%val = 1.0
   !    gpar(nkref)%low = 0.5
   !    gpar(nkref)%high = 1.0
   !    gpar(nkref)%bound = 0             
   !    gpar(nkref)%typepar = PO_TYPE
   !endif
!
!  Set  starting model for moves
   call copia_atoms(atom0,atm)
!
!  Set variable for DOC
   call set_doc(sadoc,atm%rcod(5))
!corr   call sadoc%prn(0,atm)
!corr   ndoc = count(atm%rcod(5) == 1)
!corr   gopti%doc = any(atm(:)%rcod(5) > 0)   !check for doc
!corr   if (gopti%doc .and. ndoc > 0) then
!corr       call reallocate(vetdoc,ndoc)
!corr       vetdoc = pack([(i,i=1,size(atm))],mask=atm%rcod(5) == 1)
!corr   else
!corr       call reallocate(vetdoc,0)
!corr   endif
!
   end subroutine sa_prerefine 
   
!----------------------------------------------------------------------------------------------------

   subroutine gopt_set_ncycle(info,ncy)
!
!  Assegna un valore alla variabile che determina il numero di cicli per step di temperatura
!   
   USE arrayutil
   USE rotationmod
   type(opti_info_type), intent(in)   :: info
   integer, intent(out)               :: ncy
   real                               :: const
   real                               :: cfrag
   real                               :: cdofe
   real                               :: crot
   real                               :: ctherm
   real                               :: cshifta
   real                               :: crotc
   integer                            :: flex
!corr   integer                            :: i,j
!corr   integer, dimension(numrot)         :: vrot
!
   const = 10.0
   cfrag = 3.0    ! contributo per ogni frammento oltre al primo
   cdofe = 2.0    ! contributo per ogni grado di liberta esterno
   crot  = 1.0    ! contributo per ogni rotazione
   ctherm = 0.1   ! contributo per ogni fattore termico
   cshifta = 0.1  ! contributo per ogni shift
   crotc  = 0.8   ! contributo per ogni rotazione concatenata
!   
   if(numfrag == 1 .and. numrot <= 2) then 
!           
!     Possibile formula: 20 + 10*2**numrot
      ncy = 20 + 10*2**numrot + const*(info%npb + info%npshift)
   else           
!
!  Cerca le rotazioni concatenate
!corr   vrot = 0
!corr   do i=1,numrot
!corr      if (rotation(i)%rcod > 0) then
!corr          do j=1,numrot
!corr             if (j==i) cycle
!corr             if (rotation(j)%rcod > 0) then
!corr                 if (check_container(rotation(i)%pat,rotation(j)%pat)) then
!!!                     write(6,*)'rotazione ',j,' contenuta in ',i
!!!                     write(6,*)i,' rotazione =',rotation(i)%pat
!!!                     write(6,*)j,' rotazione =',rotation(j)%pat
!corr                     vrot(i) = vrot(i) + 1          
!corr                 endif
!corr             endif    
!corr          enddo
!corr      endif
!corr   enddo   
!!!   write(6,*)'VROT=',vrot
   flex = num_connected_rotation(rotation)
!corr   write(0,*)'SUMVROT=',sum(vrot),num_connected_rotation(rotation)
! 
   ncy = nint ( const*(cfrag*(numfrag-1) + cdofe*info%dofe +        &
                crot*info%nprot  + crotc*flex   +              &
                ctherm*info%npb  + cshifta*info%npshift) )  
   endif                
!
   end subroutine gopt_set_ncycle
      
!----------------------------------------------------------------------------------------------------

   subroutine gopt_set_algo(info,num_rot,num_frag)
   type(opti_info_type), intent(inout) :: info
   integer, intent(in)                 :: num_rot,num_frag
!
   if (num_rot <= 5 .and. num_frag <= 2) then
       info%algo = BBBC
   else
       info%algo = SA_LOCAL
   endif
!
   end subroutine  gopt_set_algo

!----------------------------------------------------------------------------------------------------

   subroutine xpar_to_atm(x,doc,atom,refl)
!
!  Converte i parametri x del SA in parametri atomici atm
!   
   USE variables, only:ortom
   USE general, only:pi,rtod
   USE fastfcal
   USE atom_type_util
   USE cgeom
   USE rrestr
!
   real, dimension(:), intent(in) :: x       ! parametri SA
   logical, intent(in)            :: doc
   type(atom_type), dimension(:), intent(out) :: atom
   type(reflection_type), dimension(:), intent(in) :: refl
   integer                        :: j
   real, dimension(3)             :: xtra
   real, dimension(3)             :: xrot
   integer                        :: scodt,scodr
   integer                        :: i
   real, dimension(3)             :: pa1,pa2
   type(atom_type), dimension(size(atom))  :: atmr 
   real                           :: theta 
   integer                        :: natrot
   integer                        :: natom
!corr   integer :: ic
   !logical                        :: dynocc = .false.
                 integer :: nshift
                 real, dimension(3) :: xshift
                 type(atom_type) :: atc
   atom = atom0
   natom = size(atom)
!
!  Thermal factor
   do j=1,natom
      if (atom(j)%rcod(4) > 0) then      ! B
          atom(j)%biso = x(atom0(j)%rcod(4))
      endif
   enddo
   do i=1,nconstraints(sconstr)
      if (sconstr(i)%ref > 0) then
          select case (sconstr(i)%code) 
             case (1)
                atom(sconstr(i)%vat)%biso = x(sconstr(i)%ref)
             case (4)
          end select
      endif
   enddo
!
!  Frammenti
   do i=1,numfrag
      scodt = 0
      do j=1,3                           ! traslazione
         if (fragment(i)%rcod(j) > 0) then
             xtra(j) = x(fragment(i)%rcod(j))
             scodt = scodt + 1
         else
             xtra(j) = 0
         endif
      enddo
!      
      scodr = 0
      do j=4,6                           ! rotazione
         if (fragment(i)%rcod(j) > 0) then
             xrot(j-3) = x(fragment(i)%rcod(j))
             scodr = scodr + 1
         else
             xrot(j-3) = 0
         endif
      enddo      
      !write(6,*)'xrot=',xrot(:)
      if (scodt + scodr > 0) then
          !call move_fragment(atom,fragment(i),xtra,xrot,scodt,scodr)      
          !atom(fragment(i)%pos) = fragment(i)%atm0
          call move_fragment(atom,fragment(i)%pos,xtra,xrot,scodt,scodr)      
!      else
!
!         se non affini i DOF esterni riposiziona il frammento nell'origine      
!corr-old          atom(fragment(i)%pos) = fragment(i)%atm0
          !do ic=1,3
          !   atom(fragment(i)%pos)%xc(ic) = fragment(i)%atm0%xc(ic)
          !enddo
      endif          
   enddo   
!
!  Rotazioni
   do i=1,numrot
      if (rotation(i)%rcod > 0) then
          pa1 = matmul(ortom,atom(rotation(i)%pax(1))%xc)
          pa2 = matmul(ortom,atom(rotation(i)%pax(2))%xc)
          theta = 2.0*pi*x(rotation(i)%rcod)
          natrot = rotation(i)%nat
          atmr(:natrot) = atom(rotation(i)%pat)
          call rotate_atoms(atmr(:natrot),pa1,direction_cos(pa1,pa2),theta)
          atom(rotation(i)%pat) = atmr(:natrot)
      endif
   enddo
!
!  Shift sugli atomi
   !do i=1,natom 
   !   do j=1,3
   !      if (shift_atom(i)%rcod(j) > 0) then
   !          atom(i)%xc(j) = atom(i)%xc(j) + x(shift_atom(i)%rcod(j))
   !      endif
   !   enddo
   !enddo
   do i=1,natom 
      xshift(:) = 0
      nshift = 0
      do j=1,3
         if (shift_atom(i)%rcod(j) > 0) then
             xshift(j) =  x(shift_atom(i)%rcod(j))
             nshift = nshift + 1
         endif
      enddo
      if (nshift > 0) then
          !write(0,*)'sh1=',atom(i)%xc
          atc = cartesian_coord(atom(i))
          atc%xc(:) =  atc%xc(:) + xshift(:)
          atom(i) = fractional_coord(atc)
          !write(0,*)'sh2=',atom(i)%xc
      endif
   enddo
!   
!  Dynamical occupancy correction
!   -- modifica da fare: solo sugli atomi affinati va calcolata l'occupanza---
   if (sadoc%active()) then
       !call calcola_occ(atom,.false.,dist=DISTOCC)  
       !do i=1,numfrag
       !   atom(fragment(i)%pos)%och = fragment(i)%atm0%och
       !enddo
       !call search_overlapped_atoms(atom,.false.)
       if (sadoc%all_atoms()) then
           call compute_doc(atom,1.0)
           call fcalc_for_tfocc(refl,atom,(/(i,i=1,natom)/))       
       else
           call compute_doc(atom,1.0,sadoc%vet)
           call fcalc_for_tfocc(refl,atom(sadoc%vet),sadoc%vet)
       endif
       !call compute_doc(atom(20:20),1.0)
       !call fcalc_for_tfocc(refl,atom(20:20),(/(i,i=20,20)/))       
   endif
!
!  Orientazione preferenziale
   !if (gpo%rcod > 0) then
   !    gpo%val = x(gpo%rcod)
   !endif
!  
   end subroutine xpar_to_atm
   
!----------------------------------------------------------------------------------------------------

   subroutine search_overlapped_atoms(atom,modifi,vet)
   USE Symmetry
   USE General, only:xgg
   USE CGEOM
   type(atom_type), dimension(:), intent(inout) :: atom   
   logical, intent(in), optional                :: modifi
   logical                                      :: modif
   integer                                      :: natc
   integer, dimension(:), intent(in)            :: vet
!corr   real, dimension(3,(icent+1)*nsym*ncentr)     :: xeq   !!!FIXME
   real, dimension(3,nsymtot)                   :: xeq
   real                                         :: dd
   integer                                      :: i,j,k
   real, dimension(3) :: ktra
!corr   integer, dimension(size(atom))               :: ivetocc
!
   natc = size(atom)
   if (present(modifi)) then
       modif = modifi
   else
       modif = .true.
   endif
!
!corr   ivetocc(:) = 1
   do i=1,natc-1
      call get_equivalent1(atom(i)%xc,xeq)   !!!!!,neqa)
      if (all(vet /= i)) cycle
      do j=i+1,natc
         if (all(vet /= j)) cycle
         if (atom(i)%nz == atom(j)%nz) then
             do k=1,nsymtot
                if (k == 1 .and. j < i) cycle  ! combinazione già testata
                !dd = xdisteq(xeq(:,k),atom(j)%xc,xgg)
                call xdisteqs(atom(j)%xc,xeq(:,k),xgg,dd,ktra)
                !dd = distanzaC(atom(j)%xc,xeq(:,k),xgg)
                if (dd < DISTDOC) then
!corr                    ivetocc(i) = ivetocc(i) + 1
!corr                    ivetocc(j) = ivetocc(j) + 1
                    if (modif) then
                        !if (dd < 0.01) then   !! this is test
                        !    atom(j)%xc = atom(i)%xc   !!!FIXME - but it is necessary reconnect j atom if k/=1
                        !else
                            atom(j)%xc = atom(j)%xc + ktra  ! ora i 2 atomi sono nella stessa posizione
                        !endif
                !call xdisteqs(atom(j)%xc,xeq(:,k),xgg,dd,ktra)
                    !write(0,'(a,a,a,a,f0.4,a,i0)')'vicinid=',atom(i)%lab,atom(j)%lab,'dd=',dd,' op=',k
                    !         write(0,*)'atom=',trim(atom(i)%lab),atom(i)%xc
                    !         write(0,*)'atom=',trim(atom(j)%lab),atom(j)%xc
                    endif
                    !non funziona atom(j)%xc = xeq(:,k) - ktra
                endif
             enddo
         endif
      enddo
   enddo
!corr   do i=1,natc
!corr      atom(i)%och = atom(i)%och/ivetocc(i)
!corr   enddo
!
   end subroutine search_overlapped_atoms

!----------------------------------------------------------------------------------------------------

   subroutine search_overlapped_atoms_print(atom)
   USE Symmetry
   USE General, only:xgg,lo
   USE CGEOM
   USE strutil
   USE nr
   type(atom_type), dimension(:), intent(in) :: atom   
   integer                                      :: natc
   real, dimension(3,(icent+1)*nsym*ncentr)     :: xeq   !!!FIXME
   real                                         :: dd
   integer                                      :: i,j,k
!cor   integer                                      :: neqa
   real, dimension(3)                           :: ktra
   integer, dimension(size(atom), size(atom))   :: tableat
   real, dimension(size(atom), size(atom))   :: tabled   !!!!FIXME  da migliorare
   integer                                      :: nrem
   integer, dimension(size(atom))               :: vrem
   integer, allocatable, dimension(:)           :: iord
!
   natc = size(atom)
   tableat(:,:) = 0
   do i=1,natc
      call get_equivalent1(atom(i)%xc,xeq)   !!!!!,neqa)
      do j=1,natc
         if (i==j) cycle
         if (atom(i)%nz == atom(j)%nz) then
             do k=1,nsymtot
                if (k == 1 .and. j < i) cycle  ! combinazione già testata
                call xdisteqs(atom(j)%xc,xeq(:,k),xgg,dd,ktra)
                if (dd < DISTDOC) then
                    !write(0,'(" WARNING: Atoms ",a," and ",a," are in the same position")')   &
                    !trim(atom(i)%lab),trim(atom(j)%lab)
                    tableat(i,j) = 1
                    tableat(j,i) = 1
                else if (dd < DISTOCC + 0.3) then
                    !write(0,'(" WARNING: Critical distance between ",a," and ",a," (d=",f0.3,")")')   &
                    !trim(atom(i)%lab),trim(atom(j)%lab),dd
                    tableat(i,j) = -1
                    tableat(j,i) = -1
                    tabled(i,j) = dd
                    tabled(j,i) = dd
                endif
             enddo
         endif
      enddo
   enddo
!
   do i=1,natc-1
      do j=i+1,natc
         if (tableat(i,j) == 1) then
             write(lo,'(" WARNING: Atoms ",a," and ",a," are in the same position")')   &
             trim(atom(i)%lab),trim(atom(j)%lab)
         elseif (tableat(i,j) == -1) then
             write(lo,'(" WARNING: Critical distance between ",a," and ",a," (d=",f0.3,")")')   &
             trim(atom(i)%lab),trim(atom(j)%lab),tabled(i,j)
         endif
      enddo
   enddo
!
   nrem = 0
   do i=1,natc-1
      do j=i+1,natc
         if (abs(tableat(i,j)) > 0) then
!             
!            cosa rimuovere: i o j?
             !wej = sum(abs(tableat(i,:))) 
             !wei = sum(abs(tableat(j,:)))
             !if (wei > wej) then
             !    write(lo,*)'posso rimuovere ',trim(atom(i)%lab),wei,wej
             !    tableat(i,:) = 0
             !    tableat(:,i) = 0
             !    exit
             !else
              nrem = nrem + 1
              vrem(nrem) = j
                 !!!!write(lo,*)'posso rimuovere ',trim(atom(j)%lab)
                 tableat(j,:) = 0
                 tableat(:,j) = 0
                 cycle
             !endif
         endif
      enddo
   enddo
!
!  Stampa gli atomi da eliminare
   if (nrem > 0) then
       if (nrem > 1) then   ! ordina il vettore vrem
           allocate(iord(nrem))
           call indexx(vrem(:nrem),iord)
           vrem(:nrem) = vrem(iord)
       endif
       call write_svet(atom(vrem(:nrem))%lab,nrem,lo,' The following atoms could be removed: ')
   endif
!
   end subroutine search_overlapped_atoms_print

!----------------------------------------------------------------------------------------------------

   real function tfunk(x,hpar)
   USE Calctime    
   USE variables, only:atm,ref
   USE anti_bump
!
   real, dimension(:), intent(in) :: x
   integer, intent(in)            :: hpar
   integer, parameter             :: kpr = 0
   real :: chid
!
!  Passa x in atm
   call timestep(3)
!corr   call xpar_to_atm(x,ginfo%doc,atm,ref)
   call xpar_to_atm(x,sadoc%active(),atm,ref)
   call timestep(3)
!
   call timestep(2)
         
   tfunk = chisquared_xdata(hpar,ginfo,atm,ref)
!
!  Contributo restraints
   if (nrestraints(res_sa) > 0) then
       chid=chisquared_res(res_sa, atm, kpr=0)
       tfunk = tfunk + chid
   endif
!
!  Contributo anti-bump
   if (nrestraints(bump_sa) > 0) then
       tfunk = tfunk + bumpset%weight * antibump_function(bump_sa, atm, ginfo%doc, .false.)
   endif
         
   call timestep(2)
!   
   end function tfunk
      
!----------------------------------------------------------------------------------------------------

   real function chisquared_xdata(hpar,gopti,atom,refl)  result(chix)
   USE atom_type_util
   integer, intent(in)                                :: hpar
   type(opti_info_type), intent(in)                   :: gopti    
   type(atom_type), dimension(:), intent(in)          :: atom
   type(reflection_type), dimension(:), intent(inout) :: refl
!
   select case(gopti%cftype)
      case (CHI2PROF)

      case (CHI2FACT)
        chix = sqrt(chisquared_factor(hpar,gopti,atom,refl))/rwden

      case (CHI2I_SD)
        chix = sqrt(chisquared_int(hpar,gopti,atom,refl))/rwden

      case (CHI2I)
        chix = sqrt(chisquared_int(hpar,gopti,atom,refl))/rwden

      case (CHI2I_NEW)

      case (CHI2PROF_NEW)

   end select
!
   end function chisquared_xdata

!----------------------------------------------------------------------------------------------------

   subroutine gopt_statusbar(ntot,ntemp)
   USE Calctime
   USE messagemod
   USE simulated_anneal
   integer, intent(in)                        :: ntot
   integer, intent(in)                        :: ntemp
   real                                       :: init_time = 0
   real                                       :: end_time
   real                                       :: range_time = 1.0 ! intervallo di tempo dopo il quale updatare la statusbar
   logical                                    :: oktime 
   type(calctime_type)                        :: time
!
   call timestep(4) ! calcola il tempo impiegato dalle operazioni grafiche  
!
!  Controlla se e trascorso un tempo pari a range_time
   call cpu_time(end_time)
   oktime = end_time - init_time > range_time
   if (oktime) init_time = end_time
!   
!  Aggiorna informazioni sulla statusbar
   if (oktime .or. ntemp == numcyc_calc) then
       call write_message('Move n.',inum=ntot,pos=vmess(LINE_MOVES))
       call get_timestep(1,time)
       call write_message('Time: '//string_time(time%tot),pos=vmess(LINE_TIME))
       call progress_bar_update(cval=ntemp,fval=numcyc_calc)
   endif
!
   call timestep(4)   
!   
   end subroutine gopt_statusbar   
      
!----------------------------------------------------------------------------------------------------

   subroutine gopt_draw(sdraw,pdraw,gopti)
   USE Atomi
   USE variables, only:atm,ref,lconn
   USE atom_type_util
   USE Calctime
   USE connect_mod
   USE ilmviewmod
   !USE fouriermap
   USE factor
   logical, intent(in)                         :: sdraw,pdraw
   type(opti_info_type), intent(in)            :: gopti    
   type(atom_type), dimension(nats)            :: atmsav
   type(bond_type), dimension(:), allocatable  :: legm
   type(reflection_type), dimension(size(ref)) :: refmap
!
   call timestep(4) ! calcola il tempo impiegato dalle operazioni grafiche  
!
!  Disegna la struttura
   if (sdraw) then
!corr       if (numfrag > 1 .or. gopti%doc) atmsav(:) = atm(:)
       if (numfrag > 1 .or. sadoc%active()) atmsav(:) = atm(:)
       if (numfrag > 1) call regroup_fragment(fragment,atm)
       if (sadoc%active()) then
           call search_overlapped_atoms(atm,.true.,sadoc%vet)
!corr           call calcola_occ(atm,dist=DISTOCC)
       endif
!corr       call copia_legm(legm,lconn)
       if (numfrag > 1) then 
           call copia_legm(legm,lconn)
           call connect_fragment(fragment,atm,legm)
           call ilmview_draw(atm,legm,resize=.not.(ilmview_info(1,1) == 1))  ! no resize with visible cell
       else
           call ilmview_draw(atm,lconn,resize=.not.(ilmview_info(1,1) == 1))  ! no resize with visible cell
       endif
       if (numfrag > 1 .or. sadoc%active()) atm(:) = atmsav(:)
       !if (is_map_enabled()) then
       !    call map_set_centre(baricentro(atm))
       !    refmap(:) = ref(:)
       !    call fcalcan(refmap,atm)  ! compute fase
       !    call jav_show_map(map_type_selected(),refmap,'SA map',sogmap=sasogmap,fctype=2)
       !endif
   endif
!   
!  Disegna il profilo
   !if (pdraw) then
   !    call sa_profile(sqrt(ref%fc))
   !    if (kzoomn) then
   !        call vedinew(iAct_sav,xzleft,xzright,xzleft,xzright,lsetstyle=.false.)
   !    else
   !        call vedinew(iAct_sav,1,gopti%nco,1,gopti%nco,lsetstyle=.false.)
   !    endif
   !endif
   call timestep(4)   
!
   end subroutine gopt_draw

!----------------------------------------------------------------------------------------------------

   real function chisquared_factor(hpar,gopti,atom,refl) result(chi)
   USE Atomi, only:nats
   USE fastfcal
!   USE refinecomref, only:gpo,hklpo
!   USE RIETV
   integer, intent(in) :: hpar
   type(opti_info_type), intent(in) :: gopti
   type(atom_type), dimension(:), intent(in) :: atom
   type(reflection_type), dimension(:), intent(inout) :: refl
   integer             :: i
   real                :: scal
!  type(reflection_type), dimension(size(ref)) :: ref1
!
!  Calcola fattori di struttura
   if (hpar > 0) then       
       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
           call fcalc_from_thermalf(refl,atom(gpar(hpar)%pat),gpar(hpar)%pat)
       else
           call fcalc_from_atm(refl,atom(gpar(hpar)%pat),gpar(hpar)%pat)                          
       endif    
   else
       if (gopti%doc)call fcalc_for_tfocc(refl,atom,(/(i,i=1,nats)/))       
       call fcalc_from_atm(refl,atom,(/(i,i=1,nats)/))
       call fcalc_save_atm((/(i,i=1,nats)/))
   endif    
   !call check_fcalc(refl,atom,hpar)   ! diagnostica degli fc
!
   refl%fc = sqrt(refl%fc)
   scal = sum(refl%fo) / sum(refl%fc)
   chi = sum((refl%fo - scal*refl%fc)**2)    
!   
   end function chisquared_factor
   
!----------------------------------------------------------------------------------------------------

   real function chisquared_int(hpar,gopti,atom,refl) result(chi)
   USE Atomi, only:nats
   USE fastfcal
   integer, intent(in) :: hpar   
   type(opti_info_type), intent(in) :: gopti    
   type(atom_type), dimension(:), intent(in) :: atom
   type(reflection_type), dimension(:), intent(inout) :: refl
   integer             :: i
   real                :: scal
!  type(reflection_type), dimension(size(ref)) :: ref1

!
!  Calcola fattori di struttura
   if (hpar > 0) then       
       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
           call fcalc_from_thermalf(refl,atom(gpar(hpar)%pat),gpar(hpar)%pat)
       else
           call fcalc_from_atm(refl,atom(gpar(hpar)%pat),gpar(hpar)%pat)                          
       endif    
   else
       if (gopti%doc)call fcalc_for_tfocc(refl,atom,(/(i,i=1,nats)/))       
       call fcalc_from_atm(refl,atom,(/(i,i=1,nats)/))
       call fcalc_save_atm((/(i,i=1,nats)/))
   endif    
   !!!!call check_fcalc(ref,atom,1)   ! diagnostica degli fc
!
   !intcal = ref%m*ref%fc 
   intcal = refl%fc  ! fc al quadrato
   !intcal = ref%lp*ref%fc 
   scal = sumintoss / sum(intcal)
   !chi = sum((intoss - scal*intcal)**2)    
   chi = sum(abs(intoss - scal*intcal)) 
!   chi = sum(weichi(:)*abs(intoss - scal*intcal)) 
!   
   end function chisquared_int
               
!----------------------------------------------------------------------------------------------------
   real function chisquared_bond(kpr) result(chi)
!
!  Calcola chi2 sulle distanze di legame
!   
   USE General, only:xgg
   USE CGEOM
   USE variables, only:atm,lconn
   USE connect_mod
!
   integer, intent(in) :: kpr
   real                :: d, diffd
   real                :: delta = 0.02
   integer             :: i   
!
   chi = 0.0
   do i=1,numbonds(lconn)
      d = distanzaC(atm(lconn(i)%n1)%xc,atm(lconn(i)%n2)%xc,xgg)
      diffd = d-lconn(i)%dist
      if (diffd >= delta) then
          chi = chi + ((diffd-delta)/lconn(i)%sigma)**2
      elseif (diffd <= -delta) then
          chi = chi + ((diffd+delta)/lconn(i)%sigma)**2
      endif 
      if (kpr > 0) write(6,'(a,i0,a,i0,a,f0.4,a,f0.4)')        &
       ' at=',lconn(i)%n1,'-',lconn(i)%n2,' dcalc=',d,' datteso=',lconn(i)%dist
   enddo
!   
   end function chisquared_bond
         
!----------------------------------------------------------------------------------------------------

   real function chisquared_angle(kpr) result(chi)
!
!  Calcola chi2 sugli angoli di legame
!   
   USE General, only:xgg
   USE CGEOM
   USE variables, only:atm
   USE General, only:rtod      
   USE connect_mod
!
   integer, intent(in) :: kpr
   integer             :: i
   real                :: a, diffa
   real                :: delta = 1.2
!
   chi = 0.0
   do i=1,size(ang)
      a = rtod*angleC(atm(ang(i)%n1)%xc,atm(ang(i)%n2)%xc,atm(ang(i)%n3)%xc,xgg)
      diffa = a-ang(i)%val
      if (diffa >= delta) then
          chi = chi + ((diffa-delta)/ang(i)%sigma)**2
      elseif (diffa <= -delta) then
          chi = chi + ((diffa+delta)/ang(i)%sigma)**2
      endif
      if (kpr > 0) write(6,'(3(a,i0),a,f0.4,a,f0.4)')   &
      ' at=',ang(i)%n1,'-',ang(i)%n2,'-',ang(i)%n3,' acalc=',a,' aatteso=',ang(i)%val 
   enddo
!   
   end function chisquared_angle
   
!----------------------------------------------------------------------------------------------------        

   real function chisquared_res(rest, atom, kpr)
   USE Symmetry
   USE General, only:xgg
   USE CGEOM
   USE rrestr
   USE General, only:rtod
   type(restraint_type), dimension(:), intent(inout) :: rest
   type(atom_type), dimension(:), intent(in)         :: atom
   integer, intent(in)                               :: kpr
   integer                                           :: i,k
   real, dimension(3,nsymtot)                        :: xeq  !!!FIXME
   real                                              :: dd,dmin
   real, dimension(3)                                :: ktra
   integer                                           :: n1,n2,n3
!   real                                              :: deltaa
   integer                                           :: kloc
   real                                              :: chi2a,chi2d
   real, dimension(3) :: x3, x1eq, x3eq
   integer :: kst
   real :: d12, d23
      !integer, parameter :: SYM = 1
!
!!!!   deltad = 0.3  !!!!0.02
   !deltaa = 1.2
   !deltad = 0.01
   !deltaa = 0.6
   chi2a = 0
   chi2d = 0
   do i=1, nrestraints(res_sa)
      select case (rest(i)%code)
         case (1)   ! distance
!
!          atomi coinvolti nel restraint distanza
           n1 = rest(i)%na(1)
           n2 = rest(i)%na(2)
!
!          Genera equivalenti di n2
           call get_equivalent1(atom(n2)%xc,xeq)  
!
!          Cerca la distanza minima tra n1 e equivalenti di n2
           if (n1 == n2) then
               kst = 2                  ! restraint between the same atoms: jump operator 1
               if (nsymtot == 1) cycle  ! not possible!
           else
               kst = 1
           endif
           call xdisteqs(atom(n1)%xc,xeq(:,kst),xgg,dmin,ktra)
           do k=kst+1,nsymtot
              call xdisteqs(atom(n1)%xc,xeq(:,k),xgg,dd,ktra)
              if (dd < dmin) dmin = dd
           enddo
           rest(i)%val = dmin
!
!          Valuta contributo alla funzione costo
           chi2d =  chi2d + chi2res_value(rest(i))   !!!!!,deltad)
           if (kpr > 0) write(6,'(i5,3(a,f0.3))')i,'chi2resd=',chi2d,' dist=',dmin,' expected=',rest(i)%targ

         case (3)   ! angle
           n1 = rest(i)%na(1)
           n2 = rest(i)%na(2)
           n3 = rest(i)%na(3)
!
!          generate equivalent if sym = 1
           if (rest(i)%sym(1) == 1) then
               call distance_equivalent(atom(n1)%xc,atom(n2)%xc,xgg,d12,x1eq)
           else
               d12 = distanzaC(atom(n1)%xc,atom(n2)%xc,xgg)
               x1eq = atom(n1)%xc
           endif
           if (rest(i)%sym(3) == 1) then
               call distance_equivalent(atom(n3)%xc,atom(n2)%xc,xgg,d23,x3eq)
           else
               d23 = distanzaC(atom(n3)%xc,atom(n2)%xc,xgg)
               x3eq = atom(n3)%xc
           endif
           rest(i)%val = rtod*angleC(x1eq,atom(n2)%xc,x3eq,xgg)
!
!          Evaluate contribution to cost function
           chi2a = chi2a + chi2res_value(rest(i))   !!!!,deltaa)
           if (kpr > 0) write(6,'(4(a,f0.3))')'chi2a=',chi2a,' angl=',rest(i)%val,' expected=',rest(i)%targ,' dim=',dmin
           write(66,'(3(a,f0.3))')'chi2resa=',chi2a,' angl=',rest(i)%val,' expected=',rest(i)%targ

         case (10000)   ! angle
!
!          atomi coinvolti nel restraint angolo
           n1 = rest(i)%na(1)
           n2 = rest(i)%na(2)
           n3 = rest(i)%na(3)
!
!          Genera equivalenti di n3
           call get_equivalent1(atom(n3)%xc,xeq)  !!!!!,neqa)
!
!          Cerca l'equivalente a distanza minima tra n2-n3
           call xdisteqs(atom(n2)%xc,xeq(:,1),xgg,dmin,ktra)
           !call xdisteqs(xeq(:,1),atom(n2)%xc,xgg,dmin,ktra)
           kloc = 1
           do k=2,nsymtot
              call xdisteqs(atom(n2)%xc,xeq(:,k),xgg,dd,ktra)
              !call xdisteqs(xeq(:,k),atom(n2)%xc,xgg,dd,ktra)
              if (dd < dmin) then
                  dmin = dd
                  kloc = k
              endif
           enddo
!
!         valuta angolo n1-n2-n3
          !xeq(:,kloc) = xeq(:,kloc) + ktra(:)
          x3(:) = atom(n2)%xc + ktra(:)
            !write(6,*)'distanza=',distanzaC(atom(n2)%xc,x3,xgg),dmin
          !rest(i)%val = rtod*angleC(atom(n1)%xc,atom(n2)%xc,xeq(:,kloc),xgg)
          rest(i)%val = rtod*angleC(atom(n1)%xc,atom(n2)%xc,x3,xgg)
!
!         Valuta contributo alla funzione costo
          chi2a = chi2a + chi2res_value(rest(i))   !!!!,deltaa)
          if (kpr > 0) write(6,'(4(a,f0.3))')'chi2a=',chi2a,' angl=',rest(i)%val,' expected=',rest(i)%targ,' dim=',dmin
           !write(66,'(3(a,f0.3))')'chi2resa=',chi2a,' angl=',rest(i)%val,' expected=',rest(i)%targ
      end select
   enddo
   chisquared_res = chi2a + chi2d
   if (kpr > 0) write(6,*)'chires =',chisquared_res,chi2a,chi2d
!
   end function chisquared_res

!----------------------------------------------------------------------------------------------------        

   real function chi2res_value(rest)  !!!!!,delta)
   type(restraint_type), intent(in) :: rest
   !real, intent(in)                 :: delta
   real                             :: rplus,rless
!
   !rplus = rest%targ + delta
   !rless = rest%targ - delta
   rplus = rest%targ + rest%sigma
   rless = rest%targ - rest%sigma
   if (rest%val <= rless) then
       !chi2res_value = ((rest%val-rless)/rest%sigma)**2
       chi2res_value = rest%wei*((rest%val-rless))**2
   elseif (rest%val >= rplus) then
       !chi2res_value = ((rest%val-rplus)/rest%sigma)**2
       chi2res_value = rest%wei*((rest%val-rplus))**2
   else
       chi2res_value = 0
   endif
!
   end function chi2res_value

!----------------------------------------------------------------------------------------------------        
   
   subroutine gopt_print_info(info,unt,doc,atom)      
!
!  Stampa informazioni generali
!   
!   USE RefinecomRef, only:gpo,hklpo
   type(opti_info_type), intent(in)          :: info
   integer, intent(in)                       :: unt
   type(doc_type), intent(in)                :: doc
   type(atom_type), dimension(:), intent(in) :: atom
!
   write(unt,'(/1x,a,i0)')        'Number of parameters:         ',info%npar
   write(unt,'(3x,a,i0)')           'DOF(ext):                   ',info%dofe
   write(unt,'(3x,a,i0)')           'DOF(int):                   ',info%dofi
   if (info%npb > 0) then
       write(unt,'(3x,a,i0)')       'Thermal factors:            ',info%npb
   endif
   if (info%npshift > 0) then
       write(unt,'(3x,a,i0)')       'Shifts on atoms:            ',info%npshift
   endif
   write(unt,'(1x,a,f0.2,a,f0.2)')'Used data range:              ',info%ttmin,'-',info%ttmax
   write(unt,'(1x,a,i0)')         'Number of total reflections:  ',info%nreftot
   write(unt,'(1x,a,i0)')         'Number of used reflections:   ',info%nref
   write(unt,'(1x,a,f0.2,a)')     'Resolution:                   ',info%res,' ang'
   write(unt,'(1x,a)')            'Cost function:                '//trim(info%cost_function_name)
!corr   if (info%doc) then
!corr       write(unt,'(1x,a,a)')      'Dynamical Occupancy Correction is active'
!corr   endif
   !if (gpo%rcod > 0) then
   !    write(unt,'(1x,a,3(1x,i0))')'P.O. correction is active for plane: ',hklpo(:)
   !endif
   call doc%prn(unt,atom)
!   
   end subroutine gopt_print_info
   
!----------------------------------------------------------------------------------------------------        
   
   subroutine gopt_print_parameters(xopt,par,at,unt,title)
!
!  Stampa lista dei parametri affinati
!   
   USE General, only:twopi,rtod
   USE strutil, only:centra_str
   USE atom_type_util, only:norm_torsion
   USE rrestr
   real, dimension(:), intent(in)             :: xopt
   type(param_type), dimension(:), intent(in) :: par
   type(atom_type), dimension(:), intent(in)  :: at
   integer, intent(in)                        :: unt
   integer                                    :: i,j
   integer                                    :: scod,icod   
   integer                                    :: natts
   character(len=*), intent(in), optional     :: title
!
   if (present(title)) then
       write(unt,'(/a)')centra_str(title,80)
   else        
       write(unt,*)    
   endif        
   write(unt,'(20x,a)')'        Value        Step length          Bounds'
!
!  Stampa per traslazione e rotazione frammenti   
   do i=1,numfrag
      scod = count(fragment(i)%rcod > 0)   ! numero di paramtri affinati per il frammento i
      if (scod > 0) then
          write(unt,'(1x,a,i0)')'External DOF for fragment n.',i
          do j=1,6
             icod = fragment(i)%rcod(j)
             if (icod > 0) then             
                 write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
             endif
          enddo
      endif
   enddo
!
!  Stampa per rotazioni interne ai frammenti
   if (numrot > 0) then
       scod = count(rotation%rcod > 0)
       if (scod > 0) then
           write(unt,'(1x,a)')'Internal DOF'
           do i=1,numrot
              icod = rotation(i)%rcod
              if (icod > 0) then
                  write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),                                        &
                                                            norm_torsion(rtod*twopi*xopt(icod)+rotation(i)%torsval),  &
                                                            par(icod)%stp,                                              &
                                                            rotation(i)%trbound(1),rotation(i)%trbound(2)
              endif
           enddo
       endif
   endif
!
!  Stampa per shift sugli atomi
   scod = 0
   natts = size(at)
   do i=1,natts 
      if (count(shift_atom(i)%rcod > 0) > 0) then
          scod = 1
          exit
      endif 
   enddo
   if (scod > 0) then
       write(unt,'(1x,a)')'Shift on atoms'
       do i=1,natts  
          do j=1,3
             icod = shift_atom(i)%rcod(j)
             if (icod > 0) then
                 write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
             endif
          enddo
       enddo        
   endif
!
!  Stampa per parametri termici
!corr   scod = count(at%rcod(4) > 0)
!corr   if (scod > 0) then
   if (count(at%rcod(4) > 0) > 0) then
       write(unt,'(1x,a)')'Thermal parameters'
       do j=1,size(at)
          icod = at(j)%rcod(4)
          if (icod > 0) then      
              write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
          endif
       enddo       
   endif
   do i=1,nconstraints(sconstr)
      icod = sconstr(i)%ref
      if (icod > 0) then
          write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
      endif
   enddo
!
   end subroutine gopt_print_parameters
      
!----------------------------------------------------------------------------------------------------        

   subroutine simann(gopti,sc,bb_cond)
!
!  Routine principale: esegue multirun di SA, stampa parametri iniziali e finale, gestisce la lista dei modelli
!   
   USE simulated_anneal
   USE variables, only:atm,ref,lconn
   USE Calctime
   USE General, only:fname,knw,StructureName,lo,OUT_FILE
   USE TestModel, only:vett_test,rmsd
   USE factor
   USE prnutil
   USE compare_struct
   USE atom_type_util   
   USE bb_bc
       USE grexpomod
       USE Molcom
         USE rand_mod
   USE bb_bc
         USE messagemod
   USE model_util
   USE spginfom
   USE sinterface
   USE anti_bump
   USE Conteggi, only:npunti
   USE strutil
   USE connect_mod
   implicit none
   type(opti_info_type), intent(inout)         :: gopti    
   type(SimAnn_Conditions_type), intent(inout) :: sc
   type(bb_bc_condition), intent(in)           :: bb_cond                     
   real, dimension(gopti%npar)                 :: xopt
   real                                        :: fopt
   real                                        :: tfin
   integer                                     :: nfcnev
   integer                                     :: nacc, nobds 
   integer                                     :: i,j
   type(param_type), dimension(size(gpar))     :: gparini
   type(atom_type), dimension(:), allocatable  :: atbest
   real, dimension(7)                          :: infomodel 
   real                                        :: timerun
   type(calctime_type)                         :: time
   integer                                     :: kevent
   integer                                     :: nsol      
   real                                     :: distnsol
   integer, dimension(0:6)                       :: vcolor 
   integer :: pcol
   type(atom_type), dimension(:), allocatable  :: atmr
   logical                                     :: is_profile
   integer                                     :: nplotr
   real                                        :: rappo
   real                                        :: resval
!corr   real                                        :: cfini
   integer                                     :: xdim,ydim
   type(bb_bc_condition)                       :: bcond
   type(bond_type), dimension(:), allocatable  :: legm
   real                                        :: frun_min, frun_max, frun_ave
   type(spaceg_type)                           :: spaceg
   integer :: sum_eval
   integer :: neval
   integer :: nsol8
!
   gparini = gpar
   if (jscreen > 0) then
       if(npunti == 0) call s_s_delete(gopti%popt,'P')
       !if (npunti == 0) then
       !    gopti%popt = 'VME'
       !endif
   else
      gopti%popt = 'SE'
   endif
   if (gopti%algo == BBBC) then
       call set_bb_bc_condition(bcond,sc)
       numcyc_calc = sc%num_temps*sc%ns*sc%nt*gopti%npar*bb_cond%niter*bb_cond%nlocmax  ! calculated for progress bar
       call bb_bc_print_conditions(bb_cond,6)
       call sa_print_conditions(sc,6,'Local search conditions')
   else
       numcyc_calc = sc%t_ini*sc%ns*sc%nt*gopti%npar  ! for progress bar
       call sa_print_conditions(sc,6,'Simulated Annealing conditions')
   endif
!
   ginfo = gopti
!
!  Stampa condizioni iniziali
   write(6,'(1x,a,f0.3)')     'Initial cost function value:  ',tfunk(gpar%val,0)
   if (nrestraints(bump_sa) > 0)   &
       write(6,'(1x,a,f0.3)') 'Anti-bump cost function:      ',bumpset%weight*antibump_function(bump_sa, atm, gopti%doc, .false.)
   if (nrestraints(res_sa) > 0)   &
       write(6,'(1x,a,f0.3)') 'Restraints cost function:     ',chisquared_res(res_sa,atm,kpr=0)
   call gopt_print_parameters(gpar%val,gpar,atm,6,'Starting refined parameters')
!  
   if(print_tb)write(6,'(4(a,i0),a,f0.2)')'TB1 DOFext=',gopti%dofe,' DOFint=',gopti%dofi,   &
                                   ' NREF=',gopti%nref,'/',gopti%nreftot,' RES=',gopti%res
   if(print_tb)write(6,'(a)')'TB1 NRUN  trovati     dist      RMSD       CF       n.moves          time'   
   call reallocate_models(samodel,gopti%nrun,.false.)
   nsamodel = 0
!
!  Operazione preliminari per grafico della CF
   if (index(gopti%popt,'V') > 0) then  
       vcolor = (/color_BROWN,color_LIGHTBLUE,color_LIGHTRED,color_LIGHTGREEN,     &
                  color_LIGHTCYAN,color_LIGHTMAGENTA,color_YELLOW/)
       call gr_create_graph(grgraph)
       call gr_set_graph(grgraph,stringx='moves',stringy=trim(gopti%cost_function_name_short),xscale=10000.0)
       call gr_set_graph(grgraph,y_resize=.true.,y_resize_val=150.0)
           nplotr = gopti%nrun
       if (allocated(grplot)) then
           if (size(grplot) /= nplotr) then
               deallocate(grplot)
               allocate(grplot(nplotr))
           endif
       else
           allocate(grplot(nplotr))
       endif
       do i=1,gopti%nrun
          call gr_create_plot(grplot(i),1000)
          pcol = mod(i,size(vcolor)) 
          call gr_set_plot(grplot(i),colorline=vcolor(pcol))
       enddo
!
!      Per visualizzare la linea dell'Rwp alla fine dell'affinamento
!
       call new_viewer_size(xdim,ydim)
       call gr_open_viewer(xdim,ydim,'CF vs. number of moves',newv=.true.)
       call gr_select_viewer(1)  ! serve se no disegna su entrambi i viewers 
       call gr_setbackground(color_BRIGHTWHITE)
       call gr_select_viewer(0)
   endif
!   
!  gestione messaggi
   if (knw /= 1 .or. ModelToCompare /= 1) call clear_messages(8) ! se la struttura e' incognita la riga 8 non viene usata  
!   
   allocate(atmr(size(atm)))
   nsol = 0
   distnsol = 0
   gopti%fmin = huge(1.0)
   gopti%rmin = 1
   neval = 0
   sum_eval = 0
   call reset_totaltime()
   spaceg = init_spaceg_type(spaceg_string())  ! construct class spaceg_type
   do i=1,gopti%nrun          ! inizio loop sui run di SA
!corr      write(6,'(/2x,80("=")/,25x,"** Simulated Annealing Run n.",i0," **")')i
      write(6,'(/2x,80("=")/,25x,"** ",a," Run n.",i0," **")')trim(algoname_long(gopti%algo)),i
      call timestep(5)     ! inizio per misura tempo/run
      gpar(:)%val = gparini(:)%val
      gpar(:)%stp = gparini(:)%stp
      gopti%nrunc = i         ! memorizza a che run sono
      if (gopti%seed == 0) then
          call Init_Ran(gopti%seed)  ! seed from system clock
      else
          call Init_Ran(gopti%seed+i-1)
      endif
!
!     Applica ottimizzazione dei parametri con SA      
      if (gopti%algo == SA_LOCAL) then
          call sa(gpar, sc, gopti, tfin, xopt, fopt, nacc, nfcnev, nobds, kevent, tfunk,safunc1,-1.0)   
      elseif (gopti%algo == SA_GLOBAL) then
          !call sa_glob(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safuncg)
          call sa_glob_old(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safunc1)
      elseif (gopti%algo == HARMONYS) then
          !call hm_search(gpar,fopt,tfunk,safunc1,sc)
      elseif (gopti%algo == BBBC) then
          call bb_bc_algo(gpar,bb_cond,gopti,fopt,tfunk,safunc1,sc,nfcnev,kevent)
      endif
!      
!     Stampa risultati alla fine del run
      write(6, '(/, 25x,"** Results at the end of ",a," Run n.",i0," **")')trim(algoname_short(gopti%algo)),i
      if (gopti%algo /= BBBC) call sa_print_final(fopt,nfcnev,nacc,nobds,tfin,6)      
      call gopt_print_parameters(gpar%val,gpar,atm,6)      
      call stampa_struttura(atm,write_occ=.true.)
      if(nrestraints(res_sa) > 0)then
         call stampa_restraints(lo,res_sa,atm)
!!!!only for test
         resval=chisquared_res(res_sa,atmr,kpr=1)
!!!!only for test
      endif
      if(nrestraints(bump_sa) > 0)then
!!!!only for test
         resval = antibump_function(bump_sa, atm, gopti%doc, .true.)
         call print_antibump(lo,bump_sa,atm)
!!!!only for test
      endif
      if (knw == 1 .or. ModelToCompare /= 0) then
         write(6,*)
         call fcalcan(ref,atm)
         call fillatom(atm)
         call to_coord_ref(1)
!
!        Incrementa nsol se il risultato e una soluzione         
         if (vett_test(3) == vett_test(1)) then
             nsol = nsol + 1               
             distnsol = distnsol + vett_test(4)
         endif
      else
         ref%fc = sqrt(ref%fc) ! rimetti fo in ref%fc
      endif
!
!     Calcola tempo per ogni run      
      call timestep(5)
      call get_timestep(5,time)
      timerun = time%fin - time%ini
!
!     Calcola rapporto Rwp(calc)/Rwp(Le Bail)
          rappo = 0  ! da migliorare
!
!     Ragruppa e calcola occupanza
      atmr = atm
      if (numfrag > 1) then      ! prima raggruppa i frammenti 
          call regroup_fragment(fragment,atmr)
      endif
!corr      if (gopti%doc) then
      if (sadoc%active()) then
          call calcola_occ(atmr,dist=DISTOCC)  
          call search_overlapped_atoms(atmr,vet=sadoc%vet)
!corr          do k=1,numfrag   ! reset occupancy at the end
!corr             atmr(fragment(k)%pos)%och = fragment(k)%atm0%och
!corr          enddo
          call search_overlapped_atoms_print(atmr)
      else
          call calcola_occ(atmr)  ! dist=default (0.6)
      endif
!
!     Aggiungi modello alla lista       
      infomodel = (/fopt,vett_test(3),vett_test(4),timerun,real(i),rappo,rmsd/)
      nsamodel = nsamodel + 1
      call model_set(samodel(nsamodel),atmr,fom=infomodel)
!
!     esporta i modelli ordinati per fom
      call model_sort(samodel(1:nsamodel),1)
      call overlay_models_list(samodel(1:nsamodel),gopti%nref)  ! sovrapponi
      if (index(gopti%popt,'E') > 0) then
          do j=1,nsamodel
             call model_get(samodel(j),atmr,fom=infomodel)
             call export_model(atmr,j,int(infomodel(5)),infomodel(1),spaceg,gopti%algo)
          enddo
      endif
!
!     Aggiorna info sul minimo su tutti i run
      if (fopt < gopti%fmin) then
          gopti%fmin = fopt
          gopti%rmin = i
      endif
!      
!     Write per tabella test
      if(print_tb)write(6,"('TB1',i5,i6,'/',i0,2f10.4,f12.4,1x,i15,1x,a)")   &
       i,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,fopt,nfcnev,trim(string_time(timerun))
      if (kevent == 2) exit  ! evento stop dell'utente
      if (fopt < gopti%sogCF) exit  ! stop se R% < soglia
!
!     recompute the number of evaluation using the previous run
      neval = neval + 1
      sum_eval = sum_eval + nfcnev
      numcyc_calc = sum_eval / neval
   enddo                     ! fine loop sui run di SA
!
   call reallocate_models(samodel,nsamodel)
!
!  Ordina per CF e stampa la lista
   call print_listsa(samodel,6)
   call print_listsa(samodel,0)
!
   if (nrestraints(res_sa) > 0) then
       do i=1,nsamodel  !!!!listsa%nstru
          call model_get(samodel(i),atmr,fom=infomodel)
          atm(:) = atmr(:)
          resval=chisquared_res(res_sa,atmr,kpr=1)
       enddo
   endif
!
!  Associa connettivita' ai modelli
   if (numfrag > 1) then  ! riconnetti i frammenti fra loro
       do i=1,nsamodel
          call model_get(samodel(i),atmr)
          call copia_legm(legm,lconn)
          call connect_fragment(fragment,atmr,legm)
          call model_set(samodel(i),leg=legm)
       enddo
   else
       do i=1,nsamodel
          call model_set(samodel(i),leg=lconn)
       enddo
   endif
!
!  Seleziona il primo modello   
   call model_get(samodel(1),atbest,fom=infomodel)
   fopt = infomodel(1)
   vett_test(3) = infomodel(2)
   vett_test(4) = infomodel(3)
   timerun = infomodel(4)   
   rappo = infomodel(6)
   rmsd = infomodel(7)
!   
   atm(:) = atbest(:)         ! aggiorna il modello
!   
   if(print_tb)then
      frun_min = minval(samodel(:nsamodel)%info(1))
      frun_max = maxval(samodel(:nsamodel)%info(1))
      frun_ave = sum(samodel(:nsamodel)%info(1)) / nsamodel
      nsol8 =  count(samodel(:nsamodel)%info(7) <= 0.08)   ! number of solutions with rmsd <= 0.08
      write(6,"('TB1best ',i6,'/',i0,2f10.4,f12.4,1x,a,' NSOL=',i0,' distm=',f0.3)")   &
            int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,                     &
            fopt,trim(string_time(time%tot)),nsol,distnsol/nsol 
      write(6,"('TB1info: ','CF=',f0.3,'-',f0.3,' CF(ave)=',f0.3,' Nsol_0.08=',i0,' Nsol_0.08 per min=',f0.3)")  &
      frun_min,frun_max,frun_ave,nsol8,60*nsol8/time%tot
   endif
!corr   call cancel_list(listsa)   ! cancella tutta lista perche non serve piu  
   write(6,'(/a,i0,1x,a)')' The structure of run n.',int(infomodel(5)),' was selected:'
   call stampa_struttura(atm,title='Structure solution',write_occ=.true.)
!
   call write_message('Best structure found at run n. ',inum=int(infomodel(5)),pos=2)
   call write_message('Saved in file '//trim(StructureName)//'_best1.cif',pos=3)
   call write_message('CF = ',rnum=fopt,pos=4)
!corr   if (npunti > 0)call write_message('Ratio = ',rnum=rappo,pos=0)
   call write_message('Time: '//string_time(time%tot),pos=0)
   if (knw == 1) call write_message_found(0)
   call write_message("Click on 'Solution' to explore the other structures",pos=0)
!corr   if (gopti%doc) call search_overlapped_atoms_print(atm)
   if (sadoc%active()) call search_overlapped_atoms_print(atm)
!
   if (knw == 1 .or. ModelToCompare /= 0) then   
       call fcalcan(ref,atm)
       call fillatom(atm)
       call to_coord_ref(0)      
   else
       call fillatom(atm)
   endif
!
!  Write per tabella riepilogativa       
   if (print_tb) &
   write(6,100)  &
         '"'//trim(fname(OUT_FILE)(scan(fname(OUT_FILE),'/\',back=.true.)+1:))//'",', &
         gopti%dofe,gopti%dofi,                                           &
         '"',gopti%ttmin,gopti%ttmax,'"',gopti%nref,gopti%res,            &
         sc%nt,'"',nsol,gopti%nrun,'"',                                   &
         '"',int(vett_test(3)),int(vett_test(1)),'"',vett_test(4),rmsd,   &
         fopt,frun_min,frun_max,frun_ave,rappo, ',"'//trim(string_time(time%tot))//'"'
 100 format('TB2',a,i0,',',i0,',',a,f0.2,'-',f0.2,a,',',i0,',',f0.2,',',  &
      i0,',',a, i0,'/',i0,a,',',a,i0,'/',i0,a,7(',',f0.3),a)
!   
   end subroutine simann
         
!----------------------------------------------------------------------------------------------------

   subroutine set_bb_bc_condition(cond,sc)
   USE bb_bc
   type(bb_bc_condition), intent(out)                    :: cond
   type(SimAnn_Conditions_type), intent(inout), optional :: sc
!
   if (cond%auto) then
       select case(numrot)
          case (0)
            cond%niter = 5
          case (1:4)
            cond%niter = 20
          case (5:7)
            cond%niter = 30
          case (8:9)
            cond%niter = 40
          case (10:)
            cond%niter = 80
      
       end select
       cond%niter = cond%niter + nint(0.5*(numfrag - 1))*cond%niter
   endif
!
!  condizioni per SA all'interno del BB-BC
   if (present(sc)) then
       sc%num_temps = 5                  ! comm. per nafeso4f
       sc%nt = sc%nt / 10
       sc%temp_fact = sc%temp_fact / 4   ! comm. per nafeso4f
       sc%t_ini = 0.40*sc%t_ini 
       sc%eps = 10                       ! comm. per nafeso4f
       sc%ns = sc%ns
   endif
!
   end subroutine set_bb_bc_condition

!----------------------------------------------------------------------------------------------------

   subroutine safunc(xpar,h,hpar)
   implicit none
   real , intent(in)   :: xpar(:)
   real , intent(out)  :: h
   integer, intent(in) :: hpar
!   
   h = tfunk(xpar,hpar)
!
   end subroutine safunc

!----------------------------------------------------------------------------------------------------

   subroutine safunc_temp(xpar,h,hpar,gopti)
   USE variables
   implicit none
   real , intent(in)   :: xpar(:)
   real , intent(out)  :: h
   integer, intent(in) :: hpar
   type(opti_info_type), intent(in) :: gopti    
!
   call xpar_to_atm(xpar,sadoc%active(),atm,ref)
   h = chisquared_xdata(hpar,gopti,atm,ref)
!
   end subroutine safunc_temp

!----------------------------------------------------------------------------------------------------        

   real function safuncg(x)
   real, dimension(:), intent(in) :: x
   safuncg = tfunk(x,0)
   end function safuncg

!----------------------------------------------------------------------------------------------------        
  
   subroutine safunc1(xp,fval,grad,icod)
   USE lsqalgo
!
!  Funzione che per  icod = 1  valuta la funzione costo
!                    icod = 2  valuta il gradiente
   real, dimension(:), intent(in)  :: xp
   real, intent(out)               :: fval
   real, dimension(:), intent(out), optional :: grad
   integer, intent(in)             :: icod
!
   select case(icod)
      case (1)
        fval = tfunk(xp,0)

      case (2)
        call calc_gradient(grad,xp,0.0,safuncg,icod=2)

   end select

!   CONTAINS
! 
!   real function funcgrad(x)
!   real, dimension(:), intent(in) :: x
!   funcgrad = tfunk(x,0)
!   end function funcgrad
!!
   end subroutine safunc1

!----------------------------------------------------------------------------------------------------        

   logical function is_profile_function(cftype) result(lval)
   integer, intent(in) :: cftype
!
   select case(cftype)

      case(CHI2PROF_NEW,CHI2PROF)
           lval = .true.
      case(CHI2FACT,CHI2I,CHI2I_SD,CHI2I_NEW)
           lval = .false.
      case default 
           lval = .false.
   end select
!
   end function is_profile_function

!----------------------------------------------------------------------------------------------------        
   subroutine print_listsa(model,unitp)
!
!  Stampa riepilogativa della lista dei modelli
!   
   USE calctime
   USE TestModel, only:vett_test
   USE General, only:knw,StructureName
   USE Dialogo, only:ModelToCompare   
   type(model_type), dimension(:), intent(in) :: model
   integer, intent(in)                  :: unitp
   integer                              :: i
   character(len=10)                    :: snum
   integer                              :: nmodel
!
   nmodel = size(model)
   write(unitp,'(/3x,90("-"),/a)')'           List of structural models sorted according to cost function '
   if (knw == 1 .or. ModelToCompare /= 0) then
       write(unitp,'(a/,3x,90("-"))') &
   '      RUN        CF                Ratio       found        dist    RMSD    time         saved on'
       do i=1,nmodel
          write(snum,'(i0)')i
          write(unitp,'(2x,i5,2x,f15.6,f15.3,5x,i5,"/",i0,t55,2f10.4,2x,a,4x,a)')   &
          int(model(i)%info(5)),model(i)%info(1),model(i)%info(6),   &
          int(model(i)%info(2)),int(vett_test(1)),  &
          model(i)%info(3),model(i)%info(7),trim(string_time(model(i)%info(4))),    &
          trim(StructureName)//'_best'//trim(snum)// '.cif'
       enddo 
   else
       write(unitp,'(a/,3x,90("-"))') '      RUN        CF                Ratio         time         saved on'
       do i=1,nmodel
          write(snum,'(i0)')i
          write(unitp,'(2x,i5,2x,f15.6,f15.3,8x,a,4x,a)')   &
          int(model(i)%info(5)),model(i)%info(1),model(i)%info(6),    &
          trim(string_time(model(i)%info(4))),trim(StructureName)//'_best'//trim(snum)// '.cif'
       enddo 
   endif
 !
   end subroutine print_listsa
   
!----------------------------------------------------------------------------------------------------

   subroutine overlay_models_list(model,numref)
!
!  Fa in modo che i modelli possano essere sovrapponibili
!
   USE compare_struct
   USE factor
   USE variables, only:ref,lconn
   USE PhaseModule, only:shiftexam
   USE model_util
   USE proginterface
   type(model_type), dimension(:), intent(inout) :: model
   integer, intent(in)                           :: numref
   integer                                       :: i   
   type(atom_type), allocatable, dimension(:)    :: atm1,atm2
   type(reflection_type), dimension(numref)      :: ref1,ref2
   integer                                       :: nmodel
!
   nmodel = size(model)
   if (nmodel <= 1) return
!
   do i=1,numref
      ref1(i) = ref(i)
      ref2(i) = ref(i)
   enddo
!
!  Prendi il primo modello della lista
   call model_get(model(1),atm1)
   call fcalcan(ref1,atm1)
!
!  Sovrapponi gli altri modelli al primo
   do i=2,nmodel
      call model_get(model(i),atm2)
      call fcalcan(ref2,atm2)
      call examing(ref1,ref2)
      call drawup_models(atm1,lconn,atm2,lconn,shiftexam,kfix=.true.,kenant=.false.,kinorg=.false.)
      call model_set(model(i),atm2)
   enddo
!
   end subroutine overlay_models_list

!----------------------------------------------------------------------------------------------------

   subroutine reallocate_shift(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo fragment_type
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(shift_atom_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)              :: n
   logical, optional, intent(in)    :: savevet
   logical                          :: savev
   integer                          :: nv
   type(shift_atom_type), allocatable                :: vsav(:)
   integer                          :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual � la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(nsav))
           vsav = vetr(:nsav)
!
!          dealloca vetr
           deallocate(vetr)
!
!          rialloca vetr su n
           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
           vetr(:nsav) = vsav(:nsav)
       else
           deallocate(vetr)
           allocate(vetr(n))
       endif
   endif
!
   end subroutine reallocate_shift

!----------------------------------------------------------------------------------------------------

   integer function numref_from_res(resval,refl)   result(numr)
!   
!  Calcola il numero di riflessi associati fino risoluzione resval
!
   real, intent(in)                                :: resval
   type(reflection_type), dimension(:), intent(in) :: refl 
   real                                            :: tthrval
   real                                            :: risoluzione
   integer                                         :: i
!   
!gl!   tthrval = tthvalue(resval)
   numr = 0
   do i=1, size(refl)
      risoluzione = 0.5 / sqrt(refl(i)%slaq)
      if (risoluzione > resval) then
          numr = numr + 1
      endif
   enddo
!gl!   numr = count(refl%tthd <= tthrval)
!
   end function numref_from_res

!----------------------------------------------------------------------------------------------------
   
   subroutine export_model(atom,codefile,nrun,cfval,spg,algo)
   USE General
   USE importexport
   USE spginfom
!
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, intent(in)                                    :: codefile
   integer, intent(in)                                    :: nrun
   real, intent(in)                                       :: cfval
   type(spaceg_type), intent(in)                          :: spg
   integer, intent(in)                                    :: algo
   character(len=250)                                     :: filename
   character(len=30)                                      :: snum,snum1
   character(len=200)                                     :: commline
!
   write(snum,'(i0)')codefile
   filename = trim(StructureName)//'_best'//trim(snum)// '.cif'
   write(snum,'(f15.3)')cfval
   write(snum1,'(i0)')nrun
   commline = trim(algoname_short(algo))//' at run '//trim(snum1)//' CF = '//trim(adjustl(snum))
   call export_structure(filename,atom,ftype=CIF_FILE,comm=commline,spg=spg)
!
   end subroutine export_model

   subroutine init_rietveld(codice)
   logical codice
!- carica atom da xyzs
!-- legge i riflessi
   return
   end subroutine init_rietveld

!----------------------------------------------------------------------------------------------------

   subroutine set_doc(doc,atcode)
   USE realloc
   type(doc_type), intent(inout)     :: doc
   integer, dimension(:), intent(in) :: atcode
   integer                           :: ndoc
   integer                           :: nat
   integer                           :: i
   ndoc = count(atcode == 1)
   nat = size(atcode)
   if (ndoc == 0) then             ! DOC non active
       doc%dtype = 0
       call reallocate(doc%vet,0)
   else
       call reallocate(doc%vet,ndoc)
       if (ndoc == nat) then  ! DOC active on all atoms
           doc%dtype = 1
           doc%vet = [(i,i=1,nat)]
       else                        ! DOC active on selected atoms
           doc%dtype = 2
           doc%vet = pack([(i,i=1,nat)],mask=atcode == 1)
       endif
   endif
   end subroutine set_doc

!----------------------------------------------------------------------------------------------------

   logical function doc_active(doc) 
   class(doc_type), intent(in) :: doc
   doc_active = (doc%dtype > 0) 
   end function doc_active

!----------------------------------------------------------------------------------------------------

   logical function doc_all_atoms(doc)
   class(doc_type), intent(in) :: doc
   doc_all_atoms = (doc%dtype == 1)
   end function doc_all_atoms

!----------------------------------------------------------------------------------------------------

   subroutine doc_print(doc,kpr,atom)
   USE atom_basic
   class(doc_type), intent(in)               :: doc
   integer, intent(in)                       :: kpr
   type(atom_type), dimension(:), intent(in) :: atom
   integer, parameter                        :: MAXLENDOC = 60
   integer                                   :: natdoc
   if (.not.doc%active()) return
   if (doc%all_atoms()) then
       write(kpr,'(1x,a)')'Dynamical Occupancy Correction is active on all atoms'
   else
       natdoc = size(doc%vet)
       if (natdoc == 1) then
           write(kpr,'(1x,a,i0,a)')'Dynamical Occupancy Correction is active on selected atom: '//  &
           slabvet(doc%vet,atom%lab)
       else
           write(kpr,'(1x,a,i0,a)')'Dynamical Occupancy Correction is active on ',size(doc%vet),' selected atoms: '//  &
           slabvet(doc%vet,atom%lab,maxlen=MAXLENDOC)
       endif
   endif
   end subroutine doc_print

END MODULE sannel
   
!----------------------------------------------------------------------------------------------------

   subroutine ges_atom_parameters_win(code,atomw,shifta,shiftval)
   USE sannel
   USE variables, only: atm
   USE atom_type_util
   USE Atomi, only: nats
   USE realloc
   integer, intent(in)                       :: code
   real, save                                :: shiftatomval_copy
   type(atom_type), dimension(*), intent(in) :: atomw
   integer, dimension(*), intent(inout)      :: shifta
   real, intent(in)                          :: shiftval
   integer                                   :: i
!
   select case (code)
      case (0)        ! salva i parametri all'apertura della finestra
        call save_atoms(atm)
        call reallocate_shift(shift_atom_copy,nats)
        shift_atom_copy(:) = shift_atom(:)
        shiftatomval_copy = shiftatomval
        !write(0,*)'Atom parameters window: salva parametri'

      case (1)        ! applica modifiche su 'OK'
!
!       aggiorna i parametri
!mr        atm(:) = atomw(:)
        do i=1,nats
           shift_atom(i)%rcod(:) = shifta(i)
           atm(i) = atomw(i)
        enddo
        shiftatomval = shiftval
!
!       disegna il nuovo modello
        call fillatom(atm)
        call toMolplo3(0)
        call RotazTot
        call Disegna(1)
!
        call clear_saved_atoms()
        call reallocate_shift(shift_atom_copy,0)
        !write(0,*)'Atom parameters window: applica modifiche'
        
      case (2)        ! rispristina parametri su 'Cancel'
        call restore_atoms(atm,nats)
        call clear_saved_atoms()
        shift_atom(:) = shift_atom_copy(:)
        call reallocate_shift(shift_atom_copy,0)
        shiftatomval = shiftatomval_copy
        do i=1,nats
           if (count(shift_atom(i)%rcod > 0) > 0) then
               shifta(i) = 1
           else
               shifta(i) = 0
           endif
        enddo
!
!       disegna il modello ripristinato
        call fillatom(atm)
        call toMolplo3(0)
        call RotazTot
        call Disegna(1)
        !write(0,*)'Atom parameters window: ripristina parametri'

   end select
!
   end subroutine ges_atom_parameters_win

!!mr     END MODULE sannel
   
!----------------------------------------------------------------------------------------------------

   subroutine get_info_for_gtklist(nrow,nrcol,cfcol,strcol)
   USE sannel
!corr   USE General, only:fname
   USE General, only:StructureName
   USE model_util
   implicit none
   integer, intent(out)                                     :: nrow    ! numero di righe della gtklist
   integer, dimension(*), intent(out)                       :: nrcol   ! colonna 'Run nr.'
   real, dimension(*), intent(out)                          :: cfcol   ! colonna 'Cost Function'
   character(len=*) , intent(out)                           :: strcol  ! nome file
   integer                                                  :: i
   real, dimension(5)                                       :: infolist
!corr   integer                                                  :: nomel
!   
   nrow = nsamodel 
!   
!corr   nomel = index(fname(1),'.bin')
!corr   strcol = fname(1)(1:nomel-1) // char(0)
   strcol = trim(StructureName)//char(0)
   do i=1,nsamodel  
      call model_get(samodel(i),fom=infolist)
      nrcol(i) = nint(infolist(5))
      cfcol(i) = infolist(1)
   enddo
!
   end subroutine get_info_for_gtklist

!----------------------------------------------------------------------------------------------------

   subroutine ShowSolution(n)
   USE sannel
   USE variables
   USE ilmviewmod
   USE factor
   USE messagemod
   USE conteggi, only:npunti
   USE strutil
   USE General, only:StructureName,knw
   USE TestModel, only:vett_test, rmsd
   USE model_util
   USE Dialogo, only:ModelToCompare   
   implicit none
   integer                                    :: n
   real, dimension(7)                         :: infomodel
   type(bond_type), dimension(:), allocatable :: legm
!   
!  Seleziona il modello n dalla lista
   call model_get(samodel(n),atm,leg=legm,fom=infomodel)
!
!  Disegna il modello
   call ilmview_open()
   call ilmview_draw(atm,legm,resize=.false.)   !!mr 080214
!
   call clear_messages(ipos=2)
   if (n == 1) then
       call write_message('The best structure (trial n.'//trim(i_to_s(int(infomodel(5))))//') was selected',pos=2)
   else
       call write_message('The structure of trial n.'//trim(i_to_s(int(infomodel(5))))//' was selected',pos=2)
   endif
   call write_message('Saved in file '//trim(StructureName)//'_best'//trim(i_to_s(n))//'.cif',pos=3)
   call write_message('CF = ',rnum=infomodel(1),pos=4)
   if (npunti > 0)call write_message('Ratio = ',rnum=infomodel(6),pos=0)
   if (knw==1 .or. ModelToCompare /= 0) then
       vett_test(3) = infomodel(2)           ! trovati
       vett_test(4) = infomodel(3)           ! distanza
       rmsd = infomodel(7)
       call write_message_found1(0)
   endif
!   
   end subroutine ShowSolution

!----------------------------------------------------------------------------------------------------

   subroutine SaveSolution(n)
   USE variables
   USE atom_type_util
   USE sannel
!
   integer       ::n
   call SetToggleVal('SymmOn'//char(0), 0)
   write(6,'("Solution n. ",i0," was selected using Solutions menu")')n
   call stampa_struttura(atm,title='Structure solution',write_occ=.true.)
   call search_overlapped_atoms_print(atm)
   end subroutine SaveSolution

!----------------------------------------------------------------------------------------------------

   subroutine CancelSolution(n)
!
   integer       ::n
   call SetToggleVal('SymmOn'//char(0), 0)
   call ShowSolution(n)
   end subroutine CancelSolution

!----------------------------------------------------------------------------------------------------

   subroutine fcnprint(gopti,popt,xopt,fopt,temp,nftot,nmtemp)
!
!  Stampa informazioni alla fine di ogni step di temperatura
!  Opzioni di output: S = stampa parametri
!                     P = draw pattern su viewer principale
!                     M = draw molecule su viewer principale
!                     V = draw on secondary viewer
!                     B = manage status and progress bar
   USE variables, only:atm,ref 
   USE sannel
   USE simulated_anneal
   USE General, only:knw
   USE TestModel
   USE factor
   USE calctime
   USE compare_struct
   USE atom_type_util
   USE grexpomod
   USE Molcom
   USE messagemod
   USE strutil
   USE timer
                                                  USE anti_bump
!                
   implicit none
   type(opti_info_type), intent(in)           :: gopti    
   character(len=*), intent(in)               :: popt     ! stringa opzioni di stampa
   real, dimension(:), intent(in)             :: xopt     ! parametri minimo
   real, intent(in)                           :: fopt     ! cf per parametri minimo
   real, intent(in)                           :: temp     ! temperatura
   integer, intent(in)                        :: nftot    ! num. totale di movimenti
   integer, intent(in)                        :: nmtemp   ! num. di movimenti alla temperatura temp
   integer                                    :: nplot
   real, save                                 :: ratiopt
   character(len=300)                         :: stringa
   type(reflection_type), dimension(size(ref)) :: refc
!new   character(len=:), allocatable              :: stringa
   logical                                    :: is_anti_bump, is_restraints
   real                                       :: totaltime
   real                                       :: fbump,fres
   character(len=3) :: adv
   type(timer_type),save :: t_draw
   logical :: sdraw
!
!  Aggiorna la status e progress bar
   if (index(popt,'B') > 0) then
!
!      aggiorna ratio perche un nuovo minimo e stato trovato
           ratiopt = 0.0
       call gopt_statusbar(nftot,nmtemp)
   endif
!
!  Stampa informazioni alla fine di uno step di temperatura
   if (index(popt,'S') > 0) then    !S=brevi informazioni
       call write_message('Ratio = ',rnum=ratiopt,pos=vmess(LINE_RATIO))
       adv = 'no'
       write(6,'(a,i0)',advance=adv)'Run:',gopti%nrunc
       if (gopti%algo == SA_LOCAL) then
           write(6,'(a,f8.2)',advance=adv)'  Temp:',temp
       endif
       write(6,'(1x,a,f10.3)',advance=adv)trim(gopti%cost_function_name_short)//':',fopt
       is_anti_bump = nrestraints(bump_sa) > 0
       if (is_anti_bump) then
!corrr           fbump = bumpset%weight*antibump_function(bump_sa, atm, gopti%doc, .false.)   !!!! FIXME
           fbump = bumpset%weight*antibump_function(bump_sa, atm, sadoc%active(), .false.)   !!!! FIXME
           write(6,'(a,f10.3)',advance=adv)' A-bumpCF:',fbump
       endif
       is_restraints = nrestraints(res_sa) > 0
       if (is_restraints) then
           fres = chisquared_res(res_sa,atm,kpr=0) 
           write(6,'(a,f10.3)',advance=adv)' CF-res:',fres
       endif
       if (knw==1 .or. ModelToCompare /= 0) then
!   
!          Passa xopt in atm
           call xpar_to_atm(xopt,sadoc%active(),atm,ref)
           totaltime = get_totaltime()
!
!          calcola fasi senza ricoprire ref
           refc(:) = ref(:)  
           call fcalcan(refc,atm)
           call fillatom(atm)
           ref(:)%ph = refc(:)%ph
           call to_coord_ref(-1)
           stringa = ' (found: '//trim(i_to_s(int(vett_test(3))))//'/'//trim(i_to_s(int(vett_test(1))))//   &
           ' dist: '//trim(r_to_s(vett_test(4),3))//' RMSD: '//trim(r_to_s(rmsd,3))//')'
           call write_message( 'CF = '//trim(r_to_s(fopt,3))//trim(stringa),pos=vmess(LINE_CF))
           !adv = 'yes'
           !write(6,'(a,a10,a,i3,a,i0,a,f0.4,a,f0.4,a,i0)',advance=adv)              &
           write(6,'(a,a10,a,i3,a,i0,a,f0.4,a,f0.4,a,i0)')                          &
             '  Time:',trim(string_time(totaltime)),'  found:',int(vett_test(3)),   &
             '/',int(vett_test(1)),'  dist:',vett_test(4),' RMSD:',rmsd,'  Moves:',nftot
           if (gopti%fmin < fopt) then
               stringa = '(found: '//trim(i_to_s(int(samodel(1)%info(2))))//'/'//trim(i_to_s(int(vett_test(1))))//   &
               ' dist: '//trim(r_to_s(samodel(1)%info(3),3))//' RMSD: '//trim(r_to_s(samodel(1)%info(7),3))//')'
               call write_message('CFmin = '//trim(r_to_s(gopti%fmin,3))//' at run.'    &
                    //trim(i_to_s(gopti%rmin))//' '//trim(stringa),pos=vmess(LINE_CFMIN))
           else
               call write_message('CFmin = '//trim(r_to_s(fopt,3))//' at run.'//trim(i_to_s(gopti%nrunc)),pos=vmess(LINE_CFMIN))
           endif
       else
           call write_message('CF = ',rnum=fopt,pos=vmess(LINE_CF))
           totaltime = get_totaltime()
           write(6,'(a,a10,4x,a,i0)')'  Time:',trim(string_time(totaltime)),'Moves:',nftot
           if (gopti%fmin < fopt) then
               call write_message('CFmin = '//trim(r_to_s(gopti%fmin,3))//' at run.'  &
                    //trim(i_to_s(gopti%rmin)),pos=vmess(LINE_CFMIN))
           else
               call write_message('CFmin = '//trim(r_to_s(fopt,3))//' at run.'//trim(i_to_s(gopti%nrunc)),pos=vmess(LINE_CFMIN))
           endif
       endif   
       if (is_anti_bump) call write_message('Anti-bump CF = ',rnum=fbump,pos=vmess(LINE_CFBUMP))
       if (is_restraints) call write_message('Restraints CF = ',rnum=fres,pos=vmess(LINE_CFRES))
   endif
!   
!  Output grafico su xopt
   if (index(popt,'M') > 0) then
       sdraw = diff_time(t_draw,0.5)
   else
       sdraw = .false.
   endif
    !if (sdraw) write(0,*)'SDRAW=',sdraw
   call gopt_draw(sdraw,index(popt,'P')>0,gopti)
   !old call gopt_draw(index(popt,'M')>0,index(popt,'P')>0,gopti)
!
!  Output grafico su viewer secondario
   if (index(popt,'V') > 0) then  
       call timestep(4) ! calcola il tempo impiegato dalle operazioni grafiche  
!           
!      Aggiorna plot CF
       call gr_add_point_plot(grplot(gopti%nrunc),real(nftot),fopt)
!
!      Aggiorna linea che segnala Rwp alla fine dell'affinamento
       if(is_profile_function(gopti%cftype)) then
          nplot = size(grplot)
          if (grplot(nplot)%point(2)%x < nftot) grplot(nplot)%point(2)%x = nftot
       endif
!       
!      Se il viewer e' aperto esegui plot
       if (viewerOpened == 1) then  
           call gr_select_viewer(1)
           call gr_clear_area(0.0,1.0,0.0,1.0)
           call gr_area(0.08, 0.99, 0.15, 0.99)
           call gr_set_graph(grgraph,color=grplot(gopti%nrunc)%colorline)
           call gr_draw_graph(grgraph,grplot)
           call zflush()
           call gr_select_viewer(0)
       endif
       call timestep(4) 
   endif
!
   end subroutine fcnprint         
   
!----------------------------------------------------------------------------------------------------   
   
   subroutine sa_restorepar(hpar)
!
!  Ripristina i parametri se il movimento non � accettato
!   
   USE fastfcal
   USE sannel
   USE Atomi, only:nats
   implicit none
   integer, intent(in) :: hpar
   integer             :: i
!
   if (gpar(hpar)%typepar == THERMALF_TYPE) then 
       call fcalc_ripr_thermalf(gpar(hpar)%pat)
   else
       call fcalc_ripr_atm(gpar(hpar)%pat)
       if (sadoc%active()) then
!corr           call fcalc_ripr_thermalf((/(i,i=1,nats)/))
           call fcalc_ripr_thermalf(sadoc%vet)
       endif
   endif
!   
   end subroutine sa_restorepar

!----------------------------------------------------------------------------------------------------      
   subroutine sa_savepar(hpar)
!
!  Salva i parametri se il movimento � accettato
!   
   USE fastfcal
   USE sannel
   USE Atomi, only:nats
   implicit none
   integer, intent(in) :: hpar
   integer             :: i
!
   if (gpar(hpar)%typepar == THERMALF_TYPE) then 
       call fcalc_save_thermalf(gpar(hpar)%pat)
   else
       call fcalc_save_atm(gpar(hpar)%pat)              
       if (ginfo%doc) then
           call fcalc_save_thermalf((/(i,i=1,nats)/))
       endif
   endif    
!   
   end subroutine sa_savepar

!----------------------------------------------------------------------------------------------------
   subroutine sa_savepar_all()
   USE fastfcal
   USE Atomi, only:nats
!
   call fcalc_save_atm((/(i,i=1,nats)/))
!
   end subroutine sa_savepar_all
!----------------------------------------------------------------------------------------------------
   
   subroutine sannelexe()   
!
!  Questa routine e' chiamata quando e' attivo il comando %sannel
!
   USE sannel  
   USE simulated_anneal                
   USE General, only:ifAutomatic
   USE Direttive
   USE molcom, only:jscreen
   USE Atomi, only:nats
   USE errormod
   USE ilmviewmod
   USE bb_bc
   implicit none
   type(SimAnn_Conditions_type) :: sc                     
   type(bb_bc_condition)        :: bcond                     
   integer                      :: irefb
   type(error_type)             :: saerr
!
   if (nats == 0 .and. jscreen > 0) then
       call open_file_selector('Import starting model for Simulated Annealing' //char(0))
   endif
   if (nats == 0) return
!
   if (ifAutomatic == 1 .or. jscreen == 0) then
!
!      Inizializza e legge le direttive associate al comando %sannel 
       call c_fortran_al_lavoro(0)
       call sa_prelim(goptim,sc,bcond,.true.,irefb,saerr=saerr)
       if (saerr%signal) then
           call saerr%print()
       else
!gl!       call apri_toolbar('run')
           call apri_toolbar('run2')
           !call abilitaDrag(1)
           call sa_run(goptim,sc,bcond)
           call exitsannel()
           call RunToolbar(-1,' ')
       endif
       call c_fine_lavoro(0)
   else
       call aprisannelwin()
   endif
!
   end subroutine sannelexe

!----------------------------------------------------------------------------------------------------
   
   subroutine sannelexewin(ierr, resminGr, resmaxGr, info, sc, bcond, ifButton, bumpsetC)   
!
!  Questa routine e' chiamata alla pressione del tasto 'Simulated Annealing'
!
   USE General, only : nrefl
   USE Riflessi, only : iwork1
   USE Atomi, only:nats
   USE sannel  
   USE simulated_anneal                  
   USE variables, only:atm
   USE rotationmod
   USE Molcom, only:ifLabels
   USE General, only:dtor, resmin, resm, Msg, ERR_WINDOW, InfoOK
   USE bb_bc
   USE rrestr
   USE errormod
   implicit none
   type(SimAnn_Conditions_type)        :: sc                     
   type(bb_bc_condition)               :: bcond                     
   integer                             :: irefb, nExitCode
   real*8                              :: resminGr,resmaxGr
   type(opti_info_type), intent(inout) :: info
   integer                             :: nfrag
   integer, intent(in)                 :: ifButton    ! (=1 se arrivo qui da menubar)
   integer, intent(out)                :: ierr
   integer                             :: i
   real                                :: bump_sc
   integer                             :: nrl
   type(bump_settings)                 :: bumpsetC                     
   type(error_type)                    :: saerr
!
   if (InfoOK(6) == 0) then
       Msg = 'No reflections loaded !!!'
       call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
       ierr = -1
       return
   endif
   call ModCom(0)      
!
   if (nats == 0) then
       call open_file_selector('Import starting model for Simulated Annealing' //char(0))
       if (nats == 0) then
           ierr = -1
           return
       endif
   endif
!
   if (ifButton == 0) then
!
!      Sto chiamando SA da comando %sannel nell'input: mi leggo le direttive
       call sa_prelim(goptim,sc,bcond,.true.,irefb,saerr=saerr)
   else
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!      dealloco iwork1 che potrebbe essere stato usato per relax
!      e lo rialloco alla dimensione corretta   ! Gianluca 17/6/2014
       deallocate(iwork1)
       allocate(iwork1(nrefl))
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!      Sto chiamando SA da menubar: non mi leggo le direttive
       call sa_prelim(goptim,sc,bcond,.false.,irefb,saerr=saerr)
   endif
!
   if (saerr%signal) then
       call saerr%print()
       ierr = -1
   else
!
!      Forza il disegno con le labels
       if (IfLabels == 0) then
           call SetToggleVal('LabTog'//char(0), 1)
!           ifLabels = 1
           call Disegna(1)      
       endif      
!
!      Calcola risoluzione minima e massima per la grafica
       resminGr = resm   ! resmin     gianluca 30/11/2010
       resmaxGr = resmin ! resm
       !resmax = resvalue(thmin)
!       
!      Calcola angoli di torsione per la grafica
       !do i=1,numrot   
       !   rotation(i)%torsval = torsion_value_for_rotat(rotation(i),atm)
       !enddo
!
       bump_sc = bumpset%weight
       bumpsetC = bumpset
       nrl = nrestraints(bumptab)
!
       call passa_abres(nrl)
       do i=1, nrl
          call put_info_abres(i-1,bumptab(i)%code,bumptab(i)%targ, &
               size(bumptab(i)%na),bumptab(i)%na)
       end do
!
!      Apre finestra della grafica
       ierr = nats
       info = goptim
       nfrag = numfrag
       call passa_info(atm, fragment%nat, nfrag, numrot, nats, shiftatomval)
       do i = 1, nfrag
          call passa_frag(fragment(i)%pos, fragment(i)%rcod, fragment(i)%tlow, fragment(i)%thigh, fragment(i)%rbound, i-1)
       end do
   endif
!
   end subroutine sannelexewin

!----------------------------------------------------------------------------------------------------

   subroutine passa_rcod(i, c_rcod, c_tlow, c_thigh, c_rbound)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                            
   integer      :: i
   integer      :: c_rcod(6)
   real         :: c_tlow(3)
   real         :: c_thigh(3)
   real         :: c_rbound(2)
!
   fragment(i)%rcod = c_rcod
   fragment(i)%tlow = c_tlow
   fragment(i)%thigh = c_thigh
   fragment(i)%rbound = c_rbound
!                
   end subroutine passa_rcod

!----------------------------------------------------------------------------------------------------

   integer function get_srcod(i, j)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                
   integer      :: i
   integer      :: j
!
   get_srcod = shift_atom(i)%rcod(j)
!
   end function get_srcod

!----------------------------------------------------------------------------------------------------

   integer function get_rrcod(i)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                
   integer      :: i
!
   get_rrcod = rotation(i)%rcod
!
   end function get_rrcod

!----------------------------------------------------------------------------------------------------

   subroutine  get_rators(i, vettore, cod, val, bounds)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                
   integer      :: i, j, cod
   integer      :: vettore(4)
   real         :: val, bounds(2)
!
   do j=1, 4
      vettore(j) = rotation(i)%ators(j)
   end do
   do j=1, 2
      bounds(j) = rotation(i)%trbound(j)
   end do
   cod = rotation(i)%rcod
   val = rotation(i)%torsval
!
   end subroutine get_rators

!----------------------------------------------------------------------------------------------------

   subroutine  put_rators(i, cod, val, bounds)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                
   integer      :: i, j, cod
   real         :: val, bounds(2)
!
   do j=1, 2
      rotation(i)%trbound(j) = bounds(j)
   end do
   rotation(i)%rcod = cod
   rotation(i)%torsval = val
!
   end subroutine put_rators

!----------------------------------------------------------------------------------------------------

   subroutine  put_srcod(i, cod)
!
!
   USE sannel
   USE simulated_anneal
   implicit none
                                                                                                                
   integer      :: i, cod(3), j
!
   do j=1, 3
      shift_atom(i)%rcod(j) = cod(j)
   end do
!
   end subroutine put_srcod

!----------------------------------------------------------------------------------------------------

   subroutine finishsannel(info, sc, bcond, shiftatomv, bumpsetC)
!
!  Routine chiamata dalla grafica alla pressione del tasto 'Execute'
!
   USE sannel
   USE simulated_anneal                  
   USE rotationmod
   USE variables, only:atm
   USE bb_bc
   USE ilmviewmod
   USE anti_bump
   implicit none
   type(SimAnn_Conditions_type)         :: sc                     
   type(bb_bc_condition)                :: bcond                     
   type(opti_info_type), intent(inout)  :: info
   integer                              :: i
   real                                 :: shiftatomv
   type(bump_settings)                  :: bumpsetC    
!
   call jav_set_modify(0)
   goptim = info
   shiftatomval = shiftatomv
   bumpset = bumpsetC     
   if (bumpset%baction > 0) then
       call bump_from_table(bumptab,bump_sa,atm,fragment,bumpset)
   else    ! 'None' was selected
       call reallocate_restr(bump_sa,0)
   endif
   call sa_run(goptim,sc,bcond)
   info = goptim                           !sa_run modifica alcuni campi di gopti
!
!  Calcola nuovi valori per angoli di torsione per la grafica
   do i=1,numrot   
      rotation(i)%torsval = torsion_value_for_rotat(rotation(i),atm)
      !write(0,*)'TORS=',i,rotation(i)%torsval
   enddo
   call jav_set_modify(3)
!
   end subroutine finishsannel

!----------------------------------------------------------------------------------------------------

   subroutine exitsannel()
!
!  Routine chiamata dalla grafica alla pressione del tasto 'Exit'
!
   USE General, only: nrefl, Msg
   USE Atomi, only: nz, nzSav, nzConst
   USE Molcom, only:ifLabels, jscreen
   USE PhaseModule, only: erre
   USE sannel
   USE realloc
   USE lsqalgo
   USE model_util
!                   USE Atomi, only:iconn
!                   USE Molcom, only : nLegInd
!gl!   USE counts
!gl!   USE RefinecomRef, only:thmax,rfcond,iFirstRietveld    
!gl!   USE RIETV
   USE variables, only:ref,atm
!gl!   USE exportfiles
   USE ilmviewmod
   USE atom_type_util
   implicit none
   integer :: ifBreak, L
   real    :: scalan
!
!  dealloca parametri
!! ifort -O3 ver 11.1.069 da errore
   if (allocated(gpar)) deallocate(gpar)
!
!  Ripristina riflessi
   call reallocate(ref, nrefl)
   ref = refi
!
!  dealloca frammenti e rotazioni
   call reallocate_fragment(fragment,0)   
   call reallocate_rotation(rotation,0)
!
!  dealloca i plot
   if (allocated(grplot)) deallocate(grplot)
   if (jscreen > 0) then
       call clear_plot(grplot,1)
       call gr_close_viewer()
       call jav_set_modify(1)
       call ilmview_close()
   endif
!
!  Sposta le informazioni sui modelli in ambiente Sir
   call FromSAtoSir()
!
!  Salva il modello corrente
   call fillatom(atm)
   call PutModel
   call toMolplo3(0)
   call RotazTot
   call Disegna(1)
!
!  Calcola le fasi
         nzSav(:) = nz(:)
        if (iabs(nz(1)) > 100) then
             nz(:) = nz(:) / nzConst
         endif
         ifBreak = 0
         call sftryyyc(0, 0, nrefl, scalan, ifBreak, 2)
         nz(:) = nzSav(:)
!
!-- ripristina la finestra intera
!
         call StartStopMsg(2)
         write(Msg,10) erre*100, nrefl
   10    format('Final R value = ',f6.2,'% using all (',i9,' ) reflections')
         call OneBlankOnly(Msg)
         call shift(Msg,L)
         call PrintMess(Msg(1:L))
!
!       write(0,*) 'iconn(3)',iconn(1),nLegInd
!
!  ripristina labels come scelto dall'utente
   if (ifLabels /= ifLabelsSav) then
       call SetToggleVal('LabTog'//char(0), ifLabelsSav)
       call Disegna(1)
   endif
!
!  da sistemare: se uso la simmetria iconn e LE non sono allineati
   !call GetBonds(1)
!
!gl!   call export_files()
!       write(0,*) 'iconn(4)',iconn(1),nLegInd
!
   end subroutine exitsannel

!----------------------------------------------------------------------------------------------------

   subroutine evaluate_res(res,numref,tthmaxr,ier)
!
!  Data la risoluzione calcola il numero di riflessi e il 2theta massimo
!
   USE General, only: resmin, resm
   USE sannel, only:ginfo,refi
!
   implicit none
   real, intent(inout)  :: res      ! risoluzione
   integer, intent(out) :: numref   ! numero di riflessi
   real, intent(out)    :: tthmaxr  ! 2theta massimo
   integer              :: ier      ! >0 se res � inferiore al valore minimo
   real                 :: rhomin, rhomax, rhos
!
   rhomin = ( 0.5 / resm) ** 2
   rhomax = ( 0.5 / resmin) ** 2
   rhos   = ( 0.5 / res) ** 2
!
   tthmaxr = 0.0
   !gl!if (res < resmin) then
   if (res < resm) then
       numref = ginfo%nreftot
       ier = 1   ! segnala che res � fuori range
   else
       numref = count(refi%slaq <= rhos  .and. refi%slaq >= rhomax)   
       ier = 0
   endif
!
   end subroutine evaluate_res

!----------------------------------------------------------------------------------------------------

   subroutine check_resp(resval,nparam)    ! da ripristinare
!   
!  Controlla che la risoluzione  sia compatibile col numero di parametri
!
   USE sannel
   USE General, only:nrefl
   real, intent(inout) :: resval
   integer, intent(in) :: nparam
   integer             :: nrefpar
   integer             :: nrefval
!   
!!!   resval = 2.0
!   
!  Calcola il numero di riflessi associati alla risoluzione 
   nrefval = numref_from_res(resval,refi)
   if (nrefval < nrefl) then
!
!      Numero minimo di riflessi in base al numero di paramatri
       nrefpar = nparam * 7
       if (nrefpar > nrefl) nrefpar = nrefl
!   
       if (nrefval < nrefpar) then
           resval = 0.5 / sqrt(refi(nrefpar)%slaq)    
       endif           
   endif
!
   end subroutine check_resp

!----------------------------------------------------------------------------------------------------

   subroutine gestione_eventi(kevent)
   USE Molcom
   USE grexpomod
   USE ilmviewmod
   implicit none
   integer, intent(out) :: kevent ! se > 0 l'informazione va passata all'esterno
   integer              :: xsize,ysize
!
   if (jscreen > 0 .and. ifBrekke /= 0) then
       select case (ifBrekke)
          case (1)    ! tasto skip
            kevent = ifBrekke
          case (2)    ! tasto stop
            kevent = ifBrekke
          case (3)    ! tasto zoom + sul viewer secondario
            kevent = 0
            !call OnZoom_event(1)
          case (4)    ! tasto zoom - sul viewer secondario
            kevent = 0
            !call OnZoom_event(-1)
          case (5)    ! tasto annulla zoom sul viewer secondario
            kevent = 0
            !call OnZoom_event(0)
          case (6)    ! resize sul viewer secondario
            kevent = 0
            call new_viewer_size(xsize,ysize)
            call OnResize_event(xsize, ysize)
          case (7)    ! open viewer secondario
            kevent = 0
            !call OnOpen_event()
          case (8)    ! close viewer secondario
            kevent = 0
            !call gr_close_viewer()
          case (9)   
            kevent = 0
            call DoSingleIteration()

          case (11)  ! show structure
            kevent = 0
            call ilmview_open()

          case default
             kevent = 0
       end select
       ifBrekke = 0
   else
       kevent = 0
   endif
!
   end subroutine gestione_eventi

!------------------------------------------------------------------
!aggiunta 14/01/14
   subroutine Passa_Info_Bumptab(i,targ,code)
   USE sannel
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: i
   real, intent(in)                 :: targ
!
   bumptab(i)%code  = code
   bumptab(i)%targ  = targ
!
   end subroutine Passa_Info_Bumptab

!----------------------------------------------------------------------

   subroutine randomize_parameters()
!
!  'Randomize button'
!
   USE sannel
   USE fragmentmod
   USE variables
   USE ilmviewmod
   USE General, only:knw,cell
   USE factor
   USE compare_struct
   USE unit_cell
   implicit none
   integer :: i
!
   call randomize_molecule(atm,lconn,set_cell_type(cell),fragment,rotation)  ! randomize atm
   call set_atm_in_frag(fragment,atm)                    ! update fragment info
   do i=1,numrot                                         ! update torsion value
      rotation(i)%torsval = torsion_value_for_rotat(rotation(i),atm)
   enddo
   call ilmview_draw(atm,lconn)                          ! update graphic view
   if (knw > 0) then
       call fcalcan(ref,atm)
       call to_coord_ref(-1)
       call write_message_found1(5)
   endif
!
   end subroutine randomize_parameters

!----------------------------------------------------------------------
