MODULE atom_type_util
!
! Questo modulo contiene un insieme di routine per gestire operazioni con variabili di tipo atom_type
!
! F numatomtype                                            numero di atomi
! F atomic_number(atom)                                    restituisce il numero atomico
! F numatomspec(atom,zval or spec)                         numero di atomi di una data specie
! F number_for_specie(z,zval,nzval) result(num)            Numero di specie con Z=z
! S get_atoms_of_specie(pxen,atom,vat,natom)               Extract atoms of the same specie in a array
! F atomic_specie(atom) result(spec)                       estrai la specie atomica
! F is_atomic_specie(atom,spec)                            atom è di tipo spec?
! F is_hydrogen(atom)
! F number_of_hydrogens(atom)
! F nz_from_znumber(z)   result(nzz)                       converte il numero atomico z in nz
! F nz_from_pxen(zxen,elem)  result(nzz)                  Convert pointer to table of elements to nz
! F pxen_from_nz(nzval) result(pxen)                       estrai il puntatore al file xen dall'nz
! S elements_from_atom(atom)                               Extract elements from atoms
! F is_present_sd(atom)                                    Check if atom has sd on coordinates
! S formula_from_atom(atom,elem)                           Extract formula from atom
! S xyz_to_atom(atom,xco,bf,occr,occh,nz,label,inte)       carica modello strutturale in atom
! S frac_to_cart(at)                                       conversione di atomi atm da coordinate frazionarie a cartesiane
! S frac_to_cart_copy(atom,atomcart,orto)                  converti atom in coordinate cartesiane copiandolo in atomcart
! F cartesian_coord(atom) result(atomc)                    conversione da cristallografiche a cartesiane
! S cart_to_frac(at)                                       conversione di atomi atm da coordinate cartesiane a frazionarie
! S translate_atoms(atmnew,xtra)                           trasla atmold in atmnew
! S fillatom(atom)                                         trasferisci atom in variabili expo 
! S atom_to_xyzs(atom)                                     trasferisci atom e nats in variabili expo
! S legm_to_tors(legm,atom,tors,cart)                      genera torsioni
! S add_atoms_to_list(atom,atomadd,natnew,icon1,icon2)     combina 2 modelli atomici
! S add_atom(atom,atomadd,legm,leg)                        aggiungi un atomo alla lista
! S combine_iconn(icontot,icon1,icon2,nat1)                combina 2 connettivita
! S calcola_occ(atom,modcoor)                              calcola occ. per atom
! S set_biso(atom)                                         assegna biso ad atom
! S set_intensity(atom)                                    assegna intensita' ragionevoli agli atomi 
! S set_specie(kat,atom,kel,elem)                          Assign element kel of array at atom kat
! S stampa_struttura(at,height,title,write_occ)            stampa atom
! S save_structure_bin(unitbin,atom,legm)                  save structure model on binary file
! S read_structure_bin(unitbin,atom,legm)                  read structure model from binary file
! S applica_sym_oper(k,atom)                               applica operatore k ad atom 
! S set_atoms(atom,elem,nsymt,wave)                        set atoms according to content in elem
! F atom_symm(k,atom)                                      applica operatore k ad atom
! S create_tetrahedron(atom,dist,icon,ortoi,xpos,kcentri)  crea un tetraedro in coordinate cartesiane
! S create_octahedron (atom,dist,icon,ortoi,xpos,kcentri)  crea un ottaedro in coordinate cartesiane
! S create_square (atom,dist,icon,ortoi,xpos,kcentri)      crea un geometria quadrato planare
! S create_connectivity_special(atom,dist,legm,esd,z1,z2)  cerca legami fra atomi di specie z1 e z2 a distanza dist
! S check_angle(atom,legm,angl,kpr)                        rimuove angoli minori di angl
! S apply_symmetry(atom,atoms,icon,icons,legm,legms)       applica la simmetria 
! S translate_in_cell(atom)                                trasla in cella in gruppo di atomi
! F is_bar_out_of_cell(atom)                               controlla se il baricentro e' fuori cella
! F is_out_of_cell(atom)                                   controlla se tutti gli atomi sono fuori cella
! F is_out_of_range(atom,inf,sup,bar)                      Atoms fuori dal range?
! F baricentro(at)                                         calcola il baricentro di at
! S get_info_distance(atom1,atom2,pos,dist,errd)           Info sulla distanza tra 2 atomi
! S get_radius_molecule(atom,radmax,bar)                   Assimila la molecola ad una sfera e calcola il suo raggio
! S save_atoms(atoms)                                      genera copia degli atomi
! S restore_atoms(atoms)                                   ripristina copia degli atomi
! S clear_saved_atoms()                                    cancella la copia degli atomi
! S copia_atoms(atom1,atom2)                               Copia atom2 in atom1
! S remove_atoms_from_list(atom,veta,val,icon,legm,iord)   Cancella dalla lista tutti gli atomi per i quali vet e' uguale a val
! S remove_atoms_sym(atom,atoms,legm,legms,symm,veta,iord) Rimuovi gli atomi in vet con gli equivalenti
! S remove_atoms_vet(atom,legm,veta,iorder,keep)           Remove atoms in array veta. if keep is true only atoms in vet are kept
! F integer function is_organic(zval,nzval)                Riconosce se un composto è organico
! S sort_by_specie(atom,legm)                              Ordine atomi in base alla specie
! S force_connectivity(atom,legm,kat)                      Forza la connettivita' per l'atomo kat
! S connect_groups(atom1,atom2,legm)                       Cerca connettivita' tra 2 gruppi di atomi
! S renumber_atoms(atom,veta,string,iser)                  Renumber of atoms
! S get_minimum_distance(at,atom,kmin,dmin)                Distanza minima di at dagli atomi in atom in cartesiane
! S get_atoms_distance(at,atom,vetd,distv,vexcl)           Calcola distanza di at dagli atomi atom (in coord. cartesiane)
! S get_distance_radius                                    Trova gli atomi a distanza dist da kat
! S atom_string(atom,vet)                                  Crea stringa degli atomi dalla specie chimica
! S add_to_content(zvs,nvs)                                Aggiungi nuove specie al contenuto
! F is_in_content(zspec)                                   La specie chimica con numero atomico zspec e' nel contenuto?
! S apply_symmetry_legm(atoms,infos,legm,legms)            Applica la simmetria ai legami usando infos
! S expand_contacts(atomasym,atom,infos,legm)              Espandi i contatti di atom
! S duplicate_atoms(atom,legm,vet)                         Duplicate selected atoms with random traslation
! F operator_code(kop,tra) result(opcode)                  Genara codice per memorizzare la rotazione e la traslazione
! F tra_opcode(opcode)                                     Estrai la traslazione dal codice
! S get_limit_translation(opcode,ktramin,ktramax)          Individua la min e la max traslazione
! S init_infos(infosym)                                    inizializza informazioni sulla simmetria
! S init_for_symm(atom,legm,atoms,legms,infos)             Inizialize symmetry by copying asymmetric unit
! S find_sssr(atom,legm)                                   Find SSSR from a connection table. 
! F ring_planarity(atom,ring)                              Determine wheter ring is planar or not.
! F ring_planarity_estimate(atom,ring) result(rps)         Estimate the planarity of ring
! F function density_value(mw,vcell,nop)                   Compute density
! F linear_abs_coeff(atom,cell,nop)   result(mu)           Compute the linear absorption coefficient
! S change_bond_distance(atom,legm,k,dist,xgg)             Change bond distance of bond k
! S change_bond_angle(k,angv,atom,legm,aval,cell)          Change bond angle of angle k
! S change_torsion_angle(tors,atom,legm,tval,cell,changed) Rotate torsion angle
! F norm_torsion(ang)                                      Normalize torsion angle between -180 and 180
! S copy_atoms_sym(atoms,legms,symm,kop,symop,tcell,...)   Copy atoms applying the sym operator symop and tcell. Duplicate atoms are not removed
! S make_random(atom)                                      Make random coordinates

USE progtype, only: symminfo_type
USE atom_basic    !!!!!, only: atom_type, SIZELAB

implicit none

type chem_group_type
  integer               :: geom  ! number of atoms bond to center
  integer               :: zc    ! Z of center
  integer, dimension(4) :: za    ! Z of other atoms
  integer, dimension(4) :: btype ! bond type
  integer, dimension(4) :: cval  ! max number of bond for other atoms
end type chem_group_type

private is_organic_z, is_organic_atm

type(atom_type), dimension(:), allocatable :: atoms_copy

interface atom_symm
  module procedure atom_symm_scalar, atom_symm_scalar_xyz, atom_symm_vet
end interface

interface numatomspec
  module procedure numatomspec_s, numatomspec_z
end interface

interface is_atomic_specie
  module procedure is_atomic_specie_ss, is_atomic_specie_sv,     &
                   is_atomic_speciez_ss, is_atomic_speciez_vs, is_atomic_speciez_vv
end interface

interface operator(==)
  module procedure equal_atom
end interface

interface operator(+)
  module procedure add_xyz
end interface

interface bond_distance
  module procedure bond_distanceZ, bond_distanceAT
end interface

interface volume_per_atom
  module procedure volume_per_atom_cont, volume_per_atom_at
end interface 

interface is_organic
  module procedure is_organic_z, is_organic_atm
end interface 

private :: is_hydrogen_s, is_hydrogen_v
interface is_hydrogen
  module procedure is_hydrogen_s, is_hydrogen_v
end interface 

private :: fractional_coord_a, fractional_coord_a_o
interface fractional_coord
  module procedure fractional_coord_a, fractional_coord_a_o
end interface

private :: cartesian_coord_a, cartesian_coord_a_o
interface cartesian_coord
  module procedure cartesian_coord_a, cartesian_coord_a_o
end interface

private :: applica_sym_oper_kmat, applica_sym_oper_s, applica_sym_oper_s_scalar
interface applica_sym_oper
  module procedure applica_sym_oper_kmat, applica_sym_oper_s, applica_sym_oper_s_scalar
end interface

private :: get_atoms_from_string_lvet, get_atoms_from_string_ivet
interface get_atoms_from_string
  module procedure get_atoms_from_string_lvet, get_atoms_from_string_ivet
end interface

private :: compute_doc_all, compute_doc_vet
interface compute_doc
   module procedure compute_doc_all, compute_doc_vet
end interface

CONTAINS

   logical function equal_atom(atom1,atom2)
   type(atom_type), intent(in) :: atom1,atom2
   real, parameter             :: EPS = epsilon(1.0)
   equal_atom = all(abs(atom1%xc - atom2%xc) <= EPS)
   end function equal_atom

!----------------------------------------------------------------------

   function add_xyz(atom,xyz)
   type(atom_type), intent(in)    :: atom
   real, dimension(3), intent(in) :: xyz
   type(atom_type)                :: add_xyz
   add_xyz%xc = atom%xc + xyz
   end function add_xyz

!----------------------------------------------------------------------

   integer function numatomtype(atom)
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
!   
   if (allocated(atom)) then
       numatomtype = size(atom)
   else
       numatomtype = 0
   endif
!
   end function numatomtype

!----------------------------------------------------------------------

   integer function numatomspec_z(atom,zval)   result(nums)
!
!  Numero di atomi con Z = zval
!
   type(atom_type), dimension(:), allocatable :: atom
   integer, intent(in)                        :: zval
   if (numatomtype(atom) > 0) then
       nums = count(atomic_number(atom) == zval)
   else
       nums = 0
   endif
!
   end function numatomspec_z

!----------------------------------------------------------------------

   integer function numatomspec_s(atom,spec)   result(nums)
!
!  Numero di atomi di tipo spec
!
   type(atom_type), dimension(:), allocatable :: atom
   character(len=*), intent(in)               :: spec
   integer                                    :: nat
   integer                                    :: i
!
   nat = numatomtype(atom)
   if (nat > 0) then
       nums = 0
       do i=1,nat
          if (is_atomic_specie(atom(i),spec)) then
              nums = nums + 1
          endif
       enddo
   else
       nums = 0
   endif
!
   end function numatomspec_s

!----------------------------------------------------------------------

   integer function number_for_specie(z,zval,nzval) result(num)
!
!  Numero di specie con Z=z
!
   integer, intent(in)               :: z ! specie di cui si vuole conoscere il numero
   integer, dimension(:), intent(in) :: zval,nzval
   integer                           :: i
!
   num = 0
   do i=1,size(zval)
      if (zval(i) == z) then
          num = nzval(i)
          exit
      endif
   enddo
!
   end function number_for_specie

!---------------------------------------------------------------------      

   subroutine get_atoms_of_specie(pxen,atom,vat,natom)
!
!  Extract atoms of the same specie in a array
!
   integer, intent(in)                         :: pxen   ! specie
   type(atom_type), dimension(:), intent(in)   :: atom   ! atoms
   integer, dimension(size(atom)), intent(out) :: vat    ! array of atoms of pxen
   integer, intent(out)                        :: natom  ! number of atoms in vat
   integer :: i
!
   natom = count(atom%ptab == pxen)
   if (natom > 0) then
       vat(:natom) = pack((/(i,i=1,size(atom))/),mask = atom%ptab == pxen)
   endif
!
   end subroutine get_atoms_of_specie

!---------------------------------------------------------------------      
 
   integer elemental function atomic_number(atom)
!
!  Restituisce il numero atomico
!
   USE General, only:no
   USE Atomi, only:nzConst
   type(atom_type), intent(in) :: atom
!
   if (atom%nz /= 0) then
       atomic_number = no(abs(atom%nz/nzConst))
   else
       atomic_number = 0
   endif
!
   end function atomic_number

!----------------------------------------------------------------------

   elemental function atomic_specie(atom) result(spec)
!
!  Estrai la specie atomica da atom
!
   USE elements, only:NLEN_LAB
   type(atom_type), intent(in) :: atom
   character(len=NLEN_LAB)     :: spec
!
   spec = atomic_specie_nz(atom%nz)
!   
   end function atomic_specie

!----------------------------------------------------------------------

   logical function is_atomic_specie_ss(atom,spec) 
!
!  atom è di tipo spec?
!
   USE strutil
   type(atom_type), intent(in) :: atom
   character(len=*)            :: spec
!
   is_atomic_specie_ss = s_eqidb(spec,atomic_specie(atom))
!
   end function is_atomic_specie_ss

!----------------------------------------------------------------------

   logical function is_atomic_specie_sv(atom,spec)
!
!  atom è di tipo contenuto nel vettore spec?
!
   USE strutil
   type(atom_type), intent(in)                :: atom
   character(len=*), dimension(:), intent(in) :: spec
   integer                                    :: i
!
   is_atomic_specie_sv = .false.
   do i=1,size(spec)
      is_atomic_specie_sv = s_eqidb(spec(i),atomic_specie(atom))
      if (is_atomic_specie_sv) return
   enddo
!
   end function is_atomic_specie_sv

!----------------------------------------------------------------------

   logical function is_atomic_speciez_ss(atom,zspec)
!
!  atomic number of atom is z?
!
   type(atom_type), intent(in) :: atom
   integer, intent(in)         :: zspec
!
   is_atomic_speciez_ss = atomic_number(atom) == zspec
!
   end function is_atomic_speciez_ss

!----------------------------------------------------------------------

   logical function is_atomic_speciez_vs(atom,zspec)
!
!  atomic number of atom is z?
!
   USE strutil
   type(atom_type), intent(in), dimension(:) :: atom
   integer, intent(in)                       :: zspec
!
   is_atomic_speciez_vs = any(atomic_number(atom) == zspec)
!
   end function is_atomic_speciez_vs

!----------------------------------------------------------------------

   logical function is_atomic_speciez_vv(atom,zspec)
!
!  atom è di tipo contenuto nel vettore zspec?
!
   USE strutil
   type(atom_type), intent(in), dimension(:) :: atom
   integer, dimension(:), intent(in)         :: zspec
   integer                                   :: i
!
   is_atomic_speciez_vv = .false.
   do i=1,size(zspec)
      is_atomic_speciez_vv = is_atomic_speciez_vs(atom,zspec(i))
      if (is_atomic_speciez_vv) return
   enddo
!
   end function is_atomic_speciez_vv

!----------------------------------------------------------------------

   logical function is_hydrogen_s(atom)
   USE elements
   type(atom_type), intent(in) :: atom
   is_hydrogen_s = is_atomic_specie(atom,H_at)
   end function is_hydrogen_s

!----------------------------------------------------------------------

   logical function is_hydrogen_v(atom)
   USE elements
   type(atom_type), dimension(:), intent(in) :: atom
   is_hydrogen_v = number_of_hydrogens(atom) > 0
   end function is_hydrogen_v

!----------------------------------------------------------------------

   integer function number_of_hydrogens(atom)
   USE elements
   type(atom_type), dimension(:), intent(in) :: atom
   number_of_hydrogens = count(atomic_number(atom) == H_at)
   end function number_of_hydrogens

!----------------------------------------------------------------------

   elemental function atomic_specie_nz(nzval) result(spec)
!
!  Estrai la specie atomica da nz
!
   USE General, only:Species
   USE Atomi, only:nzConst
   USE elements, only:NLEN_LAB
   USE strutil
   integer, intent(in)     :: nzval
   character(len=NLEN_LAB) :: spec
   integer                 :: kps
!
   kps = abs(nzval/nzConst)
   if (kps /= 0) then
       spec = Species(kps)
   else
       spec = 'Q'                    !unspecified specie
   endif
!   
   end function atomic_specie_nz

!----------------------------------------------------------------------

   function nz_from_znumber(z)   result(nzz)
!
!  Converte il numero atomico z in nz
!
   USE General, only:nk,no
   USE Atomi, only:nzConst
   integer, dimension(:), intent(in) :: z
   integer, dimension(size(z))       :: nzz
   integer                           :: i,j
!
   nzz(:) = 0
!
   do i=1,size(z)
      do j=1,nk
         if (no(j) == z(i)) then
             nzz(i) = nzConst*j
             exit
         endif
      enddo
   enddo
!
   end function nz_from_znumber

!----------------------------------------------------------------------
#if 0
   function nz_from_pxen(zxen)  result(nzz)
!
!  Converte il puntatore al file xen in nz
!
   USE elements
   USE General, only:nk,Species
   USE Atomi, only:nzConst
   USE strutil
   integer, dimension(:), intent(in) :: zxen
   integer, dimension(size(zxen))    :: nzz
   integer                           :: i,j
   character(len=NLEN_LAB)           :: spec
!
   !nzz(:) = nz_from_znumber(z_from_pxen(zxen))  ! errato: no distingui H da D
   do i=1,size(zxen)
      spec = specie_from_pxen(zxen(i))
      nzz(i) = 0
      do j=1,nk
         if (s_eqi(spec,Species(j))) then
             nzz(i) = nzConst*j
             exit
         endif
      enddo
   enddo
!
   end function nz_from_pxen
#endif
!----------------------------------------------------------------------

   function nz_from_pxen(zxen,elem)  result(nzz)
!
!  Convert pointer to table of elements to nz
!
   USE elements
   USE General, only:nk,Species
   USE Atomi, only:nzConst
   USE strutil
   integer, dimension(:), intent(in) :: zxen
   integer, dimension(size(zxen))    :: nzz
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   integer                           :: i,j
   character(len=NLEN_LAB)           :: spec
   integer :: nelem
!
   nelem = numelem(elem)
   if (nelem == 0) then
       nzz = 0
       return
   endif
!
   do i=1,size(zxen)
      nzz(i) = 0
      do j=1,nk
         if (zxen(i) == elem(j)%ptab) then
             nzz(i) = nzConst*j
             exit
         endif
      enddo
   enddo
!
   end function nz_from_pxen

!----------------------------------------------------------------------

   function pxen_from_nz(nzval) result(pxen)
!
!  Estrai il puntatore al file xen dall'nz
!
   USE elements
   integer, dimension(:), intent(in) :: nzval
   integer, dimension(size(nzval))   :: pxen
   character(len=NLEN_LAB)           :: spec
   integer                           :: i
!
   do i=1,size(nzval)
      spec = atomic_specie_nz(nzval(i))
      pxen(i) = pxen_from_specie(spec)
   enddo
!
   end function pxen_from_nz

!----------------------------------------------------------------------

   subroutine elements_from_atom(atom,elem)
!
!  Extract elements from atoms
!
   USE elements  
   type(atom_type), dimension(:), intent(in) :: atom
   type(element_type), dimension(:), allocatable :: elem
!
   call elem_from_atoms(elem,pxen_from_nz(atom(:)%nz))
!
   end subroutine elements_from_atom

!----------------------------------------------------------------------

   logical function is_present_sd(atom)
!
!  Check if atom has sd on coordinates
!
   type(atom_type), intent(in) :: atom
!
   is_present_sd = any(abs(atom%xsd) > 0.0)
!
   end function is_present_sd

!----------------------------------------------------------------------

   subroutine formula_from_atom(atom,elem)
!
!  Extract formula from atom
!
   USE elements
   USE fractionm
   type(atom_type), dimension(:), intent(in) :: atom
   type(element_type), dimension(:), allocatable :: elem
   integer                                       :: i,j
   type(fract_type), dimension(:), allocatable :: fract
   integer                                       :: nel
   integer                                       :: mcm
      real, dimension(:), allocatable :: ochspec
!
   !call elem_from_atoms(elem,atom%ptab)   ! suppress ionic species from forumula
   call elem_from_atoms(elem,eleminfo(atom%ptab)%z)
   nel = numelem(elem)
   if (nel > 0) then
       allocate(ochspec(nel))
       ochspec(:) = 1
       !do i=1,nel   ! suppress ionic species 
       !   elem(i)%nw = 0
       !   do j=1,size(atom)
       !      if (atom(j)%ptab == elem(i)%ptab) then
       !          elem(i)%nw = atom(j)%ocry + elem(i)%nw
       !          ochspec(i) = ochspec(i)*atom(j)%och
       !      endif
       !   enddo
       !enddo
       do i=1,nel
          elem(i)%nw = 0
          do j=1,size(atom)
             if (eleminfo(atom(j)%ptab)%z == elem(i)%z) then
                 elem(i)%nw = atom(j)%ocry + elem(i)%nw
                 ochspec(i) = ochspec(i)*atom(j)%och
             endif
          enddo
       enddo
!
       allocate(fract(nel))
       do i=1,nel
          fract(i) = fractional(elem(i)%nw)
       enddo
       mcm = lcmv(fract(:)%den)
       elem(:)%nw = mcm * elem(:)%nw * ochspec(:)
   endif
!
   end subroutine formula_from_atom

!----------------------------------------------------------------------

   subroutine xyz_to_atom(atom,xco,bf,bij,occr,occh,nz,lab,inte)
!
!  Poni il modello strutturale in atom
!
   USE realloc
   type(atom_type), dimension(:), allocatable, intent(inout)  :: atom
   real,dimension(:,:),intent(in)              :: xco   ! coordinate e num. di atomi
   real, dimension(:), intent(in), optional    :: bf    ! fattori termici
   real, dimension(:,:), intent(in), optional  :: bij   ! fattori termici aniso
   real, dimension(:), intent(in), optional    :: occr  ! occupanza cristall.
   real, dimension(:), intent(in), optional    :: occh  ! occupanza chimica 
   real, dimension(:), intent(in), optional    :: inte  ! intensita di picco
   integer, dimension(:), intent(in), optional :: nz    ! nz
   character(len=*), dimension(:), optional    :: lab   ! label
   integer                                     :: i
   integer                                     :: num_atoms
!
   num_atoms = size(xco,2)
   call reallocate(atom,num_atoms)
   do i=1,num_atoms
!
!     x,y,z
      atom(i)%xc   = xco(:,i)
!
!     thermal factor
      if (present(bf)) atom(i)%biso = bf(i)
!
!     anisotropic thermal factor
      if (present(bij)) atom(i)%bij(:) = bij(:,i)
!
!     occupanza chimica
      if (present(occh)) atom(i)%och = occh(i)
!
!     occupanza cristallografica
      if (present(occr)) atom(i)%ocry = occr(i)
!
!     specie
      if (present(nz)) atom(i)%nz   = nz(i)
!
!     intensita di picco
      if (present(inte)) atom(i)%inte = inte(i)
!
!     label
      if (present(lab)) atom(i)%lab  = lab(i)
   enddo
!
   end subroutine xyz_to_atom

!----------------------------------------------------------------------

   subroutine frac_to_cart_copy(atom,atomcart,orto)
!
!  Converti atom in coordinate cartesiane copiandolo in atomcart
!
   USE realloc
   type(atom_type), dimension(:), intent(in)                 :: atom
   type(atom_type), dimension(:), allocatable, intent(inout) :: atomcart
   real, dimension(3,3), intent(in)                          :: orto
   integer                                                   :: nat
!   
   nat = size(atom)
   call reallocate(atomcart,nat,.false.)
   atomcart(:) = atom(:)
   call frac_to_cart(atomcart,orto)
!
   end subroutine frac_to_cart_copy

!----------------------------------------------------------------------

   subroutine frac_to_cart(at,orto)
!
!  Conversione di atomi atm da coordinate frazionarie a cartesiane
!
   type(atom_type), dimension(:), intent(inout) :: at
   real, dimension(3,3), intent(in)             :: orto
   integer                                      :: i
!
   do i=1,size(at)
      at(i)%xc = matmul(orto,at(i)%xc)
   enddo
!   
   end subroutine frac_to_cart

!----------------------------------------------------------------------------------------------------

   function cartesian_coord_a(atom) result(atomc)
!  Conversione da cristallografiche a cartesiane
!  Ortom va inizializzata ( ortom = orthomatrix(cell) )
   USE variables, only:ortom
   type(atom_type), intent(in) :: atom
   type(atom_type)             :: atomc
!
   atomc = atom
   atomc%xc = matmul(ortom,atom%xc)
!
   end function cartesian_coord_a

!----------------------------------------------------------------------------------------------------

   function cartesian_coord_a_o(atom,ortom) result(atomc)
!  Conversione da cristallografiche a cartesiane
!  Ortom va inizializzata ( ortom = orthomatrix(cell) )
   type(atom_type), intent(in)      :: atom
   real, dimension(3,3), intent(in) :: ortom
   type(atom_type)                  :: atomc
!
   atomc = atom
   atomc%xc = matmul(ortom,atom%xc)
!
   end function cartesian_coord_a_o

!----------------------------------------------------------------------------------------------------

   function fractional_coord_a(atomc) result(atomf)
!  Conversione da cristallografiche a cartesiane
!  Ortom va inizializzata ( ortoi = orthomatrixi(cell) )
   USE variables, only:ortoi
   type(atom_type), intent(in) :: atomc
   type(atom_type)             :: atomf
!
   atomf = atomc
   atomf%xc = matmul(ortoi,atomc%xc)
!
   end function fractional_coord_a

!----------------------------------------------------------------------------------------------------

   function fractional_coord_a_o(atomc,ortoi) result(atomf)
!  Conversione da cristallografiche a cartesiane
   type(atom_type), intent(in)      :: atomc
   real, dimension(3,3), intent(in) :: ortoi
   type(atom_type)                  :: atomf
!
   atomf = atomc
   atomf%xc = matmul(ortoi,atomc%xc)
!
   end function fractional_coord_a_o

!----------------------------------------------------------------------------------------------------

   subroutine cart_to_frac(at,orto)
!
!  Conversione di atomi atm da coordinate cartesiane a frazionarie
!  Orto va inizializzata. ( ortom = orthomatrixi(cell) )
!
   type(atom_type), dimension(:), intent(inout) :: at
   real, dimension(3,3), intent(in)             :: orto
   integer                                      :: i
!
   do i=1,size(at)
      at(i)%xc = matmul(orto,at(i)%xc)
   enddo
!   
   end subroutine cart_to_frac

!----------------------------------------------------------------------------------------------------

   subroutine translate_atoms(atm,xtra)
!
!  Trasla gli atomi atmold di una quantit� xtra in atmnew
!
   type(atom_type), dimension(:), intent(inout) :: atm
   real, dimension(:), intent(in)               :: xtra    ! traslazione da applicare
   integer                                      :: i
!
   do i=1,size(atm)
      atm(i)%xc = atm(i)%xc + xtra
   enddo
!
   end subroutine translate_atoms

!----------------------------------------------------------------------------------------------------

   subroutine fillatom(atom)
   USE Atomi
   type(atom_type), dimension(:), intent(in) :: atom
   integer                                   :: i,j
!
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%nz
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso
      aLab(i) = atom(i)%lab
   enddo
!
   end subroutine fillatom

!----------------------------------------------------------------------------------------------------

   subroutine legm_to_tors(legm,atom,tors,cart)
!
!  Genera angoli. Se angvali e' assegnato, solo gli angoli < di angvali vengono generati 
!
   USE cgeom
   USE General, only:cell
   USE unit_cell
   USE connect_mod
   type(bond_type), dimension(:), allocatable, intent(in)       :: legm
   type(atom_type), dimension(:), intent(in)                    :: atom
   type(torsion_type), dimension(:), allocatable, intent(inout) :: tors
   logical, intent(in), optional                                :: cart    ! se vero, atom e' in coord. cartesiane
   integer                                                      :: nleg
   logical                                                      :: carti
   integer                                                      :: i,j,k
   integer                                                      :: nat
   type(connection_type), dimension(:), allocatable             :: conn
   integer                                                      :: ndimtors
   integer                                                      :: ntors
   type(atom_type), dimension(size(atom))                       :: atomc
   integer                                                      :: n1,n2,n3,n4
   real                                                         :: val
!
   nleg = numbonds(legm)
   if (nleg > 0) then
       if (present(cart)) then
          carti = cart
       else
          carti = .false.
       endif
!
!      Converti in cartesiane
       if (.not.carti) then
           atomc(:) = atom(:)
           call frac_to_cart(atomc,orthomatrix(cell)) 
       endif
!
!      alloca inizialmente gli angoli al numero di legami x 2
       ndimtors = 2*nleg
       call reallocate_tors(tors,ndimtors,.false.)
!
       nat = size(atomc)
       call legm_to_connect(nat,legm,conn)    
!
       ntors = 0
       do i=1,nleg     ! cerca le torsioni associate ad ogni legame
          n2 = legm(i)%n1
          n3 = legm(i)%n2
          if (conn(n2)%nat == 1 .or. conn(n3)%nat == 1) cycle  ! non puo' esserci torsione
          do j=1,conn(n2)%nat
             n1 = conn(n2)%pos(j)
             if (n1 == n3) cycle
             do k=1,conn(n3)%nat
                n4 = conn(n3)%pos(k)
                if (n4 == n2) cycle
                ntors = ntors + 1
                if (ntors > ndimtors) then   ! espandi la dimensione di tors
                    ndimtors = ndimtors + nleg
                    call reallocate_tors(tors,ndimtors)
                endif
                val = Angle_Dihedral(atomc(n1)%xc,atomc(n2)%xc,atomc(n3)%xc,atomc(n4)%xc)
                tors(ntors) = torsion_type(n1,n2,n3,n4,val)
             enddo
          enddo
       enddo
!
       call reallocate_tors(tors,ntors) ! compatta tors a ntors
   endif
!
   end subroutine legm_to_tors

!---------------------------------------------------------------------------------------

   subroutine build_model_info(atom,bcal,ocal)
   USE General
   USE Atomi
!
!  Crea informazioni addizionali sul modello
!
   type(atom_type), dimension(:), intent(inout) :: atom
   logical, intent(in), optional                :: bcal,ocal
   logical                                      :: bcali,ocali
!
   if (present(bcal)) then
       bcali = bcal
   else
       bcali = .true.
   endif
   if (present(ocal)) then
       ocali = ocal
   else
       ocali = .true.
   endif
!
!  Calcola occupanza cristallog. senza modificare le coordinate     
   call calcola_occ(atom,modcoor=.false.)
   if(ocali) atom%och = 1.0  ! inizializza occ. chimica
!
!  Assegna fattori termici
   if (bcali) call set_biso(atom)
!
   nats = size(atom)
!
!  Assegna intensita in base a Z
   call set_intensity(atom)
!
   call atom_string(atom)  ! genera label
!
!  Copia informazioni da atom a varibili expo/sir      
   call fillatom(atom)
!
   end subroutine build_model_info

!---------------------------------------------------------------------------------------

   subroutine add_atoms_to_list(atom,atomadd,natnew,legm1,legm2)
!
!  Combina 2 modelli in atom
!
   USE realloc
   USE connect_mod
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom
   type(atom_type), dimension(:), intent(in)                 :: atomadd
   integer, intent(out)                                      :: natnew
   type(bond_type), dimension(:), allocatable, intent(inout), optional :: legm1
   type(bond_type), dimension(:), allocatable, intent(in),    optional :: legm2
   integer                                                   :: natcurr
   integer                                                   :: natadd
!
!  Combina gli atomi
   natcurr = numatomtype(atom)   !!!!size(atom)
   natadd = size(atomadd)
   natnew = natcurr + natadd
   call reallocate(atom,natnew)
   atom(natcurr+1:) = atomadd
!
!  Combina la connettivita' in leg
   if (present(legm1) .and. present(legm2)) then
       call combine_legm(legm1,legm2,natcurr)
   endif
!
   end subroutine add_atoms_to_list

!---------------------------------------------------------------------------------------

   subroutine add_atom(atom,atomadd,legm,leg)
   USE realloc
   USE connect_mod
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom
   type(atom_type)                                           :: atomadd
   type(bond_type), allocatable, dimension(:), intent(inout), optional :: legm
   type(bond_type), dimension(:), intent(in), optional                 :: leg
   integer                                                   :: natcurr
   type(bond_type), allocatable, dimension(:) :: leg1
!
   natcurr = numatomtype(atom)
   call reallocate(atom,natcurr+1)
   atom(natcurr+1) = atomadd
   if (present(legm) .and. present(leg)) then
       allocate(leg1(size(leg)),source=leg)
       call combine_legm(legm,leg1,0)
   endif
!
   end subroutine add_atom

!---------------------------------------------------------------------------------------

   subroutine combine_iconn(icontot,icon1,icon2,nat1)
!
!  Combina in icontot la connettivita di due modelli 
!
   integer, dimension(:), allocatable, intent(inout) :: icontot
   integer, dimension(:), intent(in)                 :: icon1,icon2
   integer, intent(in)                               :: nat1
   integer                                           :: ncon1,ncon2,ncontot
   integer                                           :: i,nc2
!
   ncon1 = icon1(1)
   ncon2 = icon2(1)
   ncontot = ncon1+ncon2
   allocate(icontot(3*ncontot+1))
   icontot(1) = ncontot
   if (ncontot > 0) then
       icontot(2:3*ncon1+1) = icon1(2:3*ncon1+1)
       nc2 = 3*ncon1+1
       do i=2,ncon2*3+1,3
          nc2 = nc2 + 1
          icontot(nc2) = icon2(i)+nat1
          nc2 = nc2 + 1
          icontot(nc2) = icon2(i+1)+nat1
          nc2 = nc2 + 1
          icontot(nc2) = icon2(i+2)
       enddo
   endif
!
   end subroutine combine_iconn

!---------------------------------------------------------------------------------------

   subroutine calcola_occ(atom,modcoor,dist)
   USE Symmetry, only:ddmin
   type(atom_type), dimension(:), intent(inout) :: atom   
   logical, optional                            :: modcoor
   real, intent(in), optional                   :: dist
   logical                                      :: modco
   integer                                      :: ksp,kspecb
   real, dimension(11)                          :: xo,xn
   integer, dimension(10)                       :: key      
   integer                                      :: i
   integer, dimension(192,3)                    :: newvet   
   integer                                      :: natc
   real                                         :: ddminsav
   integer                                      :: itype,khead,iactn
!
!  modcoor = .true. -> modifica le coordinate
   if (present(modcoor)) then
       modco = modcoor
   else          
       modco = .true.
   endif        
!
!  eventualmente modifica distanza di default
   if (present(dist))then
       ddminsav = ddmin
       ddmin = dist
   endif
!
   natc = size(atom)
!!!!!FIXME --- testare 1 dovrebbe funzionare se calcoli solo occ.
   iactn = 2
   !iactn = 1
   do i=1,natc
      xo(:3) = atom(i)%xc
      ksp=kspecb(newvet,xo,xn,key,itype,i,khead,iactn)
      atom(i)%ocry = 1.0/key(10)
      if (modco) atom(i)%xc = xn(:3)
   enddo
   if (present(dist))then
       ddmin = ddminsav
   endif
!   
   end subroutine calcola_occ

!---------------------------------------------------------------------------------------
!#define anna_b_setting 1
   elemental subroutine set_biso(atom,zval)
!
!  Assegna dei fattori termici ragionevoli agli atomi (DASH approach)
!
   USE elements
   type(atom_type), intent(inout) :: atom
   integer, intent(in), optional  :: zval  ! numero atomico 
   integer                        :: zv
!
   if (present(zval)) then
       zv = zval
   else
       zv = atomic_number(atom)
   endif
!
#if defined(anna_b_setting)
   select case (zv)
      case (H_at)
        atom%biso = 4.0

      case default
        atom%biso = 1.0

   end select
#else
   select case (zv)
      case (H_at)
        atom%biso = 6.0   
        
      case (He_at,C_at,N_at,O_at,F_at,Ne_at,P_at,S_at,Cl_at,Ar_at,Br_at,Kr_at,I_at,Xe_at,Rn_at)
        atom%biso = 3.0   

      case default
        atom%biso = 1.0   

   end select
#endif
!
   end subroutine set_biso

!---------------------------------------------------------------------------------------

   !subroutine set_intensity(atom)
   elemental subroutine set_intensity(atom)
!
!  Assegna intensita' ragionevoli agli atomi 
!
   USE Atomi, only:nzConst
   type(atom_type), intent(inout) :: atom   
   !type(atom_type), dimension(:), intent(inout) :: atom   
!
   atom%inte = nzConst*atomic_number(atom)
!
   end subroutine set_intensity

!---------------------------------------------------------------------------------------

   subroutine set_specie(kat,atom,kel,elem)
!
!  Assign element kel of array array at atom kat
!
   USE elements
   USE Atomi, only: nzconst
   integer, intent(in)                          :: kat
   type(atom_type), dimension(:), intent(inout) :: atom
   integer, intent(in)                          :: kel
   type(element_type), dimension(:), intent(in) :: elem
!
   atom(kat)%nz = kel*nzconst
   atom(kat)%ptab = elem(kel)%ptab
   call atom_string(atom,[kat])
!
   end subroutine set_specie

!---------------------------------------------------------------------------------------

   subroutine stampa_struttura(at,height,title,write_occ,kpr)
   USE strutil
   type(atom_type), dimension(:), intent(in)   :: at         ! gli atomi
   logical, optional                           :: height     ! le intensita'?
   character(len=*), intent(in), optional      :: title      ! titolo per intestazione
   logical, intent(in), optional               :: write_occ  ! occupanza cristallografica
   integer, intent(in), optional               :: kpr        ! stampa su kpr
   logical                                     :: wocc
   integer                                     :: n
   logical                                     :: heightl 
   integer                                     :: kpri
!
   if (present(kpr)) then
       kpri = kpr
   else
       kpri = 6
   endif
   if (present(height)) then
       heightl = height
   else
       heightl = .false.
   endif
   if (present(title)) then
       write(kpri,'(/a)')centra_str(title,80)
   else           
       write(kpri,*)
   endif        
   if (heightl) then
       write(kpri,'(a)')'                  Height         X           Y         Z             B       Site'
       do n=1,size(at)
          write(kpri,'(1x,i3,a1,2x,a5,2x,g10.2,3(2x,f10.4),2x,f10.3,2x,f8.3)') &
                n,')',at(n)%lab,at(n)%inte,at(n)%xc(:3),at(n)%biso,at(n)%och
       enddo
   else
       if (present(write_occ)) then
           wocc = write_occ
       else
           wocc = .false.
       endif
       if (wocc) then
           write(kpri,'(a)')'                     X           Y           Z           B       Site     Occ'
           do n=1,size(at)
              write(kpri,'(1x,i3,a1,2x,a5,3(2x,f10.4),2x,f10.3,2(2x,f8.3))') &
                    n,')',at(n)%lab,at(n)%xc(:3),at(n)%biso,at(n)%och,at(n)%ocry
           enddo
       else
           write(kpri,'(a)')'                     X           Y           Z           B       Site'
           do n=1,size(at)
              write(kpri,'(1x,i3,a1,2x,a5,3(2x,f10.4),2x,f10.3,2x,f8.3)') &
                    n,')',at(n)%lab,at(n)%xc(:3),at(n)%biso,at(n)%och
           enddo
       endif
   endif
   write(kpri,*)
!
   end subroutine stampa_struttura

!-------------------------------------------------------------------------------

   subroutine stampa_strutturan(atom,vet,title,kpr,adp_type,infos,spg)
   USE strutil
   USE ccryst
   USE spginfom
   type(atom_type), intent(in), dimension(:), allocatable :: atom   ! atomi da stampare
   integer, dimension(:), intent(in), optional            :: vet    ! cosa stampare?
   character(len=*), intent(in), optional                 :: title  ! titolo per intestazione
   integer, intent(in), optional                          :: kpr    ! file pointer
   integer, intent(in), optional                          :: adp_type  ! adp type
   type(symminfo_type), dimension(:), allocatable, optional :: infos ! symmetry info
   type(spaceg_type), intent(in), optional                  :: spg   ! space group
   integer, dimension(:), allocatable                     :: ival
   real, dimension(:), allocatable                        :: rval
   character(len=40), dimension(:), allocatable           :: sval
   integer, dimension(:), allocatable                     :: ktype
   integer                                                :: kpri
   integer, dimension(:), allocatable                     :: veti
   integer                                                :: ncol
   integer                                                :: i
   integer                                                :: adp_t
   character(len=9) :: adp_string
   logical :: lsym
!
   if (present(kpr)) then
       kpri = kpr
   else
       kpri = 6
   endif
!
   if (present(title)) then
       write(kpri,'(/a)')centra_str(title,80)
   else           
       write(kpri,*)
   endif        
!
   if (present(adp_type)) then
       adp_t = adp_type
   else
       adp_t = 1
   endif
!
   if (present(vet)) then
       ncol = size(vet)
       allocate(veti(ncol))
       veti(:) = vet(:)
   else
       ncol = 10
       allocate(veti(ncol))
       veti(:) = 1
!corr       veti(:9) = 1
!corr       veti(10) = 0
   endif
   lsym = present(infos) .and. present(spg)
   allocate(ktype(ncol),ival(ncol),rval(ncol),sval(ncol))
!
!  Titoli delle colonne
   ktype(:) = 3
   if (adp_t == 1) then
       adp_string = 'B'
   else
       adp_string = 'U'
   endif
   call set_row_list(veti,ktype,sval=(/'Number   ','Label    ','Type     ',    &
   'Intensity','Xfrac    ','Yfrac    ','Zfrac    ',adp_string,'s.o.f.   ','Symm.Op. '/),kpr=kpri)
!
!  Scrivi la tabella degli atomi
   ktype = [1,3,3,22,25,25,25,23,23,3]
   do i=1,numatomtype(atom)
      ival(1) = i
      sval(2) = atom(i)%lab
      sval(3) = atomic_specie(atom(i))
      rval(4) = atom(i)%inte
      rval(5) = atom(i)%xc(1)
      rval(6) = atom(i)%xc(2)
      rval(7) = atom(i)%xc(3)
      if (adp_t == 1) then
          rval(8) = atom(i)%biso
      else
          rval(8) = u_from_b(atom(i)%biso)
      endif
      rval(9) = atom(i)%ocry*atom(i)%och
      if (lsym) then
          sval(10) = spg%symopstr(infos(i)%oper)
      else
          sval(10) = 'x, y, z'
      endif
        write(0,*)'SYM=',trim(sval(10))
      call set_row_list(veti,ktype,ival,rval,sval,kpri)
   enddo
!
   end subroutine stampa_strutturan

!-------------------------------------------------------------------------------

   subroutine save_structure_bin(unitbin,atom,legm)
!
!  Save structure model on binary file
!
   USE connect_mod
   integer, intent(in)                                    :: unitbin ! unit of binary file
   type(atom_type), intent(in), dimension(:), allocatable :: atom    ! atomi da stampare
   type(bond_type), dimension(:), allocatable, intent(in) :: legm    ! legami
   integer                                                :: nat, numleg
!
!  write atom struct
   nat = numatomtype(atom)
   write(unitbin) nat
   if (nat > 0) write(unitbin) atom(:)   
!   
!  write legm struct
   numleg = numbonds(legm)
   write(unitbin) numleg
   if (numleg > 0) write(unitbin) legm(:)
!
   end subroutine save_structure_bin

!-------------------------------------------------------------------------------

   subroutine read_structure_bin(unitbin,atom,legm,err)
!
!  Read structure model from binary file
!
   USE connect_mod
   USE realloc
   USE errormod
   integer, intent(in)                                     :: unitbin ! unit of binary file
   type(atom_type), dimension(:), allocatable, intent(out) :: atom    ! atomi da stampare
   type(bond_type), dimension(:), allocatable, intent(out) :: legm    ! legami
   type(error_type), intent(out)                           :: err
   integer                                                 :: ier
   integer                                                 :: nat, numleg
!
!  read atom struct
   read(unitbin,iostat=ier,err=10) nat
   if (nat > 0 .and. ier == 0) then
       call reallocate(atom,nat,.false.)
       read(unitbin,iostat=ier,err=10) atom(:)
   endif
!   
!  read legm struct
   read(unitbin,iostat=ier,err=10) numleg
   if (numleg > 0 .and. ier == 0) then
       call reallocate_legm(legm,numleg,.false.)
       read(unitbin,iostat=ier,err=10) legm(:)
   endif
!
10 continue
   if (ier /= 0) then
       call err%set('Error on reading structure')
   endif
!
   end subroutine read_structure_bin
   
!-------------------------------------------------------------------------------

   function atom_symm_scalar(k,atom) result(at)
   USE symmetry, only:kmat, tmat
   integer, intent(in)         :: k
   type(atom_type), intent(in) :: atom
   type(atom_type)             :: at
   at%xc = matmul(kmat(k,:,:),atom%xc) + tmat(k,:)   ! Rot*X + Tra
   end function atom_symm_scalar

!-------------------------------------------------------------------------------

   function atom_symm_scalar_xyz(k,atx) result(xyz)
   USE symmetry, only:kmat, tmat
   integer, intent(in)            :: k
   real, dimension(3), intent(in) :: atx
   real, dimension(3)             :: xyz
   xyz(:) = matmul(kmat(k,:,:),atx) + tmat(k,:)   ! Rot*X + Tra
   end function atom_symm_scalar_xyz

!-------------------------------------------------------------------------------

   function atom_symm_vet(k,atom) result(at)
   USE symmetry, only:kmat, tmat
   integer, intent(in)         :: k
   type(atom_type), dimension(:), intent(in) :: atom
   type(atom_type), dimension(size(atom))    :: at
   integer                                   :: i
   do i=1,size(atom)
      at(i)%xc = matmul(kmat(k,:,:),atom(i)%xc) + tmat(k,:)   ! Rot*X + Tra
   enddo
   end function atom_symm_vet

!-------------------------------------------------------------------------------

   subroutine applica_sym_oper_kmat(k,atom)
   USE symmetry, only:kmat, tmat
   integer, intent(in)                          :: k
   type(atom_type), dimension(:), intent(inout) :: atom   
   integer                                      :: i
!
   do i=1,size(atom)
      atom(i)%xc = matmul(kmat(k,:,:),atom(i)%xc) + tmat(k,:)   ! Rot*X + Tra
   enddo
!
   end subroutine applica_sym_oper_kmat

!-------------------------------------------------------------------------------

   subroutine applica_sym_oper_s(atom,symop)
   USE spginfom
   type(symop_type), intent(in)                 :: symop
   type(atom_type), dimension(:), intent(inout) :: atom
   integer                                      :: i
!
   do i=1,size(atom)
      atom(i)%xc = matmul(symop%rot,atom(i)%xc) + symop%trn   ! Rot*X + Tra
   enddo
!
   end subroutine applica_sym_oper_s

!-------------------------------------------------------------------------------

   subroutine applica_sym_oper_s_scalar(atom,symop)
   USE spginfom
   type(symop_type), intent(in)   :: symop
   type(atom_type), intent(inout) :: atom
!
   atom%xc = matmul(symop%rot,atom%xc) + symop%trn   ! Rot*X + Tra
!
   end subroutine applica_sym_oper_s_scalar

!---------------------------------------------------------------------      

   subroutine create_tetrahedron(atom,dist,icon,legm,ortoi,xpos,kcentr,centro)
!
!  Crea un tetraedro in coordinate cartesiane
!
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom    ! gli atomi
   real, intent(in)                                          :: dist    ! la distanza tra atomo centrale e vertici
   integer, dimension(:), intent(out), optional              :: icon    ! la connettivita
   type(bond_type), dimension(:), allocatable, optional      :: legm
   real, dimension(3,3), optional, intent(in)                :: ortoi   ! matrice per conversione in coord. fraz.
   real, dimension(3), intent(in), optional                  :: xpos    ! posizione 
   integer, intent(in),optional                              :: kcentr  ! gestione atomo centrale
   logical, intent(in),optional                              :: centro  ! atomo al centro
   logical                                                   :: centr
   real                                                      :: xx,rr,dd,aa
   integer                                                   :: nat
!
!  atomo centrale nell'origine
   if (present(centro) .and. present(kcentr)) then
       centr = centro
   else
       centr = .true.
   endif
   if (centr) then  ! aggiungi atomo al centro
       allocate(atom(5))
       nat = 1
       atom(nat)%xc(:) = (/0.0,0.0,0.0/)
   else                   ! l'atomo kcentr deve essere al centro
       allocate(atom(4))
       nat = 0
   endif
!
!  atomi ai vertici
   xx = 0.942809041*dist   !  (4/(3*sqrt(2)))*dist
   rr = 0.333333333*dist   !  (1/3)*dist
   dd = 0.47140452*dist    !  (sqrt(2)/3)*dist
   aa = 1.632993162*dist   !  (4/sqrt(6))*dist
   atom(nat+1)%xc = (/xx,0.0,-rr/)
   atom(nat+2)%xc = (/-dd,0.5*aa,-rr/)
   atom(nat+3)%xc = (/-dd,-0.5*aa,-rr/)
   atom(nat+4)%xc = (/0.0,0.0,dist/)
!
!  Connettivita
   if (present(icon)) then
       if (centr) then
           icon(:13) = (/4,1,2,1,1,3,1,1,4,1,1,5,1/)
       else
           icon(:13) = (/4,kcentr,1,1,kcentr,2,1,kcentr,3,1,kcentr,4,1/)
       endif
   endif
   if (present(legm)) then
       call reallocate_legm(legm,4,.false.)
       if (centr) then
           legm(:)%n1 = (/1,1,1,1/)
           legm(:)%n2 = (/2,3,4,5/)
       else
           legm(:)%n1 = (/kcentr,kcentr,kcentr,kcentr/)
           legm(:)%n2 = (/1,2,3,4/)
       endif
   endif
!
   if (present(ortoi)) call cart_to_frac(atom,ortoi)    !converti in coord. frazionarie
   if (present(xpos))  call translate_atoms(atom,xpos)  !sposta in xpos
!
   end subroutine create_tetrahedron

!---------------------------------------------------------------------      

   subroutine create_octahedron (atom,dist,icon,ortoi,xpos,kcentr,centro)
!
!  Crea un ottaedro in coordinate cartesiane
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom    ! gli atomi
   real, intent(in)                                          :: dist    ! la distanza tra atomo centrale e vertici
   integer, dimension(:), intent(out), optional              :: icon    ! la connettivita
   real, dimension(3,3), optional, intent(in)                :: ortoi   ! matrice per conversione in coord. fraz.
   real, dimension(3), intent(in), optional                  :: xpos    ! posizione 
   integer, intent(in),optional                              :: kcentr  ! gestione atomo centrale
   logical, intent(in),optional                              :: centro  ! atomo al centro
   logical                                                   :: centr
   integer                                                   :: nat
!
!  atomo centrale nell'origine
   if (present(centro) .and. present(kcentr)) then
       centr = centro
   else
       centr = .true.
   endif
   if (centr) then  ! aggiungi atomo al centro
       allocate(atom(7))
       nat = 1
       atom(nat)%xc(:) = (/0.0,0.0,0.0/)
   else                   ! l'atomo kcentr deve essere al centro
       allocate(atom(6))
       nat = 0
   endif
!
!  atomi ai vertici
   atom(nat+1)%xc = (/dist,0.0,0.0/)
   atom(nat+2)%xc = (/0.0,dist,0.0/)
   atom(nat+3)%xc = (/0.0,0.0,dist/)
   atom(nat+4)%xc = (/-dist,0.0,0.0/)
   atom(nat+5)%xc = (/0.0,-dist,0.0/)
   atom(nat+6)%xc = (/0.0,0.0,-dist/)
!
   if (present(icon)) then
       if (centr) then
           icon(:19) = (/6,1,2,1,1,3,1,1,4,1,1,5,1,1,6,1,1,7,1/)
       else
           !icon(:19) = (/6,kcentr+6,1,1,kcentr+6,2,1,kcentr+6,3,1,kcentr+6,4,1,kcentr+6,5,1,kcentr+6,6,1/)
           icon(:19) = (/6,kcentr,1,1,kcentr,2,1,kcentr,3,1,kcentr,4,1,kcentr,5,1,kcentr,6,1/)
       endif
   endif
!
   if (present(ortoi)) call cart_to_frac(atom,ortoi)  !converti in coord. frazionarie
   if (present(xpos))  call translate_atoms(atom,xpos)  !sposta in xpos
!
   end subroutine create_octahedron

!---------------------------------------------------------------------      

   subroutine create_square (atom,dist,icon,ortoi,xpos,kcentr,centro)
!
!  Crea un geometria quadrato planare
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom    ! gli atomi
   real, intent(in)                                          :: dist    ! la distanza tra atomo centrale e vertici
   integer, dimension(:), intent(out), optional              :: icon    ! la connettivita
   real, dimension(3,3), optional, intent(in)                :: ortoi   ! matrice per conversione in coord. fraz.
   real, dimension(3), intent(in), optional                  :: xpos    ! posizione 
   integer, intent(in),optional                              :: kcentr  ! gestione atomo centrale
   logical, intent(in),optional                              :: centro  ! atomo al centro
   logical                                                   :: centr
   integer                                                   :: nat
!
!  atomo centrale nell'origine
   if (present(centro) .and. present(kcentr)) then
       centr = centro
   else
       centr = .true.
   endif
   if (centr) then  ! aggiungi atomo al centro
       allocate(atom(5))
       nat = 1
       atom(nat)%xc(:) = (/0.0,0.0,0.0/)
   else                   ! l'atomo kcentr deve essere al centro
       allocate(atom(4))
       nat = 0
   endif
!
!  atomi ai vertici
   atom(nat+1)%xc = (/dist,0.0,0.0/)
   atom(nat+2)%xc = (/0.0,dist,0.0/)
   atom(nat+3)%xc = (/-dist,0.0,0.0/)
   atom(nat+4)%xc = (/0.0,-dist,0.0/)
!
   if (present(icon)) then
       if (centr) then
           icon(:13) = (/4,1,2,1,1,3,1,1,4,1,1,5,1/)
       else
           icon(:13) = (/4,kcentr,1,1,kcentr,2,1,kcentr,3,1,kcentr,4,1/)
       endif
   endif
!
   if (present(ortoi)) call cart_to_frac(atom,ortoi)  !converti in coord. frazionarie
   if (present(xpos))  call translate_atoms(atom,xpos)  !sposta in xpos
!
   end subroutine create_square

!---------------------------------------------------------------------      

   subroutine create_triangle(atom, dist, angle, ortoi, xpos, leg)
   USE trig_constants
   USE progtype
   type(atom_type), dimension(3), intent(out) :: atom
   real, intent(in)                           :: dist,angle
   real, dimension(3,3), intent(in)           :: ortoi   ! matrice per conversione in coord. fraz.
   real, dimension(3), intent(in), optional   :: xpos    ! posizione 
   type(bond_type), dimension(2), intent(out) :: leg
!
   atom(1)%xc = (/0.0,0.0,0.0/)
   atom(2)%xc = (/dist,0.0,0.0/)
   atom(3)%xc = (/dist*cos(dtor*angle),dist*sin(dtor*angle),0.0/)
!
   leg(1) = bond_type(1,2,dist,0.3,0)
   leg(2) = bond_type(1,3,dist,0.3,0)
!
   call cart_to_frac(atom,ortoi)   !converti in coord. frazionarie
   call translate_atoms(atom,xpos) !sposta in xpos
!
   end subroutine create_triangle

!---------------------------------------------------------------------      

   subroutine set_figures(code,atom,legm,dist,angle,nat,distm)
   USE connect_mod
   USE General, only:cell,nk,no,nw,neutro
   USE atomi, only:nzConst
   USE variables, only:ortoi,ortom
   USE unit_cell
   USE realloc
   USE symmetry, only:nsymtot
   USE rand_mod
   integer, intent(in)                                       :: code
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom
   type(bond_type), allocatable, dimension(:), intent(out)   :: legm
   integer, intent(in), optional                             :: nat
   real, intent(in)                                          :: dist
   real, intent(in), optional                                :: angle
   integer                                                   :: nfig,resto,nfig0
   type(atom_type), dimension(:), allocatable                :: atomt
   type(bond_type), dimension(:), allocatable                :: legmt
   real, dimension(6)                                        :: xrand
   integer                                                   :: i,j
   integer                                                   :: nati,nat0,na
   real, intent(in), optional                                :: distm
   integer                                                   :: kfound,ntent
   integer, parameter                                        :: NTENTMAX = 20
!
   if (present(nat)) then
       nati = nat
   else
       nati = nint(sum(nw(:nk),mask=no(:nk) > 1) / real(nsymtot)) ! atomi in u.a.
   endif
   nfig = nati/code
   resto = mod(nati,code)
!
   ortom = orthomatrix(cell)
   ortoi = orthomatrixi(cell)
   select case (code)
       case (1)        ! atomi isolati
         nat0 = nfig
         call reallocate(atom,nfig,.false.)
         call reallocate_legm(legm,0)
         nfig0 = 1
         atom(1)%xc(:) = randvalue(3,0.0,1.0)
         ntent = 0
         do 
            nfig0 = nfig0 + 1
            atom(nfig0)%xc(:) = randvalue(3,0.0,1.0)
!
!           Controlla distanza dagli altri atomi
            if (present(distm)) then
                call get_distance_radius(atom(:nfig0),nfig0,distm,2,kfound)
                if (ntent == NTENTMAX) kfound = 0
                if (kfound > 0) then
                    nfig0 = nfig0 - 1
                    ntent = ntent + 1
                else
                    ntent = 0
                endif
                !write(0,*)'atom=',nfig0,kfound,ntent
            endif
            if (nfig0 == nfig) exit
         enddo

       case (2)        ! coppie
         nat0 = 0
         call reallocate(atom,nat0,.false.)
         call reallocate_legm(legmt,1)
         legmt(1) = bond_type(1,2,dist,0.3,0)
         allocate(atomt(2))
         do i=1,nfig
            call random_number(xrand)
            atomt(1)%xc = (/0.0,0.0,0.0/)
            atomt(2)%xc = (/dist,0.0,0.0/)
            call cart_to_frac(atomt,ortoi)
            call translate_atoms(atomt,xrand)
            call rand_rotate_atoms(atomt,xrand(4:6))
            call add_atoms_to_list(atom,atomt,nat0,legm1=legm,legm2=legmt)
         enddo

       case (3)        ! triangoli
         nat0 = 0
         call reallocate(atom,nat0,.false.)
         call reallocate_legm(legmt,2)
         allocate(atomt(3))
         do i=1,nfig
            call random_number(xrand)
            call create_triangle(atomt,dist,angle,ortoi,xrand(1:3),legmt)
            call rand_rotate_atoms(atomt,xrand(4:6))
            call add_atoms_to_list(atom,atomt,nat0,legm1=legm,legm2=legmt)
         enddo

   end select 
!
!  Gestione atomi restanti
   if (resto == 1) then
       call random_number(xrand)
       atomt(1)%xc = xrand(1:3)
       call add_atoms_to_list(atom,atomt(1:1),nat0)
   elseif (resto == 2) then
       call random_number(xrand)
       call create_triangle(atomt,dist,angle,ortoi,xrand(1:3),legmt) ! prendi solo 2 atomi
       call rand_rotate_atoms(atomt(1:2),xrand(4:6))
       call reallocate_legm(legmt,1)
       call add_atoms_to_list(atom,atomt(1:2),nat0,legm1=legm,legm2=legmt)
   endif
!
!  Assegna specie sulla base del contenuto
   na = 0
   loop_spec:do i=nk,1,-1
      if (no(i) == 1 .and. neutro == 0) cycle  ! niente H
      do j=1,nint(nw(i)/real(nsymtot))
         na = na + 1
         if (na > nat0) exit loop_spec
         atom(na)%nz = i*nzConst
      enddo
   enddo loop_spec
   if (na < nat0) atom(na:nat0)%nz = atom(na)%nz  ! atomi resudui hanno la specie dell'ultimo assegnato
   call atom_string(atom)
   !atom(:)%ocry = 1.0; atom(:)%och = 1.0
!
   end subroutine set_figures

!---------------------------------------------------------------------      

   subroutine set_atoms(atom,elem,nsymt,wave)
!
!  Set atoms according to content in elem
!
   USE elements
   USE Atomi, only:nzConst
   type(atom_type), dimension(:), intent(inout) :: atom
   type(element_type), dimension(:), intent(in) :: elem
   integer, intent(in)                          :: nsymt, wave
   integer                                      :: nat, na
   integer                                      :: nel
   integer                                      :: i,j
   nat = size(atom)
   nel = size(elem)
   na = 0
   loop_spec:do i=nel,1,-1
      if (elem(i)%z == H_at .and. wave == 0) cycle
      do j=1,nint(elem(i)%nw/nsymt)
         na = na + 1
         if (na > nat) exit loop_spec
         atom(na)%nz = i*nzConst
         atom(na)%ptab = elem(i)%ptab
      enddo
   enddo loop_spec
   if (na < nat) then ! atomi resudui hanno la specie dell'ultimo assegnato
       atom(na:nat)%nz = atom(na)%nz  
       atom(na:nat)%ptab = atom(na)%ptab  
   endif
   call atom_string(atom)
   
   end subroutine set_atoms

!---------------------------------------------------------------------      

   subroutine create_connectivity_fix(atom,legm,check,tolmin,tolmax,angl,kpr,vet,code)
!
!  Crea la connettivita senza spostare gli atomi
!  Se vet e' presente si cercano solo i legami che coinvolgono gli atomi in vet
!
   USE cgeom
   USE bondtmod
   USE connect_mod
!
   type(atom_type), dimension(:), intent(in)                           :: atom
   type(bond_type), dimension(:), allocatable, intent(inout), optional :: legm
   logical, intent(in), optional                                       :: check
   real, intent(in), optional                                          :: tolmin,tolmax
   real, intent(in), optional                                          :: angl
   integer, intent(in), optional                                       :: kpr
   integer, dimension(:), intent(in), optional                         :: vet
   integer, intent(in), optional                                       :: code
   logical                                                             :: checkb
   integer                                                             :: kprb
   integer                                                             :: nat
   integer                                                             :: i,j 
   integer, dimension(size(atom))                                      :: zval
   integer                                                             :: nleg
   integer                                                             :: ndimleg
   real                                                                :: angmin
   type(bond_type)                                                     :: legnew
   real                                                                :: tlmin,tlmax
   integer                                                             :: codevet
       integer :: iv
!
   if (present(kpr)) then
       kprb = kpr
   else
       kprb = 0
   endif
   if (present(check)) then
       checkb = check
   else
       checkb = .false.
   endif
   if (present(tolmin)) then
       tlmin = tolmin
   else
       tlmin = 0.0 
   endif
   if (present(tolmax)) then
       tlmax = tolmax
   else
       tlmax = 0.0 
   endif
   if (present(angl)) then
       angmin = angl
   else
       angmin = 55
   endif
!
   nat = size(atom)
   nleg = 0
!
   if (nat > 1) then  ! piu' di un atomo ...
!
!      genera Z di tutti gli atomi
       zval(:) = atomic_number(atom)
!
!      Crea tabella di connettivita'
       ndimleg = nat 
       call reallocate_legm(legm,ndimleg)
       if (present(vet)) then
           if (present(code)) then
               codevet = code
           else
               codevet = 1
           endif
           if (codevet == 1) then   ! only atoms in the vector vet are connected with all other atoms
               do iv=1,size(vet)
                  i = vet(iv)
                  if (zval(i) == 0) cycle
                  do j=1,nat
                     if (any(vet(1:iv) == j)) cycle ! trick: exclude i=j and all atoms connected before
                     if (zval(j) == 0) cycle
                     call create_bond(i,j,atom(i)%xc,atom(j)%xc,zval(i),zval(j),tlmin,tlmax,legnew)
                     if (legnew%n1 > 0) then  ! se 0 non c'è legame tra i due atomi
                         nleg = nleg + 1
                         if (nleg > ndimleg) then   ! check sulla dimensione di legm
                             ndimleg = ndimleg + nat/2
                             call reallocate_legm(legm,ndimleg)
                         endif
                         legm(nleg) = legnew
                     endif
                  enddo
               enddo
           else                    ! exclude bonds i-j with vet > 0 and vet(i)/=vet(j)
               do i=1,nat-1
                  if (zval(i) == 0) cycle
                  do j=i+1,nat
                     if (zval(j) == 0) cycle
                     if ((vet(i) > 0 .and. vet(j) > 0) .and. (vet(i) /= vet(j))) cycle  ! test bond i-j
                     call create_bond(i,j,atom(i)%xc,atom(j)%xc,zval(i),zval(j),tlmin,tlmax,legnew)
                     if (legnew%n1 > 0) then  ! se 0 non c'è legame tra i due atomi
                         nleg = nleg + 1
                         if (nleg > ndimleg) then   ! check sulla dimensione di legm
                             ndimleg = ndimleg + nat/2
                             call reallocate_legm(legm,ndimleg)
                         endif
                         legm(nleg) = legnew
                     endif
                  enddo
               enddo
           endif
       else
           do i=1,nat-1
              if (zval(i) == 0) cycle
              do j=i+1,nat
                 if (zval(j) == 0) cycle
                 call create_bond(i,j,atom(i)%xc,atom(j)%xc,zval(i),zval(j),tlmin,tlmax,legnew)
                 if (legnew%n1 > 0) then  ! se 0 non c'è legame tra i due atomi
                     nleg = nleg + 1
                     if (nleg > ndimleg) then   ! check sulla dimensione di legm
                         ndimleg = ndimleg + nat/2
                         call reallocate_legm(legm,ndimleg)
                     endif
                     legm(nleg) = legnew
                 endif
              enddo
           enddo
       endif
       call reallocate_legm(legm,nleg)
       if (nleg > 0) then
           if (checkb) then
               call check_angle(atom,legm,angmin,kprb)
           endif 
       endif
   endif
!
   end subroutine create_connectivity_fix

!---------------------------------------------------------------------      

   subroutine create_bond(ka1,ka2,x1,x2,z1,z2,tolmin,tolmax,leg)
!
!  Valuta se possibile legame tra ka1,ka2. Se leg%n1 = 0 allora non c'e' legame
!
   USE bondtmod
   USE connect_mod
   integer, intent(in)            :: ka1,ka2   ! atom numbers
   real, dimension(3), intent(in) :: x1,x2     ! coordinate cart
   integer, intent(in)            :: z1,z2     ! numeri atomici
   real, intent(in)               :: tolmin,tolmax        ! tolerance
   type(bond_type), intent(out)   :: leg       ! legame
   real, dimension(3)             :: xdiff
   integer                        :: k
   real                           :: dist
   real                           :: bondmin,bondmax
!
   leg%n1 = 0
   bondmax = bond_table_max(1,z1,z2)
   if (bondmax > 0.000) then
!
!      valuta distanza x1-x2 solo se la prima distanza in tabella e' > 0
       xdiff(:) = x1(:) - x2(:)
       dist = sqrt(xdiff(1)**2+xdiff(2)**2+xdiff(3)**2)
!
       bondmin = bond_table_min(1,z1,z2) + tolmin
       bondmax = bondmax + tolmax
       if (dist >= bondmin .and. dist <= bondmax) then 
           !leg = bond_type(ka1,ka2,dist,tolmin)
           leg = bond_type(ka1,ka2,dist)
       else
           do k=2,3
              bondmax = bond_table_max(k,z1,z2)
              if (bondmax > 0.000) then
                  bondmin = bond_table_min(k,z1,z2) + tolmin
                  bondmax = bondmax + tolmax
                  if (dist >= bondmin .and. dist <= bondmax) then 
                      !leg = bond_type(ka1,ka2,dist,tolmin)
                      leg = bond_type(ka1,ka2,dist)
                  endif
              else
                  exit
              endif
           enddo
       endif
   endif
!
   end subroutine create_bond
!---------------------------------------------------------------------      

   subroutine check_atoms(kat,vet,atomc,code)
!
!  Controlla che gli angoli tra kat e gli atomi in vet siano plausibili
!
   USE connect_mod
   integer, intent(in)                         :: kat
   integer, dimension(:), intent(in)           :: vet
   type(atom_type), dimension(:), intent(in)   :: atomc  ! coordinate cartesiane
   integer, intent(out)                        :: code
   integer                                     :: i
   type(bond_type), dimension(:), allocatable  :: legm
   type(angle_type), dimension(:), allocatable :: angle
   integer                                     :: nat
   real                                        :: angmin,angmax
   logical                                     :: is_angmax
!
   code = 0
   nat = size(vet)
   if (nat >= 2) then
       call reallocate_legm(legm,nat)
       do i=1,nat
          legm(i)%n1 = kat
          legm(i)%n2 = vet(i)
       enddo
!
!      Decidi l'angolo limite sulla base del numero di coordinazione
       select case (nat)
          case (2)  
            angmin = 55
            is_angmax = .false.
          case (3)
            angmin = 100
            angmax = 140
            is_angmax = .true.
          case (4:5) 
            angmin = 60
            angmax = 150
            is_angmax = .true.
          case (6:)
        !!!    angmin = 70
            angmin = 60
            is_angmax = .false.
       end select
!
!      Estrai angoli da legm 
       call legm_to_ang(legm,atomc,angle,cart=.true.)
       do i=1,size(angle)
          if (angle(i)%val < angmin) then
              code = code + 1
          else
              if (is_angmax) then
                  if (angle(i)%val > angmax) code = code + 1
              endif
          endif
       enddo
   endif
!
   end subroutine check_atoms
!---------------------------------------------------------------------      

   subroutine check_angle(atom,legm,angl,kpr)
!
!  Rimuove angoli minori di angl
!
   USE bondtmod
   USE connect_mod
   type(atom_type), dimension(:), intent(in)                           :: atom ! in coordinate cartesiane
   type(bond_type), dimension(:), allocatable, intent(inout), optional :: legm
   real, intent(in), optional                                          :: angl
   integer, intent(in), optional                                       :: kpr
   integer                                                             :: kpra
   type(angle_type), dimension(:), allocatable                         :: angle
   integer                                                             :: numang
   real                                                                :: anglim
   logical                                                             :: delang
   integer                                                             :: n1i,n2i,n3i
   integer                                                             :: n1j,n2j,n3j
   integer                                                             :: del12,del23
   integer                                                             :: nleg1,nleg3
   integer                                                             :: i,j
   real                                                                :: err12,err32
   real, parameter :: ANGLIM_METAL = 30
!
!  Genera gli angoli minori di anglim
   if (present(angl)) then
       anglim = angl
   else
       anglim = 55
   endif
   call legm_to_ang(legm,atom,angle,anglim,cart=.true.) ! generazione angoli da coordinate cartesiane
!
   if (angl > anglim_metal) then
!      Metal atoms could have low bond angles
!      Remove angle > anglim_metal and containing metal atoms
       do i=1,numangles(angle)
          if (any(is_metal(atom([angle(i)%n1,angle(i)%n2,angle(i)%n3]))) .and. angle(i)%val > ANGLIM_METAL) then
              angle(i)%n1 = -angle(i)%n1
          endif
       enddo
   endif
!
   numang = numangles(angle)
   if (numang > 0) then
       if (present(kpr)) then
           kpra = kpr
       else
           kpra = 0
       endif
       !call load_bond_table()
       if (kpra > 0) write(6,*)
       !if (kpra > 0) call print_connect(atom%lab,angle=angle,kpri=0)  
!
!      Rimuovi i legami in comune a piu' angoli
       do i=1,numang-1         
          n1i = angle(i)%n1
          if (n1i < 0) cycle ! angolo da non considerare
          n2i = angle(i)%n2
          n3i = angle(i)%n3
          del12 = 0
          del23 = 0
          do j=i+1,numang
             n1j = angle(j)%n1
             if (n1j < 0) cycle 
             n2j = angle(j)%n2
             n3j = angle(j)%n3
             delang = .false.
             if ((n1i == n1j .and. n2i == n2j) .or. (n1i == n2j .and. n2i == n1j)) then
                 del12 = 1
                 delang = .true.
             elseif ((n2i == n2j .and. n3i == n3j) .or. (n2i == n3j .and. n3i == n2j)) then
                 del23 = 1
                 delang = .true.
             endif
             if (delang) then
                 angle(j)%n1 = -angle(j)%n1
             endif
          enddo
          if (del12 == 1 .or. del23 == 1) then
              angle(i)%n1 = -angle(i)%n1
              if (del12 == 1) then
                  call remove_bond_from_atoms(legm,n1i,n2i)
                  if (kpra > 0) call print_del_bond(n1i,n2i,angle(i))
              endif
              if (del23 == 1) then
                  call remove_bond_from_atoms(legm,n2i,n3i)
                  if(kpra > 0) call print_del_bond(n2i,n3i,angle(i))
              endif
          endif
       enddo
!
!      Rimuovi legame a piu' bassa connettivita'
       do i=1,numang
          n1i = angle(i)%n1
          if (angle(i)%n1 < 0) cycle
          n2i = angle(i)%n2
          n3i = angle(i)%n3
          nleg1 = number_of_bonds(legm,n1i)
          nleg3 = number_of_bonds(legm,n3i)
          if (nleg1 > nleg3) then      ! atomo n1i piu' connesso
              call remove_bond_from_atoms(legm,n2i,n3i)
              if (kpra > 0) call print_del_bond(n2i,n3i,angle(i))
          elseif (nleg1 < nleg3) then  ! atomo n3i piu' connesso
              call remove_bond_from_atoms(legm,n2i,n1i)
              if (kpra > 0) call print_del_bond(n2i,n1i,angle(i))
          else                         ! stessa connettivita' su n1i e n3i
!
!             Elimino il legame più sbagliato
              call get_info_distance(atom(n1i),atom(n2i),errd=err12)
              call get_info_distance(atom(n3i),atom(n2i),errd=err32)
              if (err12 > err32) then
                  call remove_bond_from_atoms(legm,n2i,n1i)
                  if (kpra > 0) call print_del_bond(n2i,n1i,angle(i))
              else
                  call remove_bond_from_atoms(legm,n2i,n3i)
                  if (kpra > 0) call print_del_bond(n2i,n3i,angle(i))
              endif
          endif
       enddo
   endif
!
   CONTAINS
   
   subroutine print_del_bond(n1b,n2b,ang)
   integer, intent(in) :: n1b,n2b
   type(angle_type)    :: ang
   write(6,'(2x,a,f0.2)')'Bond '//trim(atom(n1b)%lab)//'-'//trim(atom(n2b)%lab)//' was deleted because the angle '//  &
              trim(atom(abs(ang%n1))%lab)//'-'//trim(atom(ang%n2)%lab)//'-'//trim(atom(ang%n3)%lab)//' is ',ang%val
   end subroutine print_del_bond
!
   end subroutine check_angle

!---------------------------------------------------------------------      

   subroutine create_connectivity_special(atom,dist,legm,esd,z1,z2)
!
!  Cerca legami fra atomi di specie z1 e z2 a distanza dist
!
   USE General, only:xgg
   USE cgeom
   USE connect_mod
   type(atom_type), dimension(:), intent(in)                 :: atom
   real, intent(in)                                          :: dist
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, intent(in), optional                             :: z1,z2
   real, intent(in), optional                                :: esd
   real                                                      :: esdval
   integer, dimension(size(atom))                            :: zval
   integer, dimension(size(atom),size(atom))                 :: tabconn
   integer                                                   :: i,j
   integer                                                   :: za1,za2
   real                                                      :: distij
   integer                                                   :: nat
!
   if (present(esd))then
       esdval = esd
   else
       esdval = 0.3
   endif
!
   nat = size(atom)
!
   tabconn(:,:) = 0
   if (nat > 1) then
!
!      genera Z di tutti gli atomi
       if (present(z1) .or. present(z2)) then
           zval(:) = atomic_number(atom)
           if (present(z1)) za1 = z1
           if (present(z2)) then
               za2 = z2
               if (.not.present(z1)) za1 = za2
           else
               za2 = za1
           endif
       else
!
!          nessun controllo controllo sulle specie
           zval(:) = 0
           za1 = 0
           za2 = 0
       endif
!
       do i=1,nat-1
          do j=i+1,nat
             if ((zval(i) == za1 .and. zval(j) == za2) .or. (zval(j) == za1 .and. zval(i) == za2)) then
                 distij = distanzaC(atom(i)%xc,atom(j)%xc,xgg)
                 if (abs(distij - dist) <= esdval) then
                     tabconn(i,j) = 1
                     tabconn(j,i) = 1
                 endif
             endif
          enddo
       enddo
   endif
!
   call tabconn_to_legm(tabconn,atom,esdval,legm)
!
   end subroutine create_connectivity_special

!---------------------------------------------------------------------      

   subroutine AppSymmBij(atoms, infos)
   USE Symmetry
   USE progtype
!
   type(atom_type), dimension(:)                  :: atoms
   type(symminfo_type), dimension(:), allocatable :: infos
   integer                                        :: ii, j
   real :: bijTmp(3,3) 
!
   do ii=1, size(atoms)
      if (atoms(ii)%bij(1) > 0.0) then
          bijTmp(1,1) = atoms(ii)%bij(1)
          bijTmp(2,2) = atoms(ii)%bij(2)
          bijTmp(3,3) = atoms(ii)%bij(3)
          bijTmp(1,2) = atoms(ii)%bij(4)
          bijTmp(2,1) = atoms(ii)%bij(4)
          bijTmp(1,3) = atoms(ii)%bij(5)
          bijTmp(3,1) = atoms(ii)%bij(5)
          bijTmp(2,3) = atoms(ii)%bij(6)
          bijTmp(3,2) = atoms(ii)%bij(6)
          j = infos(ii)%oper
          bijTmp(:,:) = matmul(matmul(transpose(kmat(j,:,:)),bijTmp(:,:)),kmat(j,:,:))
          atoms(ii)%bij(1) = bijTmp(1,1)
          atoms(ii)%bij(2) = bijTmp(2,2)
          atoms(ii)%bij(3) = bijTmp(3,3)
          atoms(ii)%bij(4) = bijTmp(1,2)
          atoms(ii)%bij(5) = bijTmp(1,3)
          atoms(ii)%bij(6) = bijTmp(2,3)
      endif
   enddo
!
   end subroutine AppSymmBij

!---------------------------------------------------------------------      

   integer function operator_code(kop,tra) result(opcode)
!
!  Genara codice per memorizzare la rotazione e la traslazione
!
   integer, intent(in)               :: kop
   integer, dimension(3), intent(in) :: tra
!
   opcode = kop*1000+(5+tra(1))*100+(5+tra(2))*10+(5+tra(3))
!
   end function operator_code

!---------------------------------------------------------------------      

   function tra_opcode(opcode)
!
!  Estrai la traslazione dal codice
!
   integer, intent(in)   :: opcode
   integer, dimension(3) :: tra_opcode
   integer               :: tcode
!
   tcode = mod(opcode,1000)
   tra_opcode(1) = tcode / 100 - 5
   tra_opcode(2) = mod(tcode,100) / 10 - 5
   tra_opcode(3) = mod(tcode,10) - 5
!
   end function tra_opcode

!---------------------------------------------------------------------      

   integer function op_from_opcode(opcode) result(op)
!
!  Symmetry operator from opcode
!
   integer, intent(in)               :: opcode
   !integer, dimension(:), intent(in) :: tra
   !op = (opcode - (5+tra(1))*100+(5+tra(2))*10+(5+tra(3))) / 1000
   op = opcode / 1000
   end function op_from_opcode

!---------------------------------------------------------------------      

   integer function translate_opcode(opcode,xtra) result(opnew)
!
!  Generate new opcode after translation
!
   integer, intent(in)               :: opcode
   integer, dimension(3), intent(in) :: xtra
   integer, dimension(3)             :: tra
   integer                           :: op
   tra = tra_opcode(opcode)
   op = op_from_opcode(opcode)
   opnew = operator_code(op,tra+xtra)
   end function translate_opcode

!---------------------------------------------------------------------      

   subroutine get_limit_translation(opcode,ktramin,ktramax)
!
!  Individua la min e la max traslazione
!
   integer, dimension(:), intent(in)  :: opcode
   integer, dimension(3), intent(out) :: ktramin,ktramax
   integer, dimension(3)              :: ktra
   integer                            :: i
!
   ktramin = tra_opcode(opcode(1))  
   ktramax = tra_opcode(opcode(1))  
   do i=2,size(opcode)
      ktra = tra_opcode(opcode(i))
      where (ktra < ktramin) ktramin = ktra
      where (ktra > ktramax) ktramax = ktra
   enddo
!
   end subroutine get_limit_translation

!---------------------------------------------------------------------      

   subroutine init_infos(infosym,kdim)
!
!  Inizializza informazioni sulla simmetria
!
   type(symminfo_type), dimension(:), allocatable, intent(inout) :: infosym
   integer, intent(in)                                           :: kdim
   integer                                                       :: i
!
   call reallocate_infos(infosym,kdim,.false.)
   do i=1,kdim
      infosym(i)%asym = i
      infosym(i)%oper = 1
      infosym(i)%opcode = 1555
   enddo
!
   end subroutine init_infos

!---------------------------------------------------------------------      

   logical function checkeq_infos(infosym,infosymvet) result(eq)
   type(symminfo_type), intent(in)               :: infosym
   type(symminfo_type), dimension(:), intent(in) :: infosymvet
   integer                                       :: i
!   
   eq = .false.
   do i=1,size(infosymvet)
      if (infosymvet(i)%opcode == infosym%opcode .and. infosymvet(i)%asym == infosym%asym) then
          eq = .true.
          exit
      endif
   enddo
!
   end function checkeq_infos

!---------------------------------------------------------------------      

   subroutine init_for_symm(atom,legm,atoms,legms,infos)
!
!  Inizialize symmetry by copying asymmetric unit
!
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in)  :: atom
   type(bond_type), dimension(:), allocatable, intent(in)  :: legm
   type(atom_type), dimension(:), allocatable, intent(out) :: atoms
   type(bond_type), dimension(:), allocatable, intent(out) :: legms
   type(symminfo_type), dimension(:), allocatable, intent(out) :: infos
!
   call copia_atoms(atoms,atom)
   call init_infos(infos,numatomtype(atoms))
   call copia_legm(legms,legm)
!
   end subroutine init_for_symm

!---------------------------------------------------------------------      

   integer function nsymminfo(symm)
   type(symminfo_type), dimension(:), allocatable :: symm
!   
   if (allocated(symm)) then
       nsymminfo = size(symm)
   else
       nsymminfo = 0
   endif
!
   end function nsymminfo

!---------------------------------------------------------------------      
   
   subroutine copia_infos(symm1,symm2)
   type(symminfo_type), dimension(:), allocatable, intent(out) :: symm1
   type(symminfo_type), dimension(:), allocatable, intent(in)  :: symm2
   integer                                                     :: nsinfo
!
   nsinfo = nsymminfo(symm2) 
   call reallocate_infos(symm1,nsinfo,savevet=.false.)
   if (nsinfo > 0) then
       symm1(:) = symm2(:)
   endif
!
   end subroutine copia_infos

!---------------------------------------------------------------------      

   subroutine reallocate_infos(vetr,n,savevet)
!
!  Rialloca ad n un vettore reale.
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(symminfo_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)              :: n
   logical, optional, intent(in)    :: savevet
   logical                          :: savev
   integer                          :: nv
   type(symminfo_type), allocatable :: vsav(:)
   integer                          :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
           allocate(vsav(n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
       else
           deallocate(vetr)
           allocate(vetr(n))
       endif
   endif
!
   end subroutine reallocate_infos

!--------------------------------------------------------------------------------------------------

   subroutine update_infos(infos,vet) 
   type(symminfo_type), allocatable, intent(inout) :: infos(:)
   integer, dimension(:), intent(in)                      :: vet  
   integer, dimension(size(vet))                          :: iord
   integer                                                :: j1  
   integer                                                :: i
   integer                                                :: nums
   type(symminfo_type), allocatable                :: infosnew(:)
   integer                                                :: numinfo
   integer                                                :: nat
!
   j1 = 0
   do i=1,size(vet)
      if (vet(i) > 0) then
          j1 = j1 + 1
          iord(i) = j1
      endif
   enddo
   numinfo = size(infos)
   call reallocate_infos(infosnew,numinfo,.false.)
   nums = 0
   do i=1,size(infos)
      nat = infos(i)%asym
      if (vet(nat) > 0) then
          nums = nums + 1   
          infosnew(nums)%asym = iord(nat)
          infosnew(nums)%oper = infos(i)%oper
          infosnew(nums)%opcode = infos(i)%opcode
      endif
   enddo
   call reallocate_infos(infos,nums,.false.)
   if (nums > 0) infos(:) = infosnew(:nums)
!
   end subroutine update_infos

!--------------------------------------------------------------------------------------------------
   subroutine remove_infos(infos)
!
!  Rimuovi infos(i) con infos(i)%oper uguale a 0
!
   type(symminfo_type), allocatable, intent(inout) :: infos(:)
   integer                                                :: nrem
   type(symminfo_type), allocatable                :: infosnew(:)
   integer                                                :: i
   integer                                                :: newsize
!
   nrem = count(infos(:)%oper == 0)
   if (nrem > 0) then
       newsize = size(infos) - nrem
       call reallocate_infos(infosnew,newsize,.false.)
!
       newsize = 0
       do i=1,size(infos)
          if (infos(i)%oper /= 0) then
              newsize = newsize + 1
              infosnew(newsize) = infos(i)
          endif
       enddo
       call reallocate_infos(infos,newsize,.false.)
       infos(:) = infosnew(:)
   endif
!
   end subroutine remove_infos
!--------------------------------------------------------------------------------------------------
   subroutine connect_groups(atom1,atom2,legm)
!
!  Cerca connettivita' tra 2 gruppi di atomi in coordinate cartesiane
!
   USE realloc
   USE bondtmod
   USE connect_mod
   USE cgeom
   type(atom_type), dimension(:), intent(in)                 :: atom1,atom2
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer                                                   :: nat1,nat2
   integer                                                   :: i,j  
   integer                                                   :: nleg
   integer, dimension(size(atom1))                           :: zval1
   integer, dimension(size(atom2))                           :: zval2
   type(bond_type)                                           :: legnew
   real                                                      :: tolmin,tolmax
!
   nat1 = size(atom1)
   nat2 = size(atom2)
   if (nat1 == 0 .or. nat2 == 0) return
!
!  genera Z di tutti gli atomi
   zval1(:) = atomic_number(atom1)
   zval2(:) = atomic_number(atom2)
!
   call reallocate_legm(legm,nat1*nat2,.false.)
!
   tolmin = 0.0
   tolmax = 0.0
   nleg = 0
   do i=1,nat1
      if (zval1(i) /= 0) then
          do j=1,nat2
             if (zval2(j) /= 0) then
                 call create_bond(i,j,atom1(i)%xc,atom2(j)%xc,zval1(i),zval2(j),tolmin,tolmax,legnew)
                 if (legnew%n1 > 0) then  ! se 0 non c'è legame tra i due atomi
                     nleg = nleg + 1
                     legm(nleg) = legnew
                 endif
             endif
          enddo
      endif
   enddo
!
   call reallocate_legm(legm,nleg)
!
   end subroutine connect_groups

!---------------------------------------------------------------------      

   subroutine translate_in_cell(atom,xtra)
!
!  Trasla in cella in gruppo di atomi
!
   type(atom_type), dimension(:), intent(inout) :: atom
   real, dimension(3), optional, intent(out)    :: xtra
   integer                                      :: nat
   real, dimension(3)                           :: barini,barfin
   real, dimension(3)                           :: xtrasl
!
   nat = size(atom)
!
   if (is_bar_out_of_cell(atom)) then
       barini(:) = baricentro(atom)
       barfin(:) = mod(barini(:)+10,1.0)
       xtrasl(:) = barfin(:) - barini(:)
       call translate_atoms(atom,xtrasl)
       if (present(xtra)) xtra(:) = xtrasl(:)
   else
       if (present(xtra)) xtra(:) = 0
   endif
!
   end subroutine translate_in_cell

!---------------------------------------------------------------------      

   logical function is_bar_out_of_cell(atom)
!
!  Controlla se il baricentro e' fuori cella
!
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(3) :: bar
!
   bar(:) = baricentro(atom)
   is_bar_out_of_cell = any(bar(:) >= 1.0 .or. bar(:) < 0.0) 
!
   end function is_bar_out_of_cell

!---------------------------------------------------------------------      

   logical function is_out_of_cell(atom)
!
!  Sono tutti gli atomi fuori cella?
!
   type(atom_type), dimension(:), intent(in) :: atom
   integer                                   :: nat
   integer                                   :: i
   integer                                   :: nin
   real                                      :: tol = 4.0*epsilon(1.0)
!
   nat = size(atom)
   nin = 0
   do i=1,nat
      if (any(atom(i)%xc < 0.0-tol .or. atom(i)%xc >= 1.0+tol)) then 
          nin = nin + 1
      endif
   enddo
   is_out_of_cell = nin == nat
!
   end function is_out_of_cell

!---------------------------------------------------------------------      

   logical function is_out_of_range(atom,inf,sup,code)
!
!  Atoms fuori dal range?
!
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(3), intent(in)            :: inf,sup
   integer, intent(in)                       :: code
   real, dimension(3)                        :: xbar
   integer                                   :: nat
   integer                                   :: i
   integer                                   :: nin
!
   nat = size(atom)
   nin = 0
   select case (code)
      case (1) !  is centroid out of range?
        xbar = baricentro(atom)
        is_out_of_range = any(xbar < inf .or. xbar > sup)

      case (2)               ! all atoms out of range?
        do i=1,nat
           if (any(atom(i)%xc < inf .or. atom(i)%xc > sup)) then
               nin = nin + 1
           endif
        enddo
        is_out_of_range = nin == nat

      case (3)               ! just an atom out of range?
        is_out_of_range = .false.
        do i=1,nat
           if (any(atom(i)%xc < inf .or. atom(i)%xc > sup)) then
               is_out_of_range = .true.
               exit
           endif
        enddo
   end select
!
   end function is_out_of_range

!---------------------------------------------------------------------      

   function baricentro(at)
!
!  Calcola il baricentro di at
!   
   type(atom_type), dimension(:), intent(in) :: at
   real, dimension(3)                        :: baricentro
   integer                                   :: i
   integer                                   :: numat
!
   numat = size(at)
   do i=1,3
      baricentro(i) = sum(at%xc(i)) / numat
   enddo
!   
   end function baricentro

!---------------------------------------------------------------------      

   subroutine get_info_distance(atom1,atom2,pos,dist,errd)
!
!  Info sulla distanza tra 2 atomi in coordinate cartesiane
!
   USE connect_mod
   type(atom_type), intent(in)    :: atom1,atom2 ! atomi in coordinate cartesiane
   integer, optional, intent(out) :: pos         ! puntatore alla distanza sulla tabella dei legami
   real, optional, intent(out)    :: dist        ! distanza tra i 2 atomi
   real, optional, intent(out)    :: errd        ! deviazione dal valore in tabella
   integer                        :: z1,z2
   integer                        :: kpos
   real                           :: dist12, distt, diffmin
!
   z1 = atomic_number(atom1)
   z2 = atomic_number(atom2)
!   
   dist12 = sqrt(dot_product(atom1%xc-atom2%xc,atom1%xc-atom2%xc))
!
   call bond_info(z1,z2,dist12,kpos,distt,diffmin)
   if (present(pos)) pos = kpos
   if (present(errd)) errd = diffmin
   if (present(dist)) then
       if (kpos > 0) then
           dist = distt
       else
!!!FIXME - suggest distance from radius
           dist = dist12
       endif
   endif
!
   end subroutine get_info_distance

!---------------------------------------------------------------------      

   real function bond_distanceZ(z1,z2)  result(dist)
!
!  Search distance between atoms with Z number z1 and z2
!
   USE elements
   USE bondtmod
   integer, intent(in) :: z1,z2
   if (z1 == 0 .or. z2 == 0 .or. z1 > N_ELEMENTS .or. z2 > N_ELEMENTS) then
       dist = 0.0
   else
       dist = bond_table(1,z1,z2)
       if (dist == 0.0) then   ! distance absent on table: use covelent radius
           dist = eleminfo(z1)%c_radius + eleminfo(z2)%c_radius
       endif
   endif
!
   end function bond_distanceZ

!---------------------------------------------------------------------      

   real function bond_distanceAT(at1,at2) result(dist)

!  Find distance from table, if absent compute from radius
!
   type(atom_type), intent(in) :: at1,at2
   integer                     :: z1,z2
!
   z1 = atomic_number(at1)
   z2 = atomic_number(at2)
   dist = bond_distance(z1,z2)
!
   end function bond_distanceAT

!---------------------------------------------------------------------      

   subroutine get_radius_molecule(atom,radmax,bar)
!
!  Assimila la molecola ad una sfera e calcola il suo raggio come massima distanza dal baricentro
!
   USE cgeom
   type(atom_type), dimension(:), intent(in) :: atom  ! in coordinate cartesiane
   real, intent(out)                         :: radmax
   real, dimension(3), intent(out), optional :: bar
   real, dimension(3)                        :: barc
   integer                                   :: i
   real                                      :: rad
!
   barc = baricentro(atom)
!
   radmax = distanzaC(barc,atom(1)%xc)
   do i=2,size(atom)
      rad = distanzaC(barc,atom(i)%xc)
      if (rad > radmax) radmax = rad
   enddo
!
   if (present(bar)) then
       bar(:) = barc(:)
   endif
!
   end subroutine get_radius_molecule

!---------------------------------------------------------------------      

   subroutine save_atoms(atoms)
!
!  Genera copia degli atomi
!
   USE realloc
   type(atom_type), dimension(:), intent(in) :: atoms
   integer                       :: natoms
!
   natoms = size(atoms)  
   call reallocate(atoms_copy,natoms)
   atoms_copy(:) = atoms(:)
!
   end subroutine save_atoms

!---------------------------------------------------------------------      

   subroutine restore_atoms(atoms,natoms)
!
!  Ripristina copia degli atomi
!
   USE realloc
   type(atom_type), dimension(:), allocatable, intent(inout) :: atoms
   integer, intent(out)                                      :: natoms
!
   if (allocated(atoms_copy)) then
       natoms = size(atoms_copy)
       call reallocate(atoms,natoms,.false.)
       atoms(:) = atoms_copy(:)
   endif
!
   end subroutine restore_atoms

!---------------------------------------------------------------------      

   subroutine clear_saved_atoms()
!
!  Cancella la copia degli atomi
!
   USE realloc
   call reallocate(atoms_copy,0)
   end subroutine clear_saved_atoms

!---------------------------------------------------------------------      

   subroutine copia_atoms(atom1,atom2)
!
!  Copia atom2 in atom1
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom1
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom2
   integer                                                   :: nat2,nat1
!
   nat2 = numatomtype(atom2) 
   nat1 = numatomtype(atom1) 
   if (nat1 == nat2) then
       if (nat1 /= 0) atom1 = atom2
   else
       if (allocated(atom1)) deallocate(atom1)
       if (nat2 > 0) then
           allocate(atom1(nat2),source=atom2)
       endif
   endif
!
   end subroutine copia_atoms

!---------------------------------------------------------------------      

!corr   subroutine remove_atoms_from_list(atom,veta,val,icon,legm,iord)
   subroutine remove_atoms_from_list(atom,veta,val,legm,iord)
!
!  Cancella dalla lista tutti gli atomi per i quali vet e' uguale a val
!
   USE realloc
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(inout)           :: atom
   integer, dimension(:), intent(in)                                   :: veta
   integer                                                             :: val
!corr   integer, dimension(:), intent(inout), optional                      :: icon
   type(bond_type), dimension(:), allocatable, intent(inout), optional :: legm
   integer, dimension(:), optional                                     :: iord
   integer                                                             :: nat
   integer                                                             :: nrem
   integer                                                             :: nnat
   integer                                                             :: j1,j2
   integer                                                             :: i
   integer, dimension(size(atom))                                      :: vetnew
   integer, dimension(size(atom))                                      :: iorder
!
   nrem = count(veta(:) == val)
   if (nrem > 0) then
       nat = numatomtype(atom)
       nnat = nat - nrem
       j1 = 0
       j2 = nnat
       do i=1,nat
          if (veta(i) /= val) then
              j1 = j1 + 1
              vetnew(j1) = i
              iorder(i) = j1
          else
              j2 = j2 + 1
              vetnew(j2) = i
              iorder(i) = 0
          endif
       enddo
       atom(:nat) = atom(vetnew)
       call reallocate(atom,nnat)
!
!      aggiorna la connettivita' se richiesto
!corr       if (present(icon)) then   
!corr          call remove_atoms_from_iconn(vetnew(nnat+1:),icon)
!corr       endif
       if (present(legm)) then   ! aggiorna la connettivita' se richiesto
           call remove_bond_from_atom(legm,vetnew(nnat+1:),corr=.true.)
       endif
!
!      iord e' tale che iord(vecchia posizione) = nuova posizione
       if (present(iord)) then
           do i=1,nat
              iord(i) = iorder(i)
           enddo
       endif
   else
       if (present(iord)) then
           iord(:) = [(i,i=1,size(iord))]
       endif
   endif
!
   end subroutine remove_atoms_from_list

!---------------------------------------------------------------------      

   subroutine remove_atoms_sym(atom,atoms,legm,legms,symm,veta,iorder)
!
!  Rimuovi gli atomi in vet con gli equivalenti
!
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(inout)     :: atom,atoms
   type(bond_type), dimension(:), allocatable, intent(inout)     :: legm,legms
   type(symminfo_type), dimension(:), allocatable, intent(inout) :: symm
   integer, dimension(:), intent(in)                             :: veta
   integer, dimension(:), allocatable, intent(out)               :: iorder
   integer, dimension(size(veta))                                :: vetac
   integer                                                       :: nptra
   integer                                                       :: i
!
   nptra = size(veta)
   do i=1,nptra
      vetac(i) = symm(veta(i))%asym ! prendi l'asimmetrico corrispondente
      atom(vetac(i))%nz = -999       ! marca atomi da eliminare nell'u.a.
   enddo
!
!  Elimina gli atomi da atomviews, cioè da tutta la cella
   do i=1,size(atoms)
      if (any(vetac == symm(i)%asym)) then
          atoms(i)%nz = -999
      endif
   enddo
!
!  Elimina atomi
   call update_infos(symm,atom(:)%nz)
   allocate(iorder(size(atoms)))
   call remove_atoms_from_list(atoms,atoms(:)%nz,-999,legm=legms,iord=iorder)
   call remove_atoms_from_list(atom,atom(:)%nz,-999,legm=legm)   ! elimina dall'u.a.
!
   end subroutine remove_atoms_sym

!---------------------------------------------------------------------      

   subroutine remove_atoms_vet(atom,legm,veta,iord,keep)
!
!  Remove atoms in array veta. If keep is true only atoms in vet are kept
!
   USE progtype
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, dimension(:), intent(in)                         :: veta
   integer, dimension(:), allocatable, intent(out), optional :: iord
   logical, intent(in), optional                             :: keep
   integer                                                   :: i
   logical                                                   :: keep1
!
   if (present(keep)) then
       keep1 = keep
   else
       keep1 = .false.
   endif
!
   if (keep1) then
       do i=1,size(atom)
          if (any(veta(:) == i)) cycle
          atom(i)%nz = -999
       enddo
   else
       !do i=1,size(veta)
       !   atom(veta(i))%nz = -999
       !enddo
       atom(veta(:))%nz = -999
   endif
!
   if (present(iord)) then
       allocate(iord(size(atom)))
       call remove_atoms_from_list(atom,atom%nz,-999,legm=legm,iord=iord)
   else
       call remove_atoms_from_list(atom,atom%nz,-999,legm=legm)
   endif
!
   end subroutine remove_atoms_vet

!---------------------------------------------------------------------      

   integer function is_organic_z(zval,nzval)
!
!  is_organic = 1 -> organica
!             = 2 -> metallorganica                
!             = 0 -> inorganica
!
   USE elements
   integer, dimension(:), intent(in) :: zval   ! Z delle specie atomiche
   integer, dimension(:), intent(in) :: nzval  ! numero di specie atomiche
   integer, dimension(11), parameter :: vetorg = (/C_at,H_at,B_at,N_at,O_at,P_at,S_at,F_at,Cl_at,Br_at,I_at/)
   integer, dimension(size(zval))    :: inorg
   integer                           :: i
   integer                           :: nsp
   real                              :: perci
   integer                           :: sinorg
   integer                           :: numO,numOi
!
!  Una specie, organica o mettallorg. deve contenere C (Z=6)
   if (any(zval(:) == C_at)) then
       nsp = size(zval)
!
!      le altre specie devono essere H(1),N(7),O(8),P(15),S(16),F(9),Cl(17),Br(35),I(53)
!      memorizza nel vettore inorg le specie inorganiche, cioe' diverse da vetorg
       do i=1,nsp
          if (any(vetorg(:) == zval(i))) then
              inorg(i) = 0   
          else
              inorg(i) = nzval(i)
          endif
       enddo
!
       sinorg = sum(inorg(:)) ! somma delle specie inorganiche
       if (sinorg == 0) then
!           
!          non ci sono specie inorganiche
           is_organic_z = 1
       else
!
!          Ma O potrebbero appartenere anche alla parte inorganica (es. zeoliti)
           numO = number_for_specie(O_at,zval,nzval)  ! numero di O
           if (numO > 0) then
               numOi = 0
               do i=1,nsp
                  if (inorg(i) > 0 .or. zval(i) == P_at) then   ! P is also inorganic if O are present
!                     oxygen for inorganic atom = 0.7*num. di ossidazione 
                      numOi = numOi + nint(0.7*oxidation_number(zval(i))) * nzval(i)
                  endif
               enddo
               if (numOi > numO) numOi = numO
               sinorg = sinorg + numOi
           endif
           perci=100*sinorg/sum(nzval(:))
!
           if (perci <= 20) then
               is_organic_z = 2    ! metallorganica
           else
               is_organic_z = 0    ! inorganica
           endif
       endif
   else  
!
!      se non contiene C e' inorganica
       is_organic_z = 0
   endif
!
   end function is_organic_z

!---------------------------------------------------------------------      

   integer function is_organic_atm(atom)
   USE elements
   type(atom_type), dimension(:), intent(in)     :: atom
   type(element_type), dimension(:), allocatable :: elem
!   
   call elem_from_atoms(elem,pxen_from_nz(atom%nz))
   if (numelem(elem) > 0) then
       is_organic_atm = is_organic(elem%z,nint(elem%nw))
   else
       is_organic_atm = 0
   endif
!
   end function is_organic_atm

!---------------------------------------------------------------------      

   subroutine sort_by_specie(atom,legm,iorder)
!
!  Ordine atomi in base alla specie e al seriale della label
!
   USE nr
   USE connect_mod
   USE atom_basic
   USE cgeom
   USE unit_cell
   USE General, only:cell
   type(atom_type), dimension(:), intent(inout)                        :: atom
   type(bond_type), dimension(:), allocatable, intent(inout), optional :: legm
   integer, dimension(size(atom)), optional                            :: iorder
   integer, dimension(size(atom))                                      :: iord
   integer                                                             :: nat
   integer                                                             :: i
   integer, dimension(size(atom))                                      :: iser
   type(atom_type), dimension(size(atom))                              :: atomc
   character(len=100)                                                  :: sprefix
   integer                                                             :: ierror
   integer                                                             :: nleg
!
!  Estrai dalla label il seriale. (es. C1 prima di C2, C3)
   do i=1,size(atom)
      call get_from_label(atom(i)%lab,sprefix,iser(i),ierror)
   enddo
!
!  Genera nuovo indice degli atomi in base al numero atomico
   call indexx(atomic_number(atom)*1000-iser,iord)
   nat = size(atom)
   iord = iord(nat:1:-1)
!
!  Ordina gli atomi
   atom(:) = atom(iord)
!
!  Ordina i legami
   if (present(legm)) then
       nleg = numbonds(legm)
       if (nleg > 0) then
           iord(iord) = (/(i,i=1,nat)/)  ! per i legami serve un indice inverso
           legm(:)%n1 = iord(legm(:)%n1)
           legm(:)%n2 = iord(legm(:)%n2)
!
!          ricalcola le distanze
           atomc(:) = atom(:)
           call frac_to_cart(atomc,orthomatrix(cell))
           do i=1,nleg
              legm(i)%dist = distanzaC(atomc(legm(i)%n1)%xc,atomc(legm(i)%n2)%xc)
           enddo
       endif
   endif
!
   if (present(iorder)) then
       iorder(:) = iord(:)
   endif
!
   end subroutine sort_by_specie

!---------------------------------------------------------------------      

   subroutine force_connectivity(atom,legm,kat)
!
!  Forza la connettivita' per l'atomo kat
!
   USE general, only:xgg
   USE connect_mod
   USE cgeom
   type(atom_type), dimension(:), intent(in)                 :: atom  
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer                                                   :: kat
   integer, dimension(1)                                     :: vet1,vet2
   integer                                                   :: kfind
   integer                                                   :: i
   real                                                      :: dist,distmin
!
!  Cerca l'atomo piu' vicino a kat
   distmin = huge(1.0)
   kfind = 0
   do i=1,size(atom)
      if (i == kat) cycle
      dist = distanzac(atom(i)%xc,atom(kat)%xc,xgg)
      if (dist < distmin) then
          kfind = i
          distmin = dist
      endif
   enddo
!
!  Aggiungi il legame
   if (kfind > 0) then
       vet1(1) = kat
       vet2(1) = kfind
       call add_bonds(legm,atom,vet1,vet2)
   endif
!
   end subroutine force_connectivity

!---------------------------------------------------------------------      

   subroutine renumber_atoms(atom,veta,string,iser,vetd,nd)
!
!  Renumber of atoms
!
   USE strutil
   USE Atomi, only:nzConst
   USE General, only:Species
   type(atom_type), dimension(:), intent(inout)          :: atom
   integer, dimension(:), intent(in)                     :: veta   ! atomi da rinumerare
   character(len=*), intent(in)                          :: string ! prefisso della label; se vuota si usa la specie
   integer, intent(in)                                   :: iser   ! inizio della numerazione
   integer, dimension(size(veta)), intent(out), optional :: vetd   ! restituisce le stringhe che si duplicano
   integer, optional                                     :: nd     ! numero di duplicazioni
   integer                                               :: i
   integer                                               :: kat
   logical                                               :: is_string
   integer                                               :: nspec
   integer, dimension(size(veta))                        :: vetdd
   integer                                               :: ndouble
   integer                                               :: lens
!
   ndouble = 0    ! conta le stringa che verranno eventualmente duplicate
   lens = len_trim(string)
   do i=1,size(veta)
      kat = veta(i)
      nspec = iabs(atom(kat)%nz)/nzConst   ! puntatore alla specie
      if (nspec > 0) then   ! ghost atom?
          if (lens == 0) then    ! renumber usando la specie come prefisso
              write(atom(kat)%lab,'(a,i0)')trim(Species(nspec)),iser+i-1
          else
              write(atom(kat)%lab,'(a,i0)')trim(string),iser+i-1
          endif
          is_string = string_locate(atom(kat)%lab,atom(:)%lab,vet=veta) > 0  ! controlla che la nuova stringa non esista gia'
          if (is_string) then         ! memorizza le duplicazioni di stringa
              ndouble = ndouble + 1
              vetdd(ndouble) = kat
          endif
      endif
   enddo
!
!  Salva in uscita le duplicazioni
   if (present(vetd) .and. present(nd)) then
       nd = ndouble
       vetd(:nd) = vetdd(:nd)
       !do i=1,ndouble
       !   write(0,*)'Duplicate string '//trim(atom(vetdd(i))%lab)
       !enddo
   endif
!
   end subroutine renumber_atoms

!---------------------------------------------------------------------      

   subroutine get_minimum_distance(at,atom,kmin,dmin)
!
!  Distanza minima di at dagli atomi in atom in cartesiane
!
   USE cgeom
   type(atom_type), intent(in)                 :: at    
   type(atom_type), dimension(:), intent(in)   :: atom 
   integer, intent(out)                        :: kmin ! atom(kmin) a distanza minima
   real, intent(out)                           :: dmin
   real                                        :: dist
   integer                                     :: i
!
   dmin = distanzaC(at%xc,atom(1)%xc)
   kmin = 1
   do i=2,size(atom)
      dist = distanzaC(at%xc,atom(i)%xc)
      if (dist < dmin) then
          dmin = dist
          kmin = i
      endif
   enddo
!
   end subroutine get_minimum_distance

!---------------------------------------------------------------------      

   subroutine get_atoms_distance(at,atom,vetd,distv,vexcl)
!
!  Calcola distanza di at dagli atomi in atom (in coord. cartesiane)
!
   USE nr
   USE cgeom
   type(atom_type), intent(in)                 :: at    
   type(atom_type), dimension(:), intent(in)   :: atom 
   integer, dimension(:), intent(out)          :: vetd  ! contine gli atomi ordinati dal piu' vicino
   real, dimension(:), intent(out), optional   :: distv ! le distanze corrispondenti
   integer, dimension(:), intent(in), optional :: vexcl ! atomi da escudere in atom
   integer                                     :: i
   integer                                     :: nd
   integer, dimension(size(atom))              :: vete,vetnd
   real, dimension(size(atom))                 :: distvi
!
   vete(:) = 0
   if (present(vexcl)) then
       vete(vexcl) = 1
   endif
!
   nd = 0
   do i=1,size(atom)
      if (vete(i) == 1) cycle   ! atomo da escludere
      nd = nd + 1
      vetnd(nd) = i   ! memorizza il puntatore ad atom
      distvi(nd) = distanzaC(at%xc,atom(i)%xc)
   enddo
!
!  Genera indece vetd
   call indexx(distvi(:nd),vetd(:nd))
!
!  Ordina le distanze
   if (present(distv)) then
       distv(:nd) = distvi(vetd(:nd))
   endif
!
   vetd(:nd) = vetnd(vetd(:nd))
!
   end subroutine get_atoms_distance

!---------------------------------------------------------------------      

   !subroutine get_groups_distance(atm1,atm2,dmin,dmed)
   !USE cgeom
   !type(atom_type), dimension(:), intent(in) :: atm1,atm2 
   !integer, dimension(size(atm1))            :: z1
   !integer, dimension(size(atm2))            :: z2
   !real, intent(out)                         :: dmin, dmed
   !integer                                   :: nat1, nat2
   !integer                                   :: i,j
   !real                                      :: dist
   !integer                                   :: btype
   !real                                      :: distab
!!
!   nat1 = size(atm1)
!   nat2 = size(atm2)
!   z1(:) = atomic_number(atm1)
!   z2(:) = atomic_number(atm2)
!   dmin = huge(1.0)
!   dmed = 0
!   do i=1,nat1
!      do j=1,nat2
!         dist = distanzaC(atm1(i)%xc,atm2(j)%xc)
!         call bond_info(z1(i),z2(j),dist,btype,distab)
!         if (dist < dmin) then
!             dmin = dist
!         endif
!         dmed = dmed + dist
!      enddo
!   enddo
!   dmed = dmed / (nat1*nat2)
!
!   end subroutine get_groups_distance

!---------------------------------------------------------------------      

   subroutine get_distance_radius(atom,kat,distm,mode,kfound,vat,vdist,vsym)
!
!  Trova gli atomi a distanza dist da kat
!
   USE cgeom
   USE Symmetry
   USE general, only:xgg
   USE realloc
   USE nr
   type(atom_type), dimension(:), intent(in)         :: atom       ! tutti gli atomi 
   integer, intent(in)                               :: kat        ! atomo su cui eseguire la ricerca
   real, intent(in)                                  :: distm      ! raggio della sfera entro cui cercare gli atomi
   integer, intent(in)                               :: mode       ! modalita' di ricerca (leggi sotto)
   integer, intent(out)                              :: kfound     ! numero di trovati
   integer, dimension(:), allocatable, intent(inout), optional :: vat,vsym   ! atomi trovati e operatore di simmetria corrispondente
   real, dimension(:), allocatable, intent(inout), optional    :: vdist      ! distanze
   type(atom_type)                                   :: atomk 
   real, dimension(3)                                :: ktra
   real                                              :: dd
   integer                                           :: ndist
   integer                                           :: i,k
   integer                                           :: nat
   real, dimension((icent+1)*nsym*ncentr*27)         :: disteq
   integer, dimension((icent+1)*nsym*ncentr*27)      :: vds
   integer                                           :: ndisteq
   integer, dimension(:), allocatable                :: iord
   integer, dimension(1)                             :: loc
   integer                                           :: ndim
   type(atom_type)                                   :: atom1  
   integer                                           :: k1,k2,k3
   integer                                           :: opcode
   real, dimension(3) :: diff
!
   nat = size(atom)
   ndist = 0
   atomk = atom(kat)
   ndim = nat
   if (present(vat)) call reallocate_dist(ndim)
   kfound = 0
!
   select case (mode)   ! cerca atomi senza considerare gli aquivalenti
      case (1)
        do i=1,nat
           if (i == kat) cycle
           dd = distanzaC(atomk%xc,atom(i)%xc,xgg)
           if (dd <= distm) then
               kfound = kfound + 1
               vat(kfound) = i       
               vdist(kfound) = dd
               vsym(kfound) = 1555
           endif
        enddo

      case (2)          ! cerca considerando anche gli equivalenti
        do i=1,nat
           do k=1,nsymTot
              atom1 = atom_symm(k,atom(i))
              diff(:) = nint(atomk%xc - atom1%xc)
              do k1=-1,1
                 do k2=-1,1
                    do k3=-1,1
                       ktra = (/k1,k2,k3/) + diff(:)
                       opcode = operator_code(k,nint(ktra))
                       if (opcode == 1555 .and. i==kat) cycle
                       dd = distanzaC(atomk%xc,atom1%xc+ktra,xgg)
                       if (dd <= distm) then
!
!                          Ora accetta il nuovo atomo
                           kfound = kfound + 1
                           if (present(vat)) then
                               if (ndim < kfound) then
                                   ndim = ndim + nat
                                   call reallocate_dist(ndim)
                               endif
                               vat(kfound) = i       
                               vdist(kfound) = dd
                               vsym(kfound) = opcode
                           endif
                       endif
                    enddo
                 enddo
              enddo
           enddo
        enddo
!!!!!prova
        !do i=1,nat
        !   call distance_equivalent(atomk%xc,atom(i)%xc,xgg,dd)
        !   if (dd < distm) then
        !       write(0,*)'atom:'//trim(atom(i)%lab),dd
        !   endif
        !enddo
!!!!!prova

      case (3)     ! per ogni atomo fornisce solo l'equivalente piu' vicino
        do i=1,nat
           ndisteq = 0
           do k=1,nsymTot
              !atom1(1) = atom(i)
              !call applica_sym_oper(k,atom1)
              atom1 = atom_symm(k,atom(i))
              do k1=-2,2
                 do k2=-2,2
                    do k3=-2,2
                       ktra = (/k1,k2,k3/)
                       opcode = operator_code(k,nint(ktra))
                       if (opcode == 1555 .and. i==kat) cycle
                       dd = distanzaC(atomk%xc,atom1%xc+ktra,xgg)
                       if (dd <= distm) then
                           ndisteq = ndisteq + 1
                           disteq(ndisteq) = dd
                           vds(ndisteq) = opcode
                       endif
                    enddo
                 enddo
              enddo
           enddo
           if (ndisteq > 0) then
               loc = minloc(disteq(:ndisteq))
               kfound = kfound + 1
               vat(kfound) = i       
               vdist(kfound) = disteq(loc(1))
               vsym(kfound) = vds(loc(1))
           endif
        enddo
   end select
!
   if (kfound > 0 .and. present(vat)) then
!
!      ricompatta i vettori vdist e vat
       call reallocate_dist(kfound)
!
!      Ordina le distanze dalla piu' piccola
       allocate(iord(kfound))
       call indexx(vdist,iord)
       vdist(:) = vdist(iord)
       vat(:) = vat(iord)
       vsym(:) = vsym(iord)
   endif

   contains
  
   subroutine reallocate_dist(nsize)
   integer, intent(in) :: nsize
   call reallocate(vdist,nsize)
   call reallocate(vat,nsize)
   call reallocate(vsym,nsize)
   end subroutine reallocate_dist
!
   end subroutine get_distance_radius

!---------------------------------------------------------------------      

   integer function atom_hybridization(kat,legm,zval)   result(hyb)
!
!  Calcola il tipo di ibridazione di un atomo
!  0: ibridazione non identificabile
!  1: ibridazione sp
!  2: ibridazione sp2
!  3: ibridazionne sp3
!
   USE connect_mod
   integer, intent(in)                       :: kat
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer                                   :: zval
   integer                                   :: nbond
!
   hyb = 0
   nbond = number_of_bonds(legm,kat)
!
   select case(nbond)
     case (0)     
!
!       atomo isolato -> assegna ibridazione sp3 se B,C,N,O,S
        select case (zval)
           case (5,6,7,8,16)   ! B,C,N,O,S
             hyb = 3
           case default  
             hyb = 0
        end select

     case (1)

     case (2)
     case (3)
     case (4)
     case default
      hyb = 0
   end select
!
   end function atom_hybridization

!---------------------------------------------------------------------      

   subroutine atom_string(atom,vet)
!
!  Crea stringa degli atomi dalla specie chimica
!
   USE general, only:Species
   USE Atomi, only:nzConst
   USE strutil
   USE atom_basic
   type(atom_type), dimension(:), intent(inout) :: atom
   integer, dimension(:), intent(in), optional  :: vet  ! labella solo gli atomi indicati in vet
   integer                                      :: nat
   integer, dimension(:), allocatable           :: nzz
   integer, dimension(:), allocatable           :: vnk
   integer                                      :: iz
   integer                                      :: i,j
   integer                                      :: nkk
   integer                                      :: numb
   integer                                      :: ierror
   integer                                      :: natl
   character(len=SIZELAB)                       :: spec
   integer                                      :: kat
   integer                                      :: kspec
!
   nat = size(atom)   ! numero totale di atomi
   if (present(vet)) then
!
!      Crea stringa solo per gli atomi in vet
       natl = size(vet)  ! numero di atomi da labellare
!
!      Calcola in vnk il numero di atomi per specie
       allocate(nzz(natl))
       nzz(:) = iabs(atom(vet(:))%nz/nzConst)
       nkk = maxval(nzz)
       allocate(vnk(0:nkk))
       vnk(:) = 0
!
!      Per ognuna delle specie metti in vnk il massimo numero seriale
       do j=1,nat
          if (any(vet == j)) cycle  ! non considerare gli atomi da labellare
          call get_from_label(atom(j)%lab,spec,numb,ierror) ! estrai label e seriale 
          if (ierror == 0) then
              kspec = -1
              if  (s_eqi(spec,'Q')) then  ! e' un ghost
                   kspec = 0
              else
                   do i=1,nkk
                      if (s_eqi(spec,Species(i))) then
                          kspec = i; exit
                      endif
                   enddo
              endif
              if (kspec >= 0) then
                  if (numb > vnk(kspec)) vnk(kspec) = numb 
              endif
          endif
       enddo
!
!      Genera la stringa
       do i=1,natl
          kat = vet(i)
          iz = nzz(i)
          vnk(iz) = vnk(iz) + 1
          if (iz == 0) then
              write(atom(kat)%lab,'(a,i0)')'Q',vnk(iz)  ! ghost atom
          else
              write(atom(kat)%lab,'(a,i0)')trim(Species(iz)),vnk(iz)
          endif
       enddo
   else
!
!      Crea stringa per tutti gli atomi
       allocate(nzz(nat))
       nzz(:) = iabs(atom(:)%nz/nzConst)
       nkk = maxval(nzz)
       allocate(vnk(0:nkk))
       vnk(:) = 0
       do i=1,nat
          iz = nzz(i)
          vnk(iz) = vnk(iz) + 1
          if (iz == 0) then
              write(atom(i)%lab,'(a,i0)')'Q',vnk(iz)  ! ghost atom
          else
              write(atom(i)%lab,'(a,i0)')trim(Species(iz)),vnk(iz)
          endif
       enddo
   endif
!
   end subroutine atom_string

!---------------------------------------------------------------------      

   logical function is_in_content(zspec)
!
!  La specie chimica con numero atomico zspec e' nel contenuto?
!
   USE General, only:nk,no
   integer, intent(in) :: zspec
!
   is_in_content = any(no(:nk) == zspec)
!
   end function is_in_content

!---------------------------------------------------------------------      

   subroutine merge_model(atom1,atom2,nmodif)
!
!  Combina atom1 con atom2. atom2 contiene in nz il valore di Z
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom1
   type(atom_type), dimension(:), intent(in)                 :: atom2
   integer, intent(out)                                      :: nmodif
   integer                                                   :: i,k
   integer                                                   :: nat1,nat2
   real                                                      :: epsx,epsb,epso
   integer                                                   :: natnew
!
   nat1 = numatomtype(atom1)
   nat2 = size(atom2)
   epsx = 0.00001
   epsb = 0.00001
   epso = 0.001
   nmodif = 0   ! contatore delle modifiche
!
!  Controlla se atom2 contiene piu' atomi
   if (nat2 > nat1) then
!
!      Aggiungi gli atomi in piu' in fondo alla lista di atom1
       call add_atoms_to_list(atom1,atom2(nat1+1:),natnew)
       atom1(nat1+1:)%nz = nz_from_znumber(atom1(nat1+1:)%nz)  ! genera nz da Z
       call set_intensity(atom1(nat1+1:))
       atom1(nat1+1:)%och = 1.0
       call calcola_occ(atom1(nat1+1:))
       nmodif = nmodif + nat2 - nat1
   endif
!
   do i=1,min(nat1,nat2)
!         
!     confronta coordinate 
      do k=1,3
         if (abs(atom1(i)%xc(k) - atom2(i)%xc(k)) >= epsx) then 
             atom1(i)%xc(k) = atom2(i)%xc(k)
             nmodif = nmodif + 1
         endif
      enddo
!
!     confronta b
      if (abs(atom1(i)%biso - atom2(i)%biso) >= epsb) then 
          atom1(i)%biso = atom2(i)%biso
          nmodif = nmodif + 1
      endif
!
!     confronta sof
      if (abs(atom1(i)%och - atom2(i)%och) >= epso) then 
          atom1(i)%och = atom2(i)%och
          nmodif = nmodif + 1
      endif
!
!     confronta specie chimica
      if (atomic_number(atom1(i)) /= atom2(i)%nz) then
          atom1(i:i)%nz = nz_from_znumber((/atom2(i)%nz/))
          nmodif = nmodif + 1
      endif
!
!     confronta label
      if (trim(atom1(i)%lab) /= trim(atom2(i)%lab)) then
          atom1(i)%lab = atom2(i)%lab
          nmodif = nmodif + 1
      endif
   enddo
!
   end subroutine merge_model

!---------------------------------------------------------------------      

   function plane_of_atoms(atom,veta,cart) result(coef)
!
!  Piano passante per gli atomi di veta se veta e' presente
!
   USE unit_cell
   USE general, only:cell
   USE cgeom
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, dimension(:), intent(in), optional            :: veta
   logical, intent(in)                                    :: cart
   real, dimension(4)                                     :: coef
   type(atom_type), dimension(:), allocatable             :: atomc
   integer                                                :: i
   integer                                                :: nvet
   real, dimension(:,:), allocatable                      :: xyza
!
!  numero di atomi su cui calcolae il piano
   if (present(veta)) then
       nvet = size(veta)
   else
       nvet = numatomtype(atom)
   endif
!
   if (nvet > 2) then
!
!      conversione in cartesiane
       call copia_atoms(atomc,atom)
       if (.not.cart) then
           call frac_to_cart(atomc,orthomatrix(cell))
       endif
!
!      riempi xyza per calcolo del piano
       allocate(xyza(3,nvet))
       if (present(veta)) then
           do i=1,nvet
              xyza(:,i) = atomc(veta(i))%xc(:)
           enddo
       else
           do i=1,nvet
              xyza(:,i) = atomc(i)%xc(:)
           enddo
       endif
         !xyza(:3,:) = xyz(:3,:nvet)
!
       select case (nvet) 
          case (3)
           coef(:) = plane3points(xyza(:,1),xyza(:,2),xyza(:,3),1)
 
          case (4:)
            call lsqplane(xyza,coef)

       end select
   else
       coef(:) = 0
   endif
!
   end function plane_of_atoms

!-------------------------------------------------------------------------------------------------

   subroutine apply_symmetry_legm(atoms,infos,legm,legms)
!
!  Applica la simmetria ai legami usando infos
!
   USE connect_mod
   USE Symmetry
   type(atom_type), dimension(:), allocatable, intent(in)            :: atoms
   type(symminfo_type), dimension(:), allocatable, intent(in) :: infos  ! info sulla simmetria
   type(bond_type), dimension(:), allocatable, intent(in)            :: legm   ! legm nella u.a.
   type(bond_type), dimension(:), allocatable, intent(inout)         :: legms  ! legm con simmetria
   integer                                                           :: numleg
   integer                                                           :: i,j,k,op1,op2
   integer                                                           :: n1,n2
   integer                                                           :: nat
   integer                                                           :: numaumax
   integer                                                           :: nleg_new
!
   numleg = numbonds(legm)
   nat = numatomtype(atoms)
   if (nat > 0 .and. numleg > 0) then
       numaumax = nsymtot*numleg*27
       call reallocate_legm(legms,numaumax,.false.)
       nleg_new = 0
       loop_bonds: do i=1,numleg      ! loop sui legami nell'u.a.
          n1 = legm(i)%n1
          n2 = legm(i)%n2
          do j=1,nat-1    ! loop su tutti gli atomi con simmetria
             if (infos(j)%asym == n1 .or. infos(j)%asym == n2) then
                op1 = infos(j)%opcode
                do k=j+1,nat
                   if (infos(k)%asym == n1 .or. infos(k)%asym == n2) then
                       op2 = infos(k)%opcode
                       if (op1 /= op2) cycle
                       nleg_new = nleg_new + 1
                       legms(nleg_new)%n1 = j
                       legms(nleg_new)%n2 = k
                       legms(nleg_new)%dist = legm(i)%dist
                       legms(nleg_new)%sigma = 0.3
                       if (numaumax == nleg_new) exit loop_bonds
                   endif
                enddo
             endif
          enddo
       enddo loop_bonds
       call reallocate_legm(legms,nleg_new)
   endif
!
   end subroutine  apply_symmetry_legm

!--------------------------------------------------------------------------------------------------

   subroutine expand_contacts(atomasym,cell,atom,infos,legm)
!
!  Espandi i contatti di atom
!
   USE connect_mod
   USE Symmetry, only:nsymTot
   USE realloc
   USE cgeom
   USE General, only:xgg
   USE unit_cell
   type(atom_type), dimension(:), intent(in)                            :: atomasym  ! unita' asimmetrica
   type(cell_type), intent(in)                                          :: cell
   type(atom_type), dimension(:), allocatable, intent(inout)            :: atom      ! atomi da espandere
   type(symminfo_type), dimension(:), allocatable, intent(inout) :: infos
   type(bond_type), dimension(:), allocatable, intent(inout)            :: legm
   type(atom_type), dimension(:), allocatable                           :: atomadd
   type(bond_type), dimension(:), allocatable                           :: legmadd
   type(atom_type), dimension(:), allocatable                           :: atomcart,atomcart1
   type(atom_type), dimension(1)                                        :: atomc
   type(atom_type)                                                      :: atomtra,atomsav
   integer                                                              :: opcode
   integer, dimension(size(atomasym))                                   :: zasym
   integer, dimension(size(atom))                                       :: zval
   type(bond_type)                                                      :: legnew
   integer                                                              :: nat,natasym
   integer                                                              :: nlegmax
   integer                                                              :: nleg
   integer                                                              :: k1,k2,k3
   integer, dimension(3)                                                :: ktra
   integer                                                              :: ka
   integer                                                              :: i,k
   real                                                                 :: tolmin=0.0,tolmax=0.0
   logical                                                              :: distp
   integer                                                              :: kk
!corr   real                                                                 :: dist
   integer, dimension(3)                                                :: ktramax, ktramin
   real, dimension(3)                                                   :: xtra
   real, dimension(3)                                                   :: bar
   real                                                                 :: rad
   real, dimension(3) :: dx
   real, parameter :: D2MIN = 0.6*0.6  ! square of minimum distance
!
   natasym = size(atomasym)
   nat = size(atom)
!
!  Prendi la traslazione minima e massima
   call get_limit_translation(infos%opcode,ktramin,ktramax)
!
!  Allarga di 2 i limiti di traslazione
   ktramin(:) = ktramin(:) - 1
   ktramax(:) = ktramax(:) + 1
!
!  Alloca ad un numero sufficientemente grande
   nlegmax = 4*nat
   call reallocate_legm(legmadd,nlegmax,.false.)
   call reallocate(atomadd,nlegmax,.false.)
   call reallocate_infos(infos,nat+nlegmax,.true.)
!
!  converti in cartesiane
   call frac_to_cart_copy(atom,atomcart,cell%get_ortom())
   call frac_to_cart_copy(atomasym,atomcart1,cell%get_ortom())
!
   call get_radius_molecule(atomcart,rad,bar)  ! calcola raggio e baricentro
!
!  estrai numeri atomi
   zasym(:) = atomic_number(atomasym)
   zval(:) = atomic_number(atom)
   nleg = 0
!
!  Applica operatori all' u.a. (atomasym) e cerca legami con atom
   loop_asym: do i=1,natasym   
      do k=1,nsymTot
         atomc = atomasym(i)
         call applica_sym_oper(k,atomc)
         call translate_in_cell(atomc,xtra)
         do k1=ktramin(1),ktramax(1)
            do k2=ktramin(2),ktramax(2)
               do k3=ktramin(3),ktramax(3)
                  ktra = (/k1,k2,k3/)
                  opcode = operator_code(k,nint(ktra+xtra))
                  !!!!if (any(infos(:nat)%opcode == opcode)) cycle
                  if (checkeq_infos(symminfo_type(i,k,opcode),infos(:nat))) cycle
                  atomsav = atomc(1)
                  atomsav%xc = atomsav%xc + ktra  ! conserva in atomsav le coord. cryst.
                  atomtra = cartesian_coord(atomsav,cell%get_ortom())
!
!                 check per atomi in posizione speciale
                  distp = .true.
                  do kk=1,nat
                     if (infos(kk)%asym == i) then
                         dx(:) = atomtra%xc-atomcart(kk)%xc
                         !dist = distanzaC(atomtra%xc,atomcart(kk)%xc)
                         if (dot_product(dx,dx) < D2MIN) then
                             distp = .false.
                             exit
                         endif
                     endif
                  enddo
                  if (distp) then
                      do kk=1,nleg
                         if (infos(nat+kk)%asym == i) then
                             !dist = distanzaC(atomsav%xc,atomadd(kk)%xc,xgg)
                             dx(:) = atomsav%xc-atomadd(kk)%xc
                             !if (dist < 0.01) then
                             if (dot_product(dx,matmul(xgg,dx)) < D2MIN) then
                                 distp = .false.
                                 exit
                             endif
                         endif
                      enddo
                  endif
!
                  if (distp) then
!
!                     Controlla se l'equivalente è legato ad un atomo
                      if (distanzaC(bar,atomtra%xc) > rad + 4) cycle  
                      do ka=1,nat
                         call create_bond(nat+nleg+1,ka,atomtra%xc,atomcart(ka)%xc,zasym(i),zval(ka),tolmin,tolmax,legnew)
                         if (legnew%n1 > 0) then  ! se 0 non c'è legame tra i due atomi
                             nleg = nleg + 1
                             legmadd(nleg) = legnew
                             atomadd(nleg) = atomsav
                             infos(nat+nleg)%asym = i
                             infos(nat+nleg)%oper = k
                             infos(nat+nleg)%opcode = opcode
                             if (nleg == nlegmax) exit loop_asym  ! troppi legami! esci.
                         endif
                      enddo
                  endif
               enddo
            enddo
         enddo
      enddo
   enddo loop_asym
!
   call reallocate_infos(infos,nat+nleg)
   if (nleg > 0) then
       call add_atoms_to_list(atom,atomadd(:nleg),nat)
       call reallocate_legm(legmadd,nleg)
       call combine_legm(legm,legmadd)
   endif
!
   end subroutine expand_contacts

!------------------------------------------------------------------------------------------      

   subroutine duplicate_atoms(atom,legm,vet)
!
!  Duplicate selected atoms with random traslation
!
   USE connect_mod
   USE realloc
   USE rand_mod
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom  
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, dimension(:), optional                           :: vet  ! atoms to duplicate
   integer                                                   :: nat, natnew
   type(atom_type), dimension(:), allocatable                :: atomadd
   type(bond_type), dimension(:), allocatable                :: legmadd
   integer                                                   :: nlegadd
   integer                                                   :: i
   integer                                                   :: nleg, natadd, nvet
!
   nat = numatomtype(atom)
   if (nat > 0) then
       if (present(vet)) then
!
!          extract selected atoms
           natadd = 0
           nvet = size(vet)
           allocate(atomadd(nvet))
           do i=1,nvet
              if (vet(i) > 0 .and. vet(i) <= nat) then
                  natadd = natadd + 1
                  atomadd(natadd) = atom(vet(i))
              endif
           enddo
           call translate_atoms(atomadd,randvalue(3,0.0,0.5))
           call translate_in_cell(atomadd)
!
!          extract selected bonds
           nleg = numbonds(legm)
           if (nleg > 0) then
               call extract_bonds(legm,vet,legmadd,nlegadd)
           endif
!
           call add_atoms_to_list(atom,atomadd,natnew,legm1=legm,legm2=legmadd)
           call atom_string(atom,vet=(/(i,i=nat+1,natnew)/))
       else
!
!          duplicate atoms
           natnew =nat*2
           call reallocate(atom,natnew,.true.)
           atom(nat+1:) = atom(1:nat)
           call translate_atoms(atom(nat+1:),randvalue(3,0.0,0.5))
           call translate_in_cell(atom(nat+1:))
!
!          duplicate bonds
           nleg = numbonds(legm)
           if (nleg > 0) then
               call reallocate_legm(legm,nleg*2,.true.)
               do i=1,nleg
                  legm(i+nleg) = legm(i) + nat
               enddo
           endif
           call atom_string(atom,vet=(/(i,i=nat+1,natnew)/))
       endif
   endif
!
   end subroutine duplicate_atoms

!---------------------------------------------------------------------      

   subroutine find_sssr(atom,legm,ring,nring)
!
!  Find the smallest set of smallest rings (SSSR) from a connection table. 
!  (J. Chem. Inf. Comput. Sci. 1993, 33, 657-662)
!
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in)        :: atom
   type(bond_type), dimension(:), allocatable, intent(in)        :: legm  
   type(connection_type), dimension(:), allocatable, intent(out) :: ring
   integer, intent(out)                                          :: nring
   type(bond_type), dimension(:), allocatable                    :: legmc, legmb, legmh
   type(connection_type), dimension(:), allocatable              :: connect
   integer                                                       :: nat, nleg
   logical                                                       :: found
   integer                                                       :: i,j,k, nl
   integer, dimension(:), allocatable                            :: vet2,vrem,vleg,vrem_min
   integer                                                       :: n1,n2, jpos1, jpos2
   integer                                                       :: nminstep
   integer, dimension(:), allocatable                            :: minpath 
   integer                                                       :: nrem, nrem_min, nlegh
   integer                                                       :: kat, rootat, kk, ka2
   integer                                                       :: nblock, nblock1, natblock, nlegb
   integer, dimension(:), allocatable                            :: block
   integer, dimension(1)                                         :: loc
   integer                                                       :: hide_atom 
   integer                                                       :: nring0
   integer, parameter                                            :: MAXAT_BLOCK = 100
   logical                                                       :: n2found
!#define PRINT_ENABLE 
#if defined(PRINT_ENABLE) 
   logical                                                       :: kpr = .false.
#endif
!   
   nring = 0
   nat = numatomtype(atom)
   nleg = numbonds(legm)
   if (nat == 0 .or. nleg == 0) return
   call copia_legm(legmc,legm)
   call legm_to_connect(nat,legm,connect)
   allocate(vrem(nat),vrem_min(nat))
!
!  Step 1: remove all acyclic atoms
   nrem = 0
   do i=1,nat
      call cycle_search(i,i,connect,found)
      if (.not.found) then
          nrem = nrem + 1
          vrem(nrem) = i
      endif
   enddo
   if (nrem == nat) return
   if (nrem > 0) then
#if defined(PRINT_ENABLE) 
       if (kpr) write(0,'(a,*(i4))')'Acycic atoms: ',vrem(:nrem)
#endif
       call remove_bond_from_atom(legmc,vrem(:nrem))
       call legm_to_connect(nat,legmc,connect)
   endif
!
!  Step 2: remove all open acyclic bonds from connection table
   nrem = 0
   allocate(vleg(nleg), source=0)
   do i=1,numbonds(legmc)
      n1 = legmc(i)%n1
      n2 = legmc(i)%n2
!
!     hide bond n1-n2
      do j=1,connect(n1)%nat    
         if (connect(n1)%pos(j) == n2) then
             connect(n1)%pos(j) = -connect(n1)%pos(j)
             jpos1 = j
             exit
         endif
      enddo
      do j=1,connect(n2)%nat    
         if (connect(n2)%pos(j) == n1) then
             connect(n2)%pos(j) = -connect(n2)%pos(j)
             jpos2 = j
             exit
         endif
      enddo
!     n1 and n2 are connected in cycle?
      call cycle_search(n1,n2,connect,found) 
      if (.not.found) then
#if defined(PRINT_ENABLE) 
          if (kpr) write(0,*)'Acyclic bonds:',n1,n2
#endif
          connect(n1)%pos(jpos1) = 0
          connect(n2)%pos(jpos2) = 0
          nrem = nrem + 1
          vleg(i) = 1
      endif
   enddo
   if (nrem > 0) then
       call remove_bondsv(legmc,vleg,1)
       call legm_to_connect(nat,legmc,connect)
   endif
!
!  Step 3: separate blocks saving info in vet2
   allocate(vet2(nat), source=0)
   nblock = 0
   nblock1 = -1
   do i=1,nat
      if (vet2(i) > 0) cycle                 
      if (connect(i)%nat > 0) then
          do j=1,nat
             if (vet2(j) > 0) cycle                 
             if (connect(j)%nat > 0) then
                 call cycle_search(i,j,connect,found)
                 if (found) then                 
                     if (i /= nblock1) nblock = nblock + 1
                     vet2(j) = nblock
                     nblock1 = i
                 endif
             endif
          enddo
      endif
   enddo
!
!  Number of rings (fail for 1501992.cif)
   nring = nblock + count(connect(:)%nat == 3)/2 + count(connect(:)%nat == 4) 
   call reallocate_connect(ring,nring,.false.)
!
!  Step 4: find rings
   allocate(block(nat))
   allocate(minpath(nat)) 
   nring0 = 0
   block_loop: do i=1,nblock
!
!     Get a block
      natblock = 0
      hide_atom = 0
      do j=1,nat
         if (vet2(j) == 0) cycle
         if (vet2(j) == i) then
             natblock = natblock + 1
             block(natblock) = j
         endif
      enddo
!
!     Get connectivity only for block i
      nrem = 0
      do j=1,nat
         if (vet2(j) /= i) then
             nrem = nrem + 1
             vrem(nrem) = j
         endif
      enddo
      call copia_legm(legmb,legmc)
      call remove_bond_from_atom(legmb,vrem(:nrem))
      call legm_to_connect(nat,legmb,connect)
#if defined(PRINT_ENABLE) 
      if (kpr) write(0,'(a,i0,a,i0,a,i0,a,*(i4))')    &
              'block n. ',i,' (',natblock,' atoms, ',size(legmb),' bonds): ',block(:natblock)
#endif
!
!     Check for anomalies in the block: wrong connectivity, random atoms, ...
      if (natblock > MAXAT_BLOCK .and. size(legmb) > 6*natblock) cycle
!
      rootat_loop: do   ! loop on root atoms in the block i
!
!        Choice of the root atom with the smallest ring connectivity
         loc = minloc(connect(:)%nat,mask=connect(:)%nat > 1)
         rootat = loc(1)
         if (rootat == 0) exit rootat_loop  ! (*) see personal notes
         if (connect(rootat)%nat > 4) exit
!
!        Search the smallest ring containing the root atom
         call minpath_find(rootat,rootat,connect,minpath,nminstep)
#if defined(PRINT_ENABLE) 
         if (kpr) write(0,'(i4,a,*(i4))')nminstep, 'ring: ',minpath(:nminstep)
#endif
!corr-bug 17/1/2014         if (nring0 <= nring) then
         if (nring0 < nring) then
             if (nminstep >= 3) then   ! (*) possible if rootat is an N1/N2 atom not previusly removed
                 nring0 = nring0 + 1
                 ring(nring0)%nat = nminstep
                 allocate(ring(nring0)%pos(nminstep), source=minpath(:nminstep))
             endif
         else
             exit block_loop ! errore: nring0 > nring
         endif
!
         if (hide_atom > 0) then
!
!            Restore hidden atom
#if defined(PRINT_ENABLE) 
             if (kpr) write(0,*)'restore hidden atom: ',hide_atom
#endif
             do nl=1,nlegh
                if (legmh(nl)%n1 == hide_atom) then
                    ka2 = legmh(nl)%n2
                else
                    ka2 = legmh(nl)%n1
                endif
                if (ka2 == rootat) cycle
                if (connect(ka2)%nat > 0) then
                    nlegb = size(legmb)
                    call reallocate_legm(legmb,nlegb+1)
                    legmb(nlegb+1) = legmh(nl)
                    !write(0,*)'ripristino :',legmh(nl)%n1,legmh(nl)%n2
                endif
             enddo
             call legm_to_connect(nat,legmb,connect)
             hide_atom = 0
             natblock = natblock + 1
         endif
!
!        Hiding the root atom with N3 or N4 connectivity
         if (connect(rootat)%nat == 3 .or. connect(rootat)%nat == 4) then
             call get_bonds_of_atom(legmb,rootat,vleg,nlegh)
             if (nlegh > 0) then
                 call reallocate_legm(legmh,nlegh)
                 do nl=1,nlegh
                    legmh(nl) = legmb(vleg(nl))
                 enddo
             endif
             hide_atom = rootat
#if defined(PRINT_ENABLE) 
             if (kpr) write(0,*)'hide atom:',rootat
#endif
         endif
!
!        Elimination of reducible atoms
         nrem_min = 0
         if (hide_atom > 0) then
             nrem_min = nrem_min + 1
             vrem_min(nrem_min) = hide_atom
         else
!
!            Remove the large sequence of atoms with connectivity N2
!            This loop should be on all atom not removed recursively. The if (*) evoid this 
             do j=1,nminstep
                kat = minpath(j)
                if (connect(kat)%nat == 2) then
                    nrem = 1
                    vrem(nrem) = kat
                    do k=1,nminstep-1
                       kk = mod(j+k,nminstep)
                       if (kk == 0) kk = nminstep
                       kk = minpath(kk)
                       if (connect(kk)%nat == 2) then
                           nrem = nrem + 1
                           vrem(nrem) = kk
                       else
                           exit
                       endif
                    enddo
                    if (nrem > nrem_min) then
                        nrem_min = nrem
                        vrem_min(:nrem) = vrem(:nrem)
                    endif
                endif
             enddo
         endif
!
         if (nrem_min > 0 .and. nrem_min /= natblock) then
             !connect(vrem_min(:nrem_min))%nat = 0  bug for cycles (6,5,6)  17/1/2014
             call connect_update_remove(connect,vrem_min(:nrem_min)) ! important to recognaze atoms N1
!
!            Test remaining N2 atoms
             n2found = .false.
             do j=1,nminstep
                kat = minpath(j)
                if (any(vrem_min(:nrem_min) == kat)) cycle
                if (connect(kat)%nat <= 2) then ! also N1 are possible, vedi appunti
                    n2found = .true.
!
!                   check if this atom belong to another cycle
                    call cycle_search(kat,kat,connect,found)
                    if (.not.found) then
#if defined(PRINT_ENABLE) 
                        if (kpr) write(0,*)'remove N2 atom:',kat
#endif
                        connect(kat)%nat = 0
                        nrem_min = nrem_min + 1
                        vrem_min(nrem_min) = kat
                    endif
                endif
             enddo
!
             if (hide_atom > 0 .and. .not.n2found) then
!
!                rare event: after hiding of rootat the path doesn't contain N2 atom
#if defined(PRINT_ENABLE) 
                 if (kpr) write(0,*)'Cannot find N2 atom after hiding ',hide_atom
#endif
                 exit
             endif
!
#if defined(PRINT_ENABLE) 
             if (kpr) write(0,'(a,*(i4))')'eliminate:',vrem_min(:nrem_min)
#endif
             natblock = natblock - nrem_min
             if (natblock == 0) exit rootat_loop
             call remove_bond_from_atom(legmb,vrem_min(:nrem_min))
             call legm_to_connect(nat,legmb,connect)
         else
             exit rootat_loop
         endif
      enddo rootat_loop
   enddo block_loop
!
!  update nring that could be different from nring 
   nring = nring0
!
#if defined(PRINT_ENABLE) 
   do i=1,nring
      write(0,'(a,i0,a,i0,a,*(1x,i0))')'ring n.',i,', ',ring(i)%nat,' atoms:',ring(i)%pos(:)
   enddo
#endif
!
   end subroutine find_sssr

!---------------------------------------------------------------------      

   subroutine bond_type_perception(atom,legm)
   USE connect_mod
   USE General, only:cell
   USE unit_cell
   USE cgeom
   USE elements
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm  
   type(connection_type), dimension(:), allocatable       :: ring, conn
   type(atom_type), dimension(:), allocatable             :: atomc
   integer                                                :: nat, nleg, nring
   integer :: i,j,k
   logical :: plan
   integer, dimension(:), allocatable :: varom
   integer, dimension(:), allocatable :: zval
   integer                            :: z1,z2
   integer, allocatable, dimension(:,:) :: tabconn
   real :: angle
   integer :: n1,n2,a1,a2,a3,c1,c2
   integer :: posb, ord_dist, ord_geom
   integer :: apos
   integer :: nchin
   integer, dimension(6) :: vchin
   logical :: match
   logical :: kpr = .false.
   type(chem_group_type) :: CO_group = chem_group_type(3,6,(/6,8,6,0/),(/1,2,1,0/),(/4,1,4,0/))      ! RR'C=0  (chetone)
   type(chem_group_type) :: COO_group = chem_group_type(3,6,(/8,8,6,0/),(/20,20,1,0/),(/1,2,4,0/))   ! -COOR
   type(chem_group_type) :: amide_group = chem_group_type(3,6,(/8,7,6,0/),(/2,1,1,0/),(/1,3,4,0/))   ! -CONRR'
   type(chem_group_type) :: NO2_group = chem_group_type(3,7,(/6,8,8,0/),(/1,2,2,0/),(/4,1,1,0/))     ! -NO2
   type(chem_group_type) :: CN2_group = chem_group_type(3,6,(/7,7,6,0/),(/20,20,1,0/),(/2,3,4,0/))   ! -CN2 (es. 3ptb)
   type(chem_group_type) :: CN3_group = chem_group_type(3,6,(/7,7,7,0/),(/20,20,20,0/),(/2,3,3,0/))  ! -CN3 (es. cime)
   type(chem_group_type) :: PO4_group = chem_group_type(4,15,(/8,8,8,8/),(/20,20,20,1/),(/1,1,1,2/)) ! -PO4
   type(chem_group_type) :: PO3_group = chem_group_type(4,15,(/8,8,8,6/),(/20,20,20,1/),(/1,1,1,4/)) ! -RPO3 (fosfonate, es.8atc)
   type(chem_group_type) :: sulfoN_group = chem_group_type(4,16,(/8,8,7,6/),(/2,2,1,1/),(/1,1,3,4/)) ! -SO2NRR' (es. hydrochloro)
!
   nat = numatomtype(atom)
   nleg = numbonds(legm)
   if (nat == 0 .or. nleg == 0) return
   allocate(zval(nat))
   zval(:) = atomic_number(atom)
   legm(:)%ord = 0

   call legm_to_connect(nat,legm,conn)
!
!  The algorithm judges the bond only if some species are present
   if (.not.is_atomic_specie(atom,[C_at,N_at,P_at,S_at])) then
       legm(:)%ord = 1
       return
   endif
!
!
!  Assign bond types for H (Z=1) and halogens (Z=9,17,35,53)
   do i=1,nleg
      z1 = zval(legm(i)%n1)
      z2 = zval(legm(i)%n2)
      if (any((/1,9,17,35,53/) == z1) .or. any((/1,9,17,35,53/) == z2)) then
          legm(i)%ord = 1
      endif
   enddo
!
!  Assign bond types for O (Z=8) and S (Z=16) where conn=2
   do i=1,nleg
      z1 = zval(legm(i)%n1)
      z2 = zval(legm(i)%n2)
      if ((z1 == 8 .or. z2 == 16) .and. conn(legm(i)%n1)%nat == 2) then
          legm(i)%ord = 1
          cycle
      endif
      if ((z2 == 8 .or. z2 == 16) .and. conn(legm(i)%n2)%nat == 2) then
          legm(i)%ord = 1
      endif
   enddo
!
   call frac_to_cart_copy(atom,atomc,orthomatrix(cell)) ! converti in cartesiane
!
   allocate(tabconn(nat,nat))
   call bond_to_tabconn(legm,tabconn)
!
!  Search for linear functional groups
   do i=1,nleg
      if (legm(i)%ord /= 0) cycle
      n1 = legm(i)%n1
      n2 = legm(i)%n2
      c1 = conn(n1)%nat
      c2 = conn(n2)%nat
      if (c1 == 2 .or. c2 == 2) then
          if (c1 + c2 == 4) then 
!
!             case R-CC-R, R-N=N=N, R-N=C=S
!!!!!!!!!!!!!!FIXME
              a1 = conn(n1)%pos(1)
              a2 = conn(n1)%pos(2)
              angle = rtod*angleC(atomc(a1)%xc,atomc(n1)%xc,atomc(a2)%xc)
              !write(0,*)'angle=',a1,n1,a2,angle
              !if (angle >= 175) then
              !    call get_info_distance(atomc(a1),atomc(n1),btype)
              ! !   if (btype
              !    write(0,*)'angle=',a1,n1,a2,angle,btype
              !    call get_info_distance(atomc(n1),atomc(a2),btype)
              !    write(0,*)'angle=',a1,n1,a2,angle,btype
              !endif
          elseif (c1 + c2 == 3) then ! case R-CC(H), R-CN, >=N=N
!
!             find sequence a1-a2-a3 with a3 terminal atom
              if (c1 == 1) then
                  a1 = conn(n2)%pos(1)
                  if (a1 == n1) a1 = conn(n2)%pos(2)
                  a2 = n2
                  a3 = n1
              else
                  a1 = conn(n1)%pos(1)
                  if (a1 == n2) a1 = conn(n1)%pos(2)
                  a2 = n1
                  a3 = n2
              endif
              !angle = rtod*angleC(atomc(a1)%xc,atomc(a2)%xc,atomc(a3)%xc)
              !if (angle >= 175) then
              if (is_linear(atomc(a1)%xc,atomc(a2)%xc,atomc(a3)%xc)) then
                  !write(0,*)'angle=',a1,a2,a3,' is linear'
                  if (zval(a2) == 7 .and. zval(a3) == 7 .and. zval(a1) == 6 .and. conn(a1)%nat == 3) then ! match with >=N=N
                      legm(tabconn(a1,a2))%ord = 2
                      legm(tabconn(a2,a3))%ord = 2
                  else
                      legm(tabconn(a1,a2))%ord = 1
                      legm(tabconn(a2,a3))%ord = 3
                  endif
              endif
          elseif (c1 + c2 == 5) then  ! alleni: >=C=<
!!!!!!!!!!!!!!FIXME
          endif
      endif
   enddo
!
!  Searching function groups with connectivity 4
   do i=1,nat
      if (conn(i)%nat == 4) then
          select case(zval(i))
            case (15) !Chemical groups of P
             call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,PO4_group,match)
             if (match) cycle
             call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,PO3_group,match)
             if (match) cycle

            case (16) !Chemical groups of S
             call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,sulfoN_group,match)
          end select
      endif
   enddo
!
!
   call find_sssr(atom,legm,ring,nring)
!
!  Rings are checked for aromaticity
   allocate(varom(nat), source=0)
   if (nring > 0) then
       do i=1,nring
          plan = all(conn(ring(i)%pos)%nat <= 3) ! in a aromatic ring the connectivity can't be more than 3
          if (plan) plan = ring_planarity(atomc,ring(i))
          if (plan) then
!
!             Init at -20 the bond type in ring
              do j=1,ring(i)%nat-1
                 posb = tabconn(ring(i)%pos(j),ring(i)%pos(j+1))
                 if (legm(posb)%ord == 0) legm(posb)%ord = -20   !!!FIXME - test on phuran
                 !legm(posb)%ord = -20
              enddo
              posb = tabconn(ring(i)%pos(ring(i)%nat),ring(i)%pos(1))
              if (legm(posb)%ord == 0) legm(posb)%ord = -20   !!!FIXME -test on phuran
              !legm(posb)%ord = -20 
              varom(ring(i)%pos) = 1
              if (kpr) write(0,'(a,*(i4))')'planar ring:',ring(i)%pos
          endif
       enddo
!
!      aromatic N with conn 3
       do i=1,nat
          if (varom(i) == 1 .and. zval(i) == N_at .and. conn(i)%nat == 3) then
              do j=1,conn(i)%nat
                 posb = tabconn(i,conn(i)%pos(j))
                 legm(posb)%ord = 10  !!!dopo prova con 1100765.cif
              enddo
          endif
       enddo
!
!      aromatic S
       do i=1,nat
          if (varom(i) == 1 .and. zval(i) == S_at) then
              do j=1,conn(i)%nat
                 posb = tabconn(i,conn(i)%pos(j))
                 legm(posb)%ord = 10 
              enddo
          endif
       enddo
   endif
!
!  Search for trigonal planar functional group
   do i=1,nat
      if (conn(i)%nat == 3 .and. varom(i) == 0) then
          if (is_trigonal(atomc(i)%xc,atomc(conn(i)%pos(1))%xc,atomc(conn(i)%pos(2))%xc,atomc(conn(i)%pos(3))%xc)) then
              select case (zval(i))
                case (6) !Chemical groups of C
!!!!!FIXME - match with chetone
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,CO_group,match)
                  if (match .and. kpr) write(0,*)match,'match chetone at ',i
                  if (match) cycle
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,COO_group,match)
                  if (match .and. kpr) write(0,*)match,'match COOR at ',i
                  if (match) cycle
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,amide_group,match)
                        !write(0,*)'atom=',i,'amide match=',match
                  if (match .and. kpr) write(0,*)match,'match amide at ',i
                  if (match) cycle
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,CN2_group,match)
                  if (match .and. kpr) write(0,*)match,'match CN2 at ',i
                  if (match) cycle
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,CN3_group,match)
                  if (match .and. kpr) write(0,*)match,'match CN3 at ',i
                  if (match) cycle

                case (7) !Chemical groups of N
                  call match_chemical_group(atomc,zval,legm,conn,tabconn,i,conn(i)%pos,NO2_group,match)
                  if (match .and. kpr) write(0,*)match,'match NO2 at ',i
                  if (match) cycle

                case (16) !Chemical groups of S
              end select
          endif
      endif
   enddo
!
   if (nring > 0) then
!
!      chinonic rule
       nchin = 0
       do i=1,nring
          if (ring(i)%nat == 6) then
              nchin = 0
              do j=1,ring(i)%nat
                 apos = ring(i)%pos(j)
                 if (zval(apos) == 6 .and. conn(apos)%nat == 3) then
                     do k=1,conn(apos)%nat
                        a1 = conn(apos)%pos(k)
                        if (zval(a1) == 8 .and. varom(a1) == 0) then
                            if (kpr) write(0,*)'chin. at. ',a1
                            nchin = nchin + 1
                            vchin(nchin) = apos
                            exit
                        endif
                     enddo
                 endif
              enddo
!!!! FIXME controlla legami semplici per i chinoni, sembra che non entrino nella coniugazione
              if (nchin == 2) then
                  do j=1,nchin
                     do k=1,conn(vchin(j))%nat
                        a1 = conn(vchin(j))%pos(k)
                        if (varom(a1) == 1) then
                            legm(tabconn(a1,vchin(j)))%ord = 1   !!!!-20
                            if (kpr) write(0,*)'leg=',a1,vchin(j),' single'
                        else
                            legm(tabconn(a1,vchin(j)))%ord = 2
                            if (kpr) write(0,*)'leg=',a1,vchin(j),' double'
                        endif
                     enddo
                  enddo
              endif
          endif
       enddo
   endif
!
!  Set order for remaining bonds
   do i=1,nleg
      if (legm(i)%ord == 0) then
          ord_dist = bond_type_from_table(legm(i),zval(legm(i)%n1),zval(legm(i)%n2))
          if (ord_dist >= 2) then
              call bond_type_from_geom(atomc,legm(i),conn,ord_geom)
              select case (ord_geom)
                 case (0,2) 
                   legm(i)%ord = -20
                 case default
                   legm(i)%ord = 1
              end select
          else
              legm(i)%ord = ord_dist
          endif
          !write(0,*)'legm=',legm(i)%n1,legm(i)%n2,legm(i)%ord
      endif
   enddo
!
!!!!!FIXME - test 1fx1.mol forse vanno aggiornate le distanze c-c
   call set_conjugation(legm,zval,nat,nleg)
!
!  Set as single all bonds connected to rings
   !do i=1,nat
   !   if (varom(i) == 1 .and. conn(i)%nat > 2) then
   !       do j=1,conn(i)%nat
   !          if (varom(conn(i)%pos(j)) == 0) then
   !              legm(tabconn(i,conn(i)%pos(j)))%ord = 1
   !          endif
   !       enddo
   !   endif
   !enddo
!
   where (legm(:)%ord == 10) 
          legm(:)%ord = 1 + 3
   elsewhere (legm(:)%ord == 20)
          legm(:)%ord = 2 + 3
   elsewhere (legm(:)%ord == 0)
          legm(:)%ord = 1
   endwhere
!
   if (kpr) then
       do i=1,nleg
          write(0,'(i4,1x,a,a,a,a,i0)')i,trim(atom(legm(i)%n1)%lab),'-',trim(atom(legm(i)%n2)%lab),' ord=',legm(i)%ord
       enddo
   endif
!
   end subroutine bond_type_perception

!---------------------------------------------------------------------      

   subroutine bond_type_from_geom(atom,leg,conn,order)
!
!  Extract bond order from geometry
!
!!!!FIXME - al momento non gestisce i tripli legami
   USE connect_mod
   USE cgeom
   USE trig_constants
   type(atom_type), dimension(:), intent(in)       :: atom
   type(bond_type), intent(in)                     :: leg  
   type(connection_type), dimension(:), intent(in) :: conn
   integer, intent(out)                            :: order
   integer                                         :: n1,n2,at1,at2
   real                                            :: angle
   integer                                         :: i,j
   real                                            :: anglemin
!
   n1 = leg%n1
   n2 = leg%n2
   order = 1
!
!  Don't check if atoms are connected with more then 2 bonds
   if (conn(n1)%nat <= 3 .and. conn(n2)%nat <= 3) then
       if (conn(n1)%nat == 1 .or. conn(n2)%nat == 1) then
           if (conn(n1)%nat == 1 .and. conn(n2)%nat == 1) then
               order = 0  ! orden can't be defined using geometrical consideration
           else
               if (conn(leg%n2)%nat == 1) then
                   n1 = leg%n2
                   n2 = leg%n1
               endif
               if (conn(n2)%nat == 3) then
                   if (is_trigonal(atom(n2)%xc,atom(conn(n2)%pos(1))%xc,   &
                              atom(conn(n2)%pos(2))%xc,atom(conn(n2)%pos(3))%xc)) order = 2
                                 !write(0,*)'trigonal=',n2,conn(n2)%pos,order
               else
                   angle = rtod*angleC(atom(conn(n2)%pos(1))%xc,atom(n2)%xc,atom(conn(n2)%pos(2))%xc)
                                 !write(0,*)'2angle=',conn(n2)%pos(1),n2,conn(n2)%pos(2),angle
                   if (abs(angle - 120) <= 3) then
                       order = 2
                   endif
                   !angle = 0
                   !do i=1,conn(n2)%nat
                   !   n3 = conn(n2)%pos(i)
                   !   if (n3 /= n1) then
                   !       angle = rtod*angleC(atom(n1)%xc,atom(n2)%xc,atom(n3)%xc)
                   !              write(0,*)'angle=',n1,n2,n3,angle
                   !   endif
                   !enddo
               endif
           endif
       else
           anglemin = huge(1.0)
           do i=1,conn(n1)%nat
              at1 = conn(n1)%pos(i)
              if (at1 /= n2) then
                  do j=1,conn(n2)%nat
                     at2 = conn(n2)%pos(j)
                     if (at2 /= n1) then
                         angle = angle_dihedral(atom(at1)%xc,atom(n1)%xc,atom(n2)%xc,atom(at2)%xc)
                         angle = min(abs(angle),abs(180-angle),abs(-180-angle))
                         if (angle < anglemin) anglemin = angle
                         !write(0,*)'angle=',at1,n1,n2,at2,angle,anglemin
                     endif
                  enddo
              endif
           enddo
           if (anglemin <= 15) then  ! find reason on Labute,2005 J.Chem.Inf.Mod
               order = 2
           endif
       endif
   endif
!
   end subroutine bond_type_from_geom

!---------------------------------------------------------------------      

   subroutine set_conjugation(legm,zval,nat,nleg)
!
!  Define bond type for conjugated system. This is a difficult task!
!
   USE connect_mod
   USE elements
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm  
   integer, dimension(:), intent(in)                         :: zval
   integer, intent(in)                                       :: nat, nleg
   integer, dimension(nat)                                   :: veta
   integer, dimension(nleg)                                  :: vetb,vram,block,vordmax
   integer                                                   :: i,j,k,l
   type(bond_type), dimension(:), allocatable                :: legmc
   type(connection_type), dimension(:), allocatable          :: connj
   integer                                                   :: nlc, nselb
   integer                                                   :: nram, numb, nlegc
   real                                                      :: costfmax, costf
   logical                                                   :: is_ram, is_nitrogen
   logical :: kpr = .false.
   integer :: lend
!
!  Isolate conjugated system in legmc
   call copia_legm(legmc,legm)
   where(legmc%ord /= -20) legmc%ord = 0
   call remove_bondsv(legmc,legmc%ord,0)
   call legm_to_connect(nat,legmc,connj)
   nlegc = numbonds(legmc)
   nram = 0
   do i=1,nlegc
      if (connj(legmc(i)%n1)%nat == 3 .or. connj(legmc(i)%n2)%nat == 3) then
          nram = nram + 1
          vram(nram) = i
          if (kpr) write(0,*)'ramificazione at. ',legmc(i)%n1,legmc(i)%n2
      endif
   enddo
!
   do i=1,nlegc
      if (legmc(i)%ord == -20) then
          legmc(i)%ord = 10
          veta(:) = 1
          vetb(:) = 0
          vetb(i) = 1
          call set_conjugation_rec(legmc,connj,i,veta,vetb,.false.)
!
!         Save in block the system of conjugated bonds and check for N
          numb = 0
          is_nitrogen = .false.
          do j=1,nlegc
             if (vetb(j) == 1) then
                 numb = numb + 1
                 block(numb) = j
                 if (.not.is_nitrogen) is_nitrogen = zval(legmc(j)%n1) == N_at .or. zval(legmc(j)%n2) == N_at
             endif
          enddo
               if (kpr) write(0,*)'is_nitrogen=',is_nitrogen
!
!         Just one bond: set it as double
          if (numb == 1) then
              legmc(block(numb))%ord = 20
              cycle
          endif
!
!         Ci sono ramificazioni nel blocco?
          if (nram > 0) then
              do j=1,numb
                 is_ram  = any(vram(:nram) == block(j))
                 if (is_ram) exit
              enddo
          else
              is_ram = .false.
          endif
!
!FIXME ad if on nram and Nitrogen in loop
          !if (nram > 0 .or. is_nitrogen) then
          !if (is_ram .or. is_nitrogen) then
!
!             Ci sono ramificazioni nell blocco
              !do j=1,numb
              !   is_ram  = any(vram(:nram) == block(j))
              !   if (is_ram) exit
              !enddo
              !if (is_ram .or. is_nitrogen) then
                  if (kpr) write(0,*)'conj system:',block(:numb)
                  costfmax = bond_weigth(legmc(block(:numb)),zval)
                  if (kpr) write(0,*)'start from: ',legmc(i)%n1,legmc(i)%n2,' CF=',costfmax
                  vordmax(:numb) = legmc(block(:numb))%ord
                        lend = 2
                  loop_reverse:  do l=1,lend
                  do j=1,numb
                     if (.not.(block(j) == i .and. l == 1)) then
                         nselb = block(j)
                         legmc(block(:numb))%ord = -20
                         legmc(nselb)%ord = 10
                         veta(:) = 1
                         vetb(:) = 0
                         vetb(nselb) = legmc(nselb)%ord/10
                         veta(legmc(nselb)%n1) = legmc(nselb)%ord/10
                         veta(legmc(nselb)%n2) = legmc(nselb)%ord/10
                         call set_conjugation_rec(legmc,connj,nselb,veta,vetb,l==2)
!
!                        Compute cost function
                         costf = bond_weigth(legmc(block(:numb)),zval)
                         if (kpr) then
                             write(0,*)'start from: ',legmc(nselb)%n1,legmc(nselb)%n2,' CF=',costf
                             do k=1,numb
                                write(0,*)k,'leg=',legmc(k)%n1,legmc(k)%n2,legmc(k)%ord
                             enddo
                         endif
                         if (costf > costfmax) then
                             costfmax = costf
                             vordmax(:numb) = legmc(block(:numb))%ord
                             if (kpr) write(0,*)'new max:',costfmax
                         endif
                     endif
              if (j==2 .and. .not.(is_ram .or. is_nitrogen)) exit loop_reverse  ! stop testing without ram and N
                  enddo
                     enddo loop_reverse
                  legmc(block(:numb))%ord = vordmax(:numb)
              !endif
          !endif
      endif
   enddo
!
   nlc = 0
   do i=1,nleg
      if (legm(i)%ord == -20) then
          nlc = nlc + 1
          legm(i)%ord = legmc(nlc)%ord 
      endif
   enddo
!
!corr   CONTAINS
!corr
!corr   real function cost_function()    result(cf)
!corr   cf = 0
!
!  N con singolo legame e' favorito
!corr   do k=1,nat
!corr      !!!!!if(connj(k)%nat > 0) write(0,*)k,'veta=',veta(k),' Z=',zval(k)
!corr      if (connj(k)%nat > 0 .and. zval(k) == N_at .and. veta(k) == H_at) cf = cf + 1
!corr   enddo
!
!corr   cf = cf + sum(legmc(block(:numb))%ord) / 10
!corr   !!!!!write(0,*)'start from: ',legmc(nselb)%n1,legmc(nselb)%n2,' CF=',costf
!corr   end function cost_function
!
   end subroutine set_conjugation
   
!---------------------------------------------------------------------      
  
   real function bond_weigth(leg,zval)  result(wei)
   USE connect_mod
   type(bond_type), dimension(:), intent(in) :: leg  
   integer, dimension(:), intent(in)         :: zval
   integer                                   :: i
   type(bond_type) :: legC_C = bond_type(6,6,1.33,0.3,20)
   type(bond_type) :: legC_N = bond_type(6,7,1.33,0.3,20)
!
   wei = 0
   do i=1,size(leg)
      if (is_bond_equal(leg(i),legC_C,zval)) then
          !write(0,*)'contr. leg=',i,leg(i)%n1,leg(i)%n2,8
          wei = wei + 8
      elseif (is_bond_equal(leg(i),legC_N,zval)) then
          !write(0,*)'contr. leg=',i,leg(i)%n1,leg(i)%n2,6
          wei = wei + 6
      endif
   enddo
!
   end function bond_weigth

!---------------------------------------------------------------------      

   recursive subroutine set_conjugation_rec(legm,conn,start,veta,vetb,rev)
   USE connect_mod
   type(bond_type), dimension(:), intent(inout)    :: legm  
   type(connection_type), dimension(:), intent(in) :: conn
   integer, intent(in)                             :: start 
   integer, dimension(:), intent(inout)            :: veta   ! order for each atoms
   integer, dimension(:), intent(inout)            :: vetb   ! if 1 the order has been assigned for bond
   logical, intent(in)                             :: rev
   type(bond_type)                                 :: leg
   integer                                         :: i
   integer                                         :: ini,fin,step
   leg = legm(start)
   if (rev) then
       ini = size(legm)
       fin = 1
       step = -1
   else
       ini = 1
       fin = size(legm)
       step = 1
   endif
   do i=ini,fin,step
      if (conn(legm(i)%n1)%nat == 0) exit ! chain terminated
      if (legm(i)%ord < 10 .and. legm(i)%ord >= 0) cycle   ! 0 < ord < 10
      if (i == start) cycle
      if (legm(i)%ord == 20 .or. legm(i)%ord == 10) cycle
      if ((legm(i)%n1 == leg%n1 .or. legm(i)%n2 == leg%n1) .or.     &
          (legm(i)%n1 == leg%n2 .or. legm(i)%n2 == leg%n2)) then
          if (veta(legm(i)%n1) == 2 .or. veta(legm(i)%n2) == 2) then
              legm(i)%ord = 10
          else
              legm(i)%ord = 20
              veta(legm(i)%n1) = 2
              veta(legm(i)%n2) = 2
          endif
          vetb(i) = 1
          call set_conjugation_rec(legm,conn,i,veta,vetb,rev)
      endif
   enddo
   end subroutine set_conjugation_rec
   
!---------------------------------------------------------------------      

   logical function is_trigonal(xc,x1,x2,x3)
!
!  Check if x1,x2,x3 have trigonal, planar geometry around xc
!
   USE cgeom
   real, dimension(3), intent(in) :: xc,x1,x2,x3
   real                           :: triple_product
!
   triple_product = dot_product(x1 - xc, cross_product(x2 - xc, x3 - xc))
   is_trigonal = abs(triple_product) <= 0.9
!
   end function is_trigonal

!---------------------------------------------------------------------      

   logical function is_linear(x1,x2,x3)
!
!  Check if x1,x2,x3 have linear geometry
!
   USE cgeom
   USE trig_constants, only:rtod
   real, dimension(3), intent(in) :: x1,x2,x3
!
   is_linear = rtod*angleC(x1,x2,x3) >= 170
!
   end function is_linear

!---------------------------------------------------------------------      

   subroutine match_chemical_group(atom,zval,legm,conn,tab,xc,xa,group,match)
   USE connect_mod
   USE cgeom
   type(atom_type), dimension(:), intent(in)       :: atom
   integer, dimension(:), intent(in)               :: zval
   type(bond_type), dimension(:), intent(inout)    :: legm  
   type(connection_type), dimension(:), intent(in) :: conn
   integer, dimension(:,:)                         :: tab
   integer                                         :: xc
   integer, dimension(:)                           :: xa
   type(chem_group_type)                           :: group
   logical, intent(out)                            :: match
   integer, dimension(group%geom)                  :: xvet
   integer                                         :: i,j
   integer                                         :: ngeom
   integer :: pos, bnat, cval, at
   integer, dimension(4) :: vat, jump
   integer :: btype
   integer :: posmin
   real :: dmin, dist
!
   match = .false.
!
!  Check on Z for xc
   match = zval(xc) == group%zc 
   if (match) then
!
!      Check on Z e connectivity on xa
       ngeom = group%geom
       xvet(:) = 0
       do i=1,ngeom
          do j=1,ngeom
             if (zval(xa(j)) == group%za(i) .and. conn(xa(j))%nat <= group%cval(i) .and. all(xvet(:) /= xa(j))) then
                 xvet(i) = xa(j)
                 exit
             endif
          enddo
          if (xvet(i) == 0) then
              match = .false.
              exit
          endif
       enddo
!
!      Check on connectivity
       if (match) then
           !write(0,*)'NEW: carbonilic at ',xc,xvet(:ngeom)
           jump(:) = 0
           do i=1,ngeom
              if (jump(i) == 1) cycle
              btype = group%btype(i)
              select case (btype)
                case (1,2,3)
                  pos = tab(xvet(i),xc)
!!!!!!!! FIXME - va migliorato: ripristina gli ord modificati prima di uscire
                  if (btype == 2) then
                      if (.not.is_bond_order(legm(pos),2,zval(xvet(i)),zval(xc),0.15)) then
!corr                          write(0,*)'rigettato:',bond_type_from_table(legm(pos),zval(xvet(i)),zval(xc))
                          match = .false.
                          return
                      endif
                  endif
                  legm(pos)%ord = group%btype(i)
!
!                 Set bond type also for the other bonds
                  do j=1,conn(xvet(i))%nat
                     at = conn(xvet(i))%pos(j)
                     if (at /= xc .and. zval(at) /= 6) then  ! not this for carbon
                         pos = tab(xvet(i),conn(xvet(i))%pos(j))
                         legm(pos)%ord = 1
                     endif
                  enddo

                case (20)  ! puo' essere 1 o 2
                  cval = group%cval(i)
                  bnat = 0
                  do j=1,ngeom
                     if (group%btype(j) == btype) then
                         if (conn(xvet(i))%nat <= cval) then
                             bnat = bnat + 1
                             vat(bnat) = xvet(j)
                         else
                             pos = tab(xvet(j),xc)
                             legm(pos)%ord = 1
                         endif
                         jump(j) = 1
                     endif
                  enddo
                  if (bnat == 1) then
                      pos = tab(vat(1),xc)
                      legm(pos)%ord = 1
                  else
                      dmin = distanzaC(atom(vat(1))%xc,atom(xc)%xc)
                      posmin = 1
                      do j=2,bnat
                         dist = distanzaC(atom(vat(j))%xc,atom(xc)%xc)
                         if (dist < dmin) then
                             posmin = j
                             dist = dmin
                         endif
                      enddo
                      !write(0,*)'due possibilita: calcola distanze ', vat(:bnat)
                      do j=1,bnat
                         pos = tab(vat(j),xc)
                         if (j == posmin) then
                             legm(pos)%ord = 2
                         else
                             legm(pos)%ord = 1
                         endif
                      enddo
                  endif
              end select
           enddo
       endif
   endif
!
   end subroutine match_chemical_group

!---------------------------------------------------------------------      

   logical function ring_planarity(atom,ring)
!
!  Determine wheter ring is planar or not.
!  Ref: works of Zhao and Sayle about perception of molecules, J. Chem. Inf. Coumput. Sci.
!
   USE connect_mod
   USE cgeom
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(connection_type), intent(in)                      :: ring
   integer                                                :: kat, rsize
   real                                                   :: ave, val, cutoff
   integer                                                :: i, n1, n2, n3, n4
!
   rsize = ring%nat
!
   ave = 0
   do i=1,rsize
      kat = mod(i-1,rsize)
      if (kat == 0) kat = rsize
      n1 = ring%pos(kat)
      n2 = ring%pos(i) 
      n3 = ring%pos(mod(i,rsize) + 1)
      n4 = ring%pos(mod(i+1,rsize) + 1)
      val = Angle_Dihedral(atom(n1)%xc,atom(n2)%xc,atom(n3)%xc,atom(n4)%xc)
      ave = ave + abs(val)
   enddo
   ave = ave / rsize
!
!  The same cutoff of fconv is used
   if (rsize <= 5) then
       cutoff = 10
   else
       cutoff = 15
   endif
   ring_planarity = ave <= cutoff
   !write(0,*)'ave=',ave,ring_planarity
!
   end function ring_planarity

!---------------------------------------------------------------------      

   real function ring_planarity_estimate(atom,ring) result(rps)
!
!  Estimate the planarity of ring
!
   USE connect_mod
   USE cgeom
   type(atom_type), dimension(:), allocatable, intent(in) :: atom 
   type(connection_type), intent(in)                      :: ring 
   integer                                                :: kat, rsize
   real                                                   :: val
   integer                                                :: i, n1, n2, n3, n4
!
   rsize = ring%nat
!
   rps = 0
   do i=1,rsize
      kat = mod(i-1,rsize)
      if (kat == 0) kat = rsize
      n1 = ring%pos(kat)
      n2 = ring%pos(i) 
      n3 = ring%pos(mod(i,rsize) + 1) 
      n4 = ring%pos(mod(i+1,rsize) + 1) 
      val = Angle_Dihedral(atom(n1)%xc,atom(n2)%xc,atom(n3)%xc,atom(n4)%xc)
      rps = rps + abs(val)
   enddo
   rps = rps / rsize
!
   end function ring_planarity_estimate

!---------------------------------------------------------------------      
!era MODULE geom3d
!Questo modulo contiene procedure per gestire operazioni geometriche su frammenti strutturali
!
!S coord_in_newcell                      Trasporta le coord. frazionarie in altra cella
!F Angle_Dihedral(ri,rj,rk,rn)           Calcola l'angolo di torsione definito dagli atomi ri,rj,rk,rl
!F rand_rotation_matrix(x)               Calcola Fast Random Rotation Matrix
!F rotation_matrix(l,m,n,theta)          Matrice di rotazione intorno ad un asse
!S rand_rotate_atoms(atom,xrot)          Ruota in modo random un insieme di atomi intorno al baricentro   
!S rotate_atoms(atmr,pp1,pp2,theta)      Ruota di theta un insieme di atomi intorno ad un asse pp2-pp1
!S Get_Cartesian_from_Z(ci,ri,rj,rk,rn)  Usata per conversione da Zmatrix a cartesiane

   subroutine coord_in_newcell(atom,cellold,cellnew)
!
!  Trasporta le coord. frazionarie da cellold in cellnew
!
   USE unit_cell
   type(atom_type), dimension(:), intent(inout) :: atom
   real, dimension(6), intent(in)               :: cellold,cellnew
!
!  Converti in cartesiane usando cellold
   call frac_to_cart(atom,orthomatrix(cellold))
!
!  Riconverti in coordinate frazionarie con la nuova cella
   call cart_to_frac(atom,orthomatrixi(cellnew))
!
   end subroutine coord_in_newcell

!----------------------------------------------------------------------------------------------------
   
   subroutine rand_rotate_atoms(atom,xrot)
!
!  Ruota in modo random un insieme di atomi intorno al baricentro   
!
   USE variables, only:ortom,ortoi
   USE cgeom
   type(atom_type), dimension(:), intent(inout) :: atom   !atomi prima della rotazione
   real, dimension(:), intent(in)           :: xrot   !3 random numbers tra 0-1 
   integer                                  :: i   
   real, dimension(3,3)                     :: rmat
   real, dimension(3)                       :: bar   
   integer                                  :: nat
!
   nat = size(atom) !numero di atomi
!
!  Converti il modello iniziale in coordinate cartesiane
   call frac_to_cart(atom,ortom)
!
!  Trasla il baricentro nell'origine   
   bar = baricentro(atom)
   do i=1,nat
      atom(i)%xc = atom(i)%xc - bar
   enddo
!
!  Genera matrice di rotazione random  
   !rmat = rand_rotation_matrix(real(xrot))   
   rmat = rand_rotation_matrix(xrot)   
!
!  Applica rotazione alle coordinate cartesiane   
   do i=1,nat
      atom(i)%xc = matmul(rmat,atom(i)%xc)
   enddo
!
!  Ritrasla il baricentro nella posizione originaria   
   do i=1,nat
      atom(i)%xc = atom(i)%xc + bar
   enddo
!
!  Ripristina in coordinate frazionarie      
   call cart_to_frac(atom,ortoi)   
!   
   end subroutine rand_rotate_atoms
      
!----------------------------------------------------------------------------------------------------
   
   subroutine rotate_atoms(atmr,pp1,pp2,theta,cart)
!
!  Ruota di theta un insieme di atomi intorno ad un asse per i punti p1 e p2
!
   USE variables, only:ortom,ortoi
   USE cgeom
   type(atom_type), dimension(:), intent(inout) :: atmr   ! atomi da routare
   real, dimension(3), intent(in)           :: pp1,pp2    ! 2 punti che individuano l'asse di rotazione
   real, intent(in)                         :: theta    ! angolo di rotazione in radianti
   logical, intent(in), optional            :: cart
   logical                                  :: carte
!   real, dimension(3)                       :: pd
!   real                                     :: spd
   real                                     :: l,m,n
   real, dimension(3,3)                     :: rmat
   integer                                  :: nat
   integer                                  :: i
   real, dimension(3)                       :: p1,p2
!
   if (present(cart)) then
       carte = cart
   else
       carte = .false.
   endif
!
!  p1 e p2 in coord cartesiane
   if (.not. carte) then
       !p1 = matmul(ortom,pp1)
       !p2 = matmul(ortom,pp2)
       p1 = pp1
       p2 = pp2
   else
       p1 = pp1
       p2 = pp2
   endif
!
!  Coseni direttori l,m,n
   !pd = p2 - p1   
   !spd = sqrt(sum(pd**2))
   !l = pd(1)/spd
   !m = pd(2)/spd
   !n = pd(3)/spd
   l = p2(1)
   m = p2(2)
   n = p2(3)
!
!  Calcola matrice di rotazione
   rmat = rotation_matrix(l,m,n,theta)
!
   nat = size(atmr) !numero di atomi
!
!  Converti il modello iniziale in coordinate cartesiane
   if (.not.carte) call frac_to_cart(atmr,ortom)
!
!  Fissa l'origine sull'atomo p1
   do i=1,nat
      atmr(i)%xc = atmr(i)%xc - p1
   enddo   
!
!  Applica rotazione alle coordinate cartesiane   
   do i=1,nat
      atmr(i)%xc = matmul(rmat,atmr(i)%xc)
   enddo
!
!  Ripristina l'origine
   do i=1,nat
      atmr(i)%xc = atmr(i)%xc + p1
   enddo      
!
!  Ripristina in coordinate frazionarie      
   if (.not.carte) call cart_to_frac(atmr,ortoi)   
!     
   end subroutine rotate_atoms      

!----------------------------------------------------------------------------------------------------

   integer function serial_number(atom) result(kser)
!
!  Extract the serial number of an atom
!
   USE atom_basic
   type(atom_type), intent(in) :: atom
   integer                     :: ier
!
   call get_from_label(atom%lab,numb=kser,ierror=ier)
   if (ier > 0) kser = -1  ! serial assente
!
   end function serial_number

!----------------------------------------------------------------------------------------------------

   subroutine remove_duplicate_labels(atom)
!
!  Remove duplicate labels
!
   USE strutil
   USE atom_basic
   type(atom_type), dimension(:), intent(inout) :: atom
   integer                                      :: i
   integer                                      :: maxs
   integer                                      :: numb,error
   character(len=100)                           :: prefix
!
   do i=1,size(atom)
      if (is_duplicated_label(atom,i)) then
          maxs = max_serial_number(atom,atom(i)%lab)
          call get_from_label(atom(i)%lab,prefix,numb,error)
          atom(i)%lab = trim(prefix)//i_to_s(maxs+1)
      endif
      !write(0,*)'Atom: ',trim(atom(i)%lab),max_serial_number(atom,atom(i)%lab),is_duplicated_label(atom,i)
   enddo
!
   !write(0,*)'NEW LABEL'
   !do i=1,size(atom)
   !   write(0,*)'new label:',trim(atom(i)%lab)
   !enddo
!
   end subroutine remove_duplicate_labels

!----------------------------------------------------------------------------------------------------

   logical function is_duplicated_label(atom,k)
!
!  For the label of atom k check if the label is duplicated
!
   USE strutil
   type(atom_type), dimension(:), intent(inout) :: atom
   integer, intent(in)                          :: k
   integer                                      :: i
!
   is_duplicated_label = .false.
   do i=1,k-1   ! check only up to k position
      is_duplicated_label = s_eqi(atom(i)%lab,atom(k)%lab)
      if (is_duplicated_label) exit
   enddo
!
   end function is_duplicated_label

!----------------------------------------------------------------------------------------------------

   integer function max_serial_number(atom,lab,prefix) result(maxserial)
!
!  Find the maximum serial number for a specified prefix of label
!
   USE strutil
   USE atom_basic
   type(atom_type), dimension(:), intent(inout) :: atom
   character(len=*), intent(in),optional        :: lab
   character(len=*), intent(in),optional        :: prefix
   integer                                      :: error
   character(len=SIZELAB)                       :: sprefix_lab
   integer                                      :: numb, numb_lab
   integer                                      :: i
   character(len=100)                           :: sprefix
!
   maxserial = 0
!
   if (present(lab)) then
       call get_from_label(lab,sprefix_lab,numb_lab,error)
   else
       sprefix_lab = trim(prefix)
   endif
!
   do i=1,size(atom)
      call get_from_label(atom(i)%lab,sprefix,numb,error)
      if (error == 0) then
          if (numb > maxserial .and. s_eqi(sprefix,sprefix_lab)) maxserial = numb
      endif
   enddo
!
   end function max_serial_number

!----------------------------------------------------------------------------------------------------

   elemental logical function is_metal(atom)
   USE elements
   type(atom_type), intent(in) :: atom
   integer, parameter :: notmetal(16) = [H_at,He_at,C_at,N_at,O_at,F_at,Ne_at,P_at,S_at,Cl_at,Ar_at,Br_at,Kr_at,I_at,Xe_at,Rn_at]
!
   is_metal = .not.any(notmetal(:) == atomic_number(atom))
!
   end function is_metal

!----------------------------------------------------------------------------------------------------

   real function density_value(mw,vcell,nop)
!
!  Compute density
!
   USE elements
   real, intent(in)                          :: mw     ! molecular mass of asymmetric unit
   real, intent(in)                          :: vcell  ! cell volume
   integer, intent(in)                       :: nop    ! number of symmetry operators
   real, parameter                           :: DENS_CONST = 1.660543780 !1/NA*10E-24
!
   density_value = DENS_CONST*nop*mw/vcell
!
   end function density_value

!--------------------------------------------------------------------

   real function volume_per_atom_cont(specv,nspecv,vcell) result(vol)
!
!  Compute volume per atom from cell content
!
   USE strutil
   character(len=*), dimension(:), intent(in) :: specv   ! specie coinvol
   integer, dimension(:), intent(in)          :: nspecv  ! num. di atomi per specie
   real, intent(in)                           :: vcell   ! volume della cella
   real                                       :: sumspec
   integer                                    :: i
!
!  Count non-hydrogen species
   sumspec = 0.
   do i=1,size(nspecv)
      if (.not.s_eqi(specv(i),'H ')) then
          sumspec = sumspec + nspecv(i)
      endif
   enddo
!
   if (sumspec > 0) then
       vol  = vcell  / sumspec
   else
       vol = 0.
   endif
!
   end function volume_per_atom_cont

!----------------------------------------------------------------------------------------------------

   real function volume_per_atom_at(atom,nsymt,vcell)  result(vol)
!
!  Compute volume per atom from asymmetric unit
!
   USE elements
   type(atom_type), dimension(:), intent(in) :: atom
   integer, intent(in)                       :: nsymt
   real, intent(in)                          :: vcell
   real                                      :: sumspec
   integer                                   :: i
!
!  Sum over non-H atoms
   sumspec = 0.
   do i=1,size(atom)
      if (.not.is_atomic_specie(atom(i),H_at)) then
          sumspec = sumspec + atom(i)%ocry*atom(i)%och
      endif
   enddo
   if (sumspec > 0) then
       vol  = vcell  / (nsymt*sumspec)
   else
       vol = 0.
   endif
   !vol = vcell / (nsymt * sum(atom%ocry*atom%och))
!
   end function volume_per_atom_at

!----------------------------------------------------------------------------------------------------

   real function linear_abs_coeff(atom,dens)   result(mu)
!
!  Compute the linear absorption coefficient
!
   USE elements
   USE ccryst
   type(atom_type), dimension(:), intent(in)     :: atom
   real, intent(in)                              :: dens
   type(element_type), dimension(:), allocatable :: elem
   integer                                       :: nat,nel,i,j
   real                                          :: mw   !!!!, dens
!
   call elements_from_atom(atom,elem)
   nel = numelem(elem)
   if (nel > 0) then
       nat = size(atom)
       do i=1,nel
          elem(i)%nw = 0
          do j=1,nat
             if (atom(j)%ptab == elem(i)%ptab) then
                 elem(i)%nw = atom(j)%och*atom(j)%ocry + elem(i)%nw
             endif
          enddo
       enddo
       elem(:)%nw = elem(:)%nw * elem(:)%weight
       mw = sum(elem(:)%nw)
       mu = (dens/mw) * sum(elem(:)%nw * elem%fCu(3))
   else
       mu = 0
   endif
!
   end function linear_abs_coeff

!----------------------------------------------------------------------------------------------------

   subroutine calcola_occ_new(atom,distmin)
   USE Symmetry
   USE cgeom
   USE General, only:xgg
   type(atom_type), dimension(:), intent(inout) :: atom
   real, intent(in)                             :: distmin
   real, dimension(3,nsymtot)                   :: xeq
   integer                                      :: i
   integer                                      :: nat   !,neqa
   integer, dimension(nsymtot)                  :: vets
   real, dimension(3)                           :: ktra
   real                                         :: dd
   integer                                      :: k1,k2
               !real,dimension(3) :: s
!
   nat = size(atom)
   do i=1,nat
      call get_equivalent1(atom(i)%xc,xeq)
      vets(:) = 1
      do k1=1,nsymtot
         if (vets(k1) == 1) then
             do k2=k1+1,nsymtot
                if (vets(k2) == 1) then
                    call xdisteqs(xeq(:,k1),xeq(:,k2),xgg,dd,ktra)
                    if (dd <= distmin) then
                        !write(0,*)'OP=',k1,k2,dd
                        vets(k2) = 0
                    endif
                endif
             enddo
         endif
      enddo
      atom(i)%ocry = sum(vets)/real(nsymtot)
   enddo
   do i=1,nat
      write(0,*)'OCC.'//trim(atom(i)%lab),atom(i)%ocry
   enddo
!
   end subroutine calcola_occ_new

!----------------------------------------------------------------------------------------------------

   subroutine compute_doc_all(atom,distmin)
   USE Symmetry
   USE cgeom
   USE General, only:xgg
   type(atom_type), dimension(:), intent(inout) :: atom
   real, intent(in)                             :: distmin
   real, dimension(3,nsymtot)                   :: xeq
   integer                                      :: nat
   integer                                      :: i,j,k
   real                                         :: dd
   real, dimension(3)                           :: ktra
!
   nat = size(atom)
   atom%ocry = 1.0
   do i=1,nat
      call get_equivalent1(atom(i)%xc,xeq)
      !do j=1,nat      !!!old
      do j=i,nat     !!!new
         if (atom(i)%nz == atom(j)%nz) then
             do k=1,nsymtot
                if (k==1 .and. j == i) cycle
                call xdisteqs(atom(j)%xc,xeq(:,k),xgg,dd,ktra)
                if (dd < distmin) then
                    atom(i)%ocry = atom(i)%ocry + abs(dd-distmin)
                    if (i /= j) atom(j)%ocry = atom(j)%ocry + abs(dd-distmin)   !!!new occ
                    !write(0,*)trim(atom(i)%lab)//'-'//trim(atom(j)%lab),abs(dd-distmin)
                endif
             enddo
         endif
      enddo
   enddo
   atom%ocry = 1/atom%ocry
   !do i=1,nat
   !   write(0,*)'OCC.'//trim(atom(i)%lab),atom(i)%ocry
   !enddo
!
   end subroutine compute_doc_all

!----------------------------------------------------------------------------------------------------

   subroutine compute_doc_vet(atom,distmin,vet)
   USE Symmetry
   USE cgeom
   USE General, only:xgg
   type(atom_type), dimension(:), intent(inout) :: atom
   real, intent(in)                             :: distmin
   integer, dimension(:), intent(in)            :: vet
   real, dimension(3,nsymtot)                   :: xeq
   integer                                      :: nat
   integer                                      :: i,j,k,ki,kj
   real                                         :: dd
   real, dimension(3)                           :: ktra
!
   nat = size(vet)
   atom(vet)%ocry = 1.0
   do ki=1,nat
      i = vet(ki)
      call get_equivalent1(atom(i)%xc,xeq)
      do kj=ki,nat  
         j = vet(kj)
         if (atom(i)%nz == atom(j)%nz) then
             do k=1,nsymtot
                if (k==1 .and. j == i) cycle
                call xdisteqs(atom(j)%xc,xeq(:,k),xgg,dd,ktra)
                if (dd < distmin) then
                    atom(i)%ocry = atom(i)%ocry + abs(dd-distmin)
                    if (i /= j) atom(j)%ocry = atom(j)%ocry + abs(dd-distmin)   !!!new occ
                    !write(0,*)trim(atom(i)%lab)//'-'//trim(atom(j)%lab),abs(dd-distmin)
                endif
             enddo
         endif
      enddo
   enddo
   atom(vet)%ocry = 1/atom(vet)%ocry
   !do i=1,nat
   !   write(0,*)'OCC.'//trim(atom(i)%lab),atom(i)%ocry
   !enddo
!
   end subroutine compute_doc_vet

!----------------------------------------------------------------------------------------------------

   subroutine find_duplicate_atoms(atom,legm,dist,vet,nd,kpr)
!
!  Find duplicate atoms in a distance dist
!
   USE Symmetry
   USE General, only:xgg
   USE cgeom
   USE atom_basic
   USE connect_mod
   type(atom_type), dimension(:), intent(in)              :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm  
   real, intent(in)                                       :: dist !minimum distance
   integer, intent(in)                                    :: kpr  !if  >=0 print is enabled
   integer, dimension(size(atom)), intent(out)            :: vet
   integer, dimension(size(atom),size(atom))              :: tabat
   type(connection_type), dimension(:), allocatable       :: conn
   integer                                                :: nd
   integer                                                :: i,j
   integer                                                :: nat
   integer, dimension(size(atom))                         :: groupat
   integer                                                :: natg
   real, dimension(3)                                     :: xc
   real, dimension(3)                                     :: ktra
   real                                                   :: dd
   integer                                                :: k, jat
   character(len=:), allocatable                          :: strl
   integer, dimension(1)                                  :: cmax
   integer                                                :: minb, minj
!
   nd = 0
   nat = size(atom)
!
!  Create table with 1 for couples (i,j) of atoms < dist
   tabat(:,:) = 0
   do i=1,nat-1
      loop_atom: do j=i+1,nat
         if (atom(i)%nz == atom(j)%nz) then
             do k=1,nsymtot
                xc = atom_symm(k,atom(i)%xc)
                call xdisteqs(atom(j)%xc,xc,xgg,dd,ktra)
                if (dd <= dist) then
                    tabat(i,j) = 1
                    cycle loop_atom
                endif
             enddo
         endif
      enddo loop_atom
   enddo 
!
   if (any(tabat(:,:) == 1)) then
       vet(:) = 0
       call legm_to_connect(nat,legm,conn)
       do i=1,nat-1
          if (any(vet(:nd) == i)) cycle
!
!         store in groupat atoms in the same position
          natg = 1
          groupat(natg) = i
          do j=i+1,nat
             !if (tabat(i,j) > 0) then
             if (tabat(i,j) > 0 .and. (all(vet(:nd) /= j)))then
                 natg = natg + 1
                 groupat(natg) = j
             endif
          enddo
          if (natg > 1) then
              if (kpr >= 0) then
                  strl = slabvet(groupat(:natg),atom(:)%lab)
                  write(kpr,'(a,a,a)')' WARNING: The atoms ',strl,' are in the same position'
              endif
              if (all(conn(groupat(2:natg))%nat == conn(groupat(1))%nat)) then 
!                 if atoms in group have the same number of bonds keep the atom bound to atom with lower connection
!                 so tetrahedra are preserved rispect to octahedra
                  cmax(1) = 1
                  minb = huge(1)
                  do j=1,natg
                     jat = groupat(j)
                         !write(0,*)'G=',groupat(:natg)
                     !write(0,*)jat,'VAL=',minval(conn(conn(jat)%pos)%nat)
                     if (conn(jat)%nat > 0) then  ! if jat is not an isolated atom
                         minj = minval(conn(conn(jat)%pos)%nat)
                         if (minj < minb) then
                             minb = minj
                             cmax(1) = j
                         endif
                     endif
                  enddo
                  !write(0,*)'non so cosa rimuovere, keep '//trim(atom(groupat(cmax(1)))%lab)
              else
!                 keep atom with higher connectivity
                  cmax = maxloc(conn(groupat(:natg))%nat)
                  !write(0,*)'rimuovo tutti tranne '//trim(atom(groupat(cmax(1)))%lab)
              endif
              do j=1,natg
                 if (j /= cmax(1)) then
                     nd = nd + 1
                     vet(nd) = groupat(j)
                 endif
              enddo
          endif
       enddo
            !write(0,*)'vet=',vet(:nd)
       if (nd > 0 .and. kpr >= 0) then
           write(kpr,'(/a,a)')' List of duplicate atoms: ',slabvet(vet(:nd),atom(:)%lab)
           write(kpr,'(a)')" Choose 'Delete Duplicate Atoms' in the menu Modify to delete these atoms"
       endif
   endif
!
   end subroutine find_duplicate_atoms

!----------------------------------------------------------------------------------------------------

   subroutine get_atoms_from_string_lvet(string,atom,vatom,val,defval,err)
!
!  Read string containing list of atoms and an optional numeric value
!
   USE strutil
   USE errormod
   character(len=*), intent(in)              :: string
   type(atom_type), dimension(:), intent(in) :: atom
   logical, dimension(:), intent(inout)      :: vatom   ! must be initialize outside to enable more call
   real, intent(out), optional               :: val
   real, intent(in), optional                :: defval
   type(error_type), intent(out)             :: err
   integer, dimension(size(atom))            :: iveta
   integer                                   :: natom,i
!
   call get_atoms_from_string_ivet(string,atom,iveta,natom,val,defval,err)
   do i=1,natom
      vatom(iveta(i)) = .true.
   enddo
!
   end subroutine get_atoms_from_string_lvet

!----------------------------------------------------------------------------------------------------

   subroutine get_atoms_from_string_ivet(string,atom,vatom,natom,val,defval,err)
!
!  Read string containing list of atoms and an optional numeric value
!
   USE strutil
   USE errormod
   character(len=*), intent(in)              :: string
   type(atom_type), dimension(:), intent(in) :: atom
   integer, dimension(:), intent(inout)      :: vatom 
   integer, intent(out)                      :: natom
   real, intent(out), optional               :: val
   real, intent(in), optional                :: defval
   type(error_type), intent(out)             :: err
   character(len=len_trim(string))           :: line
   character(len=20), dimension(size(atom))  :: wordv
   integer                                   :: nword
   logical                                   :: lval
   real                                      :: rval
   integer                                   :: i, nw, vpos
!
   natom = 0
   line = trim(adjustl(string))
   call s_filter(line)
   if (present(val)) val = defval
   if (len_trim(line) > 0) then
       call get_words(line,wordv,nword)
!
       nw = nword
!
!      Check if the last word is numeric
       if (present(val)) then
           call s_is_r(wordv(nword),rval,lval)
           if (lval) then
               nw = nword - 1
               val = rval
           endif
       endif
!
!      Real labels
       do i=1,nw
          vpos = position_stringv(wordv(i),atom(:)%lab)
          if (vpos == 0) then
              if (err%signal) then
                  call err%add(', '//wordv(i))
              else
                  call err%set('Undefined atom(s) '//trim(wordv(i)))
              endif
          else
              if (any(vatom(:natom) == vpos)) then  ! check for duplicate atom
                  call err%set('Duplicate atom: '//trim(wordv(i)))
                  return
              endif
              natom = natom + 1
              vatom(natom) = vpos
          endif
       enddo
   endif
!
   end subroutine get_atoms_from_string_ivet

!----------------------------------------------------------------------------------------------------

   subroutine change_bond_distance(atom,legm,k,dist,xgg)
!
!  Change bond distance of the bond k 
!
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom   ! all atoms 
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm   ! all bonds
   integer, intent(in)                                       :: k      ! pointer to bond to change
   real, intent(in)                                          :: dist
   real, dimension(3,3), intent(in)                          :: xgg
   type(connection_type), dimension(:), allocatable          :: connt
   integer, dimension(size(atom))                            :: vet1,vet2
   integer                                                   :: nat1,nat2
   integer                                                   :: nat
   logical                                                   :: in_ring
   real, dimension(3)                                        :: xtra
   integer                                                   :: i
   integer                                                   :: n1,n2
!
   n1 = legm(k)%n1
   n2 = legm(k)%n2
!corr   if (kpr > 0) write(0,*)'change bond ',k,trim(atom(n1)%lab)//'-'//trim(atom(n2)%lab)
   nat = numatomtype(atom)
   call legm_to_connect(nat,legm,connt)
   call get_atoms_legm(k,legm,connt,vet1,nat1,vet2,nat2,in_ring)
!
!  Compute translation
   xtra = (atom(vet1(1))%xc - atom(vet2(1))%xc)*(dist - legm(k)%dist) / legm(k)%dist
   if (in_ring) then
!
!      Move the less number of atoms
       call get_atoms_outside_ring(n1,n2,legm,connt,vet1,nat1)
       vet1(nat1+1) = n1; nat1 = nat1 + 1
       call get_atoms_outside_ring(n2,n1,legm,connt,vet2,nat2)
       vet2(nat2+1) = n2; nat2 = nat2 + 1
   endif
   !else
!
!  Apply translation to group with less atoms
   if (nat1 < nat2) then
       forall(i=1:nat1) atom(vet1(i))%xc = atom(vet1(i))%xc + xtra
       call bond_distance_update(n1,atom,legm,connt(n1),xgg)
       !write(0,*)'TRANS nat1'
   else  ! if nat1 >= nat2
       forall(i=1:nat2) atom(vet2(i))%xc = atom(vet2(i))%xc - xtra
       call bond_distance_update(n2,atom,legm,connt(n2),xgg)
       !write(0,*)'TRANS nat2'
   endif
   !endif
   !write(0,'(a,*(i5))')'chain 1:',vet1(:nat1)
   !write(0,'(a,*(i5))')'chain 2:',vet2(:nat2)
   !write(0,*)'IN RING=',in_ring
!
   end subroutine change_bond_distance

!----------------------------------------------------------------------------------------------------

   subroutine get_atoms_outside_ring(n1,n2,legm,conn,vetbond,nb)
!
!  For the bond n1-n2 in ring provide in vetbond the atoms connected to n1 outside the ring
!
   USE connect_mod
   integer, intent(in)                                          :: n1,n2
   type(bond_type), dimension(:), allocatable, intent(in)       :: legm
   type(connection_type), dimension(:), allocatable, intent(in) :: conn
   integer, dimension(:), intent(out)                           :: vetbond
   integer, intent(out)                                         :: nb
   integer                                                      :: i
   integer, dimension(size(conn))                               :: veta,vetb
   integer                                                      :: jat, kleg
   logical                                                      :: in_ring
   integer                                                      :: nata, natb
!
   nb = 0
   vetbond(:) = 0
   do i=1,conn(n1)%nat
      jat = conn(n1)%pos(i)
      if (jat == n2) cycle
      kleg = bond_position(legm,n1,jat)
      call get_atoms_legm(kleg,legm,conn,veta,nata,vetb,natb,in_ring)
      if (in_ring) cycle
      if (any(veta(:nata) == n2)) then
          !write(0,*)n1,jat,'vetb=',vetb(:natb)
          vetbond(vetb(:natb)) = vetb(:natb)
      else
          !write(0,*)n1,jat,'veta=',veta(:nata)
          vetbond(veta(:nata)) = veta(:nata)
      endif
   enddo
   if (count(vetbond > 0) > 0) then
       nb = count(vetbond > 0)
       vetbond(:nb) = pack(vetbond,mask=vetbond > 0)
       !write(0,*)'vetb=',vetbond(:)
   endif
!
   end subroutine get_atoms_outside_ring

!----------------------------------------------------------------------------------------------------
   
   subroutine change_bond_angle(k,angv,atom,legm,aval,cell)
!
!  Change bond angle of angle k
!
   USE connect_mod
   USE cgeom
   USE variables, only:ortom,ortoi
   USE unit_cell
   integer, intent(in)                                        :: k    ! order number of angle to modify
   type(atom_type), dimension(:), allocatable, intent(inout)  :: atom ! atoms
   type(bond_type), dimension(:), allocatable, intent(in)     :: legm ! bonds
   type(angle_type), dimension(:), allocatable, intent(inout) :: angv ! angles
   real, intent(in)                                           :: aval ! final value of angle k
   type(cell_type), intent(in)                                :: cell ! cell parameters
   type(angle_type)                                           :: ang
   integer                                                    :: nat,k1,k2
   type(connection_type), dimension(:), allocatable           :: connt
   integer, dimension(size(atom))                             :: vet12,vet21,vet23,vet32
   integer                                                    :: nat12,nat21,nat23,nat32
   logical                                                    :: in_ring1,in_ring2
   real, dimension(4)                                         :: xplan
   type(atom_type), dimension(size(atom))                     :: atmr
   type(atom_type), dimension(3)                              :: atang
   real                                                       :: diffa
   real                                                       :: dir
!
   nat = numatomtype(atom)
   call legm_to_connect(nat,legm,connt)
   ang = angv(k)
   k1 = bond_position(legm,ang%n1,ang%n2)
   call get_atoms_legm(k1,legm,connt,vet12,nat12,vet21,nat21,in_ring1)
   k2 = bond_position(legm,ang%n2,ang%n3)
   call get_atoms_legm(k2,legm,connt,vet23,nat23,vet32,nat32,in_ring2)
   ortom = cell%get_ortom()
   ortoi = cell%get_ortoi()
   atang = [cartesian_coord(atom(ang%n1)),cartesian_coord(atom(ang%n2)),cartesian_coord(atom(ang%n3))]
   xplan = plane3points(atang(1)%xc,atang(2)%xc,atang(3)%xc,ptype=1)
   if (in_ring1 .and. in_ring2) then
       atmr(1) = atom(ang%n1) 
       diffa = (ang%val-aval)*dtor/2
       call rotate_atoms(atmr(:1),atang(2)%xc,xplan(:3),diffa)
       atom(ang%n1) = atmr(1)
       atmr(1) = atom(ang%n3) 
       call rotate_atoms(atmr(:1),atang(2)%xc,xplan(:3),-diffa)
       atom(ang%n3) = atmr(1)
       angv(k)%val = aval
       call bond_angle_update(ang%n1,atom,angv,cell%get_g())
       call bond_angle_update(ang%n3,atom,angv,cell%get_g())
   else
       if (in_ring1) then
!corr          write(0,*)'bond 1 in ring',in_ring1+in_ring2     ! T
           if (legm(k2)%n1 == ang%n3) then
!corr               write(0,*)'1rotate ',vet23(:nat23)
               vet12(:nat23) = vet23(:nat23)
               nat12 = nat23
               dir = 1
           else
!corr               write(0,*)'2rotate ',vet32(:nat32)
               vet12(:nat32) = vet32(:nat32)
               nat12 = nat32
               dir = -1
           endif
       elseif(in_ring2) then
!
!          rotate n1 and connected atoms
!corr        write(0,*)'bond 2 in ring',in_ring1+in_ring2     ! T
           if (legm(k1)%n1 == ang%n1) then
               dir = 1
!corr               write(0,*)'1rotate ',vet12(:nat12)
           else
!corr               write(0,*)'2rotate ',vet21(:nat21)
               dir = -1
               vet12(:nat21) = vet21(:nat21)
               nat12 = nat21
           endif
      else
!
!          rotate the smallest number of atoms
           dir = 1
           if (legm(k1)%n1 /= ang%n1) then
               vet12(:nat21) = vet21(:nat21)
               nat12 = nat21
!corr               write(0,*)'1rotate ',vet12(:nat12)
           endif
           if (legm(k2)%n1 /= ang%n3) then
               vet23(:nat32) = vet32(:nat32)
               nat23 = nat32
!corr               write(0,*)'2rotate ',vet23(:nat23)
           endif
           if (nat12 > nat32) then
               vet12(:nat23) = vet23(:nat23)
               nat12 = nat23
               dir = -1
           endif
!corr           write(0,*)'rotate ',vet12(:nat12)
      endif
      atmr(:nat12) = atom(vet12(:nat12))
!corr      atang = [cartesian_coord(atom(ang%n1)),cartesian_coord(atom(ang%n2)),cartesian_coord(atom(ang%n3))]
!corr      xplan = plane3points(atang(1)%xc,atang(2)%xc,atang(3)%xc,ptype=1)
      call rotate_atoms(atmr(:nat12),atang(2)%xc,xplan(:3),dir*(ang%val-aval)*dtor)
      atom(vet12(:nat12)) = atmr(:nat12)
      call bond_angle_update(ang%n2,atom,angv,cell%get_g())
   endif
!
   end subroutine change_bond_angle

!----------------------------------------------------------------------------------------------------

   subroutine change_torsion_angle(tors,atom,legm,tval,cell,changed)
!
!  Rotate torsion angle
!
   USE connect_mod
   USE cgeom
   USE variables, only:ortom,ortoi
   USE unit_cell
   type(torsion_type), intent(inout)                         :: tors    ! torsion angle to modify
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom    ! atoms
   type(bond_type), dimension(:), allocatable, intent(in)    :: legm    ! bonds
   real, intent(in)                                          :: tval    ! final value of angle k
   type(cell_type), intent(in)                               :: cell    ! cell parameters
   logical, intent(out)                                      :: changed ! torsion angle was changed?
   integer                                                   :: nat
   type(connection_type), dimension(:), allocatable          :: connt
   integer                                                   :: kleg
   integer, dimension(size(atom))                            :: vet1,vet2
   integer                                                   :: nat1,nat2
   logical                                                   :: in_ring
   type(atom_type), dimension(size(atom))                    :: atmr
   real, dimension(3)                                        :: pa1,pa2
   real :: dir 
!
   nat = numatomtype(atom)
   call legm_to_connect(nat,legm,connt)
   kleg = bond_position(legm,tors%n2,tors%n3)
   call get_atoms_legm(kleg,legm,connt,vet1,nat1,vet2,nat2,in_ring)
   if (.not.in_ring) then
!
!      rotate the smallest number of atoms
       dir = 1
       if (nat2 < nat1) then
           vet1(:nat2) = vet2(:nat2)
           nat1 = nat2
           dir = -1
!corr           write(0,*)'change dir'
       endif
!corr       write(0,*)'1rotate: ',atom(vet1(:nat1))%lab,' di ',tval-tors%val,' from ',tors%val,' to ',norm_torsion(tval)
       ortom = cell%get_ortom()
       ortoi = cell%get_ortoi()
       pa1 = matmul(ortom,atom(tors%n2)%xc)
       pa2 = matmul(ortom,atom(tors%n3)%xc)
       atmr(:nat1) = atom(vet1(:nat1))
       call rotate_atoms(atmr(:nat1),pa1,direction_cos(pa1,pa2),dir*(tval - tors%val)*dtor)
       atom(vet1(:nat1)) = atmr(:nat1)
       tors%val = norm_torsion(tval)
       changed = .true.
   else
       changed = .false.
!corr       write(0,*)'bond in ring'
   endif
!
   end subroutine change_torsion_angle

!----------------------------------------------------------------------------------------------------
  
   real function norm_torsion(ang) 
!
!  Normalize torsion angle between -180 and 180
!
   real, intent(in) :: ang
   real :: mang
   mang = mod(ang+36000,360.)
   if (mang > 180) then
       norm_torsion = mang - 360
       !norm_torsion = mod(ang+36000,360.) - 360
   elseif (mang < -180) then
       norm_torsion = 360 + mang
       !norm_torsion = 360 + mod(ang,360.)
   else
       norm_torsion = mang
   endif
!
   end function norm_torsion

!----------------------------------------------------------------------------------------------------

   subroutine copy_atoms_sym(atoms,legms,symm,kop,symop,tcell,na,vet)
!
!  Copy atoms applying the sym operator symop and tcell. Duplicate atoms are not removed
!
   USE connect_mod
   USE spginfom
   type(atom_type), dimension(:), allocatable, intent(inout)     :: atoms
   type(bond_type), dimension(:), allocatable, intent(inout)     :: legms
   type(symminfo_type), dimension(:), allocatable, intent(inout) :: symm
   integer, intent(in)                                           :: kop
   type(symop_type), intent(in)                                  :: symop
   integer, dimension(3), intent(in)                             :: tcell
   integer, intent(in), optional                                 :: na
   integer, dimension(:), intent(in), optional                   :: vet
   type(atom_type), dimension(:), allocatable                    :: atom
   type(bond_type), dimension(:), allocatable                    :: legm
   integer :: nat, natnew, i, nleg
!
   nat = numatomtype(atoms)
!
   if (present(na)) then
       allocate(atom(na))
       atom = atoms(vet)
       if (numbonds(legms) > 0) call extract_bonds(legms,vet,legm,nleg)
   else
       call copia_atoms(atom,atoms)
       call copia_legm(legm,legms)
   endif
   call applica_sym_oper(atom,symop)
   call translate_atoms(atom,real(tcell))
!
   call add_atoms_to_list(atoms,atom,natnew,legms,legm)
   call reallocate_infos(symm,natnew)
   do i=nat+1,natnew
      symm(i)%asym = symm(i-nat)%asym
      symm(i)%oper = kop   ! FIXME: symm(i-nat)%oper + kop
      symm(i)%opcode = operator_code(kop,tcell)
   enddo
!
   end subroutine copy_atoms_sym

!----------------------------------------------------------------------------------------------------

   subroutine make_specie(atom,elem,radtype)
   USE elements
   USE symmetry, only:nsymtot
   USE nr
   type(atom_type), dimension(:), intent(inout)              :: atom
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   integer, intent(in)                                       :: radtype
   integer                                                   :: nat
   integer, dimension(1)                                     :: loc
   integer                                                   :: maxatom
   real                                                      :: maxinte
   integer                                                   :: i,j
   integer                                                   :: nelem,nelmin
   type(element_type), dimension(:), allocatable             :: el
   integer, dimension(:), allocatable                        :: iord
   real, dimension(:), allocatable                           :: elint,diff
   integer, dimension(:), allocatable                        :: ordiff
   real                                                      :: const
   integer                                                   :: ndiff,jj,jmin,kspec,kat
   integer, dimension(size(atom))                            :: ordat
   real, dimension(size(atom))                               :: atint
!
   nat = size(atom)
   if (nat == 0) return
!
   nelem = numelem(elem)
   if (nelem == 0) then
       atom%nz = 0
       return
   endif
!
!  Sort elements in array el
   allocate(el(nelem),source=elem)
   call ordina_elements(el,radtype,sort=-1)
!
   select case (radtype)
      case (RX_SOURCE)          ! raggi X
        allocate(elint(nelem), source=real(el%z))
        atint(:) = atom(:)%inte
      case (NEUTRON_SOURCE)     ! neutroni
        allocate(elint(nelem), source=abs(elem%fact)) 
        atint(:) = abs(atom(:)%inte)
      case (ELECTRON_SOURCE)    ! elettroni
        allocate(elint(nelem), source=elem%zeff) 
        atint(:) = atom(:)%inte
   end select
!
!  Create index of atoms sorted for intensity
   call indexx(atint,ordat)
   ordat = ordat(nat:1:-1)  ! from the larger
   maxinte = atint(ordat(1))
!
   const = maxinte / elint(1)
!
!  Do not assign H for X-Ray
   nelmin = nelem
   if (el(nelem)%z == H_at .and. radtype == RX_SOURCE) then
       nelmin = nelem - 1
   endif
!
   allocate(diff(nelmin),ordiff(nelmin))
   do i=1,nat
      kat = ordat(i)
!
      diff(:) = abs(const*elint(:nelmin) - atint(kat))
!
!     Find specie corresponding to minimum difference     
      call indexx(diff,ordiff)
      jmin = 0
      kspec = 0
      do j=1,nelmin
         jj = ordiff(j)
         if (el(jj)%nw > 0 .and. jj > jmin) then
             kspec = jj
             exit
         else
!            next jj must be > jmin (es. cannot assign P if all O have been assigned)
             jmin = jj  
         endif
      enddo
      if (kspec == 0) kspec = nelmin ! assign lighter specie
      atom(kat)%ptab = el(kspec)%ptab
      el(kspec)%nw = el(kspec)%nw - nsymtot*atom(kat)%ocry
   enddo
!
   atom(:)%nz = nz_from_pxen(atom%ptab,elem)
!
   end subroutine make_specie

!------------------------------------------------------------------------------------------

   subroutine make_random(atom)
!
!  Make random coordinates
!
   USE rand_mod
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   integer                                                   :: i
   do i=1,numatomtype(atom)
      atom(i)%xc = randvalue(3,0.,1.)
   enddo
   end subroutine make_random

END MODULE atom_type_util   
